@@ -37,6 +37,7 @@
 #include "bytestream-socks5.h"
 #include "connection.h"
 #include "debug.h"
+#include "disco.h"
 #include "namespaces.h"
 #include "presence-cache.h"
 #include "private-tubes-factory.h"
@@ -45,6 +46,9 @@
 G_DEFINE_TYPE (GabbleBytestreamFactory, gabble_bytestream_factory,
     G_TYPE_OBJECT);
 
+#define NB_SOCKS5_PROXIES_USED 5
+#define SOCKS5_PROXY_TIMEOUT 10
+
 /* properties */
 enum
 {
@@ -105,6 +109,31 @@ bytestream_id_free (gpointer v)
   g_slice_free (BytestreamIdentifier, bsid);
 }
 
+static GabbleSocks5Proxy *
+gabble_socks5_proxy_new (const gchar *jid,
+                         const gchar *host,
+                         const gchar *port)
+{
+  GabbleSocks5Proxy *proxy;
+
+  proxy = g_slice_new (GabbleSocks5Proxy);
+  proxy->jid = g_strdup (jid);
+  proxy->host = g_strdup (host);
+  proxy->port = g_strdup (port);
+
+  return proxy;
+}
+
+static void
+gabble_socks5_proxy_free (GabbleSocks5Proxy *proxy)
+{
+  g_free (proxy->jid);
+  g_free (proxy->host);
+  g_free (proxy->port);
+
+  g_slice_free (GabbleSocks5Proxy, proxy);
+}
+
 struct _GabbleBytestreamFactoryPrivate
 {
   GabbleConnection *conn;
@@ -131,6 +160,14 @@ struct _GabbleBytestreamFactoryPrivate
    * BytestreamIdentifier -> GabbleBytestreamMultiple */
   GHashTable *multiple_bytestreams;
 
+  /* List of GabbleSocks5Proxy discovered on the connection */
+  GSList *socks5_proxies;
+  /* List of GabbleSocks5Proxy found using the fallback-socks5-proxies param */
+  GSList *socks5_fallback_proxies;
+  /* List of SOCKS5's jids that have not been queried yet */
+  GSList *socks5_potential_proxies;
+  guint socks5_proxies_timer;
+
   gboolean dispose_has_run;
 };
 
@@ -173,6 +210,217 @@ gabble_bytestream_factory_init (GabbleBytestreamFactory *self)
       bytestream_id_equal, bytestream_id_free, g_object_unref);
 }
 
+static LmHandlerResult
+socks5_proxy_query_reply_cb (GabbleConnection *conn,
+                             LmMessage *sent_msg,
+                             LmMessage *reply_msg,
+                             GObject *obj,
+                             gpointer user_data)
+{
+  GabbleBytestreamFactory *self = GABBLE_BYTESTREAM_FACTORY (obj);
+  GabbleBytestreamFactoryPrivate *priv = GABBLE_BYTESTREAM_FACTORY_GET_PRIVATE (
+      self);
+  LmMessageNode *query, *streamhost;
+  const gchar *jid, *host, *port;
+  GabbleSocks5Proxy *proxy;
+  gboolean fallback = GPOINTER_TO_INT (user_data);
+
+  if (lm_message_get_sub_type (reply_msg) != LM_MESSAGE_SUB_TYPE_RESULT)
+    return LM_HANDLER_RESULT_REMOVE_MESSAGE;
+
+  query = lm_message_node_get_child_with_namespace (reply_msg->node, "query",
+      NS_BYTESTREAMS);
+  if (query == NULL)
+    return LM_HANDLER_RESULT_REMOVE_MESSAGE;
+
+  streamhost = lm_message_node_get_child (query, "streamhost");
+  if (streamhost == NULL)
+    return LM_HANDLER_RESULT_REMOVE_MESSAGE;
+
+  jid = lm_message_node_get_attribute (streamhost, "jid");
+  host = lm_message_node_get_attribute (streamhost, "host");
+  port = lm_message_node_get_attribute (streamhost, "port");
+
+  if (jid == NULL || host == NULL || port == NULL)
+    return LM_HANDLER_RESULT_REMOVE_MESSAGE;
+
+  proxy = gabble_socks5_proxy_new (jid, host, port);
+
+  if (fallback)
+    {
+      DEBUG ("Add fallback SOCKS5 proxy: %s %s:%s", jid, host, port);
+      priv->socks5_fallback_proxies = g_slist_prepend (
+          priv->socks5_fallback_proxies, proxy);
+    }
+  else
+    {
+      DEBUG ("Discovered SOCKS5 proxy: %s %s:%s", jid, host, port);
+      priv->socks5_proxies = g_slist_prepend (priv->socks5_proxies, proxy);
+    }
+
+  return LM_HANDLER_RESULT_REMOVE_MESSAGE;
+}
+
+static void
+send_proxy_query (GabbleBytestreamFactory *self,
+                  const gchar *jid,
+                  gboolean fallback)
+{
+  GabbleBytestreamFactoryPrivate *priv = GABBLE_BYTESTREAM_FACTORY_GET_PRIVATE (
+      self);
+  LmMessage *query;
+
+  DEBUG ("send SOCKS5 query to %s", jid);
+
+  query = lm_message_build (jid, LM_MESSAGE_TYPE_IQ,
+      '@', "type", "get",
+      '(', "query", "",
+        '@', "xmlns", NS_BYTESTREAMS,
+      ')', NULL);
+
+  _gabble_connection_send_with_reply (priv->conn, query,
+      socks5_proxy_query_reply_cb, G_OBJECT (self), GINT_TO_POINTER (fallback),
+      NULL);
+
+  lm_message_unref (query);
+}
+
+static void
+disco_item_found_cb (GabbleDisco *disco,
+                     GabbleDiscoItem *item,
+                     GabbleBytestreamFactory *self)
+{
+  if (tp_strdiff (item->category, "proxy") ||
+      tp_strdiff (item->type, "bytestreams"))
+    return;
+
+  send_proxy_query (self, item->jid, FALSE);
+}
+
+static void query_socks5_proxies (GabbleBytestreamFactory *self);
+
+static gboolean
+socks5_proxies_timeout_cb (gpointer data)
+{
+  GabbleBytestreamFactory *self = GABBLE_BYTESTREAM_FACTORY (data);
+  GabbleBytestreamFactoryPrivate *priv = GABBLE_BYTESTREAM_FACTORY_GET_PRIVATE (
+      self);
+
+  priv->socks5_proxies_timer = 0;
+
+  /* query more proxies if needed */
+  query_socks5_proxies (self);
+
+  return FALSE;
+}
+
+static void
+query_socks5_proxies (GabbleBytestreamFactory *self)
+{
+  GabbleBytestreamFactoryPrivate *priv = GABBLE_BYTESTREAM_FACTORY_GET_PRIVATE (
+      self);
+  guint nb_proxies_found;
+  guint nb_proxies_needed;
+  guint i;
+
+  nb_proxies_found = g_slist_length (priv->socks5_proxies) +
+    g_slist_length (priv->socks5_fallback_proxies);
+
+  if (nb_proxies_found >= NB_SOCKS5_PROXIES_USED)
+    {
+      DEBUG ("we already have discovered enough proxies (%u)",
+          nb_proxies_found);
+      return;
+    }
+
+  nb_proxies_needed = NB_SOCKS5_PROXIES_USED - nb_proxies_found;
+  DEBUG ("Need %u more proxies", nb_proxies_needed);
+
+  for (i = 0; i < nb_proxies_needed &&
+      priv->socks5_potential_proxies != NULL; i++)
+    {
+      gchar *jid;
+
+      jid = priv->socks5_potential_proxies->data;
+      send_proxy_query (self, jid, TRUE);
+
+      g_free (jid);
+      priv->socks5_potential_proxies = g_slist_delete_link (
+          priv->socks5_potential_proxies, priv->socks5_potential_proxies);
+    }
+
+  if (priv->socks5_potential_proxies != NULL && priv->socks5_proxies_timer == 0)
+    {
+      /* More proxies are available. Set a timer so we'll query then later if
+       * needed */
+      priv->socks5_proxies_timer = g_timeout_add_seconds (SOCKS5_PROXY_TIMEOUT,
+          socks5_proxies_timeout_cb, self);
+    }
+}
+
+static GSList *
+randomize_g_slist (GSList *list)
+{
+  guint len;
+  guint i;
+  GSList *new_head, *new_tail;
+
+  len = g_slist_length (list);
+  if (len <= 1)
+    return list;
+
+  i = g_random_int_range (0, len);
+  if (i == 0)
+    return list;
+
+  /* Cut the list at the i th position and make it the new head of the
+   * list */
+  new_tail = g_slist_nth (list, i - 1);
+  g_assert (new_tail != NULL);
+
+  new_head = new_tail->next;
+  g_assert (new_head != NULL);
+
+  new_tail->next = NULL;
+
+  return g_slist_concat (new_head, list);
+}
+
+static void
+conn_status_changed_cb (GabbleConnection *conn,
+                        TpConnectionStatus status,
+                        TpConnectionStatusReason reason,
+                        gpointer user_data)
+{
+  GabbleBytestreamFactory *self = GABBLE_BYTESTREAM_FACTORY (user_data);
+  GabbleBytestreamFactoryPrivate *priv = GABBLE_BYTESTREAM_FACTORY_GET_PRIVATE (
+      self);
+
+  if (status == TP_CONNECTION_STATUS_CONNECTED)
+    {
+      GStrv jids;
+      guint i;
+
+      /* we can't intialize socks5_potential_proxies in the constructor
+       * because Connection's properties are not set yet at this point */
+      g_object_get (priv->conn, "fallback-socks5-proxies", &jids, NULL);
+
+      for (i = 0; jids != NULL && jids[i] != NULL; i++)
+        {
+          priv->socks5_potential_proxies = g_slist_prepend (
+              priv->socks5_potential_proxies, g_strdup (jids[i]));
+        }
+
+      /* randomize the list to not always use the same proxies */
+      priv->socks5_potential_proxies = randomize_g_slist (
+              priv->socks5_potential_proxies);
+
+      query_socks5_proxies (self);
+
+      g_strfreev (jids);
+    }
+}
+
 static GObject *
 gabble_bytestream_factory_constructor (GType type,
                                        guint n_props,
@@ -208,6 +456,13 @@ gabble_bytestream_factory_constructor (GType type,
   lm_connection_register_message_handler (priv->conn->lmconn,
       priv->iq_socks5_cb, LM_MESSAGE_TYPE_IQ, LM_HANDLER_PRIORITY_FIRST);
 
+  /* Track SOCKS5 proxy available on the connection */
+  gabble_signal_connect_weak (priv->conn->disco, "item-found",
+      G_CALLBACK (disco_item_found_cb), G_OBJECT (self));
+
+  gabble_signal_connect_weak (priv->conn, "status-changed",
+      G_CALLBACK (conn_status_changed_cb), G_OBJECT (self));
+
   return obj;
 }
 
@@ -217,6 +472,7 @@ gabble_bytestream_factory_dispose (GObject *object)
   GabbleBytestreamFactory *self = GABBLE_BYTESTREAM_FACTORY (object);
   GabbleBytestreamFactoryPrivate *priv =
     GABBLE_BYTESTREAM_FACTORY_GET_PRIVATE (self);
+  GSList *proxies, *l;
 
   if (priv->dispose_has_run)
     return;
@@ -252,6 +508,27 @@ gabble_bytestream_factory_dispose (GObject *object)
   g_hash_table_destroy (priv->multiple_bytestreams);
   priv->multiple_bytestreams = NULL;
 
+  proxies = g_slist_concat (priv->socks5_proxies,
+      priv->socks5_fallback_proxies);
+
+  for (l = proxies; l != NULL; l = g_slist_next (l))
+    {
+      GabbleSocks5Proxy *proxy = (GabbleSocks5Proxy *) l->data;
+
+      gabble_socks5_proxy_free (proxy);
+    }
+  g_slist_free (proxies);
+
+  priv->socks5_proxies = NULL;
+  priv->socks5_fallback_proxies = NULL;
+
+  g_slist_foreach (priv->socks5_potential_proxies, (GFunc) g_free, NULL);
+  g_slist_free (priv->socks5_potential_proxies);
+  priv->socks5_potential_proxies = NULL;
+
+  if (priv->socks5_proxies_timer != 0)
+    g_source_remove (priv->socks5_proxies_timer);
+
   if (G_OBJECT_CLASS (gabble_bytestream_factory_parent_class)->dispose)
     G_OBJECT_CLASS (gabble_bytestream_factory_parent_class)->dispose (object);
 }
@@ -385,7 +662,8 @@ streaminit_parse_request (LmMessage *message,
                           gboolean *multiple)
 {
   LmMessageNode *iq = message->node;
-  LmMessageNode *feature, *x, *field, *stream_method, *si_multiple;
+  LmMessageNode *feature, *x, *si_multiple;
+  NodeIter i, j;
 
   *stream_init_id = lm_message_node_get_attribute (iq, "id");
 
@@ -432,8 +710,10 @@ streaminit_parse_request (LmMessage *message,
       return FALSE;
     }
 
-  for (field = x->children; field; field = field->next)
+  for (i = node_iter (x); i; i = node_iter_next (i))
     {
+      LmMessageNode *field = node_iter_data (i);
+
       if (tp_strdiff (lm_message_node_get_attribute (field, "var"),
             "stream-method"))
         /* some future field, ignore it */
@@ -449,9 +729,9 @@ streaminit_parse_request (LmMessage *message,
 
       /* Get the stream methods offered */
       *stream_methods = NULL;
-      for (stream_method = field->children; stream_method;
-          stream_method = stream_method->next)
+      for (j = node_iter (field); j; j = node_iter_next (j))
         {
+          LmMessageNode *stream_method = node_iter_data (j);
           LmMessageNode *value;
           const gchar *stream_method_str;
 
@@ -557,7 +837,8 @@ stream_method_supported (const gchar *stream_method)
 static GabbleBytestreamMultiple *gabble_bytestream_factory_create_multiple (
     GabbleBytestreamFactory *self, TpHandle peer_handle,
     const gchar *stream_id, const gchar *stream_init_id,
-    const gchar *peer_resource, GabbleBytestreamState state);
+    const gchar *peer_resource, const gchar *self_jid,
+    GabbleBytestreamState state);
 
 static GabbleBytestreamIBB *gabble_bytestream_factory_create_ibb (
     GabbleBytestreamFactory *fac, TpHandle peer_handle, const gchar *stream_id,
@@ -567,151 +848,19 @@ static GabbleBytestreamIBB *gabble_bytestream_factory_create_ibb (
 static GabbleBytestreamSocks5 *gabble_bytestream_factory_create_socks5 (
     GabbleBytestreamFactory *fac, TpHandle peer_handle, const gchar *stream_id,
     const gchar *stream_init_id, const gchar *peer_resource,
-    GabbleBytestreamState state);
+    const gchar *self_jid, GabbleBytestreamState state);
 
-/**
- * bytestream_factory_iq_si_cb:
- *
- * Called by loudmouth when we get an incoming <iq>. This handler is concerned
- * with Stream Initiation requests (XEP-0095).
- *
- */
-static LmHandlerResult
-bytestream_factory_iq_si_cb (LmMessageHandler *handler,
-                             LmConnection *lmconn,
-                             LmMessage *msg,
-                             gpointer user_data)
+static void
+si_tube_received (GabbleBytestreamFactory *self,
+                  LmMessage *msg,
+                  LmMessageNode *si,
+                  GabbleBytestreamIface *bytestream,
+                  TpHandle peer_handle,
+                  TpHandle room_handle,
+                  const gchar *stream_id)
 {
-  GabbleBytestreamFactory *self = GABBLE_BYTESTREAM_FACTORY (user_data);
   GabbleBytestreamFactoryPrivate *priv =
     GABBLE_BYTESTREAM_FACTORY_GET_PRIVATE (self);
-  TpHandleRepoIface *contact_repo = tp_base_connection_get_handles (
-      (TpBaseConnection *) priv->conn, TP_HANDLE_TYPE_CONTACT);
-  TpHandleRepoIface *room_repo = tp_base_connection_get_handles (
-      (TpBaseConnection *) priv->conn, TP_HANDLE_TYPE_ROOM);
-  LmMessageNode *si;
-  TpHandle peer_handle, room_handle;
-  GabbleBytestreamIface *bytestream = NULL;
-  GSList *l;
-  const gchar *profile, *from, *stream_id, *stream_init_id, *mime_type;
-  GSList *stream_methods = NULL;
-  gboolean multiple;
-  gchar *peer_resource = NULL;
-
-  if (lm_message_get_sub_type (msg) != LM_MESSAGE_SUB_TYPE_SET)
-    return LM_HANDLER_RESULT_ALLOW_MORE_HANDLERS;
-
-  si = lm_message_node_get_child_with_namespace (msg->node, "si", NS_SI);
-  if (si == NULL)
-    return LM_HANDLER_RESULT_ALLOW_MORE_HANDLERS;
-
-  /* after this point, the message is for us, so in all cases we either handle
-   * it or send an error reply */
-
-  if (!streaminit_parse_request (msg, si, &profile, &from, &stream_id,
-        &stream_init_id, &mime_type, &stream_methods, &multiple))
-    {
-      _gabble_connection_send_iq_error (priv->conn, msg,
-          XMPP_ERROR_BAD_REQUEST, "failed to parse SI request");
-      goto out;
-    }
-
-  DEBUG ("received a SI request");
-
-  peer_handle = tp_handle_lookup (contact_repo, from, NULL, NULL);
-  if (peer_handle == 0)
-    {
-      _gabble_connection_send_iq_error (priv->conn, msg,
-          XMPP_ERROR_JID_MALFORMED, NULL);
-      goto out;
-    }
-
-  room_handle = gabble_get_room_handle_from_jid (room_repo, from);
-  if (room_handle == 0)
-    {
-      /* JID is not a MUC JID so we need contact's resource */
-      gabble_decode_jid (from, NULL, NULL, &peer_resource);
-    }
-
-  if (multiple)
-    {
-      DEBUG ("Receiver supports multi bytestreams");
-
-      bytestream = (GabbleBytestreamIface *)
-          gabble_bytestream_factory_create_multiple (self, peer_handle,
-            stream_id, stream_init_id, peer_resource,
-            GABBLE_BYTESTREAM_STATE_LOCAL_PENDING);
-    }
-
-  /* check stream method */
-  for (l = stream_methods; l != NULL; l = l->next)
-    {
-      if (multiple)
-        {
-          if (stream_method_supported (l->data))
-            {
-              DEBUG ("add %s to multi-bytestream methods",
-                  (const gchar *) l->data);
-
-              gabble_bytestream_multiple_add_stream_method (
-                  GABBLE_BYTESTREAM_MULTIPLE (bytestream), l->data);
-            }
-          else
-            {
-              DEBUG ("skip unsupported stream method: %s",
-                  (const gchar *) l->data);
-            }
-        }
-      else
-        {
-          /* We create the stream according the stream method chosen.
-           * User has to accept it */
-          DEBUG ("Receiver doesn't support multi bytestreams. He chose %s",
-              (const gchar *) l->data);
-
-          bytestream = gabble_bytestream_factory_create_from_method (self,
-              l->data, peer_handle, stream_id, stream_init_id, peer_resource,
-              GABBLE_BYTESTREAM_STATE_LOCAL_PENDING);
-          if (bytestream != NULL)
-            break;
-        }
-    }
-
-  if (bytestream == NULL)
-    {
-      DEBUG ("SI request doesn't contain any supported stream methods.");
-
-      _gabble_connection_send_iq_error (priv->conn, msg,
-          XMPP_ERROR_SI_NO_VALID_STREAMS, NULL);
-      goto out;
-    }
-
-  if (multiple)
-    {
-      /* Is there at least one stream method? */
-      if (!gabble_bytestream_multiple_has_stream_method (
-            GABBLE_BYTESTREAM_MULTIPLE (bytestream)))
-        {
-          GError e = { GABBLE_XMPP_ERROR, XMPP_ERROR_SI_NO_VALID_STREAMS, "" };
-          DEBUG ("No valid stream method in the multi bytestream. Closing");
-
-          gabble_bytestream_iface_close (bytestream, &e);
-          goto out;
-        }
-    }
-
-  /* Now that we have a bytestream, it's responsible for declining the IQ
-   * if needed. */
-
-  /* We inform the right factory we received a SI request */
-  if (tp_strdiff (profile, NS_TUBES))
-    {
-      GError e = { GABBLE_XMPP_ERROR, XMPP_ERROR_SI_BAD_PROFILE, "" };
-      DEBUG ("SI profile unsupported: %s", profile);
-
-      gabble_bytestream_iface_close (bytestream, &e);
-      goto out;
-    }
 
   /* A Tubes SI request can be:
    *  - a 1-1 new tube offer
@@ -761,10 +910,193 @@ bytestream_factory_iq_si_cb (LmMessageHandler *handler,
       NODE_DEBUG (msg->node, e.message);
       gabble_bytestream_iface_close (bytestream, &e);
     }
+}
+
+/**
+ * bytestream_factory_iq_si_cb:
+ *
+ * Called by loudmouth when we get an incoming <iq>. This handler is concerned
+ * with Stream Initiation requests (XEP-0095).
+ *
+ */
+static LmHandlerResult
+bytestream_factory_iq_si_cb (LmMessageHandler *handler,
+                             LmConnection *lmconn,
+                             LmMessage *msg,
+                             gpointer user_data)
+{
+  GabbleBytestreamFactory *self = GABBLE_BYTESTREAM_FACTORY (user_data);
+  GabbleBytestreamFactoryPrivate *priv =
+    GABBLE_BYTESTREAM_FACTORY_GET_PRIVATE (self);
+  TpHandleRepoIface *contact_repo = tp_base_connection_get_handles (
+      (TpBaseConnection *) priv->conn, TP_HANDLE_TYPE_CONTACT);
+  TpHandleRepoIface *room_repo = tp_base_connection_get_handles (
+      (TpBaseConnection *) priv->conn, TP_HANDLE_TYPE_ROOM);
+  LmMessageNode *si;
+  TpHandle peer_handle = 0, room_handle;
+  GabbleBytestreamIface *bytestream = NULL;
+  GSList *l;
+  const gchar *profile, *from, *stream_id, *stream_init_id, *mime_type;
+  GSList *stream_methods = NULL;
+  gboolean multiple;
+  gchar *peer_resource = NULL;
+  gchar *self_jid = NULL;
+
+  if (lm_message_get_sub_type (msg) != LM_MESSAGE_SUB_TYPE_SET)
+    return LM_HANDLER_RESULT_ALLOW_MORE_HANDLERS;
+
+  si = lm_message_node_get_child_with_namespace (msg->node, "si", NS_SI);
+  if (si == NULL)
+    return LM_HANDLER_RESULT_ALLOW_MORE_HANDLERS;
+
+  /* after this point, the message is for us, so in all cases we either handle
+   * it or send an error reply */
+
+  if (!streaminit_parse_request (msg, si, &profile, &from, &stream_id,
+        &stream_init_id, &mime_type, &stream_methods, &multiple))
+    {
+      _gabble_connection_send_iq_error (priv->conn, msg,
+          XMPP_ERROR_BAD_REQUEST, "failed to parse SI request");
+      goto out;
+    }
+
+  DEBUG ("received a SI request");
+
+  room_handle = gabble_get_room_handle_from_jid (room_repo, from);
+
+  if (room_handle == 0)
+    {
+     /* jid is not a muc jid so we need contact's resource */
+     gabble_decode_jid (from, NULL, NULL, &peer_resource);
+
+     peer_handle = tp_handle_ensure (contact_repo, from, NULL, NULL);
+
+     /* we are not in a muc so our own jid is the one in the 'to' attribute */
+     self_jid = g_strdup (lm_message_node_get_attribute (msg->node,
+           "to"));
+    }
+  else
+    {
+      /* we are in a muc so need to get our muc jid */
+      GabbleMucChannel *muc;
+
+      peer_handle = tp_handle_ensure (contact_repo, from,
+          GUINT_TO_POINTER (GABBLE_JID_ROOM_MEMBER), NULL);
+
+      muc = gabble_muc_factory_find_text_channel (priv->conn->muc_factory,
+          room_handle);
+
+      if (muc == NULL)
+        {
+          DEBUG ("Got an IQ from a muc in which we are not. Ignoring");
+          goto out;
+        }
+
+      g_object_get (muc, "self-jid", &self_jid, NULL);
+    }
+
+  if (peer_handle == 0)
+    {
+      _gabble_connection_send_iq_error (priv->conn, msg,
+          XMPP_ERROR_JID_MALFORMED, NULL);
+      goto out;
+    }
+
+  if (multiple)
+    {
+      DEBUG ("Receiver supports multi bytestreams");
+
+      bytestream = (GabbleBytestreamIface *)
+          gabble_bytestream_factory_create_multiple (self, peer_handle,
+            stream_id, stream_init_id, peer_resource, self_jid,
+            GABBLE_BYTESTREAM_STATE_LOCAL_PENDING);
+    }
+
+  /* check stream method */
+  for (l = stream_methods; l != NULL; l = l->next)
+    {
+      if (multiple)
+        {
+          if (stream_method_supported (l->data))
+            {
+              DEBUG ("add %s to multi-bytestream methods",
+                  (const gchar *) l->data);
+
+              gabble_bytestream_multiple_add_stream_method (
+                  GABBLE_BYTESTREAM_MULTIPLE (bytestream), l->data);
+            }
+          else
+            {
+              DEBUG ("skip unsupported stream method: %s",
+                  (const gchar *) l->data);
+            }
+        }
+      else
+        {
+          /* We create the stream according the stream method chosen.
+           * User has to accept it */
+          DEBUG ("Receiver doesn't support multi bytestreams. He chose %s",
+              (const gchar *) l->data);
+
+          bytestream = gabble_bytestream_factory_create_from_method (self,
+              l->data, peer_handle, stream_id, stream_init_id, peer_resource,
+              self_jid, GABBLE_BYTESTREAM_STATE_LOCAL_PENDING);
+          if (bytestream != NULL)
+            break;
+        }
+    }
+
+  if (bytestream == NULL)
+    {
+      DEBUG ("SI request doesn't contain any supported stream methods.");
+
+      _gabble_connection_send_iq_error (priv->conn, msg,
+          XMPP_ERROR_SI_NO_VALID_STREAMS, NULL);
+      goto out;
+    }
+
+  if (multiple)
+    {
+      /* Is there at least one stream method? */
+      if (!gabble_bytestream_multiple_has_stream_method (
+            GABBLE_BYTESTREAM_MULTIPLE (bytestream)))
+        {
+          GError e = { GABBLE_XMPP_ERROR, XMPP_ERROR_SI_NO_VALID_STREAMS, "" };
+          DEBUG ("No valid stream method in the multi bytestream. Closing");
+
+          gabble_bytestream_iface_close (bytestream, &e);
+          goto out;
+        }
+    }
+
+  /* Now that we have a bytestream, it's responsible for declining the IQ
+   * if needed. */
+
+  /* We inform the right factory we received a SI request */
+  if (!tp_strdiff (profile, NS_TUBES))
+    {
+      si_tube_received (self, msg, si, bytestream, peer_handle, room_handle,
+          stream_id);
+    }
+  else if (!tp_strdiff (profile, NS_FILE_TRANSFER))
+    {
+      gabble_ft_manager_handle_si_request (priv->conn->ft_manager, bytestream,
+          peer_handle, stream_id, msg);
+    }
+  else
+    {
+      GError e = { GABBLE_XMPP_ERROR, XMPP_ERROR_SI_BAD_PROFILE, "" };
+      DEBUG ("SI profile unsupported: %s", profile);
+
+      gabble_bytestream_iface_close (bytestream, &e);
+    }
 
 out:
   g_slist_free (stream_methods);
   g_free (peer_resource);
+  g_free (self_jid);
+  if (peer_handle != 0)
+    tp_handle_unref (contact_repo, peer_handle);
 
   return LM_HANDLER_RESULT_REMOVE_MESSAGE;
 }
@@ -878,12 +1210,7 @@ handle_ibb_close_iq (GabbleBytestreamFactory *self,
     }
   else
     {
-      DEBUG ("received IBB close stanza. Bytestream closed");
-
-      g_object_set (bytestream, "state", GABBLE_BYTESTREAM_STATE_CLOSED,
-          NULL);
-
-      _gabble_connection_acknowledge_set_iq (priv->conn, msg);
+      gabble_bytestream_ibb_close_received (bytestream, msg);
     }
 
   return TRUE;
@@ -1059,9 +1386,9 @@ handle_socks5_query_iq (GabbleBytestreamFactory *self,
     GABBLE_BYTESTREAM_FACTORY_GET_PRIVATE (self);
   GabbleBytestreamSocks5 *bytestream;
   LmMessageNode *query_node;
-  LmMessageNode *child_node;
   ConstBytestreamIdentifier bsid = { NULL, NULL };
   const gchar *tmp;
+  NodeIter i;
 
   if (lm_message_get_sub_type (msg) != LM_MESSAGE_SUB_TYPE_SET)
     return FALSE;
@@ -1101,7 +1428,8 @@ handle_socks5_query_iq (GabbleBytestreamFactory *self,
     }
 
   tmp = lm_message_node_get_attribute (query_node, "mode");
-  if (tp_strdiff (tmp, "tcp"))
+  /* If this attribute is missing, the default value of "tcp" MUST be assumed */
+  if (tmp != NULL && tp_strdiff (tmp, "tcp"))
     {
       DEBUG ("non-TCP SOCKS5 bytestreams are not supported");
       _gabble_connection_send_iq_error (priv->conn, msg,
@@ -1110,13 +1438,12 @@ handle_socks5_query_iq (GabbleBytestreamFactory *self,
       return TRUE;
     }
 
-  child_node = query_node->children;
-  while (child_node)
+  for (i = node_iter (query_node); i; i = node_iter_next (i))
     {
+      LmMessageNode *child_node = node_iter_data (i);
+
       if (!tp_strdiff (child_node->name, "streamhost"))
         gabble_bytestream_socks5_add_streamhost (bytestream, child_node);
-
-      child_node = child_node->next;
     }
 
   gabble_bytestream_socks5_connect_to_streamhost (bytestream, msg);
@@ -1166,6 +1493,11 @@ bytestream_state_changed_cb (GabbleBytestreamIface *bytestream,
                              gpointer user_data)
 {
   GabbleBytestreamFactory *self = GABBLE_BYTESTREAM_FACTORY (user_data);
+  GabbleBytestreamFactoryPrivate *priv = GABBLE_BYTESTREAM_FACTORY_GET_PRIVATE (
+      self);
+
+  if (priv->dispose_has_run)
+    return;
 
   if (state == GABBLE_BYTESTREAM_STATE_CLOSED)
     {
@@ -1191,6 +1523,7 @@ gabble_bytestream_factory_create_from_method (GabbleBytestreamFactory *self,
                                               const gchar *stream_id,
                                               const gchar *stream_init_id,
                                               const gchar *peer_resource,
+                                              const gchar *self_jid,
                                               GabbleBytestreamState state)
 {
   GabbleBytestreamIface *bytestream = NULL;
@@ -1205,7 +1538,7 @@ gabble_bytestream_factory_create_from_method (GabbleBytestreamFactory *self,
     {
       bytestream = GABBLE_BYTESTREAM_IFACE (
           gabble_bytestream_factory_create_socks5 (self, peer_handle,
-            stream_id, stream_init_id, peer_resource, state));
+            stream_id, stream_init_id, peer_resource, self_jid, state));
     }
 
   return bytestream;
@@ -1281,6 +1614,7 @@ gabble_bytestream_factory_create_socks5 (GabbleBytestreamFactory *self,
                                          const gchar *stream_id,
                                          const gchar *stream_init_id,
                                          const gchar *peer_resource,
+                                         const gchar *self_jid,
                                          GabbleBytestreamState state)
 {
   GabbleBytestreamFactoryPrivate *priv;
@@ -1297,6 +1631,7 @@ gabble_bytestream_factory_create_socks5 (GabbleBytestreamFactory *self,
       "state", state,
       "stream-init-id", stream_init_id,
       "peer-resource", peer_resource,
+      "self-jid", self_jid,
       NULL);
 
   gabble_signal_connect_weak (socks5, "state-changed",
@@ -1315,6 +1650,7 @@ gabble_bytestream_factory_create_multiple (GabbleBytestreamFactory *self,
                                            const gchar *stream_id,
                                            const gchar *stream_init_id,
                                            const gchar *peer_resource,
+                                           const gchar *self_jid,
                                            GabbleBytestreamState state)
 {
   GabbleBytestreamFactoryPrivate *priv;
@@ -1332,6 +1668,7 @@ gabble_bytestream_factory_create_multiple (GabbleBytestreamFactory *self,
       "stream-init-id", stream_init_id,
       "peer-resource", peer_resource,
       "factory", self,
+      "self-jid", self_jid,
       NULL);
 
   gabble_signal_connect_weak (multiple, "state-changed",
@@ -1350,13 +1687,15 @@ streaminit_get_multiple_bytestream (GabbleBytestreamFactory *self,
                                     LmMessageNode *si,
                                     const gchar *stream_id,
                                     TpHandle peer_handle,
-                                    const gchar *peer_resource)
+                                    const gchar *peer_resource,
+                                    const gchar *self_jid)
 {
   /* If the other client supports si-multiple we have directly a list of
    * supported methods inside <value/> tags */
-  LmMessageNode *si_multi, *value;
+  LmMessageNode *si_multi;
   const gchar *stream_method;
   GabbleBytestreamMultiple *bytestream = NULL;
+  NodeIter i;
 
   si_multi = lm_message_node_get_child_with_namespace (si, "si-multiple",
       NS_SI_MULTIPLE);
@@ -1364,10 +1703,13 @@ streaminit_get_multiple_bytestream (GabbleBytestreamFactory *self,
     return NULL;
 
   bytestream = gabble_bytestream_factory_create_multiple (self, peer_handle,
-      stream_id, NULL, peer_resource, GABBLE_BYTESTREAM_STATE_INITIATING);
+      stream_id, NULL, peer_resource, self_jid,
+      GABBLE_BYTESTREAM_STATE_INITIATING);
 
-  for (value = si_multi->children; value; value = value->next)
+  for (i = node_iter (si_multi); i; i = node_iter_next (i))
     {
+      LmMessageNode *value = node_iter_data (i);
+
       if (tp_strdiff (value->name, "value"))
         continue;
 
@@ -1391,11 +1733,13 @@ streaminit_get_bytestream (GabbleBytestreamFactory *self,
                            LmMessageNode *si,
                            const gchar *stream_id,
                            TpHandle peer_handle,
-                           const gchar *peer_resource)
+                           const gchar *peer_resource,
+                           const gchar *self_jid)
 {
-  LmMessageNode *feature, *x, *field, *value;
+  LmMessageNode *feature, *x, *value;
   GabbleBytestreamIface *bytestream = NULL;
   const gchar *stream_method;
+  NodeIter i;
 
   feature = lm_message_node_get_child_with_namespace (si, "feature",
       NS_FEATURENEG);
@@ -1413,8 +1757,10 @@ streaminit_get_bytestream (GabbleBytestreamFactory *self,
       return NULL;
     }
 
-  for (field = x->children; field; field = field->next)
+  for (i = node_iter (x); i; i = node_iter_next (i))
     {
+      LmMessageNode *field = node_iter_data (i);
+
       if (tp_strdiff (lm_message_node_get_attribute (field, "var"),
             "stream-method"))
         /* some future field, ignore it */
@@ -1430,7 +1776,7 @@ streaminit_get_bytestream (GabbleBytestreamFactory *self,
 
       stream_method = lm_message_node_get_value (value);
       bytestream = gabble_bytestream_factory_create_from_method (self,
-          stream_method, peer_handle, stream_id, NULL, peer_resource,
+          stream_method, peer_handle, stream_id, NULL, peer_resource, self_jid,
           GABBLE_BYTESTREAM_STATE_INITIATING);
 
       /* no need to parse the rest of the fields, we've found the one we
@@ -1470,6 +1816,8 @@ streaminit_reply_cb (GabbleConnection *conn,
                      gpointer user_data)
 {
   GabbleBytestreamFactory *self = GABBLE_BYTESTREAM_FACTORY (obj);
+  GabbleBytestreamFactoryPrivate *priv = GABBLE_BYTESTREAM_FACTORY_GET_PRIVATE (
+      self);
   struct _streaminit_reply_cb_data *data =
     (struct _streaminit_reply_cb_data*) user_data;
   GabbleBytestreamIface *bytestream = NULL;
@@ -1481,7 +1829,9 @@ streaminit_reply_cb (GabbleConnection *conn,
   TpHandleRepoIface *room_repo = tp_base_connection_get_handles (
       (TpBaseConnection *) conn, TP_HANDLE_TYPE_ROOM);
   TpHandle peer_handle = 0;
+  TpHandle room_handle;
   gboolean success = FALSE;
+  gchar *self_jid = NULL;
 
   if (data->object != NULL)
     {
@@ -1511,11 +1861,32 @@ streaminit_reply_cb (GabbleConnection *conn,
     }
 
   peer_handle = tp_handle_ensure (contact_repo, from, NULL, NULL);
+  room_handle = gabble_get_room_handle_from_jid (room_repo, from);
 
-  if (gabble_get_room_handle_from_jid (room_repo, from) == 0)
+  if (room_handle == 0)
     {
      /* jid is not a muc jid so we need contact's resource */
      gabble_decode_jid (from, NULL, NULL, &peer_resource);
+
+     /* we are not in a muc so our own jid is the one in the 'to' attribute */
+     self_jid = g_strdup (lm_message_node_get_attribute (reply_msg->node,
+           "to"));
+    }
+  else
+    {
+      /* we are in a muc so need to get our muc jid */
+      GabbleMucChannel *muc;
+
+      muc = gabble_muc_factory_find_text_channel (priv->conn->muc_factory,
+          room_handle);
+
+      if (muc == NULL)
+        {
+          DEBUG ("Got an IQ from a muc in which we are not. Ignoring");
+          goto END;
+        }
+
+      g_object_get (muc, "self-jid", &self_jid, NULL);
     }
 
   si = lm_message_node_get_child_with_namespace (reply_msg->node, "si",
@@ -1528,14 +1899,14 @@ streaminit_reply_cb (GabbleConnection *conn,
 
   /* Try to build a multiple bytestream with fallback methods */
   bytestream = streaminit_get_multiple_bytestream (self, reply_msg, si,
-      data->stream_id, peer_handle, peer_resource);
+      data->stream_id, peer_handle, peer_resource, self_jid);
   /* FIXME: check if there is at least one stream method */
 
   if (bytestream == NULL)
     /* The other client doesn't suppport si-multiple, use the normal XEP-095
      * method */
     bytestream = streaminit_get_bytestream (self, reply_msg, si,
-        data->stream_id, peer_handle, peer_resource);
+        data->stream_id, peer_handle, peer_resource, self_jid);
 
   if (bytestream == NULL)
     goto END;
@@ -1565,9 +1936,11 @@ END:
   if (peer_resource != NULL)
     g_free (peer_resource);
 
+
   if (peer_handle != 0)
     tp_handle_unref (contact_repo, peer_handle);
 
+  g_free (self_jid);
   g_free (data->stream_id);
   g_slice_free (struct _streaminit_reply_cb_data, data);
 
@@ -1706,3 +2079,17 @@ gabble_bytestream_factory_make_multi_accept_iq (const gchar *full_jid,
 
   return msg;
 }
+
+GSList *
+gabble_bytestream_factory_get_socks5_proxies (GabbleBytestreamFactory *self)
+{
+  GabbleBytestreamFactoryPrivate *priv = GABBLE_BYTESTREAM_FACTORY_GET_PRIVATE (
+      self);
+
+  /* randomize fallback proxies to avoid to use always the same one */
+  priv->socks5_fallback_proxies = randomize_g_slist (
+      priv->socks5_fallback_proxies);
+
+  return g_slist_concat (g_slist_copy (priv->socks5_proxies),
+      g_slist_copy (priv->socks5_fallback_proxies));
+}
