@@ -21,18 +21,27 @@
 #include "config.h"
 #include "capabilities.h"
 
+#include <stdlib.h>
+#include <string.h>
+
 #include <telepathy-glib/interfaces.h>
 #include <telepathy-glib/channel-manager.h>
 
+#define DEBUG_FLAG GABBLE_DEBUG_PRESENCE
+
 #include "caps-channel-manager.h"
+#include "debug.h"
 #include "namespaces.h"
 #include "presence-cache.h"
 #include "media-channel.h"
+#include "util.h"
 
 static const Feature self_advertised_features[] =
 {
   { FEATURE_FIXED, NS_GOOGLE_FEAT_SESSION, 0},
   { FEATURE_FIXED, NS_GOOGLE_TRANSPORT_P2P, PRESENCE_CAP_GOOGLE_TRANSPORT_P2P},
+  { FEATURE_FIXED, NS_JINGLE_TRANSPORT_RAWUDP, PRESENCE_CAP_JINGLE_TRANSPORT_RAWUDP},
+  { FEATURE_FIXED, NS_JINGLE_TRANSPORT_ICEUDP, PRESENCE_CAP_JINGLE_TRANSPORT_ICEUDP},
   { FEATURE_FIXED, NS_JINGLE015, PRESENCE_CAP_JINGLE015},
   { FEATURE_FIXED, NS_JINGLE032, PRESENCE_CAP_JINGLE032},
   { FEATURE_FIXED, NS_CHAT_STATES, PRESENCE_CAP_CHAT_STATES},
@@ -42,13 +51,17 @@ static const Feature self_advertised_features[] =
   { FEATURE_FIXED, NS_IBB, PRESENCE_CAP_IBB},
   { FEATURE_FIXED, NS_TUBES, PRESENCE_CAP_SI_TUBES},
   { FEATURE_FIXED, NS_BYTESTREAMS, PRESENCE_CAP_BYTESTREAMS},
+  { FEATURE_FIXED, NS_FILE_TRANSFER, PRESENCE_CAP_SI_FILE_TRANSFER},
 
   { FEATURE_BUNDLE_COMPAT, NS_GOOGLE_FEAT_VOICE, PRESENCE_CAP_GOOGLE_VOICE},
+  { FEATURE_OPTIONAL, NS_GOOGLE_FEAT_VIDEO, PRESENCE_CAP_GOOGLE_VIDEO },
   { FEATURE_OPTIONAL, NS_JINGLE_DESCRIPTION_AUDIO,
     PRESENCE_CAP_JINGLE_DESCRIPTION_AUDIO},
   { FEATURE_OPTIONAL, NS_JINGLE_DESCRIPTION_VIDEO,
     PRESENCE_CAP_JINGLE_DESCRIPTION_VIDEO},
-  { FEATURE_OPTIONAL, NS_JINGLE_RTP, PRESENCE_CAP_JINGLE_RTP},
+  { FEATURE_OPTIONAL, NS_JINGLE_RTP, PRESENCE_CAP_JINGLE_RTP },
+  { FEATURE_OPTIONAL, NS_JINGLE_RTP_AUDIO, PRESENCE_CAP_JINGLE_RTP_AUDIO },
+  { FEATURE_OPTIONAL, NS_JINGLE_RTP_VIDEO, PRESENCE_CAP_JINGLE_RTP_VIDEO },
 
   { FEATURE_OPTIONAL, NS_OLPC_BUDDY_PROPS "+notify", PRESENCE_CAP_OLPC_1},
   { FEATURE_OPTIONAL, NS_OLPC_ACTIVITIES "+notify", PRESENCE_CAP_OLPC_1},
@@ -89,6 +102,87 @@ capabilities_get_features (GabblePresenceCapabilities caps,
   return features;
 }
 
+static gboolean
+omits_content_creators (LmMessageNode *identity)
+{
+  const gchar *name, *suffix;
+  gchar *end;
+  int ver;
+
+  if (tp_strdiff (identity->name, "identity"))
+    return FALSE;
+
+  name = lm_message_node_get_attribute (identity, "name");
+
+  if (name == NULL)
+    return FALSE;
+
+#define PREFIX "Telepathy Gabble 0.7."
+
+  if (!g_str_has_prefix (name, PREFIX))
+    return FALSE;
+
+  suffix = name + strlen (PREFIX);
+  ver = strtol (suffix, &end, 10);
+
+  if (*end != '\0')
+    return FALSE;
+
+  /* Gabble versions since 0.7.16 did not send the creator='' attribute for
+   * contents. The bug is fixed in 0.7.29.
+   */
+  if (ver >= 16 && ver < 29)
+    {
+      DEBUG ("contact is using '%s' which omits 'creator'", name);
+      return TRUE;
+    }
+  else
+    {
+      return FALSE;
+    }
+}
+
+GabblePresenceCapabilities
+capabilities_parse (LmMessageNode *query_result)
+{
+  GabblePresenceCapabilities ret = PRESENCE_CAP_NONE;
+  const gchar *var;
+  const Feature *i;
+  NodeIter j;
+
+  for (j = node_iter (query_result); j; j = node_iter_next (j))
+    {
+      LmMessageNode *child = node_iter_data (j);
+
+      if (0 != strcmp (child->name, "feature"))
+        {
+          if (omits_content_creators (child))
+            ret |= PRESENCE_CAP_JINGLE_OMITS_CONTENT_CREATOR;
+
+          continue;
+        }
+
+      var = lm_message_node_get_attribute (child, "var");
+
+      if (NULL == var)
+        continue;
+
+      for (i = self_advertised_features; i->ns != NULL; i++)
+        {
+          if (0 == strcmp (var, i->ns))
+            {
+              ret |= i->caps;
+              break;
+            }
+        }
+
+      if (i->ns == NULL)
+        DEBUG ("ignoring unknown capability %s", var);
+    }
+
+  return ret;
+}
+
 void
 capabilities_fill_cache (GabblePresenceCache *cache)
 {
