@@ -90,6 +90,7 @@ _free_field (gpointer data, gpointer user_data)
 
   g_free (field->field_name);
   g_ptr_array_foreach (field->values, (GFunc) g_free, NULL);
+  g_ptr_array_free (field->values, TRUE);
 
   g_slice_free (DataFormField, field);
 }
@@ -102,6 +103,7 @@ _free_form (gpointer data, gpointer user_data)
   g_free (form->form_type);
 
   g_ptr_array_foreach (form->fields, _free_field, NULL);
+  g_ptr_array_free (form->fields, TRUE);
 
   g_slice_free (DataForm, form);
 }
@@ -196,19 +198,17 @@ caps_hash_compute (
 static DataForm *
 _parse_dataform (LmMessageNode *node)
 {
-  LmMessageNode *field_node;
   DataForm *form;
+  NodeIter i;
 
   form = g_slice_new0 (DataForm);
   form->form_type = NULL;
   form->fields = g_ptr_array_new ();
 
-  for (field_node = node->children;
-       NULL != field_node;
-       field_node = field_node->next)
+  for (i = node_iter (node); i; i = node_iter_next (i))
     {
+      LmMessageNode *field_node = node_iter_data (i);
       const gchar *var;
-      LmMessageNode *value_node;
 
       if (! g_str_equal (field_node->name, "field"))
         continue;
@@ -220,10 +220,11 @@ _parse_dataform (LmMessageNode *node)
 
       if (g_str_equal (var, "FORM_TYPE"))
         {
-          for (value_node = field_node->children;
-               NULL != value_node;
-               value_node = value_node->next)
+          NodeIter j;
+
+          for (j = node_iter (field_node); j; j = node_iter_next (j))
             {
+              LmMessageNode *value_node = node_iter_data (j);
               const gchar *content;
 
               if (tp_strdiff (value_node->name, "value"))
@@ -241,15 +242,15 @@ _parse_dataform (LmMessageNode *node)
       else
         {
           DataFormField *field = NULL;
+          NodeIter j;
 
           field = g_slice_new0 (DataFormField);
           field->values = g_ptr_array_new ();
           field->field_name = g_strdup (var);
 
-          for (value_node = field_node->children;
-               NULL != value_node;
-               value_node = value_node->next)
+          for (j = node_iter (field_node); j; j = node_iter_next (j))
             {
+              LmMessageNode *value_node = node_iter_data (j);
               const gchar *content;
 
               if (tp_strdiff (value_node->name, "value"))
@@ -282,11 +283,13 @@ caps_hash_compute_from_lm_node (LmMessageNode *node)
   GPtrArray *features = g_ptr_array_new ();
   GPtrArray *identities = g_ptr_array_new ();
   GPtrArray *dataforms = g_ptr_array_new ();
-  LmMessageNode *child;
   gchar *str;
+  NodeIter i;
 
-  for (child = node->children; NULL != child; child = child->next)
+  for (i = node_iter (node); i; i = node_iter_next (i))
     {
+      LmMessageNode *child = node_iter_data (i);
+
       if (g_str_equal (child->name, "identity"))
         {
           const gchar *category;
