@@ -108,6 +108,17 @@ connection_got_self_initial_avatar_cb (GObject *obj,
   update_own_avatar_sha1 (conn, sha1, NULL);
 }
 
+/* Jabber prescribes no MIME type for avatars, but XEP-0153 says support
+ * for image/png is REQUIRED, with image/jpeg and image/gif RECOMMENDED */
+static const char *mimetypes[] = {
+    "image/png", "image/jpeg", "image/gif", NULL };
+
+/* Jabber has no min/max width/height or max size, but XEP-0153 says
+ * you SHOULD use 32-96px either way, and no more than 8K of data */
+#define AVATAR_MIN_PX 32
+#define AVATAR_REC_PX 64
+#define AVATAR_MAX_PX 96
+#define AVATAR_MAX_BYTES 8192
 
 /**
  * gabble_connection_get_avatar_requirements
@@ -125,19 +136,13 @@ static void
 gabble_connection_get_avatar_requirements (TpSvcConnectionInterfaceAvatars *iface,
                                            DBusGMethodInvocation *context)
 {
-  /* Jabber prescribes no MIME type for avatars, but XEP-0153 says support
-   * for image/png is REQUIRED, with image/jpeg and image/gif RECOMMENDED */
-  static const char *mimetypes[] = {
-      "image/png", "image/jpeg", "image/gif", NULL };
 
   TP_BASE_CONNECTION_ERROR_IF_NOT_CONNECTED (TP_BASE_CONNECTION (iface),
       context);
 
-  /* Jabber has no min/max width/height or max size, but XEP-0153 says
-   * you SHOULD use 32-96px either way, and no more than 8K of data */
-
   tp_svc_connection_interface_avatars_return_from_get_avatar_requirements (
-      context, mimetypes, 32, 32, 96, 96, 8192);
+      context, mimetypes, AVATAR_MIN_PX, AVATAR_MIN_PX,
+      AVATAR_MAX_PX, AVATAR_MAX_PX, AVATAR_MAX_BYTES);
 }
 
 /* begin deprecated code */
@@ -160,8 +165,9 @@ _got_self_avatar_for_get_avatar_tokens (GObject *obj,
   g_free (context->ret[context->my_index]);
   context->ret[context->my_index] = g_strdup (sha1);
 
-  /* FIXME: I'm not entirely sure why gcc warns without this cast from
-   * (gchar **) to (const gchar **) */
+  /* Cast to (const gchar **) necessary because no-one understands 'const'
+   * in C.
+   */
   tp_svc_connection_interface_avatars_return_from_get_avatar_tokens (
       context->invocation, (const gchar **)context->ret);
   g_strfreev (context->ret);
@@ -258,8 +264,9 @@ gabble_connection_get_avatar_tokens (TpSvcConnectionInterfaceAvatars *iface,
       return;
     }
 
-  /* FIXME: I'm not entirely sure why gcc warns without this cast from
-   * (gchar **) to (const gchar **) */
+  /* Cast to (const gchar **) necessary because no-one understands 'const'
+   * in C.
+   */
   tp_svc_connection_interface_avatars_return_from_get_avatar_tokens (
       invocation, (const gchar **)ret);
   g_strfreev (ret);
@@ -480,7 +487,25 @@ _request_avatar_cb (GabbleVCardManager *self,
 
   if (NULL == vcard)
     {
-      dbus_g_method_return_error (context, vcard_error);
+      GError tp_error = { TP_ERRORS, TP_ERROR_NOT_AVAILABLE,
+          vcard_error->message };
+
+      if (vcard_error->domain == GABBLE_XMPP_ERROR)
+        {
+          switch (vcard_error->code)
+            {
+            case XMPP_ERROR_NOT_AUTHORIZED:
+            case XMPP_ERROR_FORBIDDEN:
+              tp_error.code = TP_ERROR_PERMISSION_DENIED;
+              break;
+            case XMPP_ERROR_ITEM_NOT_FOUND:
+              tp_error.code = TP_ERROR_DOES_NOT_EXIST;
+              break;
+            }
+          /* what other mappings make sense here? */
+        }
+
+      dbus_g_method_return_error (context, &tp_error);
       goto out;
     }
 
@@ -726,7 +751,19 @@ _set_avatar_cb2 (GabbleVCardManager *manager,
 
   if (NULL == vcard)
     {
-      dbus_g_method_return_error (ctx->invocation, vcard_error);
+      GError tp_error = { TP_ERRORS, TP_ERROR_NOT_AVAILABLE,
+          vcard_error->message };
+
+      /* Google Talk has been observed to return bad-request when the avatar is
+       * too big. It's not clear what other XMPP errors make sense here, or how
+       * to map them.
+       */
+      if (vcard_error->domain == GABBLE_XMPP_ERROR)
+        if (vcard_error->code == XMPP_ERROR_BAD_REQUEST ||
+            vcard_error->code == XMPP_ERROR_NOT_ACCEPTABLE)
+          tp_error.code = TP_ERROR_INVALID_ARGUMENT;
+
+      dbus_g_method_return_error (ctx->invocation, &tp_error);
     }
   else
     {
@@ -886,3 +923,36 @@ conn_avatars_iface_init (gpointer g_iface, gpointer iface_data)
 #undef IMPLEMENT
 }
 
+static TpDBusPropertiesMixinPropImpl props[] = {
+      { "MinimumAvatarWidth", GUINT_TO_POINTER (AVATAR_MIN_PX), NULL },
+      { "RecommendedAvatarWidth", GUINT_TO_POINTER (AVATAR_REC_PX), NULL },
+      { "MaximumAvatarWidth", GUINT_TO_POINTER (AVATAR_MAX_PX), NULL },
+      { "MinimumAvatarHeight", GUINT_TO_POINTER (AVATAR_MIN_PX), NULL },
+      { "RecommendedAvatarHeight", GUINT_TO_POINTER (AVATAR_REC_PX), NULL },
+      { "MaximumAvatarHeight", GUINT_TO_POINTER (AVATAR_MAX_PX), NULL },
+      { "MaximumAvatarBytes", GUINT_TO_POINTER (AVATAR_MAX_BYTES), NULL },
+      /* special-cased - it's the only one with a non-guint value */
+      { "SupportedAvatarMIMETypes", NULL, NULL },
+      { NULL }
+};
+TpDBusPropertiesMixinPropImpl *conn_avatars_properties = props;
+
+void
+conn_avatars_properties_getter (GObject *object,
+                                GQuark interface,
+                                GQuark name,
+                                GValue *value,
+                                gpointer getter_data)
+{
+  GQuark q_mime_types = g_quark_from_static_string (
+      "SupportedAvatarMIMETypes");
+
+  if (name == q_mime_types)
+    {
+      g_value_set_static_boxed (value, mimetypes);
+    }
+  else
+    {
+      g_value_set_uint (value, GPOINTER_TO_UINT (getter_data));
+    }
+}
