@@ -29,6 +29,7 @@
 #include <telepathy-glib/errors.h>
 
 #include "connection.h"
+#include "debug.h"
 
 G_DEFINE_TYPE(GabbleConnectionManager,
     gabble_connection_manager,
@@ -45,15 +46,23 @@ static TpBaseConnection *_gabble_connection_manager_new_connection (
     TpBaseConnectionManager *self, const gchar *proto,
     TpIntSet *params_present, void *parsed_params, GError **error);
 
+static void
+gabble_connection_manager_finalize (GObject *object)
+{
+  gabble_debug_free ();
+}
+
 static void
 gabble_connection_manager_class_init (GabbleConnectionManagerClass *klass)
 {
+  GObjectClass *object_class = G_OBJECT_CLASS (klass);
   TpBaseConnectionManagerClass *base_class =
     (TpBaseConnectionManagerClass *) klass;
 
   base_class->new_connection = _gabble_connection_manager_new_connection;
   base_class->cm_dbus_name = "gabble";
-  base_class->protocol_params = gabble_protocols;
+  base_class->protocol_params = gabble_connection_manager_get_protocols ();
+  object_class->finalize = gabble_connection_manager_finalize;
 }
 
 /* private data */
@@ -80,6 +89,8 @@ struct _GabbleParams {
   guint fallback_stun_port;
   gboolean ignore_ssl_errors;
   gchar *alias;
+  GStrv fallback_socks5_proxies;
+  guint keepalive_interval;
 };
 
 enum {
@@ -102,10 +113,13 @@ enum {
     JABBER_PARAM_FALLBACK_STUN_PORT,
     JABBER_PARAM_IGNORE_SSL_ERRORS,
     JABBER_PARAM_ALIAS,
+    JABBER_PARAM_FALLBACK_SOCKS5_PROXIES,
+    JABBER_PARAM_KEEPALIVE_INTERVAL,
+
     LAST_JABBER_PARAM
 };
 
-static const TpCMParamSpec jabber_params[] = {
+static TpCMParamSpec jabber_params[] = {
   { "account", DBUS_TYPE_STRING_AS_STRING, G_TYPE_STRING,
     TP_CONN_MGR_PARAM_FLAG_REQUIRED | TP_CONN_MGR_PARAM_FLAG_REGISTER, NULL,
     G_STRUCT_OFFSET(GabbleParams, account),
@@ -120,8 +134,7 @@ static const TpCMParamSpec jabber_params[] = {
     /* FIXME: validate the server properly */
     tp_cm_param_filter_string_nonempty, NULL },
 
-  { "resource", DBUS_TYPE_STRING_AS_STRING, G_TYPE_STRING,
-    TP_CONN_MGR_PARAM_FLAG_HAS_DEFAULT, GABBLE_PARAMS_DEFAULT_RESOURCE,
+  { "resource", DBUS_TYPE_STRING_AS_STRING, G_TYPE_STRING, 0, NULL,
     G_STRUCT_OFFSET(GabbleParams, resource),
     /* FIXME: validate the resource according to the RFC */
     tp_cm_param_filter_string_nonempty, NULL },
@@ -131,7 +144,7 @@ static const TpCMParamSpec jabber_params[] = {
     G_STRUCT_OFFSET(GabbleParams, priority), NULL, NULL },
 
   { "port", DBUS_TYPE_UINT16_AS_STRING, G_TYPE_UINT,
-    0, GUINT_TO_POINTER(0),
+    TP_CONN_MGR_PARAM_FLAG_HAS_DEFAULT, GUINT_TO_POINTER(5222),
     G_STRUCT_OFFSET(GabbleParams, port),
     tp_cm_param_filter_uint_nonzero, NULL },
 
@@ -197,6 +210,15 @@ static const TpCMParamSpec jabber_params[] = {
     /* setting a 0-length alias makes no sense */
     tp_cm_param_filter_string_nonempty, NULL },
 
+  { "fallback-socks5-proxies", "as", 0,
+    TP_CONN_MGR_PARAM_FLAG_HAS_DEFAULT, NULL,
+    G_STRUCT_OFFSET (GabbleParams, fallback_socks5_proxies),
+    NULL, NULL },
+
+  { "keepalive-interval", "u", G_TYPE_UINT,
+    TP_CONN_MGR_PARAM_FLAG_HAS_DEFAULT, GUINT_TO_POINTER (30),
+    G_STRUCT_OFFSET (GabbleParams, keepalive_interval), NULL, NULL },
+
   { NULL, NULL, 0, 0, NULL, 0 }
 };
 
@@ -219,6 +241,7 @@ free_params (void *p)
   g_free (params->fallback_conference_server);
   g_free (params->stun_server);
   g_free (params->alias);
+  g_strfreev (params->fallback_socks5_proxies);
 
   g_slice_free (GabbleParams, params);
 }
@@ -228,6 +251,19 @@ const TpCMProtocolSpec gabble_protocols[] = {
   { NULL, NULL }
 };
 
+const gchar *default_socks5_proxies[] = GABBLE_PARAMS_DEFAULT_SOCKS5_PROXIES;
+
+const TpCMProtocolSpec *
+gabble_connection_manager_get_protocols (void)
+{
+  jabber_params[JABBER_PARAM_FALLBACK_SOCKS5_PROXIES].gtype = G_TYPE_STRV;
+  jabber_params[JABBER_PARAM_FALLBACK_SOCKS5_PROXIES].def =
+    default_socks5_proxies;
+
+  return gabble_protocols;
+}
+
+
 #define SET_PROPERTY_IF_PARAM_SET(prop, param, member) \
   if (tp_intset_is_member (params_present, param)) \
     { \
@@ -286,6 +322,10 @@ _gabble_connection_manager_new_connection (TpBaseConnectionManager *self,
                               JABBER_PARAM_IGNORE_SSL_ERRORS,
                               params->ignore_ssl_errors);
   SET_PROPERTY_IF_PARAM_SET ("alias", JABBER_PARAM_ALIAS, params->alias);
+  SET_PROPERTY_IF_PARAM_SET ("fallback-socks5-proxies",
+      JABBER_PARAM_FALLBACK_SOCKS5_PROXIES, params->fallback_socks5_proxies);
+  SET_PROPERTY_IF_PARAM_SET ("keepalive-interval",
+      JABBER_PARAM_KEEPALIVE_INTERVAL, params->keepalive_interval);
 
   /* split up account into username, stream-server and resource */
   if (!_gabble_connection_set_properties_from_account (conn, params->account,
