@@ -30,6 +30,7 @@
 #include <telepathy-glib/dbus-properties-mixin.h>
 
 #include "error.h"
+#include "ft-manager.h"
 #include "jingle-factory.h"
 #include "muc-factory.h"
 #include "olpc-gadget-manager.h"
@@ -38,10 +39,31 @@
 G_BEGIN_DECLS
 
 /* Default parameters for optional parameters */
-#define GABBLE_PARAMS_DEFAULT_RESOURCE                   "Telepathy"
 #define GABBLE_PARAMS_DEFAULT_HTTPS_PROXY_PORT           443
 #define GABBLE_PARAMS_DEFAULT_STUN_PORT                  3478
 #define GABBLE_PARAMS_DEFAULT_FALLBACK_STUN_SERVER       "stun.collabora.co.uk"
+/* List of public SOCKS5 proxies:
+ * http://coccinella.im/servers/servers_by_proxy_bytestreams.html */
+#define GABBLE_PARAMS_DEFAULT_SOCKS5_PROXIES             \
+    { "proxy.jabber.org", "proxy.jabberfr.org",\
+      "proxy65.rooyee.biz", "proxy.jabbim.cz",\
+      "proxy.911910.cn",\
+      "proxy.downtempo.de",\
+      "proxy.im.flosoft.biz",\
+      "proxy.jabber.bluendo.com", "proxy.jabber.dk", "proxy.jabber.freenet.de",\
+      "proxy.fsinf.at", "proxy.jabber.minus273.org",\
+      "proxy.jabber.planetteamspeak.com", "proxy.jabber.tf-network.de",\
+      "proxy.jabjab.de", "proxy.jabster.pl",\
+      "proxy.schokokeks.org",\
+      "proxy.ubuntu-jabber.de", "proxy.ubuntu-jabber.net",\
+      "proxy65.unstable.nl", "proxy.verdammung.org", "proxy.vke.ru",\
+      "proxy.vodka-pomme.net", "proxy.jabbernet.eu",\
+      NULL }
+
+/* These proxies appeared to be the same as proxy.jabbernet.eu
+    "proxy.jabberchat.eu", "proxy.shady.nl", "proxy.nedbsd.be",\
+    "proxy.nedbsd.eu", "proxy.nedbsd.nl", "proxy.4business.nl"\
+*/
 
 /* order must match array of statuses in conn-presence.c */
 /* in increasing order of presence */
@@ -85,7 +107,7 @@ typedef enum
 {
   GABBLE_LIST_HANDLE_PUBLISH = 1,
   GABBLE_LIST_HANDLE_SUBSCRIBE,
-  GABBLE_LIST_HANDLE_KNOWN,
+  GABBLE_LIST_HANDLE_STORED,
   GABBLE_LIST_HANDLE_DENY
 } GabbleListHandle;
 
@@ -162,6 +184,9 @@ struct _GabbleConnection {
     /* jingle factory */
     GabbleJingleFactory *jingle_factory;
 
+    /* file transfer manager */
+    GabbleFtManager *ft_manager;
+
     /* temporary, for requestotron support */
     GPtrArray *channel_factories;
     GPtrArray *channel_managers;
@@ -199,6 +224,8 @@ GType gabble_connection_get_type (void);
   (G_TYPE_INSTANCE_GET_CLASS ((obj), GABBLE_TYPE_CONNECTION, \
                               GabbleConnectionClass))
 
+gchar *gabble_connection_get_full_jid (GabbleConnection *conn);
+
 gboolean _gabble_connection_set_properties_from_account (
     GabbleConnection *conn, const gchar *account, GError **error);
 gboolean _gabble_connection_send (GabbleConnection *conn, LmMessage *msg,
