@@ -1,6 +1,6 @@
 
 #ifndef __DEBUG_H__
-#define __DEBUG_H_
+#define __DEBUG_H__
 
 #include "config.h"
 
@@ -29,11 +29,13 @@ typedef enum
   GABBLE_DEBUG_OLPC          = 1 << 14,
   GABBLE_DEBUG_BYTESTREAM    = 1 << 16,
   GABBLE_DEBUG_LOCATION      = 1 << 17,
+  GABBLE_DEBUG_FT            = 1 << 18,
 } GabbleDebugFlags;
 
 void gabble_debug_set_flags_from_env (void);
 void gabble_debug_set_flags (GabbleDebugFlags flags);
 gboolean gabble_debug_flag_is_set (GabbleDebugFlags flag);
+void gabble_debug_free (void);
 void gabble_debug (GabbleDebugFlags flag, const gchar *format, ...)
     G_GNUC_PRINTF (2, 3);
 
@@ -67,6 +69,8 @@ G_STMT_START { \
 
 #endif /* DEBUG_FLAG */
 
+#define gabble_debug_free() G_STMT_START { } G_STMT_END
+
 #endif /* ENABLE_DEBUG */
 
 #endif /* __DEBUG_H__ */
