@@ -92,8 +92,6 @@ gabble_disco_error_quark (void)
   return quark;
 }
 
-#define GABBLE_DISCO_GET_PRIVATE(o) ((o)->priv)
-
 static void
 gabble_disco_init (GabbleDisco *obj)
 {
@@ -150,7 +148,7 @@ gabble_disco_get_property (GObject    *object,
                                 GParamSpec *pspec)
 {
   GabbleDisco *chan = GABBLE_DISCO (object);
-  GabbleDiscoPrivate *priv = GABBLE_DISCO_GET_PRIVATE (chan);
+  GabbleDiscoPrivate *priv = chan->priv;
 
   switch (property_id) {
     case PROP_CONNECTION:
@@ -169,7 +167,7 @@ gabble_disco_set_property (GObject     *object,
                            GParamSpec   *pspec)
 {
   GabbleDisco *chan = GABBLE_DISCO (object);
-  GabbleDiscoPrivate *priv = GABBLE_DISCO_GET_PRIVATE (chan);
+  GabbleDiscoPrivate *priv = chan->priv;
 
   switch (property_id) {
     case PROP_CONNECTION:
@@ -195,7 +193,7 @@ gabble_disco_constructor (GType type, guint n_props,
   obj = G_OBJECT_CLASS (gabble_disco_parent_class)-> constructor (type,
       n_props, props);
   disco = GABBLE_DISCO (obj);
-  priv = GABBLE_DISCO_GET_PRIVATE (disco);
+  priv = disco->priv;
 
   g_signal_connect (priv->connection, "status-changed",
       G_CALLBACK (gabble_disco_conn_status_changed_cb), disco);
@@ -209,7 +207,7 @@ static void
 gabble_disco_dispose (GObject *object)
 {
   GabbleDisco *self = GABBLE_DISCO (object);
-  GabbleDiscoPrivate *priv = GABBLE_DISCO_GET_PRIVATE (self);
+  GabbleDiscoPrivate *priv = self->priv;
   GSList *l;
 
   if (priv->dispose_has_run)
@@ -282,7 +280,7 @@ delete_request (GabbleDiscoRequest *request)
   g_assert (NULL != request);
   g_assert (GABBLE_IS_DISCO (disco));
 
-  priv = GABBLE_DISCO_GET_PRIVATE (disco);
+  priv = disco->priv;
 
   g_assert (NULL != g_list_find (priv->requests, request));
 
@@ -381,7 +379,7 @@ request_reply_cb (GabbleConnection *conn, LmMessage *sent_msg,
 {
   GabbleDiscoRequest *request = (GabbleDiscoRequest *) user_data;
   GabbleDisco *disco = GABBLE_DISCO (object);
-  GabbleDiscoPrivate *priv = GABBLE_DISCO_GET_PRIVATE (disco);
+  GabbleDiscoPrivate *priv = disco->priv;
   LmMessageNode *query_node;
   GError *err = NULL;
 
@@ -460,8 +458,7 @@ gabble_disco_request (GabbleDisco *self, GabbleDiscoType type,
  * @type: type of request
  * @jid: Jabber ID to request on
  * @node: node to request on @jid, or NULL
- * @timeout: the time until the request fails, in milliseconds (1/1000ths of
- *           a second)
+ * @timeout: the time until the request fails, in seconds
  * @callback: #GabbleDiscoCb to call on request fullfilment
  * @object: GObject to bind request to. the callback will not be
  *          called if this object has been unrefed. NULL if not needed
@@ -478,7 +475,7 @@ gabble_disco_request_with_timeout (GabbleDisco *self, GabbleDiscoType type,
                                    gpointer user_data, GObject *object,
                                    GError **error)
 {
-  GabbleDiscoPrivate *priv = GABBLE_DISCO_GET_PRIVATE (self);
+  GabbleDiscoPrivate *priv = self->priv;
   GabbleDiscoRequest *request;
   LmMessage *msg;
   LmMessageNode *lm_node;
@@ -521,7 +518,7 @@ gabble_disco_request_with_timeout (GabbleDisco *self, GabbleDiscoType type,
   else
     {
       request->timer_id =
-          g_timeout_add (timeout, timeout_request, request);
+          g_timeout_add_seconds (timeout, timeout_request, request);
       lm_message_unref (msg);
       return request;
     }
@@ -535,7 +532,7 @@ gabble_disco_cancel_request (GabbleDisco *disco, GabbleDiscoRequest *request)
   g_return_if_fail (GABBLE_IS_DISCO (disco));
   g_return_if_fail (NULL != request);
 
-  priv = GABBLE_DISCO_GET_PRIVATE (disco);
+  priv = disco->priv;
 
   g_return_if_fail (NULL != g_list_find (priv->requests, request));
 
@@ -569,10 +566,11 @@ item_info_cb (GabbleDisco *disco,
               GError *error,
               gpointer user_data)
 {
-  LmMessageNode *identity, *feature, *field, *value_node;
+  LmMessageNode *identity, *value_node;
   const char *category, *type, *var, *name, *value;
   GHashTable *keys;
   GabbleDiscoItem item;
+  NodeIter i;
 
   GabbleDiscoPipeline *pipeline = (GabbleDiscoPipeline *) user_data;
 
@@ -605,8 +603,10 @@ item_info_cb (GabbleDisco *disco,
 
   keys = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free);
 
-  for (feature = result->children; feature; feature = feature->next)
+  for (i = node_iter (result); i; i = node_iter_next (i))
     {
+      LmMessageNode *feature = node_iter_data (i);
+
       if (0 == strcmp (feature->name, "feature"))
         {
           var = lm_message_node_get_attribute (feature, "var");
@@ -617,9 +617,12 @@ item_info_cb (GabbleDisco *disco,
         {
           if (lm_message_node_has_namespace (feature, NS_X_DATA, NULL))
             {
-              for (field = feature->children;
-                   field; field = field->next)
+              NodeIter j;
+
+              for (j = node_iter (feature); j; j = node_iter_next (j))
                 {
+                  LmMessageNode *field = node_iter_data (j);
+
                   if (0 != strcmp (field->name, "field"))
                     continue;
 
@@ -713,10 +716,10 @@ disco_items_cb (GabbleDisco *disco,
           GError *error,
           gpointer user_data)
 {
-  LmMessageNode *iter;
   const char *item_jid;
   gpointer key, value;
   GabbleDiscoPipeline *pipeline = (GabbleDiscoPipeline *) user_data;
+  NodeIter i;
 
   pipeline->list_request = NULL;
 
@@ -726,12 +729,14 @@ disco_items_cb (GabbleDisco *disco,
       goto out;
     }
 
-  for (iter = result->children; iter; iter = iter->next)
+  for (i = node_iter (result); i; i = node_iter_next (i))
     {
-      if (0 != strcmp (iter->name, "item"))
+      LmMessageNode *item = node_iter_data (i);
+
+      if (0 != strcmp (item->name, "item"))
         continue;
 
-      item_jid = lm_message_node_get_attribute (iter, "jid");
+      item_jid = lm_message_node_get_attribute (item, "jid");
 
       if (NULL != item_jid &&
           !g_hash_table_lookup_extended (pipeline->remaining_items, item_jid,
@@ -852,7 +857,7 @@ static void
 services_cb (gpointer pipeline, GabbleDiscoItem *item, gpointer user_data)
 {
   GabbleDisco *disco = GABBLE_DISCO (user_data);
-  GabbleDiscoPrivate *priv = GABBLE_DISCO_GET_PRIVATE (disco);
+  GabbleDiscoPrivate *priv = disco->priv;
   GabbleDiscoItem *my_item = g_new0 (GabbleDiscoItem, 1);
 
   my_item->jid = g_strdup (item->jid);
@@ -874,7 +879,7 @@ static void
 end_cb (gpointer pipeline, gpointer user_data)
 {
   GabbleDisco *disco = GABBLE_DISCO (user_data);
-  GabbleDiscoPrivate *priv = GABBLE_DISCO_GET_PRIVATE (disco);
+  GabbleDiscoPrivate *priv = disco->priv;
 
   gabble_disco_pipeline_destroy (pipeline);
   priv->service_cache = g_slist_reverse (priv->service_cache);
@@ -889,7 +894,7 @@ gabble_disco_conn_status_changed_cb (GabbleConnection *conn,
                                      gpointer data)
 {
   GabbleDisco *disco = GABBLE_DISCO (data);
-  GabbleDiscoPrivate *priv = GABBLE_DISCO_GET_PRIVATE (disco);
+  GabbleDiscoPrivate *priv = disco->priv;
 
   if (status == TP_CONNECTION_STATUS_CONNECTED)
     {
@@ -919,7 +924,7 @@ gabble_disco_service_find (GabbleDisco *disco,
   GSList *l;
 
   g_assert (GABBLE_IS_DISCO (disco));
-  priv = GABBLE_DISCO_GET_PRIVATE (disco);
+  priv = disco->priv;
 
   for (l = priv->service_cache; l; l = g_slist_next (l))
     {
