@@ -42,6 +42,7 @@
 #include "presence.h"
 #include "presence-cache.h"
 #include "roster.h"
+#include "util.h"
 
 static void channel_iface_init (gpointer, gpointer);
 static void chat_state_iface_init (gpointer, gpointer);
@@ -103,12 +104,17 @@ struct _GabbleIMChannelPrivate
   gchar *peer_jid;
   gboolean send_nick;
 
+  /* FALSE unless at least one chat state notification has been sent; <gone/>
+   * will only be sent when the channel closes if this is TRUE. This prevents
+   * opening a channel and closing it immediately sending a spurious <gone/> to
+   * the peer.
+   */
+  gboolean send_gone;
+
   gboolean closed;
   gboolean dispose_has_run;
 };
 
-#define GABBLE_IM_CHANNEL_GET_PRIVATE(obj) ((obj)->priv)
-
 static void
 gabble_im_channel_init (GabbleIMChannel *self)
 {
@@ -143,7 +149,7 @@ gabble_im_channel_constructor (GType type, guint n_props,
 
   obj = G_OBJECT_CLASS (gabble_im_channel_parent_class)->
            constructor (type, n_props, props);
-  priv = GABBLE_IM_CHANNEL_GET_PRIVATE (GABBLE_IM_CHANNEL (obj));
+  priv = GABBLE_IM_CHANNEL (obj)->priv;
   conn = (TpBaseConnection *) priv->conn;
   contact_handles = tp_base_connection_get_handles (conn,
       TP_HANDLE_TYPE_CONTACT);
@@ -184,7 +190,7 @@ gabble_im_channel_get_property (GObject    *object,
                                 GParamSpec *pspec)
 {
   GabbleIMChannel *chan = GABBLE_IM_CHANNEL (object);
-  GabbleIMChannelPrivate *priv = GABBLE_IM_CHANNEL_GET_PRIVATE (chan);
+  GabbleIMChannelPrivate *priv = chan->priv;
   TpBaseConnection *base_conn = (TpBaseConnection *) priv->conn;
 
   switch (property_id) {
@@ -259,7 +265,7 @@ gabble_im_channel_set_property (GObject     *object,
                                 GParamSpec   *pspec)
 {
   GabbleIMChannel *chan = GABBLE_IM_CHANNEL (object);
-  GabbleIMChannelPrivate *priv = GABBLE_IM_CHANNEL_GET_PRIVATE (chan);
+  GabbleIMChannelPrivate *priv = chan->priv;
 
   switch (property_id) {
     case PROP_OBJECT_PATH:
@@ -387,14 +393,36 @@ gabble_im_channel_class_init (GabbleIMChannelClass *gabble_im_channel_class)
   tp_message_mixin_init_dbus_properties (object_class);
 }
 
+static void
+emit_closed_and_send_gone (GabbleIMChannel *self)
+{
+  GabbleIMChannelPrivate *priv = self->priv;
+  GabblePresence *presence;
+
+  if (priv->send_gone)
+    {
+      presence = gabble_presence_cache_get (priv->conn->presence_cache,
+          priv->handle);
+
+      if (presence && (presence->caps & PRESENCE_CAP_CHAT_STATES))
+        gabble_message_util_send_chat_state (G_OBJECT (self), priv->conn,
+            LM_MESSAGE_SUB_TYPE_NORMAL, TP_CHANNEL_CHAT_STATE_GONE,
+            priv->peer_jid, NULL);
+
+      priv->send_gone = FALSE;
+    }
+
+  DEBUG ("Emitting Closed");
+  tp_svc_channel_emit_closed (self);
+}
+
 static void
 gabble_im_channel_dispose (GObject *object)
 {
   GabbleIMChannel *self = GABBLE_IM_CHANNEL (object);
-  GabbleIMChannelPrivate *priv = GABBLE_IM_CHANNEL_GET_PRIVATE (self);
+  GabbleIMChannelPrivate *priv = self->priv;
   GabblePresence *presence;
   GabbleRosterSubscription subscription;
-  gboolean cap_chat_states = FALSE;
 
   if (priv->dispose_has_run)
     return;
@@ -407,11 +435,6 @@ gabble_im_channel_dispose (GObject *object)
   presence = gabble_presence_cache_get (priv->conn->presence_cache,
       priv->handle);
 
-  if (presence && presence->caps & PRESENCE_CAP_CHAT_STATES)
-    {
-      cap_chat_states = TRUE;
-    }
-
   if ((GABBLE_ROSTER_SUBSCRIPTION_TO & subscription) == 0)
     {
       if (NULL != presence)
@@ -423,17 +446,7 @@ gabble_im_channel_dispose (GObject *object)
     }
 
   if (!priv->closed)
-      {
-        if (cap_chat_states)
-          {
-          /* Set the chat state of the channel on gone
-           * (Channel.Interface.ChatState) */
-          gabble_message_util_send_chat_state (G_OBJECT (self), priv->conn,
-              0, TP_CHANNEL_CHAT_STATE_GONE, priv->peer_jid, NULL);
-          }
-
-        tp_svc_channel_emit_closed (self);
-      }
+    emit_closed_and_send_gone (self);
 
   if (G_OBJECT_CLASS (gabble_im_channel_parent_class)->dispose)
     G_OBJECT_CLASS (gabble_im_channel_parent_class)->dispose (object);
@@ -443,7 +456,7 @@ static void
 gabble_im_channel_finalize (GObject *object)
 {
   GabbleIMChannel *self = GABBLE_IM_CHANNEL (object);
-  GabbleIMChannelPrivate *priv = GABBLE_IM_CHANNEL_GET_PRIVATE (self);
+  GabbleIMChannelPrivate *priv = self->priv;
   TpBaseConnection *conn = (TpBaseConnection *) priv->conn;
   TpHandleRepoIface *contact_handles = tp_base_connection_get_handles (conn,
       TP_HANDLE_TYPE_CONTACT);
@@ -477,7 +490,7 @@ _gabble_im_channel_send_message (GObject *object,
   gint state = -1;
 
   g_assert (GABBLE_IS_IM_CHANNEL (self));
-  priv = GABBLE_IM_CHANNEL_GET_PRIVATE (self);
+  priv = self->priv;
 
   presence = gabble_presence_cache_get (priv->conn->presence_cache,
       priv->handle);
@@ -485,6 +498,7 @@ _gabble_im_channel_send_message (GObject *object,
   if (presence && (presence->caps & PRESENCE_CAP_CHAT_STATES))
     {
       state = TP_CHANNEL_CHAT_STATE_ACTIVE;
+      priv->send_gone = TRUE;
     }
 
   /* We don't support providing successful delivery reports. */
@@ -508,6 +522,7 @@ _gabble_im_channel_receive (GabbleIMChannel *chan,
                             TpHandle sender,
                             const char *from,
                             time_t timestamp,
+                            const gchar *id,
                             const char *text,
                             TpChannelTextSendError send_error,
                             TpDeliveryStatus delivery_status)
@@ -517,15 +532,25 @@ _gabble_im_channel_receive (GabbleIMChannel *chan,
   TpMessage *msg;
 
   g_assert (GABBLE_IS_IM_CHANNEL (chan));
-  priv = GABBLE_IM_CHANNEL_GET_PRIVATE (chan);
+  priv = chan->priv;
   base_conn = (TpBaseConnection *) priv->conn;
 
-  /* update peer's full JID if it's changed */
-  if (send_error != GABBLE_TEXT_CHANNEL_SEND_NO_ERROR &&
-      0 != strcmp (from, priv->peer_jid))
+  if (send_error == GABBLE_TEXT_CHANNEL_SEND_NO_ERROR)
     {
-      g_free (priv->peer_jid);
-      priv->peer_jid = g_strdup (from);
+      /* update peer's full JID if it's changed */
+      if (tp_strdiff (from, priv->peer_jid))
+        {
+          g_free (priv->peer_jid);
+          priv->peer_jid = g_strdup (from);
+        }
+    }
+  else
+    {
+      /* strip off the resource (if any), since we just failed to send to it */
+      char *slash = strchr (priv->peer_jid, '/');
+
+      if (slash != NULL)
+        *slash = '\0';
     }
 
   msg = tp_message_new (base_conn, 2, 2);
@@ -537,6 +562,9 @@ _gabble_im_channel_receive (GabbleIMChannel *chan,
   if (timestamp != 0)
     tp_message_set_uint64 (msg, 0, "message-sent", timestamp);
 
+  if (id != NULL)
+    tp_message_set_string (msg, 0, "message-token", id);
+
   /* Body */
   tp_message_set_string (msg, 1, "content-type", "text/plain");
   tp_message_set_string (msg, 1, "content", text);
@@ -547,11 +575,22 @@ _gabble_im_channel_receive (GabbleIMChannel *chan,
           sender);
       tp_message_set_uint64 (msg, 0, "message-received", time (NULL));
 
+      /* Ensure that all incoming messages have an ID, either from the
+       * protocol or just a locally generated UUID */
+      if (id == NULL)
+        {
+          gchar *tmp = gabble_generate_id ();
+
+          tp_message_set_string (msg, 0, "message-token", tmp);
+          g_free (tmp);
+        }
+
       tp_message_mixin_take_received (G_OBJECT (chan), msg);
     }
   else
     {
       TpMessage *delivery_report = tp_message_new (base_conn, 1, 1);
+      gchar *tmp;
 
       tp_message_set_uint32 (delivery_report, 0, "message-type",
           TP_CHANNEL_TEXT_MESSAGE_TYPE_DELIVERY_REPORT);
@@ -560,10 +599,17 @@ _gabble_im_channel_receive (GabbleIMChannel *chan,
       tp_message_set_uint64 (delivery_report, 0, "message-received",
           time (NULL));
 
+      tmp = gabble_generate_id ();
+      tp_message_set_string (delivery_report, 0, "message-token", tmp);
+      g_free (tmp);
+
       tp_message_set_uint32 (delivery_report, 0, "delivery-status",
           delivery_status);
       tp_message_set_uint32 (delivery_report, 0, "delivery-error", send_error);
 
+      if (id != NULL)
+        tp_message_set_string (delivery_report, 0, "delivery-token", id);
+
       /* We're getting a send error, so the original sender of the echoed
        * message must be us! */
       tp_message_set_handle (msg, 0, "message-sender", TP_HANDLE_TYPE_CONTACT,
@@ -589,7 +635,7 @@ _gabble_im_channel_state_receive (GabbleIMChannel *chan,
 
   g_assert (state < NUM_TP_CHANNEL_CHAT_STATES);
   g_assert (GABBLE_IS_IM_CHANNEL (chan));
-  priv = GABBLE_IM_CHANNEL_GET_PRIVATE (chan);
+  priv = chan->priv;
 
   tp_svc_channel_interface_chat_state_emit_chat_state_changed (
       (TpSvcChannelInterfaceChatState *) chan,
@@ -608,16 +654,12 @@ gabble_im_channel_close (TpSvcChannel *iface,
 {
   GabbleIMChannel *self = GABBLE_IM_CHANNEL (iface);
   GabbleIMChannelPrivate *priv;
-  GabblePresence *presence;
 
   g_assert (GABBLE_IS_IM_CHANNEL (self));
 
   DEBUG ("called on %p", self);
 
-  priv = GABBLE_IM_CHANNEL_GET_PRIVATE (self);
-
-  presence = gabble_presence_cache_get (priv->conn->presence_cache,
-      priv->handle);
+  priv = self->priv;
 
   if (priv->closed)
     {
@@ -653,16 +695,7 @@ gabble_im_channel_close (TpSvcChannel *iface,
           priv->closed = TRUE;
         }
 
-      DEBUG ("Emitting Closed");
-      tp_svc_channel_emit_closed (self);
-
-      if (presence && (presence->caps & PRESENCE_CAP_CHAT_STATES))
-        {
-          /* Set the chat state of the channel to gone
-           * (Channel.Interface.ChatState) */
-          gabble_message_util_send_chat_state (G_OBJECT (self), priv->conn,
-              0, TP_CHANNEL_CHAT_STATE_GONE, priv->peer_jid, NULL);
-        }
+      emit_closed_and_send_gone (self);
     }
 
   tp_svc_channel_return_from_close (context);
@@ -722,7 +755,7 @@ gabble_im_channel_get_handle (TpSvcChannel *iface,
   GabbleIMChannelPrivate *priv;
 
   g_assert (GABBLE_IS_IM_CHANNEL (self));
-  priv = GABBLE_IM_CHANNEL_GET_PRIVATE (self);
+  priv = self->priv;
 
   tp_svc_channel_return_from_get_handle (context, TP_HANDLE_TYPE_CONTACT,
       priv->handle);
@@ -761,7 +794,7 @@ gabble_im_channel_set_chat_state (TpSvcChannelInterfaceChatState *iface,
   GError *error = NULL;
 
   g_assert (GABBLE_IS_IM_CHANNEL (self));
-  priv = GABBLE_IM_CHANNEL_GET_PRIVATE (self);
+  priv = self->priv;
 
   presence = gabble_presence_cache_get (priv->conn->presence_cache,
       priv->handle);
@@ -784,10 +817,13 @@ gabble_im_channel_set_chat_state (TpSvcChannelInterfaceChatState *iface,
           g_set_error (&error, TP_ERRORS, TP_ERROR_INVALID_ARGUMENT,
               "you may not explicitly set the Gone state");
         }
+      else if (gabble_message_util_send_chat_state (G_OBJECT (self), priv->conn,
+          LM_MESSAGE_SUB_TYPE_NORMAL, state, priv->peer_jid, &error))
+        {
+          priv->send_gone = TRUE;
+        }
 
-      if (error != NULL ||
-          !gabble_message_util_send_chat_state (G_OBJECT (self), priv->conn,
-              0, state, priv->peer_jid, &error))
+      if (error != NULL)
         {
           dbus_g_method_return_error (context, error);
           g_error_free (error);
