@@ -76,17 +76,17 @@ struct _GabbleJingleContentPrivate
 
   GabbleJingleTransportIface *transport;
 
+  /* Whether we've got the codecs (intersection) ready. */
   gboolean media_ready;
-  gboolean transport_ready;
 
-  guint timer_id;
+  /* Whether we have at least one local candidate. */
+  gboolean have_local_candidates;
+
   guint gtalk4_event_id;
 
   gboolean dispose_has_run;
 };
 
-#define GABBLE_JINGLE_CONTENT_GET_PRIVATE(o) ((o)->priv)
-
 #define DEFAULT_CONTENT_TIMEOUT 60000
 
 /* lookup tables */
@@ -95,6 +95,7 @@ G_DEFINE_TYPE(GabbleJingleContent, gabble_jingle_content, G_TYPE_OBJECT);
 
 static void new_transport_candidates_cb (GabbleJingleTransportIface *trans,
     GList *candidates, GabbleJingleContent *content);
+static void _maybe_ready (GabbleJingleContent *self);
 
 static void
 gabble_jingle_content_init (GabbleJingleContent *obj)
@@ -104,11 +105,12 @@ gabble_jingle_content_init (GabbleJingleContent *obj)
          GabbleJingleContentPrivate);
   obj->priv = priv;
 
+  DEBUG ("%p", obj);
+
   priv->state = JINGLE_CONTENT_STATE_EMPTY;
   priv->created_by_us = TRUE;
   priv->media_ready = FALSE;
-  priv->transport_ready = FALSE;
-  priv->timer_id = 0;
+  priv->have_local_candidates = FALSE;
   priv->gtalk4_event_id = 0;
   priv->dispose_has_run = FALSE;
 
@@ -120,23 +122,14 @@ static void
 gabble_jingle_content_dispose (GObject *object)
 {
   GabbleJingleContent *content = GABBLE_JINGLE_CONTENT (object);
-  GabbleJingleContentPrivate *priv = GABBLE_JINGLE_CONTENT_GET_PRIVATE (content);
+  GabbleJingleContentPrivate *priv = content->priv;
 
   if (priv->dispose_has_run)
     return;
 
-  DEBUG ("dispose called");
+  DEBUG ("%p", object);
   priv->dispose_has_run = TRUE;
 
-  /* If we're in the middle of content-add/-accept when the session is
-   * terminated, we'll get disposed without being explicitly removed from
-   * the session. So, remove the timer here. */
-  if (priv->timer_id != 0)
-    {
-      g_source_remove (priv->timer_id);
-      priv->timer_id = 0;
-    }
-
   if (priv->gtalk4_event_id != 0)
     {
       g_source_remove (priv->gtalk4_event_id);
@@ -169,7 +162,7 @@ gabble_jingle_content_get_property (GObject *object,
                                     GParamSpec *pspec)
 {
   GabbleJingleContent *self = GABBLE_JINGLE_CONTENT (object);
-  GabbleJingleContentPrivate *priv = GABBLE_JINGLE_CONTENT_GET_PRIVATE (self);
+  GabbleJingleContentPrivate *priv = self->priv;
 
   switch (property_id) {
     case PROP_CONNECTION:
@@ -212,7 +205,7 @@ gabble_jingle_content_set_property (GObject *object,
                                     GParamSpec *pspec)
 {
   GabbleJingleContent *self = GABBLE_JINGLE_CONTENT (object);
-  GabbleJingleContentPrivate *priv = GABBLE_JINGLE_CONTENT_GET_PRIVATE (self);
+  GabbleJingleContentPrivate *priv = self->priv;
 
   switch (property_id) {
     case PROP_CONNECTION:
@@ -282,83 +275,57 @@ gabble_jingle_content_class_init (GabbleJingleContentClass *cls)
 
   /* property definitions */
   param_spec = g_param_spec_object ("connection", "GabbleConnection object",
-                                    "Gabble connection object used for exchanging "
-                                    "messages.",
-                                    GABBLE_TYPE_CONNECTION,
-                                    G_PARAM_CONSTRUCT_ONLY |
-                                    G_PARAM_READWRITE |
-                                    G_PARAM_STATIC_NICK |
-                                    G_PARAM_STATIC_BLURB);
+      "Gabble connection object used for exchanging messages.",
+      GABBLE_TYPE_CONNECTION,
+      G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
   g_object_class_install_property (object_class, PROP_CONNECTION, param_spec);
 
   param_spec = g_param_spec_object ("session", "GabbleJingleSession object",
-                                    "Jingle session object that owns this content.",
-                                    GABBLE_TYPE_JINGLE_SESSION,
-                                    G_PARAM_CONSTRUCT_ONLY |
-                                    G_PARAM_READWRITE |
-                                    G_PARAM_STATIC_NICK |
-                                    G_PARAM_STATIC_BLURB);
+      "Jingle session object that owns this content.",
+      GABBLE_TYPE_JINGLE_SESSION,
+      G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
   g_object_class_install_property (object_class, PROP_SESSION, param_spec);
 
   param_spec = g_param_spec_string ("name", "Content name",
-                                    "A unique content name in the session.",
-                                    NULL,
-                                    G_PARAM_CONSTRUCT_ONLY |
-                                    G_PARAM_READWRITE |
-                                    G_PARAM_STATIC_NAME |
-                                    G_PARAM_STATIC_BLURB);
+      "A unique content name in the session.",
+      NULL,
+      G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
   g_object_class_install_property (object_class, PROP_NAME, param_spec);
 
   param_spec = g_param_spec_string ("content-ns", "Content namespace",
-                                    "Namespace identifying the content type.",
-                                    NULL,
-                                    G_PARAM_READWRITE |
-                                    G_PARAM_STATIC_NAME |
-                                    G_PARAM_STATIC_BLURB);
+      "Namespace identifying the content type.",
+      NULL,
+      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
   g_object_class_install_property (object_class, PROP_CONTENT_NS, param_spec);
 
   param_spec = g_param_spec_string ("transport-ns", "Transport namespace",
-                                    "Namespace identifying the transport type.",
-                                    NULL,
-                                    G_PARAM_READWRITE |
-                                    G_PARAM_STATIC_NAME |
-                                    G_PARAM_STATIC_BLURB);
+      "Namespace identifying the transport type.",
+      NULL,
+      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
   g_object_class_install_property (object_class, PROP_TRANSPORT_NS, param_spec);
 
-
-
   param_spec = g_param_spec_uint ("senders", "Stream senders",
-                                  "Valid senders for the stream.",
-                                  0, G_MAXUINT32, JINGLE_CONTENT_SENDERS_NONE,
-                                  G_PARAM_READWRITE |
-                                  G_PARAM_STATIC_NAME |
-                                  G_PARAM_STATIC_BLURB);
+      "Valid senders for the stream.",
+      0, G_MAXUINT32, JINGLE_CONTENT_SENDERS_NONE,
+      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
   g_object_class_install_property (object_class, PROP_SENDERS, param_spec);
 
   param_spec = g_param_spec_uint ("state", "Content state",
-                                  "The current state that the content is in.",
-                                  0, G_MAXUINT32, JINGLE_CONTENT_STATE_EMPTY,
-                                  G_PARAM_READWRITE |
-                                  G_PARAM_STATIC_NAME |
-                                  G_PARAM_STATIC_BLURB);
+      "The current state that the content is in.",
+      0, G_MAXUINT32, JINGLE_CONTENT_STATE_EMPTY,
+      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
   g_object_class_install_property (object_class, PROP_STATE, param_spec);
 
   param_spec = g_param_spec_string ("disposition", "Content disposition",
-                                    "Distinguishes between 'session' and other "
-                                    "contents.",
-                                    NULL,
-                                    G_PARAM_READWRITE |
-                                    G_PARAM_STATIC_NAME |
-                                    G_PARAM_STATIC_BLURB);
+      "Distinguishes between 'session' and other contents.",
+      NULL,
+      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
   g_object_class_install_property (object_class, PROP_DISPOSITION, param_spec);
 
   param_spec = g_param_spec_boolean ("locally-created", "Locally created",
-                                     "True if the content was created by the "
-                                     "local client.",
-                                     FALSE,
-                                     G_PARAM_READABLE |
-                                     G_PARAM_STATIC_NAME |
-                                     G_PARAM_STATIC_BLURB);
+      "True if the content was created by the local client.",
+      FALSE,
+      G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
   g_object_class_install_property (object_class, PROP_LOCALLY_CREATED, param_spec);
 
   /* signal definitions */
@@ -379,6 +346,10 @@ gabble_jingle_content_class_init (GabbleJingleContentClass *cls)
     NULL, NULL,
     g_cclosure_marshal_VOID__POINTER, G_TYPE_NONE, 1, G_TYPE_POINTER);
 
+  /* This signal serves as notification that the GabbleJingleContent is now
+   * meaningless; everything holding a reference should drop it after receiving
+   * 'removed'.
+   */
   signals[REMOVED] = g_signal_new ("removed",
     G_OBJECT_CLASS_TYPE (cls),
     G_SIGNAL_RUN_LAST | G_SIGNAL_DETAILED,
@@ -451,7 +422,7 @@ static gboolean
 send_gtalk4_transport_accept (gpointer user_data)
 {
   GabbleJingleContent *c = GABBLE_JINGLE_CONTENT (user_data);
-  GabbleJingleContentPrivate *priv = GABBLE_JINGLE_CONTENT_GET_PRIVATE (c);
+  GabbleJingleContentPrivate *priv = c->priv;
   LmMessageNode *sess_node, *tnode;
   LmMessage *msg = gabble_jingle_session_new_message (c->session,
       JINGLE_ACTION_TRANSPORT_ACCEPT, &sess_node);
@@ -469,14 +440,12 @@ void
 gabble_jingle_content_parse_add (GabbleJingleContent *c,
     LmMessageNode *content_node, gboolean google_mode, GError **error)
 {
-  GabbleJingleContentPrivate *priv = GABBLE_JINGLE_CONTENT_GET_PRIVATE (c);
+  GabbleJingleContentPrivate *priv = c->priv;
   const gchar *name, *creator, *senders, *disposition;
   LmMessageNode *trans_node, *desc_node;
   GType transport_type = 0;
   GabbleJingleTransportIface *trans = NULL;
-  JingleDialect dialect;
-
-  g_object_get (c->session, "dialect", &dialect, NULL);
+  JingleDialect dialect = gabble_jingle_session_get_dialect (c->session);
 
   desc_node = lm_message_node_get_child_any_ns (content_node, "description");
   trans_node = lm_message_node_get_child_any_ns (content_node, "transport");
@@ -494,8 +463,12 @@ gabble_jingle_content_parse_add (GabbleJingleContent *c,
       if (creator == NULL)
           creator = "initiator";
 
-      if (name == NULL)
-          name = "gtalk";
+      /* the google protocols don't give the contents names, so put in a dummy
+       * value if none was set by the session*/
+      if (priv->name == NULL)
+        name = priv->name = g_strdup ("gtalk");
+      else
+        name = priv->name;
 
       if (trans_node == NULL)
         {
@@ -506,6 +479,14 @@ gabble_jingle_content_parse_add (GabbleJingleContent *c,
           g_object_set (c->session, "dialect", JINGLE_DIALECT_GTALK3, NULL);
           transport_type = gabble_jingle_factory_lookup_transport (
               c->conn->jingle_factory, "");
+
+          /* in practice we do support gtalk-p2p, so this can't happen */
+          if (G_UNLIKELY (transport_type == 0))
+            {
+              SET_BAD_REQ ("gtalk-p2p transport unsupported");
+              return;
+            }
+
           priv->transport_ns = g_strdup ("");
         }
     }
@@ -516,6 +497,10 @@ gabble_jingle_content_parse_add (GabbleJingleContent *c,
           SET_BAD_REQ ("missing required content attributes or elements");
           return;
         }
+
+      /* In proper protocols the name comes from the stanza */
+      g_assert (priv->name == NULL);
+      priv->name = g_strdup (name);
     }
 
   /* if we didn't set it to google-p2p implicitly already, detect it */
@@ -575,20 +560,17 @@ gabble_jingle_content_parse_add (GabbleJingleContent *c,
   g_assert (priv->transport == NULL);
   priv->transport = trans;
 
-  g_assert (priv->name == NULL);
-  priv->name = g_strdup (name);
-
   g_assert (priv->creator == NULL);
   priv->creator = g_strdup (creator);
 
   priv->state = JINGLE_CONTENT_STATE_NEW;
 
-  /* GTALK4 seems to require "transport-accept" for acknowledging
-   * the transport type? */
+  /* GTalk4 seems to require "transport-accept" for acknowledging
+   * the transport type. wjt confirms that this is apparently necessary for
+   * incoming calls to work.
+   */
   if (dialect == JINGLE_DIALECT_GTALK4)
-    {
-      priv->gtalk4_event_id = g_idle_add (send_gtalk4_transport_accept, c);
-    }
+    priv->gtalk4_event_id = g_idle_add (send_gtalk4_transport_accept, c);
 
   return;
 }
@@ -597,27 +579,21 @@ void
 gabble_jingle_content_parse_accept (GabbleJingleContent *c,
     LmMessageNode *content_node, gboolean google_mode, GError **error)
 {
-  GabbleJingleContentPrivate *priv = GABBLE_JINGLE_CONTENT_GET_PRIVATE (c);
+  GabbleJingleContentPrivate *priv = c->priv;
   const gchar *senders;
   LmMessageNode *trans_node, *desc_node;
-  JingleDialect dialect;
+  JingleDialect dialect = gabble_jingle_session_get_dialect (c->session);
   JingleContentSenders newsenders;
 
   desc_node = lm_message_node_get_child_any_ns (content_node, "description");
   trans_node = lm_message_node_get_child_any_ns (content_node, "transport");
   senders = lm_message_node_get_attribute (content_node, "senders");
 
-  g_object_get (c->session, "dialect", &dialect, NULL);
-
-  /* FIXME: if we examine dialect manually, we don't need google_mode param flag */
-  if (google_mode)
+  if (JINGLE_IS_GOOGLE_DIALECT (dialect) && trans_node == NULL)
     {
-      if (trans_node == NULL)
-        {
-          DEBUG ("no transport node, assuming GTalk3 dialect");
-          /* gtalk lj0.3 assumes google-p2p transport */
-          g_object_set (c->session, "dialect", JINGLE_DIALECT_GTALK3, NULL);
-        }
+      DEBUG ("no transport node, assuming GTalk3 dialect");
+      /* gtalk lj0.3 assumes google-p2p transport */
+      g_object_set (c->session, "dialect", JINGLE_DIALECT_GTALK3, NULL);
     }
 
   if (senders == NULL)
@@ -641,69 +617,94 @@ gabble_jingle_content_parse_accept (GabbleJingleContent *c,
   if (*error != NULL)
       return;
 
-  if (priv->timer_id != 0)
-      g_source_remove (priv->timer_id);
-  priv->timer_id = 0;
-
   priv->state = JINGLE_CONTENT_STATE_ACKNOWLEDGED;
   g_object_notify ((GObject *) c, "state");
 }
 
 void
-gabble_jingle_content_produce_node (GabbleJingleContent *c,
-  LmMessageNode *parent, gboolean full)
+gabble_jingle_content_parse_description_info (GabbleJingleContent *c,
+    LmMessageNode *content_node, GError **error)
 {
-  GabbleJingleContentPrivate *priv = GABBLE_JINGLE_CONTENT_GET_PRIVATE (c);
+  GabbleJingleContentPrivate *priv = c->priv;
+  LmMessageNode *desc_node;
+  desc_node = lm_message_node_get_child_any_ns (content_node, "description");
+  if (desc_node == NULL)
+    {
+      SET_BAD_REQ ("invalid description-info action");
+      return;
+    }
+
+  if (priv->created_by_us && priv->state < JINGLE_CONTENT_STATE_ACKNOWLEDGED)
+    {
+      /* The stream was created by us and the other side didn't acknowledge it
+       * yet, thus we don't have their codec information, thus the
+       * description-info isn't meaningful and can be ignored */
+      DEBUG ("Ignoring description-info as we didn't receive the codecs yet");
+      return;
+    }
+
+  parse_description (c, desc_node, error);
+}
+
+
+void
+gabble_jingle_content_produce_node (GabbleJingleContent *c,
+    LmMessageNode *parent,
+    gboolean include_description,
+    gboolean include_transport,
+    LmMessageNode **trans_node_out)
+{
+  GabbleJingleContentPrivate *priv = c->priv;
   LmMessageNode *content_node, *trans_node;
-  JingleDialect dialect;
+  JingleDialect dialect = gabble_jingle_session_get_dialect (c->session);
   void (*produce_desc)(GabbleJingleContent *, LmMessageNode *) =
     GABBLE_JINGLE_CONTENT_GET_CLASS (c)->produce_description;
 
-  g_object_get (c->session, "dialect", &dialect, NULL);
-
   if ((dialect == JINGLE_DIALECT_GTALK3) ||
       (dialect == JINGLE_DIALECT_GTALK4))
     {
-      /* content-* isn't used in GTalk anyways, so we always have to include
-       * the full content description */
-      g_assert (full == TRUE);
-
       content_node = parent;
     }
   else
     {
-      gboolean session_created_by_us;
-
       content_node = lm_message_node_add_child (parent, "content", NULL);
       lm_message_node_set_attributes (content_node,
           "name", priv->name,
           "senders", produce_senders (priv->senders),
           NULL);
 
-      g_object_get (c->session, "local-initiator", &session_created_by_us,
-        NULL);
-
-      if (priv->created_by_us == session_created_by_us)
+      if (gabble_jingle_content_creator_is_initiator (c))
         lm_message_node_set_attribute (content_node, "creator", "initiator");
       else
         lm_message_node_set_attribute (content_node, "creator", "responder");
     }
 
-  if (!full)
-    return;
+  if (include_description)
+    produce_desc (c, content_node);
 
-  produce_desc (c, content_node);
+  if (include_transport)
+    {
+      if (dialect == JINGLE_DIALECT_GTALK3)
+        {
+          /* GTalk 03 doesn't use a transport, but assumes gtalk-p2p */
+          trans_node = parent;
+        }
+      else
+        {
+          trans_node = lm_message_node_add_child (content_node, "transport", NULL);
+          lm_message_node_set_attribute (trans_node, "xmlns", priv->transport_ns);
+        }
 
-  /* We can do it here, don't need to call into transport object for this */
-  trans_node = lm_message_node_add_child (content_node, "transport", NULL);
-  lm_message_node_set_attribute (trans_node, "xmlns", priv->transport_ns);
+      if (trans_node_out != NULL)
+        *trans_node_out = trans_node;
+    }
 }
 
 void
 gabble_jingle_content_update_senders (GabbleJingleContent *c,
     LmMessageNode *content_node, GError **error)
 {
-  GabbleJingleContentPrivate *priv = GABBLE_JINGLE_CONTENT_GET_PRIVATE (c);
+  GabbleJingleContentPrivate *priv = c->priv;
   JingleContentSenders senders;
 
   senders = parse_senders (lm_message_node_get_attribute (content_node, "senders"));
@@ -722,7 +723,7 @@ void
 gabble_jingle_content_parse_transport_info (GabbleJingleContent *self,
   LmMessageNode *trans_node, GError **error)
 {
-  GabbleJingleContentPrivate *priv = GABBLE_JINGLE_CONTENT_GET_PRIVATE (self);
+  GabbleJingleContentPrivate *priv = self->priv;
 
   gabble_jingle_transport_iface_parse_candidates (priv->transport, trans_node, error);
 }
@@ -731,9 +732,24 @@ gabble_jingle_content_parse_transport_info (GabbleJingleContent *self,
 void
 gabble_jingle_content_add_candidates (GabbleJingleContent *self, GList *li)
 {
-  GabbleJingleContentPrivate *priv = GABBLE_JINGLE_CONTENT_GET_PRIVATE (self);
+  GabbleJingleContentPrivate *priv = self->priv;
 
-  gabble_jingle_transport_iface_add_candidates (priv->transport, li);
+  DEBUG ("called");
+
+  gabble_jingle_transport_iface_new_local_candidates (priv->transport, li);
+
+  if (!priv->have_local_candidates)
+    {
+      priv->have_local_candidates = TRUE;
+      /* Maybe we were waiting for at least one candidate? */
+      _maybe_ready (self);
+    }
+
+  /* If the content exists on the wire, let the transport send this candidate
+   * if it wants to.
+   */
+  if (priv->state > JINGLE_CONTENT_STATE_EMPTY)
+    gabble_jingle_transport_iface_send_candidates (priv->transport, FALSE);
 }
 
 /* Returns whether the content is ready to be signalled (initiated, for local
@@ -741,43 +757,36 @@ gabble_jingle_content_add_candidates (GabbleJingleContent *self, GList *li)
 gboolean
 gabble_jingle_content_is_ready (GabbleJingleContent *self)
 {
-  GabbleJingleContentPrivate *priv = GABBLE_JINGLE_CONTENT_GET_PRIVATE (self);
+  GabbleJingleContentPrivate *priv = self->priv;
 
-  /* If it's created by us, media ready and not signalled,
-   * it's ready to be added. */
-  if (priv->created_by_us && priv->media_ready &&
-      (priv->state == JINGLE_CONTENT_STATE_EMPTY))
-          return TRUE;
+  if (priv->created_by_us)
+    {
+      /* If it's created by us, media ready, not signalled, and we have
+       * at least one local candidate, it's ready to be added. */
+      if (priv->media_ready && priv->have_local_candidates &&
+          (priv->state == JINGLE_CONTENT_STATE_EMPTY))
+        return TRUE;
+    }
+  else
+    {
+      /* If it's created by peer, media and transports ready,
+       * and not acknowledged yet, it's ready for acceptance. */
+      if (priv->media_ready &&
+          gabble_jingle_transport_iface_can_accept (priv->transport) &&
+          (priv->state == JINGLE_CONTENT_STATE_NEW))
+        return TRUE;
+    }
 
-  /* If it's created by peer, media and transports ready,
-   * and not acknowledged yet, it's ready for acceptance. */
-  if (!priv->created_by_us && priv->media_ready && priv->transport_ready &&
-      (priv->state == JINGLE_CONTENT_STATE_NEW))
-          return TRUE;
-
-  return FALSE;
-}
-
-static gboolean
-timeout_content (gpointer data)
-{
-  GabbleJingleContent *c = data;
-
-  c->priv->timer_id = 0;
-  DEBUG ("content timed out");
-
-  /* we're handling it as if it were rejected */
-  gabble_jingle_content_remove (c, FALSE);
   return FALSE;
 }
 
 static void
 send_content_add_or_accept (GabbleJingleContent *self)
 {
-  GabbleJingleContentPrivate *priv = GABBLE_JINGLE_CONTENT_GET_PRIVATE (self);
+  GabbleJingleContentPrivate *priv = self->priv;
   LmMessage *msg;
-  LmMessageNode *sess_node;
-  JingleAction action = JINGLE_ACTION_UNKNOWN;
+  LmMessageNode *sess_node, *transport_node;
+  JingleAction action;
   JingleContentState new_state = JINGLE_CONTENT_STATE_EMPTY;
 
   g_assert (gabble_jingle_content_is_ready (self));
@@ -787,10 +796,6 @@ send_content_add_or_accept (GabbleJingleContent *self)
       /* TODO: set a timer for acknowledgement */
       action = JINGLE_ACTION_CONTENT_ADD;
       new_state = JINGLE_CONTENT_STATE_SENT;
-
-      g_assert (priv->timer_id == 0);
-      priv->timer_id = g_timeout_add (DEFAULT_CONTENT_TIMEOUT,
-        timeout_content, self);
     }
   else
     {
@@ -800,7 +805,10 @@ send_content_add_or_accept (GabbleJingleContent *self)
 
   msg = gabble_jingle_session_new_message (self->session,
       action, &sess_node);
-  gabble_jingle_content_produce_node (self, sess_node, TRUE);
+  gabble_jingle_content_produce_node (self, sess_node, TRUE, TRUE,
+      &transport_node);
+  gabble_jingle_transport_iface_inject_candidates (priv->transport,
+      transport_node);
   gabble_jingle_session_send (self->session, msg, NULL, NULL);
 
   priv->state = new_state;
@@ -810,7 +818,7 @@ send_content_add_or_accept (GabbleJingleContent *self)
 static void
 _maybe_ready (GabbleJingleContent *self)
 {
-  GabbleJingleContentPrivate *priv = GABBLE_JINGLE_CONTENT_GET_PRIVATE (self);
+  GabbleJingleContentPrivate *priv = self->priv;
   JingleSessionState state;
 
   if (!gabble_jingle_content_is_ready (self))
@@ -838,7 +846,7 @@ _maybe_ready (GabbleJingleContent *self)
           send_content_add_or_accept (self);
 
           /* if neccessary, transmit the candidates */
-          gabble_jingle_transport_iface_retransmit_candidates (priv->transport,
+          gabble_jingle_transport_iface_send_candidates (priv->transport,
               FALSE);
         }
       else
@@ -851,20 +859,58 @@ _maybe_ready (GabbleJingleContent *self)
     }
 }
 
+void
+gabble_jingle_content_maybe_send_description (GabbleJingleContent *self)
+{
+  GabbleJingleContentPrivate *priv = self->priv;
+
+  /* If we didn't send the content yet there is no reason to send a
+   * description-info to update it */
+  if (priv->state < JINGLE_CONTENT_STATE_SENT)
+    return;
+
+  if (gabble_jingle_session_defines_action (self->session,
+          JINGLE_ACTION_DESCRIPTION_INFO))
+    {
+      LmMessageNode *sess_node;
+      LmMessage *msg = gabble_jingle_session_new_message (self->session,
+          JINGLE_ACTION_DESCRIPTION_INFO, &sess_node);
+
+      gabble_jingle_content_produce_node (self, sess_node, TRUE, FALSE, NULL);
+      gabble_jingle_session_send (self->session, msg, NULL, NULL);
+    }
+  else
+    {
+      DEBUG ("not sending description-info, speaking an old dialect");
+    }
+}
+
+
 /* Used when session-initiate is sent (so all initial contents transmit their
  * candidates), and when we detect gtalk3 after we've transmitted some
  * candidates. */
 void
-gabble_jingle_content_retransmit_candidates (GabbleJingleContent *self)
+gabble_jingle_content_retransmit_candidates (GabbleJingleContent *self,
+    gboolean all)
 {
-  gabble_jingle_transport_iface_retransmit_candidates (self->priv->transport, TRUE);
+  gabble_jingle_transport_iface_send_candidates (self->priv->transport, all);
 }
 
+void
+gabble_jingle_content_inject_candidates (GabbleJingleContent *self,
+    LmMessageNode *transport_node)
+{
+  gabble_jingle_transport_iface_inject_candidates (self->priv->transport,
+      transport_node);
+}
+
+
 /* Called by a subclass when the media is ready (e.g. we got local codecs) */
 void
 _gabble_jingle_content_set_media_ready (GabbleJingleContent *self)
 {
-  GabbleJingleContentPrivate *priv = GABBLE_JINGLE_CONTENT_GET_PRIVATE (self);
+  GabbleJingleContentPrivate *priv = self->priv;
+
 
   priv->media_ready = TRUE;
 
@@ -875,21 +921,17 @@ void
 gabble_jingle_content_set_transport_state (GabbleJingleContent *self,
     JingleTransportState state)
 {
-  GabbleJingleContentPrivate *priv = GABBLE_JINGLE_CONTENT_GET_PRIVATE (self);
+  GabbleJingleContentPrivate *priv = self->priv;
 
   g_object_set (priv->transport, "state", state, NULL);
 
-  if (state == JINGLE_TRANSPORT_STATE_CONNECTED)
-    {
-      priv->transport_ready = TRUE;
-      _maybe_ready (self);
-    }
+  _maybe_ready (self);
 }
 
 GList *
 gabble_jingle_content_get_remote_candidates (GabbleJingleContent *c)
 {
-  GabbleJingleContentPrivate *priv = GABBLE_JINGLE_CONTENT_GET_PRIVATE (c);
+  GabbleJingleContentPrivate *priv = c->priv;
 
   return gabble_jingle_transport_iface_get_remote_candidates (priv->transport);
 }
@@ -898,12 +940,10 @@ gboolean
 gabble_jingle_content_change_direction (GabbleJingleContent *c,
     JingleContentSenders senders)
 {
-  GabbleJingleContentPrivate *priv = GABBLE_JINGLE_CONTENT_GET_PRIVATE (c);
+  GabbleJingleContentPrivate *priv = c->priv;
   LmMessage *msg;
   LmMessageNode *sess_node;
-  JingleDialect dialect;
-
-  g_object_get (c->session, "dialect", &dialect, NULL);
+  JingleDialect dialect = gabble_jingle_session_get_dialect (c->session);
 
   if (JINGLE_IS_GOOGLE_DIALECT (dialect))
     {
@@ -915,7 +955,7 @@ gabble_jingle_content_change_direction (GabbleJingleContent *c,
 
   msg = gabble_jingle_session_new_message (c->session,
       JINGLE_ACTION_CONTENT_MODIFY, &sess_node);
-  gabble_jingle_content_produce_node (c, sess_node, FALSE); 
+  gabble_jingle_content_produce_node (c, sess_node, FALSE, FALSE, NULL);
   gabble_jingle_session_send (c->session, msg, NULL, NULL);
 
   /* FIXME: actually check whether remote end accepts our content-modify */
@@ -923,61 +963,60 @@ gabble_jingle_content_change_direction (GabbleJingleContent *c,
 }
 
 static void
-_on_remove_reply (GabbleJingleSession *sess, gboolean success,
-    LmMessage *reply, gpointer user_data)
+_on_remove_reply (GObject *c_as_obj,
+    gboolean success,
+    LmMessage *reply)
 {
-  GabbleJingleContent *c = GABBLE_JINGLE_CONTENT (user_data);
-  GabbleJingleContentPrivate *priv = GABBLE_JINGLE_CONTENT_GET_PRIVATE (c);
+  GabbleJingleContent *c = GABBLE_JINGLE_CONTENT (c_as_obj);
+  GabbleJingleContentPrivate *priv = c->priv;
 
   g_assert (priv->state == JINGLE_CONTENT_STATE_REMOVING);
 
-  g_signal_emit (c, signals[REMOVED], 0);
+  DEBUG ("%p", c);
 
-  g_object_unref (c);
+  /* Everything holding a reference to a content should drop it after receiving
+   * 'removed'.
+   */
+  g_signal_emit (c, signals[REMOVED], 0);
 }
 
 void
 gabble_jingle_content_remove (GabbleJingleContent *c, gboolean signal_peer)
 {
-  GabbleJingleContentPrivate *priv = GABBLE_JINGLE_CONTENT_GET_PRIVATE (c);
+  GabbleJingleContentPrivate *priv = c->priv;
   LmMessage *msg;
   LmMessageNode *sess_node;
 
-  DEBUG ("called for content %s", priv->name);
-
-  if (priv->timer_id != 0)
-    {
-      g_source_remove (priv->timer_id);
-      priv->timer_id = 0;
-    }
-
-  if (priv->state == JINGLE_CONTENT_STATE_REMOVING)
-    {
-      DEBUG ("ignoring request to remove content which is already being removed");
-      return;
-    }
-
-  priv->state = JINGLE_CONTENT_STATE_REMOVING;
-  g_object_notify ((GObject *) c, "state");
+  DEBUG ("called for %p (%s)", c, priv->name);
 
   /* If we were already signalled and removal is not a side-effect of
    * something else (sesssion termination, or removal by peer),
    * we have to signal removal to the peer. */
   if (signal_peer && (priv->state != JINGLE_CONTENT_STATE_EMPTY))
     {
+      if (priv->state == JINGLE_CONTENT_STATE_REMOVING)
+        {
+          DEBUG ("ignoring request to remove content which is already being removed");
+          return;
+        }
+
+      priv->state = JINGLE_CONTENT_STATE_REMOVING;
+      g_object_notify ((GObject *) c, "state");
+
       msg = gabble_jingle_session_new_message (c->session,
           JINGLE_ACTION_CONTENT_REMOVE, &sess_node);
-      gabble_jingle_content_produce_node (c, sess_node, FALSE);
-      g_object_ref (c);
-      gabble_jingle_session_send (c->session, msg, _on_remove_reply, c);
+      gabble_jingle_content_produce_node (c, sess_node, FALSE, FALSE, NULL);
+      gabble_jingle_session_send (c->session, msg, _on_remove_reply,
+          (GObject *) c);
     }
   else
     {
+      DEBUG ("signalling removed with %u refs", G_OBJECT (c)->ref_count);
+      /* Everything holding a reference to a content should drop it after receiving
+       * 'removed'.
+       */
       g_signal_emit (c, signals[REMOVED], 0);
     }
-
-  /* At this point content could be unreffed by REMOVED handler
-   * and disposed of; don't do anything else with it. */
 }
 
 gboolean
@@ -986,3 +1025,42 @@ gabble_jingle_content_is_created_by_us (GabbleJingleContent *c)
   return c->priv->created_by_us;
 }
 
+gboolean
+gabble_jingle_content_creator_is_initiator (GabbleJingleContent *c)
+{
+  gboolean session_created_by_us;
+
+  g_object_get (c->session, "local-initiator", &session_created_by_us, NULL);
+
+  return (c->priv->created_by_us == session_created_by_us);
+}
+
+const gchar *
+gabble_jingle_content_get_name (GabbleJingleContent *self)
+{
+  return self->priv->name;
+}
+
+const gchar *
+gabble_jingle_content_get_ns (GabbleJingleContent *self)
+{
+  return self->priv->content_ns;
+}
+
+const gchar *
+gabble_jingle_content_get_transport_ns (GabbleJingleContent *self)
+{
+  return self->priv->transport_ns;
+}
+
+const gchar *
+gabble_jingle_content_get_disposition (GabbleJingleContent *self)
+{
+  return self->priv->disposition;
+}
+
+JingleTransportType
+gabble_jingle_content_get_transport_type (GabbleJingleContent *c)
+{
+  return gabble_jingle_transport_iface_get_transport_type (c->priv->transport);
+}
