@@ -27,6 +27,7 @@
 
 #include <lib/gibber/gibber-resolver.h>
 #include <loudmouth/loudmouth.h>
+#include <libsoup/soup.h>
 
 #define DEBUG_FLAG GABBLE_DEBUG_MEDIA
 
@@ -35,6 +36,8 @@
 #include "jingle-media-rtp.h"
 #include "jingle-session.h"
 #include "jingle-transport-google.h"
+#include "jingle-transport-rawudp.h"
+#include "jingle-transport-iceudp.h"
 #include "namespaces.h"
 #include "util.h"
 
@@ -56,6 +59,10 @@ enum
   LAST_PROPERTY
 };
 
+/* The 'session' map is keyed by:
+ * "<peer's handle>\n<peer's resource>\n<session id>"
+ */
+#define SESSION_MAP_KEY_FORMAT "%u\n%s\n%s"
 
 struct _GabbleJingleFactoryPrivate
 {
@@ -64,8 +71,11 @@ struct _GabbleJingleFactoryPrivate
   LmMessageHandler *jingle_info_cb;
   GHashTable *content_types;
   GHashTable *transports;
+
+  /* instances of SESSION_MAP_KEY_FORMAT => GabbleJingleSession. */
   GHashTable *sessions;
   GibberResolver *resolver;
+  SoupSession *soup;
 
   gchar *stun_server;
   guint16 stun_port;
@@ -73,22 +83,40 @@ struct _GabbleJingleFactoryPrivate
   guint16 fallback_stun_port;
   gchar *relay_token;
   gboolean get_stun_from_jingle;
+  gchar *relay_server;
+  guint16 relay_http_port;
+  guint16 relay_udp;
+  guint16 relay_tcp;
+  guint16 relay_ssltcp;
+
   gboolean dispose_has_run;
 };
 
-#define GABBLE_JINGLE_FACTORY_GET_PRIVATE(o) ((o)->priv)
-
 static LmHandlerResult jingle_cb (LmMessageHandler *handler,
     LmConnection *lmconn, LmMessage *message, gpointer user_data);
 static GabbleJingleSession *create_session (GabbleJingleFactory *fac,
-    const gchar *sid, TpHandle peer, const gchar *peer_resource);
+    const gchar *sid,
+    TpHandle peer,
+    const gchar *peer_resource,
+    gboolean local_hold);
 
 static void session_terminated_cb (GabbleJingleSession *sess,
-    gboolean local_terminator, GabbleJingleFactory *fac);
+    gboolean local_terminator, TpChannelGroupChangeReason reason,
+    GabbleJingleFactory *fac);
 
 static void connection_status_changed_cb (GabbleConnection *conn,
     guint status, guint reason, GabbleJingleFactory *self);
 
+#define RELAY_HTTP_TIMEOUT 5
+
+static gboolean test_mode = FALSE;
+
+void
+gabble_jingle_factory_set_test_mode (void)
+{
+  test_mode = TRUE;
+}
+
 static void
 gabble_jingle_factory_init (GabbleJingleFactory *obj)
 {
@@ -111,6 +139,7 @@ gabble_jingle_factory_init (GabbleJingleFactory *obj)
   priv->conn = NULL;
   priv->dispose_has_run = FALSE;
   priv->resolver = gibber_resolver_get_resolver ();
+  priv->relay_http_port = 80;
 }
 
 typedef struct {
@@ -206,20 +235,12 @@ take_stun_server (GabbleJingleFactory *self,
       G_OBJECT (self));
 }
 
-/*
- * jingle_info_cb
- *
- * Called by loudmouth when we get an incoming <iq>. This handler
- * is concerned only with Jingle info queries.
- */
+
 static LmHandlerResult
-jingle_info_cb (LmMessageHandler *handler,
-                LmConnection *lmconn,
-                LmMessage *message,
-                gpointer user_data)
+got_jingle_info_stanza (GabbleJingleFactory *fac,
+    LmMessage *message)
 {
-  GabbleJingleFactory *fac = GABBLE_JINGLE_FACTORY (user_data);
-  GabbleJingleFactoryPrivate *priv = GABBLE_JINGLE_FACTORY_GET_PRIVATE (fac);
+  GabbleJingleFactoryPrivate *priv = fac->priv;
   LmMessageSubType sub_type;
   LmMessageNode *query_node, *node;
 
@@ -288,21 +309,83 @@ jingle_info_cb (LmMessageHandler *handler,
 
   if (node != NULL)
     {
-      node = lm_message_node_get_child (node, "token");
+      LmMessageNode *subnode;
 
-      if (node != NULL)
+      subnode = lm_message_node_get_child (node, "token");
+
+      if (subnode != NULL)
         {
           const gchar *token;
 
-          token = lm_message_node_get_value (node);
-
+          token = lm_message_node_get_value (subnode);
           if (token != NULL)
             {
-              DEBUG ("jingle info: got relay token %s", token);
+              DEBUG ("jingle info: got Google relay token %s", token);
               g_free (fac->priv->relay_token);
               fac->priv->relay_token = g_strdup (token);
             }
         }
+
+      subnode = lm_message_node_get_child (node, "server");
+
+      if (subnode != NULL)
+        {
+          const gchar *server;
+          const gchar *port;
+
+          server = lm_message_node_get_attribute (subnode, "host");
+
+          if (server != NULL)
+            {
+              DEBUG ("jingle info: got relay server %s", server);
+              g_free (fac->priv->relay_server);
+              fac->priv->relay_server = g_strdup (server);
+            }
+
+          if (test_mode)
+            {
+              /* this is not part of the real protocol, but we can't listen on
+               * port 80 in an unprivileged regression test */
+              port = lm_message_node_get_attribute (subnode,
+                  "gabble-test-http-port");
+
+              if (port != NULL)
+                {
+                  DEBUG ("jingle info: diverting 'Google' HTTP requests to "
+                      "port %s", port);
+                  fac->priv->relay_http_port = atoi (port);
+                }
+            }
+
+          /* FIXME: these are not really actually used anywhere at
+           * the moment, because we get the same info when creating
+           * relay session. */
+          port = lm_message_node_get_attribute (subnode, "udp");
+
+          if (port != NULL)
+            {
+              DEBUG ("jingle info: got relay udp port %s", port);
+              fac->priv->relay_udp = atoi (port);
+            }
+
+          port = lm_message_node_get_attribute (subnode, "tcp");
+
+          if (port != NULL)
+            {
+              DEBUG ("jingle info: got relay tcp port %s", port);
+              fac->priv->relay_tcp = atoi (port);
+            }
+
+          port = lm_message_node_get_attribute (subnode, "tcpssl");
+
+          if (port != NULL)
+            {
+              DEBUG ("jingle info: got relay tcpssl port %s", port);
+              fac->priv->relay_ssltcp = atoi (port);
+            }
+
+        }
+
     }
 
   if (sub_type == LM_MESSAGE_SUB_TYPE_SET)
@@ -313,10 +396,39 @@ jingle_info_cb (LmMessageHandler *handler,
   return LM_HANDLER_RESULT_REMOVE_MESSAGE;
 }
 
+/*
+ * jingle_info_cb
+ *
+ * Called by loudmouth when we get an incoming <iq>. This handler
+ * is concerned only with Jingle info queries.
+ */
+static LmHandlerResult
+jingle_info_cb (LmMessageHandler *handler,
+                LmConnection *lmconn,
+                LmMessage *message,
+                gpointer user_data)
+{
+  GabbleJingleFactory *fac = GABBLE_JINGLE_FACTORY (user_data);
+
+  return got_jingle_info_stanza (fac, message);
+}
+
+static LmHandlerResult
+jingle_info_reply_cb (GabbleConnection *conn,
+    LmMessage *sent_msg,
+    LmMessage *reply_msg,
+    GObject *factory_obj,
+    gpointer user_data)
+{
+  GabbleJingleFactory *fac = GABBLE_JINGLE_FACTORY (factory_obj);
+
+  return got_jingle_info_stanza (fac, reply_msg);
+}
+
 static void
 jingle_info_send_request (GabbleJingleFactory *fac)
 {
-  GabbleJingleFactoryPrivate *priv = GABBLE_JINGLE_FACTORY_GET_PRIVATE (fac);
+  GabbleJingleFactoryPrivate *priv = fac->priv;
   TpBaseConnection *base = (TpBaseConnection *) priv->conn;
   LmMessage *msg;
   LmMessageNode *node;
@@ -332,7 +444,8 @@ jingle_info_send_request (GabbleJingleFactory *fac)
   node = lm_message_node_add_child (msg->node, "query", NULL);
   lm_message_node_set_attribute (node, "xmlns", NS_GOOGLE_JINGLE_INFO);
 
-  if (!_gabble_connection_send (priv->conn, msg, &error))
+  if (!_gabble_connection_send_with_reply (priv->conn, msg,
+        jingle_info_reply_cb, G_OBJECT (fac), fac, &error))
     {
       DEBUG ("jingle info send failed: %s\n", error->message);
       g_error_free (error);
@@ -346,7 +459,7 @@ static void
 gabble_jingle_factory_dispose (GObject *object)
 {
   GabbleJingleFactory *fac = GABBLE_JINGLE_FACTORY (object);
-  GabbleJingleFactoryPrivate *priv = GABBLE_JINGLE_FACTORY_GET_PRIVATE (fac);
+  GabbleJingleFactoryPrivate *priv = fac->priv;
 
   if (priv->dispose_has_run)
     return;
@@ -354,6 +467,12 @@ gabble_jingle_factory_dispose (GObject *object)
   DEBUG ("dispose called");
   priv->dispose_has_run = TRUE;
 
+  if (priv->soup != NULL)
+    {
+      g_object_unref (priv->soup);
+      priv->soup = NULL;
+    }
+
   g_hash_table_destroy (priv->sessions);
   priv->sessions = NULL;
 
@@ -366,6 +485,7 @@ gabble_jingle_factory_dispose (GObject *object)
   g_free (fac->priv->stun_server);
   g_free (fac->priv->fallback_stun_server);
   g_free (fac->priv->relay_token);
+  g_free (fac->priv->relay_server);
 
   if (G_OBJECT_CLASS (gabble_jingle_factory_parent_class)->dispose)
     G_OBJECT_CLASS (gabble_jingle_factory_parent_class)->dispose (object);
@@ -378,7 +498,7 @@ gabble_jingle_factory_get_property (GObject *object,
                                    GParamSpec *pspec)
 {
   GabbleJingleFactory *chan = GABBLE_JINGLE_FACTORY (object);
-  GabbleJingleFactoryPrivate *priv = GABBLE_JINGLE_FACTORY_GET_PRIVATE (chan);
+  GabbleJingleFactoryPrivate *priv = chan->priv;
 
   switch (property_id) {
     case PROP_CONNECTION:
@@ -397,7 +517,7 @@ gabble_jingle_factory_set_property (GObject *object,
                                    GParamSpec *pspec)
 {
   GabbleJingleFactory *chan = GABBLE_JINGLE_FACTORY (object);
-  GabbleJingleFactoryPrivate *priv = GABBLE_JINGLE_FACTORY_GET_PRIVATE (chan);
+  GabbleJingleFactoryPrivate *priv = chan->priv;
 
   switch (property_id) {
     case PROP_CONNECTION:
@@ -422,7 +542,7 @@ gabble_jingle_factory_constructor (GType type,
       constructor (type, n_props, props);
 
   self = GABBLE_JINGLE_FACTORY (obj);
-  priv = GABBLE_JINGLE_FACTORY_GET_PRIVATE (self);
+  priv = self->priv;
 
   /* FIXME: why was this in _constructed in media factory? */
   g_signal_connect (priv->conn,
@@ -430,6 +550,8 @@ gabble_jingle_factory_constructor (GType type,
 
   jingle_media_rtp_register (self);
   jingle_transport_google_register (self);
+  jingle_transport_rawudp_register (self);
+  jingle_transport_iceudp_register (self);
 
   return obj;
 }
@@ -468,13 +590,12 @@ connection_status_changed_cb (GabbleConnection *conn,
                               guint reason,
                               GabbleJingleFactory *self)
 {
-  GabbleJingleFactoryPrivate *priv = GABBLE_JINGLE_FACTORY_GET_PRIVATE (self);
+  GabbleJingleFactoryPrivate *priv = self->priv;
 
   switch (status)
     {
     case TP_CONNECTION_STATUS_CONNECTING:
       g_assert (priv->conn != NULL);
-      g_assert (priv->conn->lmconn != NULL);
 
       g_assert (priv->jingle_cb == NULL);
       g_assert (priv->jingle_info_cb == NULL);
@@ -548,55 +669,100 @@ connection_status_changed_cb (GabbleConnection *conn,
     }
 }
 
-
-static gboolean
-sid_in_use (GabbleJingleFactory *factory, const gchar *sid)
+static gchar *
+make_session_map_key (TpHandle peer,
+    const gchar *resource,
+    const gchar *sid)
 {
-  GabbleJingleFactoryPrivate *priv = GABBLE_JINGLE_FACTORY_GET_PRIVATE (factory);
-  gpointer key, value;
-
-  return g_hash_table_lookup_extended (priv->sessions, sid, &key, &value);
+  return g_strdup_printf (SESSION_MAP_KEY_FORMAT, peer, resource, sid);
 }
 
 static gchar *
-get_unique_sid (GabbleJingleFactory *factory)
+get_unique_sid_for (GabbleJingleFactory *factory,
+    TpHandle peer,
+    const gchar *resource,
+    gchar **key)
 {
   guint32 val;
   gchar *sid = NULL;
-  gboolean unique = FALSE;
+  gchar *key_;
 
-  while (!unique)
+  do
     {
       val = g_random_int_range (1000000, G_MAXINT);
 
       g_free (sid);
       sid = g_strdup_printf ("%u", val);
-
-      unique = !sid_in_use (factory, sid);
+      key_ = make_session_map_key (peer, resource, sid);
     }
+  while (g_hash_table_lookup (factory->priv->sessions, key_) != NULL);
 
+  *key = key_;
   return sid;
 }
 
-static void
-register_session (GabbleJingleFactory *factory,
-                  const gchar *sid,
-                  GabbleJingleSession *sess)
+static GabbleJingleSession *
+ensure_session (GabbleJingleFactory *self,
+    const gchar *sid,
+    const gchar *from,
+    JingleAction action,
+    JingleDialect dialect,
+    gboolean *new_session,
+    GError **error)
 {
-  GabbleJingleFactoryPrivate *priv = GABBLE_JINGLE_FACTORY_GET_PRIVATE (factory);
-  gchar *sid_copy;
+  GabbleJingleFactoryPrivate *priv = self->priv;
+  TpHandleRepoIface *contact_repo = tp_base_connection_get_handles (
+      (TpBaseConnection *) priv->conn, TP_HANDLE_TYPE_CONTACT);
+  const gchar *resource;
+  gchar *key;
+  GabbleJingleSession *sess;
+  TpHandle peer;
 
-  sid_copy = g_strdup (sid);
-  g_assert (g_hash_table_lookup (priv->sessions, sid_copy) == NULL);
-  g_hash_table_insert (priv->sessions, sid_copy, sess);
-}
+  resource = strchr (from, '/');
 
-void
-_jingle_factory_unregister_session (GabbleJingleFactory *factory,
-                                    const gchar *sid)
-{
-  GabbleJingleFactoryPrivate *priv = GABBLE_JINGLE_FACTORY_GET_PRIVATE (factory);
-  g_hash_table_remove (priv->sessions, sid);
+  if (resource == NULL || *resource == '\0')
+    {
+      g_set_error (error, GABBLE_XMPP_ERROR,
+          XMPP_ERROR_BAD_REQUEST, "IQ sender '%s' has no resource", from);
+      return NULL;
+    }
+
+  resource++;
+
+  peer = tp_handle_ensure (contact_repo, from, NULL, error);
+
+  if (peer == 0)
+    {
+      g_prefix_error (error, "Couldn't parse sender '%s': ", from);
+      return NULL;
+    }
+
+  key = make_session_map_key (peer, resource, sid);
+  sess = g_hash_table_lookup (priv->sessions, key);
+  g_free (key);
+
+  if (sess == NULL)
+    {
+      if (action == JINGLE_ACTION_SESSION_INITIATE)
+        {
+          sess = create_session (self, sid, peer, resource, FALSE);
+          g_object_set (sess, "dialect", dialect, NULL);
+          *new_session = TRUE;
+        }
+      else
+        {
+          g_set_error (error, GABBLE_XMPP_ERROR,
+              XMPP_ERROR_JINGLE_UNKNOWN_SESSION, "session %s is unknown", sid);
+          return NULL;
+        }
+    }
+  else
+    {
+      *new_session = FALSE;
+    }
+
+  tp_handle_unref (contact_repo, peer);
+  return sess;
 }
 
 static LmHandlerResult
@@ -606,10 +772,9 @@ jingle_cb (LmMessageHandler *handler,
            gpointer user_data)
 {
   GabbleJingleFactory *self = GABBLE_JINGLE_FACTORY (user_data);
-  GabbleJingleFactoryPrivate *priv =
-      GABBLE_JINGLE_FACTORY_GET_PRIVATE (self);
+  GabbleJingleFactoryPrivate *priv = self->priv;
   GError *error = NULL;
-  const gchar *sid;
+  const gchar *sid, *from;
   GabbleJingleSession *sess;
   gboolean new_session = FALSE;
   JingleAction action;
@@ -617,51 +782,42 @@ jingle_cb (LmMessageHandler *handler,
 
   /* see if it's a jingle message and detect dialect */
   sid = gabble_jingle_session_detect (msg, &action, &dialect);
-  if (sid == NULL)
-    {
-      return LM_HANDLER_RESULT_ALLOW_MORE_HANDLERS;
-    }
+  from = lm_message_node_get_attribute (lm_message_get_node (msg), "from");
+
+  if (sid == NULL || from == NULL)
+    return LM_HANDLER_RESULT_ALLOW_MORE_HANDLERS;
+
+  sess = ensure_session (self, sid, from, action, dialect, &new_session,
+      &error);
 
-  sess = g_hash_table_lookup (priv->sessions, sid);
   if (sess == NULL)
-    {
-      if (action != JINGLE_ACTION_SESSION_INITIATE)
-        {
-          g_set_error (&error, GABBLE_XMPP_ERROR,
-              XMPP_ERROR_JINGLE_OUT_OF_ORDER, "session not initiated yet");
-          goto REQUEST_ERROR;
-        }
-      new_session = TRUE;
-      sess = create_session (self, sid, 0, NULL);
-      g_object_set (sess, "dialect", dialect, NULL);
-    }
+    goto REQUEST_ERROR;
 
   /* now act on the message */
-  if (gabble_jingle_session_parse (sess, action, msg, &error))
-    {
-      if (new_session)
-        {
-          g_signal_emit (self, signals[NEW_SESSION], 0, sess);
-        }
-
-      /* all went well, we can acknowledge the IQ */
-      _gabble_connection_acknowledge_set_iq (priv->conn, msg);
-
-      return LM_HANDLER_RESULT_REMOVE_MESSAGE;
-    }
-
-  /* on parse error */
-  g_assert (error != NULL);
+  if (!gabble_jingle_session_parse (sess, action, msg, &error))
+    goto REQUEST_ERROR;
 
   if (new_session)
-      _jingle_factory_unregister_session (self, sid);
+    g_signal_emit (self, signals[NEW_SESSION], 0, sess);
+
+  /* all went well, we can acknowledge the IQ */
+  _gabble_connection_acknowledge_set_iq (priv->conn, msg);
+
+  return LM_HANDLER_RESULT_REMOVE_MESSAGE;
 
 REQUEST_ERROR:
-  _gabble_connection_send_iq_error (
-    priv->conn, msg, error->code, error->message);
+  g_assert (error != NULL);
+
+  DEBUG ("NAKing with error: %s", error->message);
+  _gabble_connection_send_iq_error (priv->conn, msg, error->code,
+      error->message);
 
   g_error_free (error);
 
+  if (sess != NULL && new_session)
+    gabble_jingle_session_terminate (sess, TP_CHANNEL_GROUP_CHANGE_REASON_NONE,
+        NULL);
+
   return LM_HANDLER_RESULT_REMOVE_MESSAGE;
 }
 
@@ -672,40 +828,58 @@ REQUEST_ERROR:
  */
 static GabbleJingleSession *
 create_session (GabbleJingleFactory *fac,
-    const gchar *sid, TpHandle peer, const gchar *peer_resource)
+    const gchar *sid,
+    TpHandle peer,
+    const gchar *peer_resource,
+    gboolean local_hold)
 {
-  GabbleJingleFactoryPrivate *priv =
-      GABBLE_JINGLE_FACTORY_GET_PRIVATE (fac);
+  GabbleJingleFactoryPrivate *priv = fac->priv;
   GabbleJingleSession *sess;
   gboolean local_initiator;
+  gchar *sid_, *key;
+
+  g_assert (peer != 0);
+  g_assert (peer_resource != NULL);
 
   if (sid != NULL)
     {
-      g_assert (NULL == g_hash_table_lookup (priv->sessions, sid));
+      key = make_session_map_key (peer, peer_resource, sid);
+      sid_ = g_strdup (sid);
+
       local_initiator = FALSE;
     }
   else
     {
-      sid = get_unique_sid (fac);
+      sid_ = get_unique_sid_for (fac, peer, peer_resource, &key);
+
       local_initiator = TRUE;
     }
 
-  sess = gabble_jingle_session_new (priv->conn, sid, local_initiator, peer,
-      peer_resource);
+  /* Either we should have found the existing session when the IQ arrived, or
+   * get_unique_sid_for should have ensured the key is fresh. */
+  g_assert (NULL == g_hash_table_lookup (priv->sessions, key));
 
-  g_signal_connect (sess, "terminated",
-    (GCallback) session_terminated_cb, fac);
+  sess = gabble_jingle_session_new (priv->conn, sid_, local_initiator, peer,
+      peer_resource, local_hold);
+  g_signal_connect (sess, "terminated", (GCallback) session_terminated_cb, fac);
+
+  /* Takes ownership of key */
+  g_hash_table_insert (priv->sessions, key, sess);
+
+  DEBUG ("new session (%u, %s, %s) @ %p", peer, peer_resource, sid_, sess);
+
+  g_free (sid_);
 
-  DEBUG ("new session %s @ %p created", sid, sess);
-  register_session (fac, sid, sess);
   return sess;
 }
 
 GabbleJingleSession *
 gabble_jingle_factory_create_session (GabbleJingleFactory *fac,
-    TpHandle peer, const gchar *peer_resource)
+    TpHandle peer,
+    const gchar *peer_resource,
+    gboolean local_hold)
 {
-  return create_session (fac, NULL, peer, peer_resource);
+  return create_session (fac, NULL, peer, peer_resource, local_hold);
 }
 
 void
@@ -749,14 +923,19 @@ gabble_jingle_factory_lookup_content_type (GabbleJingleFactory *self,
 
 static void
 session_terminated_cb (GabbleJingleSession *session,
-    gboolean local_terminator, GabbleJingleFactory *factory)
+                       gboolean local_terminator,
+                       TpChannelGroupChangeReason reason,
+                       GabbleJingleFactory *factory)
 {
-  const gchar *sid;
-  DEBUG ("removing terminated session");
+  gchar *key = make_session_map_key (session->peer,
+      gabble_jingle_session_get_peer_resource (session),
+      gabble_jingle_session_get_sid (session));
 
-  g_object_get (session, "session-id", &sid, NULL);
+  DEBUG ("removing terminated session with key %s", key);
 
-  _jingle_factory_unregister_session (factory, sid);
+  g_warn_if_fail (g_hash_table_remove (factory->priv->sessions, key));
+
+  g_free (key);
 }
 
 const gchar *
@@ -793,3 +972,261 @@ gabble_jingle_factory_get_stun_server (GabbleJingleFactory *self,
 
   return TRUE;
 }
+
+typedef struct
+{
+  GPtrArray *relays;
+  guint component;
+  guint requests_to_do;
+  GabbleJingleFactoryRelaySessionCb callback;
+  gpointer user_data;
+} RelaySessionData;
+
+static RelaySessionData *
+relay_session_data_new (guint requests_to_do,
+                        GabbleJingleFactoryRelaySessionCb callback,
+                        gpointer user_data)
+{
+  RelaySessionData *rsd = g_slice_new0 (RelaySessionData);
+
+  rsd->relays = g_ptr_array_sized_new (requests_to_do);
+  rsd->component = 1;
+  rsd->requests_to_do = requests_to_do;
+  rsd->callback = callback;
+  rsd->user_data = user_data;
+
+  return rsd;
+}
+
+/* This is a GSourceFunc */
+static gboolean
+relay_session_data_call (gpointer p)
+{
+  RelaySessionData *rsd = p;
+
+  g_assert (rsd->callback != NULL);
+
+  rsd->callback (rsd->relays, rsd->user_data);
+
+  return FALSE;
+}
+
+/* This is a GDestroyNotify */
+static void
+relay_session_data_destroy (gpointer p)
+{
+  RelaySessionData *rsd = p;
+
+  g_ptr_array_foreach (rsd->relays, (GFunc) g_hash_table_destroy, NULL);
+  g_ptr_array_free (rsd->relays, TRUE);
+
+  g_slice_free (RelaySessionData, rsd);
+}
+
+static void
+translate_relay_info (GPtrArray *relays,
+                      const gchar *relay_ip,
+                      const gchar *username,
+                      const gchar *password,
+                      const gchar *static_type,
+                      const gchar *port_string,
+                      guint component)
+{
+  GHashTable *asv;
+  guint port = 0;
+
+  if (port_string == NULL)
+    {
+      DEBUG ("no relay port for %s found", static_type);
+      return;
+    }
+
+  port = atoi (port_string);
+
+  if (port == 0 || port > G_MAXUINT16)
+    {
+      DEBUG ("failed to parse relay port '%s' for %s", port_string,
+          static_type);
+      return;
+    }
+
+  DEBUG ("type=%s ip=%s port=%u username=%s password=%s component=%u",
+      static_type, relay_ip, port, username, password, component);
+  /* keys are static, values are slice-allocated */
+  asv = g_hash_table_new_full (g_str_hash, g_str_equal,
+      NULL, (GDestroyNotify) tp_g_value_slice_free);
+  g_hash_table_insert (asv, "ip",
+      tp_g_value_slice_new_string (relay_ip));
+  g_hash_table_insert (asv, "type",
+      tp_g_value_slice_new_static_string (static_type));
+  g_hash_table_insert (asv, "port",
+      tp_g_value_slice_new_uint (port));
+  g_hash_table_insert (asv, "username",
+      tp_g_value_slice_new_string (username));
+  g_hash_table_insert (asv, "password",
+      tp_g_value_slice_new_string (password));
+  g_hash_table_insert (asv, "component",
+      tp_g_value_slice_new_uint (component));
+
+  g_ptr_array_add (relays, asv);
+}
+
+static void
+on_http_response (SoupSession *soup,
+                  SoupMessage *msg,
+                  gpointer user_data)
+{
+  RelaySessionData *rsd = user_data;
+
+  if (msg->status_code != 200)
+    {
+      DEBUG ("Google session creation failed, relaying not used: %d %s",
+          msg->status_code, msg->reason_phrase);
+    }
+  else
+    {
+      /* parse a=b lines into GHashTable
+       * (key, value both borrowed from items of the strv 'lines') */
+      GHashTable *map = g_hash_table_new (g_str_hash, g_str_equal);
+      gchar **lines;
+      guint i;
+      const gchar *relay_ip;
+      const gchar *relay_udp_port;
+      const gchar *relay_tcp_port;
+      const gchar *relay_ssltcp_port;
+      const gchar *username;
+      const gchar *password;
+
+      DEBUG ("Response from Google:\n====\n%s\n====",
+          msg->response_body->data);
+
+      lines = g_strsplit (msg->response_body->data, "\n", 0);
+
+      if (lines != NULL)
+        {
+          for (i = 0; lines[i] != NULL; i++)
+            {
+              gchar *delim = strchr (lines[i], '=');
+              size_t len;
+
+              if (delim == NULL || delim == lines[i])
+                {
+                  /* ignore empty keys or lines without '=' */
+                  continue;
+                }
+
+              len = strlen (lines[i]);
+
+              if (lines[i][len - 1] == '\r')
+                {
+                  lines[i][len - 1] = '\0';
+                }
+
+              *delim = '\0';
+              g_hash_table_insert (map, lines[i], delim + 1);
+            }
+        }
+
+      relay_ip = g_hash_table_lookup (map, "relay.ip");
+      relay_udp_port = g_hash_table_lookup (map, "relay.udp_port");
+      relay_tcp_port = g_hash_table_lookup (map, "relay.tcp_port");
+      relay_ssltcp_port = g_hash_table_lookup (map, "relay.ssltcp_port");
+      username = g_hash_table_lookup (map, "username");
+      password = g_hash_table_lookup (map, "password");
+
+      if (relay_ip == NULL)
+        {
+          DEBUG ("No relay.ip found");
+        }
+      else if (username == NULL)
+        {
+          DEBUG ("No username found");
+        }
+      else if (password == NULL)
+        {
+          DEBUG ("No password found");
+        }
+      else
+        {
+          translate_relay_info (rsd->relays, relay_ip, username, password,
+              "udp", relay_udp_port, rsd->component);
+          translate_relay_info (rsd->relays, relay_ip, username, password,
+              "tcp", relay_tcp_port, rsd->component);
+          translate_relay_info (rsd->relays, relay_ip, username, password,
+              "tls", relay_ssltcp_port, rsd->component);
+        }
+
+      g_strfreev (lines);
+      g_hash_table_destroy (map);
+    }
+
+  rsd->component++;
+
+  if ((--rsd->requests_to_do) == 0)
+    {
+      relay_session_data_call (rsd);
+      relay_session_data_destroy (rsd);
+    }
+}
+
+void
+gabble_jingle_factory_create_google_relay_session (
+    GabbleJingleFactory *fac,
+    guint components,
+    GabbleJingleFactoryRelaySessionCb callback,
+    gpointer user_data)
+{
+  GabbleJingleFactoryPrivate *priv = fac->priv;
+  gchar *url;
+  guint i;
+  RelaySessionData *rsd;
+
+  g_return_if_fail (callback != NULL);
+
+  rsd = relay_session_data_new (components, callback, user_data);
+
+  if (fac->priv->relay_server == NULL)
+    {
+      DEBUG ("No relay server provided, not creating google relay session");
+      g_idle_add_full (G_PRIORITY_DEFAULT, relay_session_data_call, rsd,
+          relay_session_data_destroy);
+      return;
+    }
+
+  if (fac->priv->relay_token == NULL)
+    {
+      DEBUG ("No relay token provided, not creating google relay session");
+      g_idle_add_full (G_PRIORITY_DEFAULT, relay_session_data_call, rsd,
+          relay_session_data_destroy);
+      return;
+    }
+
+  if (priv->soup == NULL)
+    {
+      priv->soup = soup_session_async_new ();
+
+      /* If we don't get answer in a few seconds, relay won't do
+       * us much help anyways. */
+      g_object_set (priv->soup, "timeout", RELAY_HTTP_TIMEOUT, NULL);
+    }
+
+  url = g_strdup_printf ("http://%s:%d/create_session",
+      fac->priv->relay_server, fac->priv->relay_http_port);
+
+  for (i = 0; i < components; i++)
+    {
+      SoupMessage *msg = soup_message_new ("GET", url);
+
+      DEBUG ("Trying to create a new relay session on %s", url);
+
+      /* libjingle sets both headers, so shall we */
+      soup_message_headers_append (msg->request_headers,
+          "X-Talk-Google-Relay-Auth", fac->priv->relay_token);
+      soup_message_headers_append (msg->request_headers,
+          "X-Google-Relay-Auth", fac->priv->relay_token);
+
+      soup_session_queue_message (priv->soup, msg, on_http_response, rsd);
+    }
+
+  g_free (url);
+}
