@@ -80,8 +80,11 @@ struct _GabbleJingleSession {
 };
 
 GabbleJingleSession *gabble_jingle_session_new (GabbleConnection *connection,
-    const gchar *session_id, gboolean local_initiator, TpHandle peer,
-    const gchar *peer_resource);
+    const gchar *session_id,
+    gboolean local_initiator,
+    TpHandle peer,
+    const gchar *peer_resource,
+    gboolean local_hold);
 
 const gchar * gabble_jingle_session_detect (LmMessage *message,
     JingleAction *action, JingleDialect *dialect);
@@ -91,7 +94,8 @@ LmMessage *gabble_jingle_session_new_message (GabbleJingleSession *sess,
     JingleAction action, LmMessageNode **sess_node);
 
 void gabble_jingle_session_accept (GabbleJingleSession *sess);
-void gabble_jingle_session_terminate (GabbleJingleSession *sess);
+gboolean gabble_jingle_session_terminate (GabbleJingleSession *sess,
+    TpChannelGroupChangeReason reason, GError **error);
 void gabble_jingle_session_remove_content (GabbleJingleSession *sess,
     GabbleJingleContent *c);
 
@@ -101,12 +105,29 @@ gabble_jingle_session_add_content (GabbleJingleSession *sess, JingleMediaType mt
 
 GType gabble_jingle_session_get_content_type (GabbleJingleSession *);
 GList *gabble_jingle_session_get_contents (GabbleJingleSession *sess);
+const gchar *gabble_jingle_session_get_peer_resource (
+    GabbleJingleSession *sess);
+const gchar *gabble_jingle_session_get_sid (GabbleJingleSession *sess);
+JingleDialect gabble_jingle_session_get_dialect (GabbleJingleSession *sess);
 
-typedef void (*JingleReplyHandler) (GabbleJingleSession *, gboolean success,
-    LmMessage *reply, gpointer user_data);
-void gabble_jingle_session_send (GabbleJingleSession *sess, LmMessage *msg,
-    JingleReplyHandler cb, gpointer user_data);
+gboolean gabble_jingle_session_can_modify_contents (GabbleJingleSession *sess);
 
+typedef void (*JingleReplyHandler) (GObject *, gboolean success,
+    LmMessage *reply);
+void gabble_jingle_session_send (GabbleJingleSession *sess,
+    LmMessage *msg,
+    JingleReplyHandler cb,
+    GObject *weak_object);
+
+void gabble_jingle_session_set_local_hold (GabbleJingleSession *sess,
+    gboolean held);
+
+gboolean gabble_jingle_session_get_remote_hold (GabbleJingleSession *sess);
+
+gboolean gabble_jingle_session_get_remote_ringing (GabbleJingleSession *sess);
+
+gboolean gabble_jingle_session_defines_action (GabbleJingleSession *sess,
+    JingleAction action);
 
 #endif /* __JINGLE_SESSION_H__ */
 
