@@ -80,11 +80,6 @@ struct _GabbleJingleTransportGooglePrivate
   gboolean dispose_has_run;
 };
 
-#define GABBLE_JINGLE_TRANSPORT_GOOGLE_GET_PRIVATE(o) ((o)->priv)
-
-static void transmit_candidates (GabbleJingleTransportGoogle *transport,
-    GList *candidates);
-
 static void
 gabble_jingle_transport_google_init (GabbleJingleTransportGoogle *obj)
 {
@@ -100,7 +95,7 @@ static void
 gabble_jingle_transport_google_dispose (GObject *object)
 {
   GabbleJingleTransportGoogle *trans = GABBLE_JINGLE_TRANSPORT_GOOGLE (object);
-  GabbleJingleTransportGooglePrivate *priv = GABBLE_JINGLE_TRANSPORT_GOOGLE_GET_PRIVATE (trans);
+  GabbleJingleTransportGooglePrivate *priv = trans->priv;
 
   if (priv->dispose_has_run)
     return;
@@ -128,7 +123,7 @@ gabble_jingle_transport_google_get_property (GObject *object,
                                              GParamSpec *pspec)
 {
   GabbleJingleTransportGoogle *trans = GABBLE_JINGLE_TRANSPORT_GOOGLE (object);
-  GabbleJingleTransportGooglePrivate *priv = GABBLE_JINGLE_TRANSPORT_GOOGLE_GET_PRIVATE (trans);
+  GabbleJingleTransportGooglePrivate *priv = trans->priv;
 
   switch (property_id) {
     case PROP_CONTENT:
@@ -153,8 +148,7 @@ gabble_jingle_transport_google_set_property (GObject *object,
                                              GParamSpec *pspec)
 {
   GabbleJingleTransportGoogle *trans = GABBLE_JINGLE_TRANSPORT_GOOGLE (object);
-  GabbleJingleTransportGooglePrivate *priv =
-      GABBLE_JINGLE_TRANSPORT_GOOGLE_GET_PRIVATE (trans);
+  GabbleJingleTransportGooglePrivate *priv = trans->priv;
 
   switch (property_id) {
     case PROP_CONTENT:
@@ -236,15 +230,20 @@ parse_candidates (GabbleJingleTransportIface *obj,
     LmMessageNode *transport_node, GError **error)
 {
   GabbleJingleTransportGoogle *t = GABBLE_JINGLE_TRANSPORT_GOOGLE (obj);
-  GabbleJingleTransportGooglePrivate *priv =
-    GABBLE_JINGLE_TRANSPORT_GOOGLE_GET_PRIVATE (t);
+  GabbleJingleTransportGooglePrivate *priv = t->priv;
   GList *candidates = NULL;
-  LmMessageNode *node;
+  JingleMediaType media_type;
+  JingleDialect dialect;
+  NodeIter i;
 
-  for (node = transport_node->children; node; node = node->next)
+  g_object_get (priv->content, "media-type", &media_type, NULL);
+  dialect = gabble_jingle_session_get_dialect (priv->content->session);
+
+  for (i = node_iter (transport_node); i; i = node_iter_next (i))
     {
+      LmMessageNode *node = node_iter_data (i);
       const gchar *name, *address, *user, *pass, *str;
-      guint port, net, gen, component = 1;
+      guint port, net, gen, component;
       gdouble pref;
       JingleTransportProtocol proto;
       JingleCandidateType ctype;
@@ -254,9 +253,35 @@ parse_candidates (GabbleJingleTransportIface *obj,
           continue;
 
       name = lm_message_node_get_attribute (node, "name");
-      if (name == NULL || tp_strdiff (name, "rtp"))
+      if (name == NULL)
           break;
 
+      if (g_str_has_prefix (name, "video_"))
+        {
+          if (media_type != JINGLE_MEDIA_TYPE_VIDEO)
+            continue;
+
+          if (!tp_strdiff (name, "video_rtp"))
+            component = 1;
+          else if (!tp_strdiff (name, "video_rtcp"))
+            component = 2;
+          else
+            break;
+        }
+      else
+        {
+          if (media_type != JINGLE_MEDIA_TYPE_AUDIO
+              && JINGLE_IS_GOOGLE_DIALECT (dialect))
+            continue;
+
+          if (!tp_strdiff (name, "rtp"))
+            component = 1;
+          else if (!tp_strdiff (name, "rtcp"))
+            component = 2;
+          else
+            break;
+        }
+
       address = lm_message_node_get_attribute (node, "address");
       if (address == NULL)
           break;
@@ -349,13 +374,13 @@ parse_candidates (GabbleJingleTransportIface *obj,
       if (str != NULL)
           component = atoi (str);
 
-      c = jingle_candidate_new (component, address, port, proto,
-          pref, ctype, user, pass, net, gen);
+      c = jingle_candidate_new (proto, ctype, NULL, component,
+          address, port, gen, pref, user, pass, net);
 
       candidates = g_list_append (candidates, c);
     }
 
-  if (node != NULL)
+  if (i != NULL)
     {
       DEBUG ("not all nodes were processed, reporting error");
       /* rollback these */
@@ -373,45 +398,23 @@ parse_candidates (GabbleJingleTransportIface *obj,
 }
 
 static void
-transmit_candidates (GabbleJingleTransportGoogle *transport, GList *candidates)
+transmit_candidates (GabbleJingleTransportGoogle *transport,
+    const gchar *name,
+    GList *candidates)
 {
-  GabbleJingleTransportGooglePrivate *priv =
-    GABBLE_JINGLE_TRANSPORT_GOOGLE_GET_PRIVATE (transport);
-  JingleDialect dialect;
+  GabbleJingleTransportGooglePrivate *priv = transport->priv;
   GList *li;
   LmMessage *msg;
   LmMessageNode *trans_node, *sess_node;
 
+  if (candidates == NULL)
+    return;
+
   msg = gabble_jingle_session_new_message (priv->content->session,
     JINGLE_ACTION_TRANSPORT_INFO, &sess_node);
 
-  g_object_get (priv->content->session, "dialect", &dialect, NULL);
-
-  if (dialect == JINGLE_DIALECT_GTALK3)
-    {
-      trans_node = sess_node;
-    }
-  else if (dialect == JINGLE_DIALECT_GTALK4)
-    {
-      trans_node = lm_message_node_add_child (sess_node, "transport", NULL);
-      lm_message_node_set_attribute (trans_node, "xmlns", NS_GOOGLE_TRANSPORT_P2P);
-    }
-  else
-    {
-      const gchar *cname, *cns;
-
-      g_object_get (GABBLE_JINGLE_CONTENT (priv->content),
-          "name", &cname, "content-ns", &cns, NULL);
-
-      /* we need the <content> ... */
-      trans_node = lm_message_node_add_child (sess_node, "content", NULL);
-      lm_message_node_set_attribute (trans_node, "xmlns", cns);
-      lm_message_node_set_attribute (trans_node, "name", cname);
-
-      /* .. and the <transport> node */
-      trans_node = lm_message_node_add_child (trans_node, "transport", NULL);
-      lm_message_node_set_attribute (trans_node, "xmlns", priv->transport_ns);
-    }
+  gabble_jingle_content_produce_node (priv->content, sess_node, FALSE, TRUE,
+      &trans_node);
 
   for (li = candidates; li; li = li->next)
     {
@@ -419,13 +422,6 @@ transmit_candidates (GabbleJingleTransportGoogle *transport, GList *candidates)
       gchar port_str[16], pref_str[16], comp_str[16], *type_str, *proto_str;
       LmMessageNode *cnode;
 
-      // FIXME
-      if (c->component == 2)
-        {
-          DEBUG ("ignoring RTCP component");
-          continue;
-        }
-
       sprintf (port_str, "%d", c->port);
       sprintf (pref_str, "%lf", c->preference);
       sprintf (comp_str, "%d", c->component);
@@ -468,69 +464,101 @@ transmit_candidates (GabbleJingleTransportGoogle *transport, GList *candidates)
           "protocol", proto_str,
           "type", type_str,
           "component", comp_str,
-          "name", "rtp",
           "network", "0",
           "generation", "0",
           NULL);
+
+      lm_message_node_set_attribute (cnode, "name", name);
     }
 
-  _gabble_connection_send (priv->content->conn, msg, NULL);
+  _gabble_connection_send_with_reply (priv->content->conn, msg, NULL, NULL,
+      NULL, NULL);
   lm_message_unref (msg);
 }
 
+/* Groups @candidates into rtp and rtcp and sends each group in its own
+ * transport-info. This works around old Gabble, which rejected transport-info
+ * stanzas containing non-rtp candidates.
+ */
+static void
+group_and_transmit_candidates (GabbleJingleTransportGoogle *transport,
+    GList *candidates)
+{
+  GabbleJingleTransportGooglePrivate *priv = transport->priv;
+  GList *rtp_candidates = NULL;
+  GList *rtcp_candidates = NULL;
+  JingleDialect dialect;
+  JingleMediaType media;
+  GList *li;
+
+  for (li = candidates; li != NULL; li = g_list_next (li))
+    {
+      JingleCandidate *c = li->data;
+
+      if (c->component == 1)
+        rtp_candidates = g_list_prepend (rtp_candidates, c);
+      else if (c->component == 2)
+        rtcp_candidates = g_list_prepend (rtcp_candidates, c);
+      else
+        DEBUG ("Ignoring unknown component %d", c->component);
+    }
+
+  dialect = gabble_jingle_session_get_dialect (priv->content->session);
+  g_object_get (priv->content, "media-type", &media, NULL);
+
+  if (media == JINGLE_MEDIA_TYPE_VIDEO && JINGLE_IS_GOOGLE_DIALECT (dialect))
+    {
+      transmit_candidates (transport, "video_rtp", rtp_candidates);
+      transmit_candidates (transport, "video_rtcp", rtcp_candidates);
+    }
+  else
+    {
+      transmit_candidates (transport, "rtp", rtp_candidates);
+      transmit_candidates (transport, "rtcp", rtcp_candidates);
+    }
+
+  g_list_free (rtp_candidates);
+  g_list_free (rtcp_candidates);
+}
+
 /* Takes in a list of slice-allocated JingleCandidate structs */
 static void
-add_candidates (GabbleJingleTransportIface *obj, GList *new_candidates)
+new_local_candidates (GabbleJingleTransportIface *obj, GList *new_candidates)
 {
   GabbleJingleTransportGoogle *transport =
     GABBLE_JINGLE_TRANSPORT_GOOGLE (obj);
-  GabbleJingleTransportGooglePrivate *priv =
-    GABBLE_JINGLE_TRANSPORT_GOOGLE_GET_PRIVATE (transport);
-  JingleContentState state;
-
-  g_object_get (priv->content, "state", &state, NULL);
-
-  if (state > JINGLE_CONTENT_STATE_EMPTY)
-    {
-      DEBUG ("content already signalled, transmitting candidates");
-      transmit_candidates (transport, new_candidates);
-      priv->pending_candidates = NULL;
-    }
-  else
-    {
-      DEBUG ("content not signalled yet, waiting with candidates");
-
-      /* if we already have pending candidates, the new ones will
-       * be in the local_candidates list after them. but these
-       * are the first pending ones, we must mark them. */
-      if (priv->pending_candidates == NULL)
-        priv->pending_candidates = new_candidates;
-  }
+  GabbleJingleTransportGooglePrivate *priv = transport->priv;
 
   priv->local_candidates = g_list_concat (priv->local_candidates,
       new_candidates);
+
+  /* If all previous candidates have been signalled, set the new
+   * ones as pending. If there are existing pending candidates,
+   * the new ones will just be appended to that list. */
+  if (priv->pending_candidates == NULL)
+    priv->pending_candidates = new_candidates;
 }
 
 static void
-retransmit_candidates (GabbleJingleTransportIface *obj, gboolean all)
+send_candidates (GabbleJingleTransportIface *obj, gboolean all)
 {
   GabbleJingleTransportGoogle *transport =
     GABBLE_JINGLE_TRANSPORT_GOOGLE (obj);
-  GabbleJingleTransportGooglePrivate *priv =
-    GABBLE_JINGLE_TRANSPORT_GOOGLE_GET_PRIVATE (transport);
+  GabbleJingleTransportGooglePrivate *priv = transport->priv;
 
   if (all)
     {
       /* for gtalk3, we might have to retransmit everything */
-      transmit_candidates (transport, priv->local_candidates);
+      group_and_transmit_candidates (transport, priv->local_candidates);
       priv->pending_candidates = NULL;
     }
   else
     {
-      /* in case content was ready after we wanted to transmit
-       * them originally, we are called to retranmit them */
-      if (priv->pending_candidates != NULL) {
-          transmit_candidates (transport, priv->pending_candidates);
+      /* If the content became ready after we wanted to transmit
+       * these originally, we are called to transmit when it them */
+      if (priv->pending_candidates != NULL)
+        {
+          group_and_transmit_candidates (transport, priv->pending_candidates);
           priv->pending_candidates = NULL;
       }
     }
@@ -541,21 +569,32 @@ get_remote_candidates (GabbleJingleTransportIface *iface)
 {
   GabbleJingleTransportGoogle *transport =
     GABBLE_JINGLE_TRANSPORT_GOOGLE (iface);
-  GabbleJingleTransportGooglePrivate *priv =
-    GABBLE_JINGLE_TRANSPORT_GOOGLE_GET_PRIVATE (transport);
+  GabbleJingleTransportGooglePrivate *priv = transport->priv;
 
   return priv->remote_candidates;
 }
 
+static JingleTransportType
+get_transport_type (void)
+{
+  return JINGLE_TRANSPORT_GOOGLE_P2P;
+}
+
 static void
 transport_iface_init (gpointer g_iface, gpointer iface_data)
 {
   GabbleJingleTransportIfaceClass *klass = (GabbleJingleTransportIfaceClass *) g_iface;
 
   klass->parse_candidates = parse_candidates;
-  klass->add_candidates = add_candidates;
-  klass->retransmit_candidates = retransmit_candidates;
+
+  klass->new_local_candidates = new_local_candidates;
+  /* Not implementing inject_candidates: gtalk-p2p candidates are always sent
+   * in transport-info or equivalent.
+   */
+  klass->send_candidates = send_candidates;
+
   klass->get_remote_candidates = get_remote_candidates;
+  klass->get_transport_type = get_transport_type;
 }
 
 void
