@@ -34,12 +34,15 @@
 
 #define DEBUG_FLAG GABBLE_DEBUG_MEDIA
 
+#include "extensions/extensions.h"
+
 #include "caps-channel-manager.h"
 #include "connection.h"
 #include "debug.h"
 #include "jingle-factory.h"
 #include "jingle-media-rtp.h"
 #include "jingle-session.h"
+
 #include "media-channel.h"
 #include "namespaces.h"
 #include "util.h"
@@ -59,7 +62,6 @@ enum
   LAST_PROPERTY
 };
 
-typedef struct _GabbleMediaFactoryPrivate GabbleMediaFactoryPrivate;
 struct _GabbleMediaFactoryPrivate
 {
   GabbleConnection *conn;
@@ -68,28 +70,22 @@ struct _GabbleMediaFactoryPrivate
   GPtrArray *channels;
   guint channel_index;
 
-  GHashTable *session_chans;
-
   gboolean dispose_has_run;
 };
 
-#define GABBLE_MEDIA_FACTORY_GET_PRIVATE(o) \
-  (G_TYPE_INSTANCE_GET_PRIVATE ((o), GABBLE_TYPE_MEDIA_FACTORY, \
-                                GabbleMediaFactoryPrivate))
-
 static void
 gabble_media_factory_init (GabbleMediaFactory *fac)
 {
-  GabbleMediaFactoryPrivate *priv = GABBLE_MEDIA_FACTORY_GET_PRIVATE (fac);
+  GabbleMediaFactoryPrivate *priv = G_TYPE_INSTANCE_GET_PRIVATE (fac,
+      GABBLE_TYPE_MEDIA_FACTORY, GabbleMediaFactoryPrivate);
+
+  fac->priv = priv;
 
   priv->channels = g_ptr_array_sized_new (1);
   priv->channel_index = 0;
 
   priv->conn = NULL;
   priv->dispose_has_run = FALSE;
-
-  priv->session_chans = g_hash_table_new_full (g_str_hash, g_str_equal,
-                                               g_free, NULL);
 }
 
 
@@ -100,7 +96,7 @@ static void
 gabble_media_factory_dispose (GObject *object)
 {
   GabbleMediaFactory *fac = GABBLE_MEDIA_FACTORY (object);
-  GabbleMediaFactoryPrivate *priv = GABBLE_MEDIA_FACTORY_GET_PRIVATE (fac);
+  GabbleMediaFactoryPrivate *priv = fac->priv;
 
   if (priv->dispose_has_run)
     return;
@@ -109,18 +105,8 @@ gabble_media_factory_dispose (GObject *object)
   priv->dispose_has_run = TRUE;
 
   gabble_media_factory_close_all (fac);
-  g_assert (priv->channels == NULL);
-
-  /* Use a temporary variable because we don't want
-   * media_channel_closed_cb to remove the channel from the hash table a
-   * second time */
-  if (priv->session_chans)
-    {
-      GHashTable *tmp = priv->session_chans;
-      priv->session_chans = NULL;
-      g_assert (g_hash_table_size (tmp) == 0);
-      g_hash_table_destroy (tmp);
-    }
+  g_assert (priv->channels->len == 0);
+  g_ptr_array_free (priv->channels, TRUE);
 
   if (G_OBJECT_CLASS (gabble_media_factory_parent_class)->dispose)
     G_OBJECT_CLASS (gabble_media_factory_parent_class)->dispose (object);
@@ -133,7 +119,7 @@ gabble_media_factory_get_property (GObject    *object,
                                    GParamSpec *pspec)
 {
   GabbleMediaFactory *fac = GABBLE_MEDIA_FACTORY (object);
-  GabbleMediaFactoryPrivate *priv = GABBLE_MEDIA_FACTORY_GET_PRIVATE (fac);
+  GabbleMediaFactoryPrivate *priv = fac->priv;
 
   switch (property_id) {
     case PROP_CONNECTION:
@@ -152,7 +138,7 @@ gabble_media_factory_set_property (GObject      *object,
                                    GParamSpec   *pspec)
 {
   GabbleMediaFactory *fac = GABBLE_MEDIA_FACTORY (object);
-  GabbleMediaFactoryPrivate *priv = GABBLE_MEDIA_FACTORY_GET_PRIVATE (fac);
+  GabbleMediaFactoryPrivate *priv = fac->priv;
 
   switch (property_id) {
     case PROP_CONNECTION:
@@ -191,51 +177,26 @@ gabble_media_factory_class_init (GabbleMediaFactoryClass *gabble_media_factory_c
 
 }
 
-static GabbleMediaChannel *new_media_channel (GabbleMediaFactory *fac,
-    GabbleJingleSession *sess, TpHandle maybe_peer);
-static void media_channel_closed_cb (GabbleMediaChannel *chan,
-    gpointer user_data);
-
-static gboolean
-_remove_sid_mapping (gpointer key, gpointer value, gpointer user_data)
-{
-  GabbleMediaChannel *chan = GABBLE_MEDIA_CHANNEL (value);
-  GabbleMediaChannel *target_chan = GABBLE_MEDIA_CHANNEL (user_data);
-
-  if (chan == target_chan) return TRUE;
-  return FALSE;
-}
-
 /**
  * media_channel_closed_cb:
  *
  * Signal callback for when a media channel is closed. Removes the references
- * that #GabbleMediaFactory holds to them. Also removes all the sessions for
- * the closed channel.
+ * that #GabbleMediaFactory holds to them.
  */
 static void
 media_channel_closed_cb (GabbleMediaChannel *chan, gpointer user_data)
 {
   GabbleMediaFactory *fac = GABBLE_MEDIA_FACTORY (user_data);
-  GabbleMediaFactoryPrivate *priv = GABBLE_MEDIA_FACTORY_GET_PRIVATE (fac);
+  GabbleMediaFactoryPrivate *priv = fac->priv;
 
   tp_channel_manager_emit_channel_closed_for_object (fac,
       TP_EXPORTABLE_CHANNEL (chan));
 
-  if (priv->channels != NULL)
-    {
-      DEBUG ("removing media channel %p with ref count %d",
-          chan, G_OBJECT (chan)->ref_count);
+  DEBUG ("removing media channel %p with ref count %d",
+      chan, G_OBJECT (chan)->ref_count);
 
-      g_ptr_array_remove (priv->channels, chan);
-      g_object_unref (chan);
-    }
-
-  if (priv->session_chans != NULL)
-    {
-      g_hash_table_foreach_remove (priv->session_chans, _remove_sid_mapping,
-          chan);
-    }
+  g_ptr_array_remove (priv->channels, chan);
+  g_object_unref (chan);
 }
 
 /**
@@ -246,7 +207,10 @@ media_channel_closed_cb (GabbleMediaChannel *chan, gpointer user_data)
 static GabbleMediaChannel *
 new_media_channel (GabbleMediaFactory *fac,
                    GabbleJingleSession *sess,
-                   TpHandle maybe_peer)
+                   TpHandle maybe_peer,
+                   gboolean peer_in_rp,
+                   gboolean initial_audio,
+                   gboolean initial_video)
 {
   GabbleMediaFactoryPrivate *priv;
   TpBaseConnection *conn;
@@ -255,7 +219,7 @@ new_media_channel (GabbleMediaFactory *fac,
 
   g_assert (GABBLE_IS_MEDIA_FACTORY (fac));
 
-  priv = GABBLE_MEDIA_FACTORY_GET_PRIVATE (fac);
+  priv = fac->priv;
   conn = (TpBaseConnection *) priv->conn;
 
   object_path = g_strdup_printf ("%s/MediaChannel%u",
@@ -264,10 +228,12 @@ new_media_channel (GabbleMediaFactory *fac,
 
   chan = g_object_new (GABBLE_TYPE_MEDIA_CHANNEL,
                        "connection", priv->conn,
-                       "factory", fac,
                        "object-path", object_path,
                        "session", sess,
                        "initial-peer", maybe_peer,
+                       "peer-in-rp", peer_in_rp,
+                       "initial-audio", initial_audio,
+                       "initial-video", initial_video,
                        NULL);
 
   DEBUG ("object path %s", object_path);
@@ -284,34 +250,18 @@ new_media_channel (GabbleMediaFactory *fac,
 static void
 gabble_media_factory_close_all (GabbleMediaFactory *fac)
 {
-  GabbleMediaFactoryPrivate *priv = GABBLE_MEDIA_FACTORY_GET_PRIVATE (fac);
+  GabbleMediaFactoryPrivate *priv = fac->priv;
+  GPtrArray *tmp = gabble_g_ptr_array_copy (priv->channels);
+  guint i;
 
   DEBUG ("closing channels");
 
-  if (priv->channels != NULL)
+  for (i = 0; i < tmp->len; i++)
     {
-      GPtrArray *tmp = priv->channels;
-      guint i;
+      GabbleMediaChannel *chan = g_ptr_array_index (tmp, i);
 
-      priv->channels = NULL;
-
-      for (i = 0; i < tmp->len; i++)
-        {
-          GabbleMediaChannel *chan = g_ptr_array_index (tmp, i);
-
-          DEBUG ("about to unref channel with ref_count %d",
-                   G_OBJECT (chan)->ref_count);
-
-          g_object_unref (chan);
-        }
-
-      g_ptr_array_free (tmp, TRUE);
-    }
-
-  if (priv->session_chans != NULL)
-    {
-      g_hash_table_destroy (priv->session_chans);
-      priv->session_chans = NULL;
+      DEBUG ("closing %p", chan);
+      gabble_media_channel_close (chan);
     }
 
   if (priv->status_changed_id != 0)
@@ -330,7 +280,29 @@ new_jingle_session_cb (GabbleJingleFactory *jf, GabbleJingleSession *sess, gpoin
   if (gabble_jingle_session_get_content_type (sess) ==
       GABBLE_TYPE_JINGLE_MEDIA_RTP)
     {
-      GabbleMediaChannel *chan = new_media_channel (self, sess, sess->peer);
+      GabbleMediaChannel *chan = new_media_channel (self, sess, sess->peer,
+          FALSE, FALSE, FALSE);
+      GList *cs;
+
+      /* FIXME: we need this detection to properly adjust nat-traversal on
+       * the channel. We hope all contents will have the same transport... */
+      cs = gabble_jingle_session_get_contents (sess);
+
+      if (cs != NULL)
+        {
+          GabbleJingleContent *c = cs->data;
+          gchar *ns;
+
+          g_object_get (c, "transport-ns", &ns, NULL);
+
+          if (!tp_strdiff (ns, NS_JINGLE_TRANSPORT_ICEUDP))
+              g_object_set (chan, "nat-traversal", "ice-udp", NULL);
+          else if (!tp_strdiff (ns, NS_JINGLE_TRANSPORT_RAWUDP))
+              g_object_set (chan, "nat-traversal", "none", NULL);
+
+          g_list_free (cs);
+        }
+
       tp_channel_manager_emit_new_channel (self,
           TP_EXPORTABLE_CHANNEL (chan), NULL);
     }
@@ -342,7 +314,7 @@ connection_status_changed_cb (GabbleConnection *conn,
                               guint reason,
                               GabbleMediaFactory *self)
 {
-  GabbleMediaFactoryPrivate *priv = GABBLE_MEDIA_FACTORY_GET_PRIVATE (self);
+  GabbleMediaFactoryPrivate *priv = self->priv;
 
   switch (status)
     {
@@ -364,7 +336,7 @@ gabble_media_factory_constructed (GObject *object)
   void (*chain_up) (GObject *) =
       G_OBJECT_CLASS (gabble_media_factory_parent_class)->constructed;
   GabbleMediaFactory *self = GABBLE_MEDIA_FACTORY (object);
-  GabbleMediaFactoryPrivate *priv = GABBLE_MEDIA_FACTORY_GET_PRIVATE (self);
+  GabbleMediaFactoryPrivate *priv = self->priv;
 
   if (chain_up != NULL)
     chain_up (object);
@@ -380,7 +352,7 @@ gabble_media_factory_foreach_channel (TpChannelManager *manager,
                                       gpointer user_data)
 {
   GabbleMediaFactory *fac = GABBLE_MEDIA_FACTORY (manager);
-  GabbleMediaFactoryPrivate *priv = GABBLE_MEDIA_FACTORY_GET_PRIVATE (fac);
+  GabbleMediaFactoryPrivate *priv = fac->priv;
   guint i;
 
   for (i = 0; i < priv->channels->len; i++)
@@ -402,6 +374,8 @@ static const gchar * const media_channel_fixed_properties[] = {
 static const gchar * const named_channel_allowed_properties[] = {
     TP_IFACE_CHANNEL ".TargetHandle",
     TP_IFACE_CHANNEL ".TargetID",
+    GABBLE_IFACE_CHANNEL_TYPE_STREAMED_MEDIA_FUTURE ".InitialAudio",
+    GABBLE_IFACE_CHANNEL_TYPE_STREAMED_MEDIA_FUTURE ".InitialVideo",
     NULL
 };
 
@@ -443,6 +417,58 @@ typedef enum
 } RequestMethod;
 
 
+typedef struct
+{
+    GabbleMediaFactory *self;
+    GabbleMediaChannel *channel;
+    gpointer request_token;
+} MediaChannelRequest;
+
+
+static MediaChannelRequest *
+media_channel_request_new (GabbleMediaFactory *self,
+    GabbleMediaChannel *channel,
+    gpointer request_token)
+{
+  MediaChannelRequest *mcr = g_slice_new0 (MediaChannelRequest);
+
+  mcr->self = self;
+  mcr->channel = channel;
+  mcr->request_token = request_token;
+
+  return mcr;
+}
+
+static void
+media_channel_request_free (MediaChannelRequest *mcr)
+{
+  g_slice_free (MediaChannelRequest, mcr);
+}
+
+static void
+media_channel_request_succeeded_cb (MediaChannelRequest *mcr,
+    GPtrArray *streams)
+{
+  GSList *request_tokens;
+
+  request_tokens = g_slist_prepend (NULL, mcr->request_token);
+  tp_channel_manager_emit_new_channel (mcr->self,
+      TP_EXPORTABLE_CHANNEL (mcr->channel), request_tokens);
+  g_slist_free (request_tokens);
+
+  media_channel_request_free (mcr);
+}
+
+static void
+media_channel_request_failed_cb (MediaChannelRequest *mcr,
+    GError *error)
+{
+  tp_channel_manager_emit_request_failed (mcr->self, mcr->request_token,
+      error->domain, error->code, error->message);
+
+  media_channel_request_free (mcr);
+}
+
 static gboolean
 gabble_media_factory_requestotron (TpChannelManager *manager,
                                    gpointer request_token,
@@ -450,18 +476,19 @@ gabble_media_factory_requestotron (TpChannelManager *manager,
                                    RequestMethod method)
 {
   GabbleMediaFactory *self = GABBLE_MEDIA_FACTORY (manager);
-  GabbleMediaFactoryPrivate *priv = GABBLE_MEDIA_FACTORY_GET_PRIVATE (self);
+  GabbleMediaFactoryPrivate *priv = self->priv;
   TpHandleType handle_type;
   TpHandle handle;
   GabbleMediaChannel *channel = NULL;
   GError *error = NULL;
-  GSList *request_tokens;
-  gboolean require_target_handle;
-  gboolean add_peer_to_remote_pending;
+  gboolean require_target_handle, add_peer_to_remote_pending;
+  gboolean initial_audio, initial_video;
 
   /* Supported modes of operation:
    * - RequestChannel(None, 0):
    *     channel is anonymous;
+   *     caller may optionally use AddMembers to add the peer to RemotePending
+   *     without sending them any XML;
    *     caller uses RequestStreams to set the peer and start the call.
    * - RequestChannel(Contact, n) where n != 0:
    *     channel has TargetHandle=n;
@@ -502,6 +529,11 @@ gabble_media_factory_requestotron (TpChannelManager *manager,
   handle = tp_asv_get_uint32 (request_properties,
       TP_IFACE_CHANNEL ".TargetHandle", NULL);
 
+  initial_audio = tp_asv_get_boolean (request_properties,
+      GABBLE_IFACE_CHANNEL_TYPE_STREAMED_MEDIA_FUTURE ".InitialAudio", NULL);
+  initial_video = tp_asv_get_boolean (request_properties,
+      GABBLE_IFACE_CHANNEL_TYPE_STREAMED_MEDIA_FUTURE ".InitialVideo", NULL);
+
   switch (handle_type)
     {
     case TP_HANDLE_TYPE_NONE:
@@ -521,7 +553,7 @@ gabble_media_factory_requestotron (TpChannelManager *manager,
               &error))
         goto error;
 
-      channel = new_media_channel (self, NULL, 0);
+      channel = new_media_channel (self, NULL, 0, FALSE, FALSE, FALSE);
       break;
 
     case TP_HANDLE_TYPE_CONTACT:
@@ -545,6 +577,9 @@ gabble_media_factory_requestotron (TpChannelManager *manager,
 
               if (peer == handle)
                 {
+                  /* Per the spec, we ignore InitialAudio and InitialVideo when
+                   * looking for an existing channel.
+                   */
                   tp_channel_manager_emit_request_already_satisfied (self,
                       request_token, TP_EXPORTABLE_CHANNEL (channel));
                   return TRUE;
@@ -552,30 +587,19 @@ gabble_media_factory_requestotron (TpChannelManager *manager,
             }
         }
 
-      channel = new_media_channel (self, NULL, handle);
-
-      if (add_peer_to_remote_pending)
-        {
-          if (!_gabble_media_channel_add_member ((GObject *) channel, handle,
-                "", &error))
-            {
-              gabble_media_channel_close (channel);
-              goto error;
-            }
-        }
-
+      channel = new_media_channel (self, NULL, handle,
+          add_peer_to_remote_pending, initial_audio, initial_video);
       break;
-
     default:
       return FALSE;
     }
 
   g_assert (channel != NULL);
 
-  request_tokens = g_slist_prepend (NULL, request_token);
-  tp_channel_manager_emit_new_channel (self,
-      TP_EXPORTABLE_CHANNEL (channel), request_tokens);
-  g_slist_free (request_tokens);
+  gabble_media_channel_request_initial_streams (channel,
+      (GFunc) media_channel_request_succeeded_cb,
+      (GFunc) media_channel_request_failed_cb,
+      media_channel_request_new (self, channel, request_token));
 
   return TRUE;
 
