@@ -46,6 +46,7 @@
 #include "jingle-media-rtp.h"
 #include "media-channel.h"
 #include "namespaces.h"
+#include "util.h"
 
 static void stream_handler_iface_init (gpointer, gpointer);
 static void dbus_properties_iface_init (gpointer, gpointer);
@@ -62,10 +63,6 @@ G_DEFINE_TYPE_WITH_CODE(GabbleMediaStream,
 /* signal enum */
 enum
 {
-    DESTROY,
-    NEW_ACTIVE_CANDIDATE_PAIR,
-    NEW_NATIVE_CANDIDATE,
-    SUPPORTED_CODECS,
     ERROR,
     UNHOLD_FAILED,
 
@@ -106,22 +103,27 @@ struct _GabbleMediaStreamPrivate
   guint media_type;
 
   GValue native_codecs;     /* intersected codec list */
-  GValue native_candidates;
+
+  /* Whether we're waiting for a codec intersection from the streaming
+   * implementation. If FALSE, SupportedCodecs is a no-op.
+   */
+  gboolean awaiting_intersection;
 
   GValue remote_codecs;
   GValue remote_candidates;
 
   guint remote_candidate_count;
 
-  /* signal handler ID for content REMOVED signal */
-  gulong removed_id;
-
   /* source ID for initial codecs/candidates getter */
   gulong initial_getter_id;
 
   gchar *nat_traversal;
   /* GPtrArray(GValueArray(STRING, UINT)) */
   GPtrArray *stun_servers;
+  /* GPtrArray(GHashTable(string => GValue)) */
+  GPtrArray *relay_info;
+
+  gboolean on_hold;
 
   /* These are really booleans, but gboolean is signed. Thanks, GLib */
   unsigned closed:1;
@@ -132,8 +134,6 @@ struct _GabbleMediaStreamPrivate
   unsigned created_locally:1;
 };
 
-#define GABBLE_MEDIA_STREAM_GET_PRIVATE(obj) ((obj)->priv)
-
 static void push_remote_codecs (GabbleMediaStream *stream);
 static void push_remote_candidates (GabbleMediaStream *stream);
 static void push_playing (GabbleMediaStream *stream);
@@ -147,6 +147,8 @@ static void content_state_changed_cb (GabbleJingleContent *c,
      GParamSpec *pspec, GabbleMediaStream *stream);
 static void content_senders_changed_cb (GabbleJingleContent *c,
      GParamSpec *pspec, GabbleMediaStream *stream);
+static void remote_state_changed_cb (GabbleJingleSession *session,
+    GabbleMediaStream *stream);
 static void content_removed_cb (GabbleJingleContent *content,
       GabbleMediaStream *stream);
 static void update_direction (GabbleMediaStream *stream, GabbleJingleContent *c);
@@ -154,20 +156,38 @@ static void update_sending (GabbleMediaStream *stream, gboolean start_sending);
 
 GabbleMediaStream *
 gabble_media_stream_new (const gchar *object_path,
-                         GabbleJingleContent *content,
-                         const gchar *name,
-                         guint id,
-                         const gchar *nat_traversal)
+    GabbleJingleContent *content,
+    const gchar *name,
+    guint id,
+    const gchar *nat_traversal,
+    const GPtrArray *relay_info,
+    gboolean local_hold)
 {
+  GPtrArray *empty = NULL;
+  GabbleMediaStream *result;
+
   g_return_val_if_fail (GABBLE_IS_JINGLE_MEDIA_RTP (content), NULL);
 
-  return g_object_new (GABBLE_TYPE_MEDIA_STREAM,
+  if (relay_info == NULL)
+    {
+      empty = g_ptr_array_sized_new (0);
+      relay_info = empty;
+    }
+
+  result = g_object_new (GABBLE_TYPE_MEDIA_STREAM,
       "object-path", object_path,
       "content", content,
       "name", name,
       "id", id,
       "nat-traversal", nat_traversal,
+      "relay-info", relay_info,
+      "local-hold", local_hold,
       NULL);
+
+  if (empty != NULL)
+    g_ptr_array_free (empty, TRUE);
+
+  return result;
 }
 
 TpMediaStreamType
@@ -192,10 +212,6 @@ gabble_media_stream_init (GabbleMediaStream *self)
   g_value_take_boxed (&priv->native_codecs,
       dbus_g_type_specialized_construct (codec_list_type));
 
-  g_value_init (&priv->native_candidates, candidate_list_type);
-  g_value_take_boxed (&priv->native_candidates,
-      dbus_g_type_specialized_construct (candidate_list_type));
-
   g_value_init (&priv->remote_codecs, codec_list_type);
   g_value_take_boxed (&priv->remote_codecs,
       dbus_g_type_specialized_construct (codec_list_type));
@@ -211,7 +227,7 @@ static gboolean
 _get_initial_codecs_and_candidates (gpointer user_data)
 {
   GabbleMediaStream *stream = GABBLE_MEDIA_STREAM (user_data);
-  GabbleMediaStreamPrivate *priv = GABBLE_MEDIA_STREAM_GET_PRIVATE (stream);
+  GabbleMediaStreamPrivate *priv = stream->priv;
 
   priv->initial_getter_id = 0;
 
@@ -245,7 +261,7 @@ gabble_media_stream_constructor (GType type, guint n_props,
   obj = G_OBJECT_CLASS (gabble_media_stream_parent_class)->
            constructor (type, n_props, props);
   stream = GABBLE_MEDIA_STREAM (obj);
-  priv = GABBLE_MEDIA_STREAM_GET_PRIVATE (stream);
+  priv = stream->priv;
 
   g_assert (priv->content != NULL);
 
@@ -294,6 +310,10 @@ gabble_media_stream_constructor (GType type, guint n_props,
           MAKE_COMBINED_DIRECTION (TP_MEDIA_STREAM_DIRECTION_BIDIRECTIONAL,
             0), NULL);
     }
+  else
+    {
+      priv->awaiting_intersection = TRUE;
+    }
 
   return obj;
 }
@@ -305,7 +325,7 @@ gabble_media_stream_get_property (GObject    *object,
                                   GParamSpec *pspec)
 {
   GabbleMediaStream *stream = GABBLE_MEDIA_STREAM (object);
-  GabbleMediaStreamPrivate *priv = GABBLE_MEDIA_STREAM_GET_PRIVATE (stream);
+  GabbleMediaStreamPrivate *priv = stream->priv;
 
   switch (property_id) {
     case PROP_OBJECT_PATH:
@@ -348,8 +368,7 @@ gabble_media_stream_get_property (GObject    *object,
       g_value_set_boolean (value, priv->created_locally);
       break;
     case PROP_RELAY_INFO:
-      /* a stub implementation, for now */
-      g_value_take_boxed (value, g_ptr_array_sized_new (0));
+      g_value_set_boxed (value, priv->relay_info);
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
@@ -364,7 +383,7 @@ gabble_media_stream_set_property (GObject      *object,
                                   GParamSpec   *pspec)
 {
   GabbleMediaStream *stream = GABBLE_MEDIA_STREAM (object);
-  GabbleMediaStreamPrivate *priv = GABBLE_MEDIA_STREAM_GET_PRIVATE (stream);
+  GabbleMediaStreamPrivate *priv = stream->priv;
 
   switch (property_id) {
     case PROP_OBJECT_PATH:
@@ -395,12 +414,14 @@ gabble_media_stream_set_property (GObject      *object,
         }
       break;
     case PROP_COMBINED_DIRECTION:
+      DEBUG ("changing combined direction from %u to %u",
+          stream->combined_direction, g_value_get_uint (value));
       stream->combined_direction = g_value_get_uint (value);
       break;
     case PROP_CONTENT:
       g_assert (priv->content == NULL);
 
-      priv->content = g_value_get_object (value);
+      priv->content = g_value_dup_object (value);
 
         {
           guint jtype;
@@ -420,27 +441,38 @@ gabble_media_stream_set_property (GObject      *object,
         }
 
       DEBUG ("%p: connecting to content %p signals", stream, priv->content);
-      g_signal_connect (priv->content, "new-candidates",
-          (GCallback) new_remote_candidates_cb, stream);
+
+      gabble_signal_connect_weak (priv->content, "new-candidates",
+          (GCallback) new_remote_candidates_cb, object);
 
       /* we need this also, if we're the initiator of the stream
        * (so remote codecs arrive later) */
-      g_signal_connect (priv->content, "remote-codecs",
-          (GCallback) new_remote_codecs_cb, stream);
+      gabble_signal_connect_weak (priv->content, "remote-codecs",
+          (GCallback) new_remote_codecs_cb, object);
 
-      g_signal_connect (priv->content, "notify::state",
-          (GCallback) content_state_changed_cb, stream);
+      gabble_signal_connect_weak (priv->content, "notify::state",
+          (GCallback) content_state_changed_cb, object);
 
-      g_signal_connect (priv->content, "notify::senders",
-          (GCallback) content_senders_changed_cb, stream);
+      gabble_signal_connect_weak (priv->content, "notify::senders",
+          (GCallback) content_senders_changed_cb, object);
 
-      priv->removed_id = g_signal_connect (priv->content, "removed",
-          (GCallback) content_removed_cb, stream);
+      gabble_signal_connect_weak (priv->content->session,
+          "remote-state-changed", (GCallback) remote_state_changed_cb, object);
+
+      gabble_signal_connect_weak (priv->content, "removed",
+          (GCallback) content_removed_cb, object);
       break;
     case PROP_NAT_TRAVERSAL:
       g_assert (priv->nat_traversal == NULL);
       priv->nat_traversal = g_value_dup_string (value);
       break;
+    case PROP_RELAY_INFO:
+      g_assert (priv->relay_info == NULL);
+      priv->relay_info = g_value_dup_boxed (value);
+      break;
+    case PROP_LOCAL_HOLD:
+      priv->local_hold = g_value_get_boolean (value);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
       break;
@@ -455,10 +487,6 @@ gabble_media_stream_class_init (GabbleMediaStreamClass *gabble_media_stream_clas
 {
   GObjectClass *object_class = G_OBJECT_CLASS (gabble_media_stream_class);
   GParamSpec *param_spec;
-  GType transport_list_type =
-      TP_ARRAY_TYPE_MEDIA_STREAM_HANDLER_TRANSPORT_LIST;
-  GType codec_list_type =
-      TP_ARRAY_TYPE_MEDIA_STREAM_HANDLER_CODEC_LIST;
 
   g_type_class_add_private (gabble_media_stream_class,
       sizeof (GabbleMediaStreamPrivate));
@@ -554,7 +582,7 @@ gabble_media_stream_class_init (GabbleMediaStreamClass *gabble_media_stream_clas
 
   param_spec = g_param_spec_boolean ("local-hold", "Local hold?",
       "True if resources used for this stream have been freed.", FALSE,
-      G_PARAM_READABLE |
+      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY |
       G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB | G_PARAM_STATIC_NICK);
   g_object_class_install_property (object_class, PROP_LOCAL_HOLD, param_spec);
 
@@ -577,7 +605,7 @@ gabble_media_stream_class_init (GabbleMediaStreamClass *gabble_media_stream_clas
   param_spec = g_param_spec_boxed ("relay-info", "Relay info",
       "Array of mappings containing relay server information",
       TP_ARRAY_TYPE_STRING_VARIANT_MAP_LIST,
-      G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
+      G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
   g_object_class_install_property (object_class, PROP_RELAY_INFO, param_spec);
 
   param_spec = g_param_spec_string ("nat-traversal", "NAT traversal",
@@ -594,42 +622,6 @@ gabble_media_stream_class_init (GabbleMediaStreamClass *gabble_media_stream_clas
 
   /* signals not exported by D-Bus interface */
 
-  signals[DESTROY] =
-    g_signal_new ("destroy",
-                  G_OBJECT_CLASS_TYPE (gabble_media_stream_class),
-                  G_SIGNAL_RUN_LAST | G_SIGNAL_DETAILED,
-                  0,
-                  NULL, NULL,
-                  g_cclosure_marshal_VOID__VOID,
-                  G_TYPE_NONE, 0);
-
-  signals[NEW_ACTIVE_CANDIDATE_PAIR] =
-    g_signal_new ("new-active-candidate-pair",
-                  G_OBJECT_CLASS_TYPE (gabble_media_stream_class),
-                  G_SIGNAL_RUN_LAST | G_SIGNAL_DETAILED,
-                  0,
-                  NULL, NULL,
-                  gabble_marshal_VOID__STRING_STRING,
-                  G_TYPE_NONE, 2, G_TYPE_STRING, G_TYPE_STRING);
-
-  signals[NEW_NATIVE_CANDIDATE] =
-    g_signal_new ("new-native-candidate",
-                  G_OBJECT_CLASS_TYPE (gabble_media_stream_class),
-                  G_SIGNAL_RUN_LAST | G_SIGNAL_DETAILED,
-                  0,
-                  NULL, NULL,
-                  gabble_marshal_VOID__STRING_BOXED,
-                  G_TYPE_NONE, 2, G_TYPE_STRING, transport_list_type);
-
-  signals[SUPPORTED_CODECS] =
-    g_signal_new ("supported-codecs",
-                  G_OBJECT_CLASS_TYPE (gabble_media_stream_class),
-                  G_SIGNAL_RUN_LAST | G_SIGNAL_DETAILED,
-                  0,
-                  NULL, NULL,
-                  g_cclosure_marshal_VOID__BOXED,
-                  G_TYPE_NONE, 1, codec_list_type);
-
   signals[ERROR] =
     g_signal_new ("error",
                   G_OBJECT_CLASS_TYPE (gabble_media_stream_class),
@@ -649,7 +641,7 @@ void
 gabble_media_stream_dispose (GObject *object)
 {
   GabbleMediaStream *self = GABBLE_MEDIA_STREAM (object);
-  GabbleMediaStreamPrivate *priv = GABBLE_MEDIA_STREAM_GET_PRIVATE (self);
+  GabbleMediaStreamPrivate *priv = self->priv;
 
   DEBUG ("called");
 
@@ -662,19 +654,12 @@ gabble_media_stream_dispose (GObject *object)
       priv->initial_getter_id = 0;
     }
 
-  _gabble_media_stream_close (self);
-
-  g_signal_emit (self, signals[DESTROY], 0);
+  gabble_media_stream_close (self);
 
   priv->dispose_has_run = TRUE;
 
-  /* If content wasn't removed already, it will emit REMOVED signal
-   * later on. We don't want to catch that. */
-  if (priv->removed_id)
-    {
-      g_signal_handler_disconnect (priv->content, priv->removed_id);
-      priv->removed_id = 0;
-    }
+  g_object_unref (priv->content);
+  priv->content = NULL;
 
   if (G_OBJECT_CLASS (gabble_media_stream_parent_class)->dispose)
     G_OBJECT_CLASS (gabble_media_stream_parent_class)->dispose (object);
@@ -684,13 +669,19 @@ void
 gabble_media_stream_finalize (GObject *object)
 {
   GabbleMediaStream *self = GABBLE_MEDIA_STREAM (object);
-  GabbleMediaStreamPrivate *priv = GABBLE_MEDIA_STREAM_GET_PRIVATE (self);
+  GabbleMediaStreamPrivate *priv = self->priv;
 
   g_free (priv->object_path);
   g_free (priv->nat_traversal);
 
+  /* FIXME: use correct macro when available */
+  if (priv->stun_servers != NULL)
+    g_boxed_free (tp_type_dbus_array_su (), priv->stun_servers);
+
+  if (priv->relay_info != NULL)
+    g_boxed_free (TP_ARRAY_TYPE_STRING_VARIANT_MAP_LIST, priv->relay_info);
+
   g_value_unset (&priv->native_codecs);
-  g_value_unset (&priv->native_candidates);
 
   g_value_unset (&priv->remote_codecs);
   g_value_unset (&priv->remote_candidates);
@@ -714,7 +705,7 @@ gabble_media_stream_codec_choice (TpSvcMediaStreamHandler *iface,
 
   g_assert (GABBLE_IS_MEDIA_STREAM (self));
 
-  priv = GABBLE_MEDIA_STREAM_GET_PRIVATE (self);
+  priv = self->priv;
 
   tp_svc_media_stream_handler_return_from_codec_choice (context);
 }
@@ -730,7 +721,7 @@ gabble_media_stream_error (GabbleMediaStream *self,
 
   g_assert (GABBLE_IS_MEDIA_STREAM (self));
 
-  priv = GABBLE_MEDIA_STREAM_GET_PRIVATE (self);
+  priv = self->priv;
 
   DEBUG ( "Media.StreamHandler::Error called, error %u (%s) -- emitting signal",
       errno, message);
@@ -796,7 +787,7 @@ gabble_media_stream_hold_state (TpSvcMediaStreamHandler *iface,
                                 DBusGMethodInvocation *context)
 {
   GabbleMediaStream *self = GABBLE_MEDIA_STREAM (iface);
-  GabbleMediaStreamPrivate *priv = GABBLE_MEDIA_STREAM_GET_PRIVATE (self);
+  GabbleMediaStreamPrivate *priv = self->priv;
 
   DEBUG ("%p: %s", self, hold_state ? "held" : "unheld");
   priv->local_hold = hold_state;
@@ -819,7 +810,7 @@ gabble_media_stream_unhold_failure (TpSvcMediaStreamHandler *iface,
                                     DBusGMethodInvocation *context)
 {
   GabbleMediaStream *self = GABBLE_MEDIA_STREAM (iface);
-  GabbleMediaStreamPrivate *priv = GABBLE_MEDIA_STREAM_GET_PRIVATE (self);
+  GabbleMediaStreamPrivate *priv = self->priv;
 
   DEBUG ("%p", self);
 
@@ -847,7 +838,7 @@ gabble_media_stream_native_candidates_prepared (TpSvcMediaStreamHandler *iface,
 
   g_assert (GABBLE_IS_MEDIA_STREAM (self));
 
-  priv = GABBLE_MEDIA_STREAM_GET_PRIVATE (self);
+  priv = self->priv;
 
   tp_svc_media_stream_handler_return_from_native_candidates_prepared (context);
 }
@@ -865,15 +856,8 @@ gabble_media_stream_new_active_candidate_pair (TpSvcMediaStreamHandler *iface,
                                                const gchar *remote_candidate_id,
                                                DBusGMethodInvocation *context)
 {
-  GabbleMediaStream *self = GABBLE_MEDIA_STREAM (iface);
-  GabbleMediaStreamPrivate *priv;
-
-  g_assert (GABBLE_IS_MEDIA_STREAM (self));
-
-  priv = GABBLE_MEDIA_STREAM_GET_PRIVATE (self);
-
-  g_signal_emit (self, signals[NEW_ACTIVE_CANDIDATE_PAIR], 0,
-                 native_candidate_id, remote_candidate_id);
+  DEBUG ("called (%s, %s); this is a no-op on Jingle", native_candidate_id,
+      remote_candidate_id);
 
   tp_svc_media_stream_handler_return_from_new_active_candidate_pair (context);
 }
@@ -894,19 +878,12 @@ gabble_media_stream_new_native_candidate (TpSvcMediaStreamHandler *iface,
   GabbleMediaStream *self = GABBLE_MEDIA_STREAM (iface);
   GabbleMediaStreamPrivate *priv;
   JingleSessionState state;
-  GPtrArray *candidates;
-  GValue candidate = { 0, };
-  GValueArray *transport;
-  guint component_id;
-  const gchar *addr;
-  GType candidate_struct_type = TP_STRUCT_TYPE_MEDIA_STREAM_HANDLER_CANDIDATE;
-  JingleCandidate *c;
-  GList *li;
+  GList *li = NULL;
   guint i;
 
   g_assert (GABBLE_IS_MEDIA_STREAM (self));
 
-  priv = GABBLE_MEDIA_STREAM_GET_PRIVATE (self);
+  priv = self->priv;
 
   g_object_get (priv->content->session, "state", &state, NULL);
 
@@ -919,106 +896,60 @@ gabble_media_stream_new_native_candidate (TpSvcMediaStreamHandler *iface,
       return;
     }
 
-  candidates = g_value_get_boxed (&priv->native_candidates);
-
-  g_value_init (&candidate, candidate_struct_type);
-  g_value_take_boxed (&candidate,
-      dbus_g_type_specialized_construct (candidate_struct_type));
-
-  dbus_g_type_struct_set (&candidate,
-      0, candidate_id,
-      1, transports,
-      G_MAXUINT);
-
-  if (transports->len < 1)
-    {
-      GError only_one = { TP_ERRORS, TP_ERROR_INVALID_ARGUMENT, "google p2p "
-          "connections only support the concept of one transport per "
-          "candidate" };
-      DEBUG ("%s: number of transports was not 1; "
-          "rejecting", G_STRFUNC);
-      dbus_g_method_return_error (context, &only_one);
-      return;
-    }
-
-  if (transports->len > 1)
-    {
-      DEBUG ("google p2p connections only support the concept of one "
-          "transport per candidate, ignoring other components");
-    }
-
   for (i = 0; i < transports->len; i++)
     {
+      GValueArray *transport;
       guint component;
+      const gchar *addr;
+      JingleCandidate *c;
 
       transport = g_ptr_array_index (transports, i);
       component = g_value_get_uint (g_value_array_get_nth (transport, 0));
 
-      /* Accept component 0 because old farsight1 stream-engine didn't set the
-       * component */
-      if (component == 0 || component == 1)
+      /* Farsight 1 compatibility */
+      if (component == 0)
+        component = 1;
+
+      /* We understand RTP and RTCP, and silently ignore the rest */
+      if ((component != 1) && (component != 2))
+        continue;
+
+      addr = g_value_get_string (g_value_array_get_nth (transport, 1));
+      if (!strcmp (addr, "127.0.0.1"))
         {
-          break;
-        }
-      else
-        {
-          transport = NULL;
+          DEBUG ("ignoring native localhost candidate");
+          continue;
         }
+
+      c = jingle_candidate_new (
+          /* protocol */
+          g_value_get_uint (g_value_array_get_nth (transport, 3)),
+          /* candidate type, we're relying on 1:1 candidate type mapping */
+          g_value_get_uint (g_value_array_get_nth (transport, 7)),
+          /* id */
+          candidate_id,
+          /* component */
+          component,
+          /* address */
+          g_value_get_string (g_value_array_get_nth (transport, 1)),
+          /* port */
+          g_value_get_uint (g_value_array_get_nth (transport, 2)),
+          /* generation */
+          0,
+          /* preference */
+          g_value_get_double (g_value_array_get_nth (transport, 6)),
+          /* username */
+          g_value_get_string (g_value_array_get_nth (transport, 8)),
+          /* password */
+          g_value_get_string (g_value_array_get_nth (transport, 9)),
+          /* network */
+          0);
+
+      li = g_list_prepend (li, c);
     }
 
-
-  if (transport == NULL)
-    {
-      GError only_one = { TP_ERRORS, TP_ERROR_INVALID_ARGUMENT, "You need"
-          " at least a component 1." };
-      DEBUG ("%s: number of transports was not 1; rejecting", G_STRFUNC);
-      dbus_g_method_return_error (context, &only_one);
-      return;
-    }
-
-
-  addr = g_value_get_string (g_value_array_get_nth (transport, 1));
-  if (!strcmp (addr, "127.0.0.1"))
-    {
-      DEBUG ("ignoring native localhost candidate");
-      tp_svc_media_stream_handler_return_from_new_native_candidate (context);
-      return;
-    }
-
-  component_id = g_value_get_uint (g_value_array_get_nth (transport, 0));
-  /* Old farsight1 s-e didn't set the component, make sure it's sane */
-  if (component_id == 0)
-      component_id = 1;
-
-  g_ptr_array_add (candidates, g_value_get_boxed (&candidate));
-
-  DEBUG ("put 1 native candidate from stream-engine into cache");
-
-  c = jingle_candidate_new (component_id,
-      /* address */
-      g_value_get_string (g_value_array_get_nth (transport, 1)),
-      /* port */
-      g_value_get_uint (g_value_array_get_nth (transport, 2)),
-      /* protocol */
-      g_value_get_uint (g_value_array_get_nth (transport, 3)),
-      /* preference */
-      g_value_get_double (g_value_array_get_nth (transport, 6)),
-      /* candidate type, we're relying on 1:1 candidate type mapping */
-      g_value_get_uint (g_value_array_get_nth (transport, 7)),
-      /* username */
-      g_value_get_string (g_value_array_get_nth (transport, 8)),
-      /* password */
-      g_value_dup_string (g_value_array_get_nth (transport, 9)),
-      /* FIXME: network is hardcoded for now */
-      0,
-      /* FIXME: generation is also hardcoded for now */
-      0);
-
-  li = g_list_prepend (NULL, c);
-  gabble_jingle_content_add_candidates (priv->content, li);
-
-  g_signal_emit (self, signals[NEW_NATIVE_CANDIDATE], 0,
-                 candidate_id, transports);
+  if (li != NULL)
+    gabble_jingle_content_add_candidates (priv->content, li);
 
   tp_svc_media_stream_handler_return_from_new_native_candidate (context);
 }
@@ -1042,7 +973,7 @@ gabble_media_stream_ready (TpSvcMediaStreamHandler *iface,
 
   g_assert (GABBLE_IS_MEDIA_STREAM (self));
 
-  priv = GABBLE_MEDIA_STREAM_GET_PRIVATE (self);
+  priv = self->priv;
 
   DEBUG ("ready called");
 
@@ -1054,6 +985,13 @@ gabble_media_stream_ready (TpSvcMediaStreamHandler *iface,
       push_remote_candidates (self);
       push_playing (self);
       push_sending (self);
+
+      /* If a new stream is added while the call's on hold, it will have
+       * local_hold set at construct time. So once tp-fs has called Ready(), we
+       * should let it know this stream's on hold.
+       */
+      if (priv->local_hold)
+        gabble_media_stream_hold (self, priv->local_hold);
     }
   else
     {
@@ -1064,34 +1002,17 @@ gabble_media_stream_ready (TpSvcMediaStreamHandler *iface,
   gabble_media_stream_set_local_codecs (iface, codecs, context);
 }
 
-static void
-pass_local_codecs (GabbleMediaStream *stream, const GPtrArray *codecs,
-    gboolean intersection)
+static gboolean
+pass_local_codecs (GabbleMediaStream *stream,
+                   const GPtrArray *codecs,
+                   gboolean ready,
+                   GError **error)
 {
-  GabbleMediaStreamPrivate *priv = GABBLE_MEDIA_STREAM_GET_PRIVATE (stream);
+  GabbleMediaStreamPrivate *priv = stream->priv;
   GList *li = NULL;
   JingleCodec *c;
   guint i;
 
-  /* if content is created by us, we want all the codecs, else we want the
-   * intersection. */
-  if (gabble_jingle_content_is_created_by_us (priv->content))
-    {
-      if (intersection)
-        {
-          DEBUG ("we already sent our codecs, ignoring codec intersection");
-          return;
-        }
-    }
-  else
-    {
-      if (!intersection)
-        {
-          DEBUG ("ignoring local codecs, waiting for codec intersection");
-          return;
-        }
-    }
-
   DEBUG ("putting list of %d supported codecs from stream-engine into cache",
       codecs->len);
 
@@ -1124,7 +1045,8 @@ pass_local_codecs (GabbleMediaStream *stream, const GPtrArray *codecs,
       li = g_list_append (li, c);
     }
 
-  jingle_media_rtp_set_local_codecs (GABBLE_JINGLE_MEDIA_RTP (priv->content), li);
+  return jingle_media_rtp_set_local_codecs (
+      GABBLE_JINGLE_MEDIA_RTP (priv->content), li, ready, error);
 }
 
 /**
@@ -1139,8 +1061,27 @@ gabble_media_stream_set_local_codecs (TpSvcMediaStreamHandler *iface,
                                       DBusGMethodInvocation *context)
 {
   GabbleMediaStream *self = GABBLE_MEDIA_STREAM (iface);
+  GError *error = NULL;
+
+  DEBUG ("called");
+
+  if (gabble_jingle_content_is_created_by_us (self->priv->content))
+    {
+      if (!pass_local_codecs (self, codecs, self->priv->created_locally,
+          &error))
+        {
+          DEBUG ("failed: %s", error->message);
+
+          dbus_g_method_return_error (context, error);
+          g_error_free (error);
+          return;
+        }
+    }
+  else
+    {
+      DEBUG ("ignoring local codecs, waiting for codec intersection");
+    }
 
-  pass_local_codecs (self, codecs, FALSE);
   tp_svc_media_stream_handler_return_from_set_local_codecs (context);
 }
 
@@ -1156,7 +1097,7 @@ gabble_media_stream_stream_state (TpSvcMediaStreamHandler *iface,
                                   DBusGMethodInvocation *context)
 {
   GabbleMediaStream *self = GABBLE_MEDIA_STREAM (iface);
-  GabbleMediaStreamPrivate *priv = GABBLE_MEDIA_STREAM_GET_PRIVATE (self);
+  GabbleMediaStreamPrivate *priv = self->priv;
   JingleTransportState ts = JINGLE_TRANSPORT_STATE_DISCONNECTED;
 
   switch (connection_state) {
@@ -1194,20 +1135,95 @@ gabble_media_stream_supported_codecs (TpSvcMediaStreamHandler *iface,
                                       DBusGMethodInvocation *context)
 {
   GabbleMediaStream *self = GABBLE_MEDIA_STREAM (iface);
+  GabbleMediaStreamPrivate *priv = self->priv;
+  GError *error = NULL;
+
+  DEBUG ("called");
+
+  if (priv->awaiting_intersection)
+    {
+      if (!pass_local_codecs (self, codecs, TRUE, &error))
+        {
+          DEBUG ("failed: %s", error->message);
+
+          dbus_g_method_return_error (context, error);
+          g_error_free (error);
+          return;
+        }
+
+      priv->awaiting_intersection = FALSE;
+    }
+  else
+    {
+      /* If we created the stream, we don't need to send the intersection. If
+       * we didn't create it, but have already sent the intersection once, we
+       * don't need to send it again. In either case, extra calls to
+       * SupportedCodecs are in response to an incoming description-info, which
+       * can only change parameters and which XEP-0167 §10 says is purely
+       * advisory.
+       */
+      DEBUG ("we already sent, or don't need to send, our codecs");
+    }
 
-  pass_local_codecs (self, codecs, TRUE);
-  g_signal_emit (self, signals[SUPPORTED_CODECS], 0, codecs);
   tp_svc_media_stream_handler_return_from_supported_codecs (context);
 }
 
+/**
+ * gabble_media_stream_codecs_updated
+ *
+ * Implements D-Bus method CodecsUpdated
+ * on interface org.freedesktop.Telepathy.Media.StreamHandler
+ */
+static void
+gabble_media_stream_codecs_updated (TpSvcMediaStreamHandler *iface,
+                                    const GPtrArray *codecs,
+                                    DBusGMethodInvocation *context)
+{
+  GabbleMediaStream *self = GABBLE_MEDIA_STREAM (iface);
+  gboolean codecs_set =
+      (g_value_get_boxed (&self->priv->native_codecs) != NULL);
+  GError *error = NULL;
+
+  if (!codecs_set)
+    {
+      GError e = { TP_ERRORS, TP_ERROR_NOT_AVAILABLE,
+          "CodecsUpdated may only be called once an initial set of codecs "
+          "has been set" };
+
+      dbus_g_method_return_error (context, &e);
+      return;
+    }
+
+  if (self->priv->awaiting_intersection)
+    {
+      /* When awaiting an intersection the initial set of codecs should be set
+       * by calling SupportedCodecs as that is the canonical set of codecs,
+       * updates are only meaningful afterwards */
+      tp_svc_media_stream_handler_return_from_codecs_updated (context);
+      return;
+    }
+
+  if (pass_local_codecs (self, codecs, self->priv->created_locally, &error))
+    {
+      tp_svc_media_stream_handler_return_from_codecs_updated (context);
+    }
+  else
+    {
+      DEBUG ("failed: %s", error->message);
+
+      dbus_g_method_return_error (context, error);
+      g_error_free (error);
+    }
+}
+
 void
-_gabble_media_stream_close (GabbleMediaStream *stream)
+gabble_media_stream_close (GabbleMediaStream *stream)
 {
   GabbleMediaStreamPrivate *priv;
 
   g_assert (GABBLE_IS_MEDIA_STREAM (stream));
 
-  priv = GABBLE_MEDIA_STREAM_GET_PRIVATE (stream);
+  priv = stream->priv;
 
   if (!priv->closed)
     {
@@ -1220,7 +1236,7 @@ static void
 new_remote_codecs_cb (GabbleJingleContent *content,
     GList *clist, GabbleMediaStream *stream)
 {
-  GabbleMediaStreamPrivate *priv = GABBLE_MEDIA_STREAM_GET_PRIVATE (stream);
+  GabbleMediaStreamPrivate *priv;
   GList *li;
   GPtrArray *codecs;
   GType codec_struct_type = TP_STRUCT_TYPE_MEDIA_STREAM_HANDLER_CODEC;
@@ -1229,11 +1245,20 @@ new_remote_codecs_cb (GabbleJingleContent *content,
 
   g_assert (GABBLE_IS_MEDIA_STREAM (stream));
 
-  priv = GABBLE_MEDIA_STREAM_GET_PRIVATE (stream);
+  priv = stream->priv;
 
   codecs = g_value_get_boxed (&priv->remote_codecs);
 
-  g_assert (codecs->len == 0);
+  if (codecs->len != 0)
+    {
+      /* We already had some codecs; let's free the old list and make a new,
+       * empty one to fill in.
+       */
+      g_value_reset (&priv->remote_codecs);
+      codecs = dbus_g_type_specialized_construct (
+          TP_ARRAY_TYPE_MEDIA_STREAM_HANDLER_CODEC_LIST);
+      g_value_take_boxed (&priv->remote_codecs, codecs);
+    }
 
   for (li = clist; li; li = li->next)
     {
@@ -1271,12 +1296,10 @@ push_remote_codecs (GabbleMediaStream *stream)
 {
   GabbleMediaStreamPrivate *priv;
   GPtrArray *codecs;
-  GType codec_list_type =
-      TP_ARRAY_TYPE_MEDIA_STREAM_HANDLER_CODEC_LIST;
 
   g_assert (GABBLE_IS_MEDIA_STREAM (stream));
 
-  priv = GABBLE_MEDIA_STREAM_GET_PRIVATE (stream);
+  priv = stream->priv;
 
   if (!priv->ready)
     return;
@@ -1289,16 +1312,13 @@ push_remote_codecs (GabbleMediaStream *stream)
                    codecs->len);
 
   tp_svc_media_stream_handler_emit_set_remote_codecs (stream, codecs);
-
-  g_value_take_boxed (&priv->remote_codecs,
-      dbus_g_type_specialized_construct (codec_list_type));
 }
 
 static void
 new_remote_candidates_cb (GabbleJingleContent *content,
     GList *clist, GabbleMediaStream *stream)
 {
-  GabbleMediaStreamPrivate *priv = GABBLE_MEDIA_STREAM_GET_PRIVATE (stream);
+  GabbleMediaStreamPrivate *priv = stream->priv;
   GPtrArray *candidates;
   GList *li;
 
@@ -1340,8 +1360,11 @@ new_remote_candidates_cb (GabbleJingleContent *content,
       g_value_take_boxed (&candidate,
           dbus_g_type_specialized_construct (candidate_struct_type));
 
-      /* FIXME: is this naming scheme sensible? */
-      candidate_id = g_strdup_printf ("R%d", ++priv->remote_candidate_count);
+      if (c->id == NULL)
+        /* FIXME: is this naming scheme sensible? */
+        candidate_id = g_strdup_printf ("R%d", ++priv->remote_candidate_count);
+      else
+        candidate_id = c->id;
 
       dbus_g_type_struct_set (&candidate,
           0, candidate_id,
@@ -1361,7 +1384,7 @@ content_state_changed_cb (GabbleJingleContent *c,
                           GParamSpec *pspec,
                           GabbleMediaStream *stream)
 {
-  GabbleMediaStreamPrivate *priv = GABBLE_MEDIA_STREAM_GET_PRIVATE (stream);
+  GabbleMediaStreamPrivate *priv = stream->priv;
   JingleContentState state;
 
   g_object_get (c, "state", &state, NULL);
@@ -1399,7 +1422,7 @@ push_remote_candidates (GabbleMediaStream *stream)
 
   g_assert (GABBLE_IS_MEDIA_STREAM (stream));
 
-  priv = GABBLE_MEDIA_STREAM_GET_PRIVATE (stream);
+  priv = stream->priv;
 
   candidates = g_value_get_boxed (&priv->remote_candidates);
 
@@ -1434,7 +1457,7 @@ push_playing (GabbleMediaStream *stream)
 
   g_assert (GABBLE_IS_MEDIA_STREAM (stream));
 
-  priv = GABBLE_MEDIA_STREAM_GET_PRIVATE (stream);
+  priv = stream->priv;
 
   if (!priv->ready)
     return;
@@ -1450,19 +1473,22 @@ static void
 push_sending (GabbleMediaStream *stream)
 {
   GabbleMediaStreamPrivate *priv;
+  gboolean emit;
 
   g_assert (GABBLE_IS_MEDIA_STREAM (stream));
 
-  priv = GABBLE_MEDIA_STREAM_GET_PRIVATE (stream);
+  priv = stream->priv;
 
   if (!priv->ready)
     return;
 
-  DEBUG ("stream %s emitting SetStreamSending(%s)",
-      stream->name, priv->sending ? "true" : "false");
+  emit = (priv->sending && !(priv->on_hold));
+  DEBUG ("stream %s emitting SetStreamSending(%s); sending=%s, on_hold=%s",
+      stream->name, emit ? "true" : "false", priv->sending ? "true" : "false",
+      priv->on_hold ? "true" : "false");
 
   tp_svc_media_stream_handler_emit_set_stream_sending (
-      stream, priv->sending);
+      stream, emit);
 }
 
 static void
@@ -1527,10 +1553,23 @@ content_senders_changed_cb (GabbleJingleContent *c,
   update_direction (stream, c);
 }
 
+static void
+remote_state_changed_cb (GabbleJingleSession *session,
+    GabbleMediaStream *stream)
+{
+  GabbleMediaStreamPrivate *priv = stream->priv;
+  gboolean old_hold = priv->on_hold;
+
+  priv->on_hold = gabble_jingle_session_get_remote_hold (session);
+
+  if (old_hold != priv->on_hold)
+    push_sending (stream);
+}
+
 static void
 content_removed_cb (GabbleJingleContent *content, GabbleMediaStream *stream)
 {
-  _gabble_media_stream_close (stream);
+  gabble_media_stream_close (stream);
 }
 
 
@@ -1538,7 +1577,7 @@ gboolean
 gabble_media_stream_change_direction (GabbleMediaStream *stream,
     guint requested_dir, GError **error)
 {
-  GabbleMediaStreamPrivate *priv = GABBLE_MEDIA_STREAM_GET_PRIVATE (stream);
+  GabbleMediaStreamPrivate *priv = stream->priv;
   CombinedStreamDirection new_combined_dir;
   TpMediaStreamDirection current_dir;
   TpMediaStreamPendingSend pending_send;
@@ -1580,6 +1619,8 @@ gabble_media_stream_change_direction (GabbleMediaStream *stream,
       update_sending (stream, start_sending);
     }
 
+  DEBUG ("current_dir: %u, requested_dir: %u", current_dir, requested_dir);
+
   /* short-circuit sending a request if we're not asking for anything new */
   if (current_dir == requested_dir)
     return TRUE;
@@ -1642,7 +1683,7 @@ gabble_media_stream_accept_pending_local_send (GabbleMediaStream *stream)
 static void
 update_sending (GabbleMediaStream *stream, gboolean start_sending)
 {
-  GabbleMediaStreamPrivate *priv = GABBLE_MEDIA_STREAM_GET_PRIVATE (stream);
+  GabbleMediaStreamPrivate *priv = stream->priv;
   gboolean new_sending;
 
   new_sending =
@@ -1677,6 +1718,7 @@ stream_handler_iface_init (gpointer g_iface, gpointer iface_data)
   IMPLEMENT(stream_state,);
   IMPLEMENT(supported_codecs,);
   IMPLEMENT(unhold_failure,);
+  IMPLEMENT(codecs_updated,);
 #undef IMPLEMENT
 }
 
@@ -1789,3 +1831,12 @@ dbus_properties_iface_init (gpointer g_iface,
   /* set not implemented in this class */
 #undef IMPLEMENT
 }
+
+GabbleJingleMediaRtp *
+gabble_media_stream_get_content (GabbleMediaStream *self)
+{
+  /* FIXME: we should fix this whole class up. It relies throughout on
+   *        self->priv->content actually secretly being a GabbleJingleMediaRtp.
+   */
+  return GABBLE_JINGLE_MEDIA_RTP (self->priv->content);
+}
