@@ -344,7 +344,7 @@ muc_ready_cb (GabbleMucChannel *text_chan,
       /* Announce text and tubes text_chan together */
       GHashTable *channels;
 
-      channels = g_hash_table_new(g_direct_hash, g_direct_equal);
+      channels = g_hash_table_new (g_direct_hash, g_direct_equal);
       g_hash_table_insert (channels, text_chan, requests_satisfied_text);
       g_hash_table_insert (channels, tubes_chan, requests_satisfied_tubes);
 
@@ -790,7 +790,7 @@ muc_factory_message_cb (LmMessageHandler *handler,
   TpHandleRepoIface *room_repo = tp_base_connection_get_handles (conn,
       TP_HANDLE_TYPE_ROOM);
 
-  const gchar *from, *body, *subject;
+  const gchar *from, *body, *subject, *id;
   time_t stamp;
   TpChannelTextMessageType msgtype;
   TpHandleRepoIface *handle_source;
@@ -804,7 +804,7 @@ muc_factory_message_cb (LmMessageHandler *handler,
   LmMessageNode *subj_node;
 
   if (!gabble_message_util_parse_incoming_message (message, &from, &stamp,
-        &msgtype, &body, &state, &send_error, &delivery_status))
+        &msgtype, &id, &body, &state, &send_error, &delivery_status))
     return LM_HANDLER_RESULT_REMOVE_MESSAGE;
 
   if (conn_olpc_process_activity_properties_message (priv->conn, message,
@@ -878,7 +878,7 @@ muc_factory_message_cb (LmMessageHandler *handler,
 
   if (body != NULL)
     _gabble_muc_channel_receive (chan, msgtype, handle_type, handle, stamp,
-        body, message, send_error, delivery_status);
+        id, body, message, send_error, delivery_status);
 
   if (send_error == GABBLE_TEXT_CHANNEL_SEND_NO_ERROR)
     {
@@ -1252,12 +1252,27 @@ gabble_muc_factory_foreach_channel (TpChannelManager *manager,
   GabbleMucFactory *fac = GABBLE_MUC_FACTORY (manager);
   GabbleMucFactoryPrivate *priv = GABBLE_MUC_FACTORY_GET_PRIVATE (fac);
   struct _ForeachData data;
+  GHashTableIter iter;
+  gpointer value;
 
   data.user_data = user_data;
   data.foreach = foreach;
 
   g_hash_table_foreach (priv->text_channels, _foreach_slave, &data);
-  g_hash_table_foreach (priv->tubes_channels, _foreach_slave, &data);
+
+  g_hash_table_iter_init (&iter, priv->tubes_channels);
+  while (g_hash_table_iter_next (&iter, NULL, &value))
+  {
+    TpExportableChannel *chan = TP_EXPORTABLE_CHANNEL (value);
+
+    /* Add channels of type Channel.Type.Tubes */
+    foreach (chan, user_data);
+
+    /* Add channels of type Channel.Type.{Stream|DBus}Tube which live in the
+     * GabbleTubesChannel object */
+    gabble_tubes_channel_foreach (GABBLE_TUBES_CHANNEL (chan), foreach,
+        user_data);
+  }
 }
 
 
@@ -1382,13 +1397,13 @@ gabble_muc_factory_foreach_channel_class (TpChannelManager *manager,
 
   /* Muc Channel.Type.StreamTube */
   g_value_set_static_string (channel_type_value,
-      GABBLE_IFACE_CHANNEL_TYPE_STREAM_TUBE);
+      TP_IFACE_CHANNEL_TYPE_STREAM_TUBE);
   func (manager, table, gabble_tube_stream_channel_get_allowed_properties (),
       user_data);
 
   /* Muc Channel.Type.DBusTube */
   g_value_set_static_string (channel_type_value,
-      GABBLE_IFACE_CHANNEL_TYPE_DBUS_TUBE);
+      TP_IFACE_CHANNEL_TYPE_DBUS_TUBE);
   func (manager, table, gabble_tube_dbus_channel_get_allowed_properties (),
       user_data);
 
@@ -1601,12 +1616,12 @@ handle_stream_tube_channel_request (GabbleMucFactory *self,
 
   /* "Service" is a mandatory, not-fixed property */
   service = tp_asv_get_string (request_properties,
-            GABBLE_IFACE_CHANNEL_TYPE_STREAM_TUBE ".Service");
+            TP_IFACE_CHANNEL_TYPE_STREAM_TUBE ".Service");
   if (service == NULL)
     {
       g_set_error (error, TP_ERRORS, TP_ERROR_NOT_IMPLEMENTED,
           "Request does not contain the mandatory property '%s'",
-          GABBLE_IFACE_CHANNEL_TYPE_STREAM_TUBE ".Service");
+          TP_IFACE_CHANNEL_TYPE_STREAM_TUBE ".Service");
       return FALSE;
     }
 
@@ -1632,12 +1647,12 @@ handle_dbus_tube_channel_request (GabbleMucFactory *self,
 
   /* "ServiceName" is a mandatory, not-fixed property */
   service = tp_asv_get_string (request_properties,
-      GABBLE_IFACE_CHANNEL_TYPE_DBUS_TUBE ".ServiceName");
+      TP_IFACE_CHANNEL_TYPE_DBUS_TUBE ".ServiceName");
   if (service == NULL)
     {
       g_set_error (error, TP_ERRORS, TP_ERROR_NOT_IMPLEMENTED,
           "Request does not contain the mandatory property '%s'",
-          GABBLE_IFACE_CHANNEL_TYPE_DBUS_TUBE ".ServiceName");
+          TP_IFACE_CHANNEL_TYPE_DBUS_TUBE ".ServiceName");
       return FALSE;
     }
 
@@ -1664,8 +1679,8 @@ gabble_muc_factory_request (GabbleMucFactory *self,
 
    if (tp_strdiff (channel_type, TP_IFACE_CHANNEL_TYPE_TEXT) &&
        tp_strdiff (channel_type, TP_IFACE_CHANNEL_TYPE_TUBES) &&
-       tp_strdiff (channel_type, GABBLE_IFACE_CHANNEL_TYPE_STREAM_TUBE) &&
-       tp_strdiff (channel_type, GABBLE_IFACE_CHANNEL_TYPE_DBUS_TUBE))
+       tp_strdiff (channel_type, TP_IFACE_CHANNEL_TYPE_STREAM_TUBE) &&
+       tp_strdiff (channel_type, TP_IFACE_CHANNEL_TYPE_DBUS_TUBE))
      return FALSE;
 
   /* validity already checked by TpBaseConnection */
@@ -1685,13 +1700,13 @@ gabble_muc_factory_request (GabbleMucFactory *self,
           request_properties, require_new, handle, &error))
         return TRUE;
     }
-  else if (!tp_strdiff (channel_type, GABBLE_IFACE_CHANNEL_TYPE_STREAM_TUBE))
+  else if (!tp_strdiff (channel_type, TP_IFACE_CHANNEL_TYPE_STREAM_TUBE))
     {
       if (handle_stream_tube_channel_request (self, request_token,
           request_properties, require_new, handle, &error))
         return TRUE;
     }
-  else if (!tp_strdiff (channel_type, GABBLE_IFACE_CHANNEL_TYPE_DBUS_TUBE))
+  else if (!tp_strdiff (channel_type, TP_IFACE_CHANNEL_TYPE_DBUS_TUBE))
     {
       if (handle_dbus_tube_channel_request (self, request_token,
           request_properties, require_new, handle, &error))
