@@ -109,12 +109,24 @@ struct _GabbleRosterItem
   gboolean ask_subscribe;
   GoogleItemType google_type;
   gchar *name;
+  gchar *alias_for;
   TpHandleSet *groups;
   /* if not NULL, an edit attempt is already "in-flight" so instead of
    * sending off another, store required edits here until the one we
    * already sent is acknowledged - this prevents some race conditions
    */
   GabbleRosterItemEdit *unsent_edits;
+
+  /* If non-zero, the GSource id for a call to flicker_prevention_timeout. */
+  guint flicker_prevention_id;
+};
+
+typedef struct _FlickerPreventionCtx FlickerPreventionCtx;
+struct _FlickerPreventionCtx
+{
+  GabbleRoster *roster;
+  TpHandle handle;
+  GabbleRosterItem *item;
 };
 
 static void channel_manager_iface_init (gpointer, gpointer);
@@ -128,6 +140,7 @@ static void gabble_roster_set_property (GObject *object, guint property_id,
 static void gabble_roster_get_property (GObject *object, guint property_id,
     GValue *value, GParamSpec *pspec);
 
+static void roster_item_cancel_flicker_timeout (GabbleRosterItem *item);
 static void _gabble_roster_item_free (GabbleRosterItem *item);
 static void item_edit_free (GabbleRosterItemEdit *edits);
 static void gabble_roster_close_all (GabbleRoster *roster);
@@ -137,8 +150,6 @@ G_DEFINE_TYPE_WITH_CODE (GabbleRoster, gabble_roster, G_TYPE_OBJECT,
       channel_manager_iface_init);
     G_IMPLEMENT_INTERFACE (GABBLE_TYPE_CAPS_CHANNEL_MANAGER, NULL));
 
-#define GABBLE_ROSTER_GET_PRIVATE(o) ((o)->priv)
-
 static void
 gabble_roster_class_init (GabbleRosterClass *gabble_roster_class)
 {
@@ -197,7 +208,7 @@ void
 gabble_roster_dispose (GObject *object)
 {
   GabbleRoster *self = GABBLE_ROSTER (object);
-  GabbleRosterPrivate *priv = GABBLE_ROSTER_GET_PRIVATE (self);
+  GabbleRosterPrivate *priv = self->priv;
 
   if (priv->dispose_has_run)
     return;
@@ -233,7 +244,7 @@ void
 gabble_roster_finalize (GObject *object)
 {
   GabbleRoster *self = GABBLE_ROSTER (object);
-  GabbleRosterPrivate *priv = GABBLE_ROSTER_GET_PRIVATE (self);
+  GabbleRosterPrivate *priv = self->priv;
 
   DEBUG ("called with %p", object);
 
@@ -250,7 +261,7 @@ gabble_roster_get_property (GObject    *object,
                             GParamSpec *pspec)
 {
   GabbleRoster *roster = GABBLE_ROSTER (object);
-  GabbleRosterPrivate *priv = GABBLE_ROSTER_GET_PRIVATE (roster);
+  GabbleRosterPrivate *priv = roster->priv;
 
   switch (property_id) {
     case PROP_CONNECTION:
@@ -269,7 +280,7 @@ gabble_roster_set_property (GObject     *object,
                             GParamSpec   *pspec)
 {
   GabbleRoster *roster = GABBLE_ROSTER (object);
-  GabbleRosterPrivate *priv = GABBLE_ROSTER_GET_PRIVATE (roster);
+  GabbleRosterPrivate *priv = roster->priv;
 
   switch (property_id) {
     case PROP_CONNECTION:
@@ -289,6 +300,10 @@ _gabble_roster_item_free (GabbleRosterItem *item)
   tp_handle_set_destroy (item->groups);
   item_edit_free (item->unsent_edits);
   g_free (item->name);
+  g_free (item->alias_for);
+
+  roster_item_cancel_flicker_timeout (item);
+
   g_slice_free (GabbleRosterItem, item);
 }
 
@@ -342,23 +357,25 @@ _parse_item_subscription (LmMessageNode *item_node)
 static TpHandleSet *
 _parse_item_groups (LmMessageNode *item_node, TpBaseConnection *conn)
 {
-  LmMessageNode *group_node;
   TpHandleRepoIface *group_repo = tp_base_connection_get_handles (
       conn, TP_HANDLE_TYPE_GROUP);
   TpHandleSet *groups = tp_handle_set_new (group_repo);
   TpHandle handle;
+  NodeIter i;
 
-  for (group_node = item_node->children;
-      NULL != group_node;
-      group_node = group_node->next)
+  for (i = node_iter (item_node); i; i = node_iter_next (i))
     {
+      LmMessageNode *group_node = node_iter_data (i);
+      const gchar *value;
+
       if (0 != strcmp (group_node->name, "group"))
         continue;
 
-      if (NULL == group_node->value)
+      value = lm_message_node_get_value (group_node);
+      if (NULL == value)
         continue;
 
-      handle = tp_handle_ensure (group_repo, group_node->value, NULL, NULL);
+      handle = tp_handle_ensure (group_repo, value, NULL, NULL);
       if (!handle)
         continue;
       tp_handle_set_add (groups, handle);
@@ -398,7 +415,8 @@ _parse_google_item_type (LmMessageNode *item_node)
 
   g_assert (item_node != NULL);
 
-  google_type = lm_message_node_get_attribute (item_node, "gr:t");
+  google_type = lm_message_node_get_attribute_with_namespace (item_node, "t",
+      NS_GOOGLE_ROSTER);
 
   if (NULL == google_type)
     return GOOGLE_ITEM_TYPE_NORMAL;
@@ -414,23 +432,23 @@ _parse_google_item_type (LmMessageNode *item_node)
   return GOOGLE_ITEM_TYPE_NORMAL;
 }
 
-static gboolean
-_google_roster_item_should_keep (LmMessageNode *item_node,
-                                 GabbleRosterItem *item)
+static gchar *
+_extract_google_alias_for (LmMessageNode *item_node)
 {
-  const gchar *attr;
+  return g_strdup (lm_message_node_get_attribute_with_namespace (item_node,
+        "alias-for", NS_GOOGLE_ROSTER));
+}
 
-  /* skip automatically subscribed Google roster items */
-  attr = lm_message_node_get_attribute (item_node, "gr:autosub");
-
-  if (!tp_strdiff (attr, "true"))
-    return FALSE;
-
-  /* skip email addresses that replied to an invite */
-  attr = lm_message_node_get_attribute (item_node, "gr:alias-for");
-
-  if (attr != NULL)
-    return FALSE;
+static gboolean
+_google_roster_item_should_keep (const gchar *jid,
+    GabbleRosterItem *item)
+{
+  /* hide hidden items */
+  if (item->google_type == GOOGLE_ITEM_TYPE_HIDDEN)
+    {
+      DEBUG ("hiding %s: gr:t='H'", jid);
+      return FALSE;
+    }
 
   /* allow items that we've requested a subscription from */
   if (item->ask_subscribe)
@@ -440,6 +458,7 @@ _google_roster_item_should_keep (LmMessageNode *item_node,
     return TRUE;
 
   /* discard anything else */
+  DEBUG ("hiding %s: no subscription", jid);
   return FALSE;
 }
 
@@ -447,7 +466,7 @@ static GabbleRosterItem *
 _gabble_roster_item_get (GabbleRoster *roster,
                          TpHandle handle)
 {
-  GabbleRosterPrivate *priv = GABBLE_ROSTER_GET_PRIVATE (roster);
+  GabbleRosterPrivate *priv = roster->priv;
   TpHandleRepoIface *contact_repo = tp_base_connection_get_handles (
       (TpBaseConnection *) priv->conn, TP_HANDLE_TYPE_CONTACT);
   TpHandleRepoIface *group_repo = tp_base_connection_get_handles (
@@ -489,7 +508,7 @@ static void
 _gabble_roster_item_remove (GabbleRoster *roster,
                             TpHandle handle)
 {
-  GabbleRosterPrivate *priv = GABBLE_ROSTER_GET_PRIVATE (roster);
+  GabbleRosterPrivate *priv = roster->priv;
   TpHandleRepoIface *contact_repo = tp_base_connection_get_handles (
       (TpBaseConnection *) priv->conn, TP_HANDLE_TYPE_CONTACT);
 
@@ -583,7 +602,7 @@ _gabble_roster_item_update (GabbleRoster *roster,
                             GHashTable *group_updates,
                             gboolean google_roster_mode)
 {
-  GabbleRosterPrivate *priv = GABBLE_ROSTER_GET_PRIVATE (roster);
+  GabbleRosterPrivate *priv = roster->priv;
   GabbleRosterItem *item;
   const gchar *ask, *name;
   TpIntSet *new_groups, *added_to, *removed_from, *removed_from2;
@@ -614,22 +633,8 @@ _gabble_roster_item_update (GabbleRoster *roster,
   if (google_roster_mode)
     {
       item->google_type = _parse_google_item_type (node);
-
-      /* discard roster item if strange, just hide it if it's hidden */
-      if (item->google_type == GOOGLE_ITEM_TYPE_HIDDEN)
-        {
-          DEBUG ("Google roster: caching hidden contact %d (%s)",
-              contact_handle,
-              lm_message_node_get_attribute (node, "jid"));
-          item->subscription = GABBLE_ROSTER_SUBSCRIPTION_NONE;
-        }
-      else if (!_google_roster_item_should_keep (node, item))
-        {
-          DEBUG ("Google roster: discarding odd contact %d (%s)",
-              contact_handle,
-              lm_message_node_get_attribute (node, "jid"));
-          item->subscription = GABBLE_ROSTER_SUBSCRIPTION_REMOVE;
-        }
+      g_free (item->alias_for);
+      item->alias_for = _extract_google_alias_for (node);
     }
 
   if (item->subscription == GABBLE_ROSTER_SUBSCRIPTION_REMOVE)
@@ -718,7 +723,7 @@ _gabble_roster_message_new (GabbleRoster *roster,
                             LmMessageSubType sub_type,
                             LmMessageNode **query_return)
 {
-  GabbleRosterPrivate *priv = GABBLE_ROSTER_GET_PRIVATE (roster);
+  GabbleRosterPrivate *priv = roster->priv;
   LmMessage *message;
   LmMessageNode *query_node;
 
@@ -781,7 +786,7 @@ _gabble_roster_item_to_message (GabbleRoster *roster,
                                 LmMessageNode **item_return,
                                 GabbleRosterItem *item)
 {
-  GabbleRosterPrivate *priv = GABBLE_ROSTER_GET_PRIVATE (roster);
+  GabbleRosterPrivate *priv = roster->priv;
   TpHandleRepoIface *contact_repo = tp_base_connection_get_handles (
       (TpBaseConnection *) priv->conn, TP_HANDLE_TYPE_CONTACT);
   LmMessage *message;
@@ -846,7 +851,7 @@ static void
 gabble_roster_emit_new_channel (GabbleRoster *self,
                                 GabbleRosterChannel *channel)
 {
-  GabbleRosterPrivate *priv = GABBLE_ROSTER_GET_PRIVATE (self);
+  GabbleRosterPrivate *priv = self->priv;
   GSList *requests_satisfied;
 
   requests_satisfied = g_hash_table_lookup (priv->queued_requests, channel);
@@ -899,7 +904,7 @@ _gabble_roster_create_channel (GabbleRoster *roster,
                                TpHandle handle,
                                gpointer request_token)
 {
-  GabbleRosterPrivate *priv = GABBLE_ROSTER_GET_PRIVATE (roster);
+  GabbleRosterPrivate *priv = roster->priv;
   TpBaseConnection *conn = (TpBaseConnection *) priv->conn;
   TpHandleRepoIface *handle_repo = tp_base_connection_get_handles (conn,
       handle_type);
@@ -973,7 +978,7 @@ _gabble_roster_get_channel (GabbleRoster *roster,
                             gboolean *created,
                             gpointer request_token)
 {
-  GabbleRosterPrivate *priv = GABBLE_ROSTER_GET_PRIVATE (roster);
+  GabbleRosterPrivate *priv = roster->priv;
   TpHandleRepoIface *handle_repo = tp_base_connection_get_handles (
       (TpBaseConnection *) priv->conn, handle_type);
   GabbleRosterChannel *chan;
@@ -1022,7 +1027,7 @@ _gabble_roster_emit_one (gpointer key,
   GabbleRoster *roster = data_struct->roster;
   GabbleRosterChannel *chan = GABBLE_ROSTER_CHANNEL (value);
 #ifdef ENABLE_DEBUG
-  GabbleRosterPrivate *priv = GABBLE_ROSTER_GET_PRIVATE (roster);
+  GabbleRosterPrivate *priv = roster->priv;
   TpHandleRepoIface *handle_repo = tp_base_connection_get_handles (
       (TpBaseConnection *) priv->conn, data_struct->handle_type);
   TpHandle handle = GPOINTER_TO_UINT (key);
@@ -1043,7 +1048,7 @@ _gabble_roster_emit_one (gpointer key,
 static void
 _gabble_roster_received (GabbleRoster *roster)
 {
-  GabbleRosterPrivate *priv = GABBLE_ROSTER_GET_PRIVATE (roster);
+  GabbleRosterPrivate *priv = roster->priv;
 
   g_assert (priv->list_channels != NULL);
 
@@ -1098,6 +1103,116 @@ _update_group (gpointer key,
   return TRUE;
 }
 
+static FlickerPreventionCtx *
+flicker_prevention_ctx_new (GabbleRoster *roster,
+    TpHandle handle,
+    GabbleRosterItem *item)
+{
+  FlickerPreventionCtx *ret = g_slice_new (FlickerPreventionCtx);
+
+  /* Not taking a ref to the roster. The context is owned by the Item (well, by
+   * its timeout) which is owned by the roster.
+   */
+  ret->roster = roster;
+  /* Not taking a ref to the handle; we borrow the roster's ref, which is
+   * released after the GabbleRosterItem is freed, at which point this context
+   * will be destroyed.
+   */
+  ret->handle = handle;
+  ret->item = item;
+
+  return ret;
+}
+
+static void
+flicker_prevention_ctx_free (gpointer ctx_)
+{
+  FlickerPreventionCtx *ctx = ctx_;
+
+  ctx->roster = NULL;
+  ctx->handle = 0;
+  ctx->item = NULL;
+
+  g_slice_free (FlickerPreventionCtx, ctx);
+}
+
+/* As described in roster/test-google-roster.py, we work around a Google Talk
+ * server bug to avoid contacts flickering off and onto
+ * subscribe:remote-pending when you try to subscribe to someone's presence.
+ *
+ * When we see a roster item with subscription=none/from and ask=subscribe:
+ *  * if no call to this function is scheduled, we schedule a call
+ *  * if one is already scheduled, we cancel it.
+ *
+ * When we see a roster item with subscription=none/from and no ask=subscribe:
+ *  * if a call to this timeout is scheduled, do nothing, in case the contact
+ *    flickers back to ask=subscribe before this fires;
+ *  * if a call to this timeout is not scheduled, remove the contact from the
+ *    subscribe list.
+ *
+ * This way, our subscription being cancelled or our subscription requests
+ * being rescinded will show up on the subscribe list, albeit with a slight lag
+ * in certain situations in case we're just seeing the Google talk server bug.
+ */
+static gboolean
+flicker_prevention_timeout (gpointer ctx_)
+{
+  FlickerPreventionCtx *ctx = ctx_;
+  GabbleRosterItem *item = ctx->item;
+
+  DEBUG ("called for %u", ctx->handle);
+
+  if (item->subscription == GABBLE_ROSTER_SUBSCRIPTION_NONE
+      && !item->ask_subscribe)
+    {
+      GabbleRosterChannel *sub_chan = _gabble_roster_get_channel (ctx->roster,
+          TP_HANDLE_TYPE_LIST, GABBLE_LIST_HANDLE_SUBSCRIBE, NULL, NULL);
+      TpIntSet *rem = tp_intset_new_containing (ctx->handle);
+
+      DEBUG ("removing %u from subscribe", ctx->handle);
+      tp_group_mixin_change_members ((GObject *) sub_chan, "", NULL, rem, NULL,
+          NULL, 0, 0);
+
+      tp_intset_destroy (rem);
+    }
+  else
+    {
+      DEBUG ("subscription=%s and ask_subscribe=%s, nothing to do",
+          _subscription_to_string (item->subscription),
+          item->ask_subscribe ? "true" : "false");
+    }
+
+  ctx->item->flicker_prevention_id = 0;
+
+  return FALSE;
+}
+
+static void
+roster_item_ensure_flicker_timeout (GabbleRoster *roster,
+    TpHandle handle,
+    GabbleRosterItem *item)
+{
+  if (item->flicker_prevention_id == 0)
+    {
+      FlickerPreventionCtx *ctx = flicker_prevention_ctx_new (roster, handle,
+          item);
+
+      item->flicker_prevention_id = g_timeout_add_seconds_full (
+          G_PRIORITY_DEFAULT, 1, flicker_prevention_timeout, ctx,
+          flicker_prevention_ctx_free);
+    }
+}
+
+static void
+roster_item_cancel_flicker_timeout (GabbleRosterItem *item)
+{
+  if (item->flicker_prevention_id != 0)
+    {
+      g_source_remove (item->flicker_prevention_id);
+      item->flicker_prevention_id = 0;
+    }
+}
+
 /**
  * gabble_roster_iq_cb
  *
@@ -1106,13 +1221,10 @@ _update_group (gpointer key,
  * if queries other than rosters are received.
  */
 static LmHandlerResult
-gabble_roster_iq_cb (LmMessageHandler *handler,
-                     LmConnection *lmconn,
-                     LmMessage *message,
-                     gpointer user_data)
+got_roster_iq (GabbleRoster *roster,
+    LmMessage *message)
 {
-  GabbleRoster *roster = GABBLE_ROSTER (user_data);
-  GabbleRosterPrivate *priv = GABBLE_ROSTER_GET_PRIVATE (roster);
+  GabbleRosterPrivate *priv = roster->priv;
   TpBaseConnection *conn = (TpBaseConnection *) priv->conn;
   TpHandleRepoIface *contact_repo = tp_base_connection_get_handles (conn,
       TP_HANDLE_TYPE_CONTACT);
@@ -1121,8 +1233,6 @@ gabble_roster_iq_cb (LmMessageHandler *handler,
   const gchar *from;
   gboolean google_roster = FALSE;
 
-  g_assert (lmconn == priv->conn->lmconn);
-
   if (priv->list_channels == NULL)
     return LM_HANDLER_RESULT_ALLOW_MORE_HANDLERS;
 
@@ -1153,7 +1263,8 @@ gabble_roster_iq_cb (LmMessageHandler *handler,
     {
       const char *gr_ext;
 
-      gr_ext = lm_message_node_get_attribute (query_node, "gr:ext");
+      gr_ext = lm_message_node_get_attribute_with_namespace (query_node, "ext",
+          NS_GOOGLE_ROSTER);
 
       if (!tp_strdiff (gr_ext, GOOGLE_ROSTER_VERSION))
         google_roster = TRUE;
@@ -1165,10 +1276,9 @@ gabble_roster_iq_cb (LmMessageHandler *handler,
    * it's a roster push. either way, parse the items. */
   switch (sub_type)
     {
-      LmMessageNode *item_node;
       TpIntSet *pub_add, *pub_rem,
                *sub_add, *sub_rem, *sub_rp,
-               *known_add, *known_rem,
+               *stored_add, *stored_rem,
                *deny_add, *deny_rem;
       TpHandleSet *referenced_handles;
       GArray *removed;
@@ -1176,6 +1286,7 @@ gabble_roster_iq_cb (LmMessageHandler *handler,
       GabbleRosterChannel *pub_chan, *sub_chan, *chan;
       GHashTable *group_update_table;
       guint i;
+      NodeIter j;
 
     case LM_MESSAGE_SUB_TYPE_RESULT:
     case LM_MESSAGE_SUB_TYPE_SET:
@@ -1186,8 +1297,8 @@ gabble_roster_iq_cb (LmMessageHandler *handler,
       sub_add = tp_intset_new ();
       sub_rem = tp_intset_new ();
       sub_rp = tp_intset_new ();
-      known_add = tp_intset_new ();
-      known_rem = tp_intset_new ();
+      stored_add = tp_intset_new ();
+      stored_rem = tp_intset_new ();
       group_update_table = g_hash_table_new_full (NULL, NULL, NULL,
           (GDestroyNotify)_group_mem_update_destroy);
       removed = g_array_new (FALSE, FALSE, sizeof (TpHandle));
@@ -1211,12 +1322,11 @@ gabble_roster_iq_cb (LmMessageHandler *handler,
           GABBLE_LIST_HANDLE_SUBSCRIBE, NULL, NULL);
 
       /* iterate every sub-node, which we expect to be <item>s */
-      for (item_node = query_node->children;
-           item_node;
-           item_node = item_node->next)
+      for (j = node_iter (query_node); j; j = node_iter_next (j))
         {
           const char *jid;
           GabbleRosterItem *item;
+          LmMessageNode *item_node = node_iter_data (j);
 
           if (strcmp (item_node->name, "item"))
             {
@@ -1266,7 +1376,10 @@ gabble_roster_iq_cb (LmMessageHandler *handler,
             {
             case GABBLE_ROSTER_SUBSCRIPTION_FROM:
             case GABBLE_ROSTER_SUBSCRIPTION_BOTH:
-              tp_intset_add (pub_add, handle);
+              if (google_roster && !_google_roster_item_should_keep (jid, item))
+                tp_intset_add (pub_rem, handle);
+              else
+                tp_intset_add (pub_add, handle);
               break;
             case GABBLE_ROSTER_SUBSCRIPTION_NONE:
             case GABBLE_ROSTER_SUBSCRIPTION_TO:
@@ -1290,7 +1403,13 @@ gabble_roster_iq_cb (LmMessageHandler *handler,
             {
             case GABBLE_ROSTER_SUBSCRIPTION_TO:
             case GABBLE_ROSTER_SUBSCRIPTION_BOTH:
-              tp_intset_add (sub_add, handle);
+              if (google_roster && !_google_roster_item_should_keep (jid, item))
+                tp_intset_add (sub_rem, handle);
+              else
+                tp_intset_add (sub_add, handle);
+
+              roster_item_cancel_flicker_timeout (item);
+
               break;
             case GABBLE_ROSTER_SUBSCRIPTION_NONE:
             case GABBLE_ROSTER_SUBSCRIPTION_FROM:
@@ -1303,49 +1422,55 @@ gabble_roster_iq_cb (LmMessageHandler *handler,
                     }
                   else
                     {
+                      if (item->flicker_prevention_id == 0)
+                        roster_item_ensure_flicker_timeout (roster, handle, item);
+                      else
+                        roster_item_cancel_flicker_timeout (item);
+
                       tp_intset_add (sub_rp, handle);
                     }
                 }
+              else if (item->flicker_prevention_id == 0)
+                {
+                  /* We're not expecting this contact's ask=subscribe to
+                   * flicker off and on again, so let's remove them immediately.
+                   */
+                  tp_intset_add (sub_rem, handle);
+                }
               else
                 {
-                  /* don't remove remote pending, presence="unsubscribed"
-                   * does that */
-                  if (!tp_handle_set_is_member (sub_chan->group.remote_pending,
-                        handle))
-                    {
-                      tp_intset_add (sub_rem, handle);
-                    }
+                  DEBUG ("delaying removal of %s from pending", jid);
                 }
               break;
             case GABBLE_ROSTER_SUBSCRIPTION_REMOVE:
-              if (!tp_handle_set_is_member (sub_chan->group.remote_pending,
-                    handle))
-                {
-                  tp_intset_add (sub_rem, handle);
-                }
+              tp_intset_add (sub_rem, handle);
               break;
             default:
               g_assert_not_reached ();
             }
 
-          /* handle known list changes */
+          /* handle stored list changes */
           switch (item->subscription)
             {
             case GABBLE_ROSTER_SUBSCRIPTION_NONE:
             case GABBLE_ROSTER_SUBSCRIPTION_TO:
             case GABBLE_ROSTER_SUBSCRIPTION_FROM:
             case GABBLE_ROSTER_SUBSCRIPTION_BOTH:
-              if (item->google_type == GOOGLE_ITEM_TYPE_HIDDEN)
-                  tp_intset_add (known_rem, handle);
+              if (google_roster &&
+                  /* Don't hide contacts from stored if they're remote pending.
+                   * This works around Google Talk flickering ask="subscribe"
+                   * when you try to subscribe to someone; see
+                   * test-google-roster.py.
+                   */
+                  !tp_handle_set_is_member (sub_chan->group.remote_pending,
+                      handle) &&
+                  !_google_roster_item_should_keep (jid, item))
+                tp_intset_add (stored_rem, handle);
               else
-                  tp_intset_add (known_add, handle);
+                tp_intset_add (stored_add, handle);
               break;
             case GABBLE_ROSTER_SUBSCRIPTION_REMOVE:
-              if (!tp_handle_set_is_member (sub_chan->group.remote_pending,
-                    handle))
-                {
-                  tp_intset_add (known_rem, handle);
-                }
+              tp_intset_add (stored_rem, handle);
               break;
             default:
               g_assert_not_reached ();
@@ -1389,13 +1514,13 @@ gabble_roster_iq_cb (LmMessageHandler *handler,
       tp_group_mixin_change_members ((GObject *) sub_chan,
             "", sub_add, sub_rem, NULL, sub_rp, 0, 0);
 
-      handle = GABBLE_LIST_HANDLE_KNOWN;
+      handle = GABBLE_LIST_HANDLE_STORED;
       chan = _gabble_roster_get_channel (roster, TP_HANDLE_TYPE_LIST, handle,
           NULL, NULL);
 
-      DEBUG ("calling change members on known channel");
+      DEBUG ("calling change members on stored channel");
       tp_group_mixin_change_members ((GObject *) chan,
-            "", known_add, known_rem, NULL, NULL, 0, 0);
+            "", stored_add, stored_rem, NULL, NULL, 0, 0);
 
       DEBUG ("calling change members on any group channels");
       g_hash_table_foreach_remove (group_update_table, _update_group, roster);
@@ -1423,8 +1548,8 @@ gabble_roster_iq_cb (LmMessageHandler *handler,
       tp_intset_destroy (sub_add);
       tp_intset_destroy (sub_rem);
       tp_intset_destroy (sub_rp);
-      tp_intset_destroy (known_add);
-      tp_intset_destroy (known_rem);
+      tp_intset_destroy (stored_add);
+      tp_intset_destroy (stored_rem);
       g_array_free (removed, TRUE);
       g_hash_table_destroy (group_update_table);
       tp_handle_set_destroy (referenced_handles);
@@ -1453,6 +1578,19 @@ gabble_roster_iq_cb (LmMessageHandler *handler,
   return LM_HANDLER_RESULT_REMOVE_MESSAGE;
 }
 
+static LmHandlerResult
+gabble_roster_iq_cb (LmMessageHandler *handler,
+                     LmConnection *lmconn,
+                     LmMessage *message,
+                     gpointer user_data)
+{
+  GabbleRoster *roster = GABBLE_ROSTER (user_data);
+  GabbleRosterPrivate *priv = roster->priv;
+
+  g_assert (lmconn == priv->conn->lmconn);
+
+  return got_roster_iq (roster, message);
+}
 
 static void
 _gabble_roster_send_presence_ack (GabbleRoster *roster,
@@ -1460,7 +1598,7 @@ _gabble_roster_send_presence_ack (GabbleRoster *roster,
                                   LmMessageSubType sub_type,
                                   gboolean changed)
 {
-  GabbleRosterPrivate *priv = GABBLE_ROSTER_GET_PRIVATE (roster);
+  GabbleRosterPrivate *priv = roster->priv;
   LmMessage *reply;
 
   if (!changed)
@@ -1511,7 +1649,7 @@ gabble_roster_presence_cb (LmMessageHandler *handler,
                            gpointer user_data)
 {
   GabbleRoster *roster = GABBLE_ROSTER (user_data);
-  GabbleRosterPrivate *priv = GABBLE_ROSTER_GET_PRIVATE (roster);
+  GabbleRosterPrivate *priv = roster->priv;
   TpBaseConnection *conn = (TpBaseConnection *) priv->conn;
   TpHandleRepoIface *contact_repo = tp_base_connection_get_handles (conn,
       TP_HANDLE_TYPE_CONTACT);
@@ -1677,7 +1815,7 @@ cancel_queued_requests (gpointer k,
 static void
 gabble_roster_close_all (GabbleRoster *self)
 {
-  GabbleRosterPrivate *priv = GABBLE_ROSTER_GET_PRIVATE (self);
+  GabbleRosterPrivate *priv = self->priv;
 
   DEBUG ("closing channels");
 
@@ -1730,6 +1868,17 @@ gabble_roster_close_all (GabbleRoster *self)
     }
 }
 
+static LmHandlerResult
+roster_received_cb (GabbleConnection *conn,
+    LmMessage *sent_msg,
+    LmMessage *reply_msg,
+    GObject *roster_obj,
+    gpointer user_data)
+{
+  GabbleRoster *roster = GABBLE_ROSTER (user_data);
+
+  return got_roster_iq (roster, reply_msg);
+}
 
 static void
 connection_status_changed_cb (GabbleConnection *conn,
@@ -1766,7 +1915,8 @@ connection_status_changed_cb (GabbleConnection *conn,
 
           message = _gabble_roster_message_new (self, LM_MESSAGE_SUB_TYPE_GET,
               NULL);
-          _gabble_connection_send (self->priv->conn, message, NULL);
+          _gabble_connection_send_with_reply (self->priv->conn, message,
+              roster_received_cb, G_OBJECT (self), self, NULL);
           lm_message_unref (message);
         }
       break;
@@ -1815,7 +1965,7 @@ gabble_roster_foreach_channel (TpChannelManager *manager,
                                gpointer data)
 {
   GabbleRoster *roster = GABBLE_ROSTER (manager);
-  GabbleRosterPrivate *priv = GABBLE_ROSTER_GET_PRIVATE (roster);
+  GabbleRosterPrivate *priv = roster->priv;
   struct foreach_data foreach;
 
   foreach.func = func;
@@ -1833,7 +1983,7 @@ gabble_roster_associate_request (GabbleRoster *self,
                                  GabbleRosterChannel *channel,
                                  gpointer request)
 {
-  GabbleRosterPrivate *priv = GABBLE_ROSTER_GET_PRIVATE (self);
+  GabbleRosterPrivate *priv = self->priv;
   GSList *list = g_hash_table_lookup (priv->queued_requests, channel);
 
   g_hash_table_steal (priv->queued_requests, channel);
@@ -1895,7 +2045,7 @@ roster_item_apply_edits (GabbleRoster *roster,
   gboolean altered = FALSE, ret;
   GabbleRosterItem edited_item;
   TpIntSet *intset;
-  GabbleRosterPrivate *priv = GABBLE_ROSTER_GET_PRIVATE (roster);
+  GabbleRosterPrivate *priv = roster->priv;
   TpHandleRepoIface *group_repo = tp_base_connection_get_handles (
       (TpBaseConnection *) priv->conn, TP_HANDLE_TYPE_GROUP);
   GabbleRosterItemEdit *edits = item->unsent_edits;
@@ -2074,7 +2224,7 @@ GabbleRosterSubscription
 gabble_roster_handle_get_subscription (GabbleRoster *roster,
                                        TpHandle handle)
 {
-  GabbleRosterPrivate *priv = GABBLE_ROSTER_GET_PRIVATE (roster);
+  GabbleRosterPrivate *priv = roster->priv;
   TpHandleRepoIface *contact_repo = tp_base_connection_get_handles (
       (TpBaseConnection *) priv->conn, TP_HANDLE_TYPE_CONTACT);
   GabbleRosterItem *item;
@@ -2099,7 +2249,7 @@ gabble_roster_handle_set_blocked (GabbleRoster *roster,
                                   gboolean blocked,
                                   GError **error)
 {
-  GabbleRosterPrivate *priv = GABBLE_ROSTER_GET_PRIVATE (roster);
+  GabbleRosterPrivate *priv = roster->priv;
   TpHandleRepoIface *contact_repo = tp_base_connection_get_handles (
       (TpBaseConnection *) priv->conn, TP_HANDLE_TYPE_CONTACT);
   GabbleRosterItem *item;
@@ -2168,7 +2318,7 @@ gboolean
 gabble_roster_handle_has_entry (GabbleRoster *roster,
                                 TpHandle handle)
 {
-  GabbleRosterPrivate *priv = GABBLE_ROSTER_GET_PRIVATE (roster);
+  GabbleRosterPrivate *priv = roster->priv;
   TpHandleRepoIface *contact_repo = tp_base_connection_get_handles (
       (TpBaseConnection *) priv->conn, TP_HANDLE_TYPE_CONTACT);
   GabbleRosterItem *item;
@@ -2187,7 +2337,7 @@ const gchar *
 gabble_roster_handle_get_name (GabbleRoster *roster,
                                TpHandle handle)
 {
-  GabbleRosterPrivate *priv = GABBLE_ROSTER_GET_PRIVATE (roster);
+  GabbleRosterPrivate *priv = roster->priv;
   TpHandleRepoIface *contact_repo = tp_base_connection_get_handles (
       (TpBaseConnection *) priv->conn, TP_HANDLE_TYPE_CONTACT);
   GabbleRosterItem *item;
@@ -2211,7 +2361,7 @@ gabble_roster_handle_set_name (GabbleRoster *roster,
                                const gchar *name,
                                GError **error)
 {
-  GabbleRosterPrivate *priv = GABBLE_ROSTER_GET_PRIVATE (roster);
+  GabbleRosterPrivate *priv = roster->priv;
   TpHandleRepoIface *contact_repo = tp_base_connection_get_handles (
       (TpBaseConnection *) priv->conn, TP_HANDLE_TYPE_CONTACT);
   GabbleRosterItem *item;
@@ -2266,7 +2416,7 @@ gabble_roster_handle_remove (GabbleRoster *roster,
                              TpHandle handle,
                              GError **error)
 {
-  GabbleRosterPrivate *priv = GABBLE_ROSTER_GET_PRIVATE (roster);
+  GabbleRosterPrivate *priv = roster->priv;
   TpHandleRepoIface *contact_repo = tp_base_connection_get_handles (
       (TpBaseConnection *) priv->conn, TP_HANDLE_TYPE_CONTACT);
   GabbleRosterItem *item;
@@ -2319,7 +2469,7 @@ gabble_roster_handle_add (GabbleRoster *roster,
                           TpHandle handle,
                           GError **error)
 {
-  GabbleRosterPrivate *priv = GABBLE_ROSTER_GET_PRIVATE (roster);
+  GabbleRosterPrivate *priv = roster->priv;
   TpHandleRepoIface *contact_repo = tp_base_connection_get_handles (
       (TpBaseConnection *) priv->conn, TP_HANDLE_TYPE_CONTACT);
   GabbleRosterItem *item;
@@ -2379,7 +2529,7 @@ gabble_roster_handle_add_to_group (GabbleRoster *roster,
                                    TpHandle group,
                                    GError **error)
 {
-  GabbleRosterPrivate *priv = GABBLE_ROSTER_GET_PRIVATE (roster);
+  GabbleRosterPrivate *priv = roster->priv;
   TpHandleRepoIface *contact_repo = tp_base_connection_get_handles (
       (TpBaseConnection *) priv->conn, TP_HANDLE_TYPE_CONTACT);
   TpHandleRepoIface *group_repo = tp_base_connection_get_handles (
@@ -2441,7 +2591,7 @@ gabble_roster_handle_remove_from_group (GabbleRoster *roster,
                                         TpHandle group,
                                         GError **error)
 {
-  GabbleRosterPrivate *priv = GABBLE_ROSTER_GET_PRIVATE (roster);
+  GabbleRosterPrivate *priv = roster->priv;
   TpHandleRepoIface *contact_repo = tp_base_connection_get_handles (
       (TpBaseConnection *) priv->conn, TP_HANDLE_TYPE_CONTACT);
   TpHandleRepoIface *group_repo = tp_base_connection_get_handles (
@@ -2612,8 +2762,7 @@ gabble_roster_request (GabbleRoster *self,
       handle == GABBLE_LIST_HANDLE_DENY &&
       !(self->priv->conn->features & GABBLE_CONNECTION_FEATURES_GOOGLE_ROSTER))
     {
-      /* FIXME: should this be NotImplemented? */
-      g_set_error (&error, TP_ERRORS, TP_ERROR_NOT_AVAILABLE,
+      g_set_error (&error, TP_ERRORS, TP_ERROR_NOT_IMPLEMENTED,
           "This server does not have Google roster extensions, so there's "
           "no deny list");
       goto error;
