@@ -72,9 +72,9 @@ G_DEFINE_TYPE_WITH_CODE (GabbleTubeDBus, gabble_tube_dbus, G_TYPE_OBJECT,
       tp_dbus_properties_mixin_iface_init);
     G_IMPLEMENT_INTERFACE (TP_TYPE_SVC_CHANNEL, channel_iface_init);
     G_IMPLEMENT_INTERFACE (GABBLE_TYPE_TUBE_IFACE, tube_iface_init);
-    G_IMPLEMENT_INTERFACE (GABBLE_TYPE_SVC_CHANNEL_TYPE_DBUS_TUBE,
+    G_IMPLEMENT_INTERFACE (TP_TYPE_SVC_CHANNEL_TYPE_DBUS_TUBE,
       dbustube_iface_init);
-    G_IMPLEMENT_INTERFACE (GABBLE_TYPE_SVC_CHANNEL_INTERFACE_TUBE,
+    G_IMPLEMENT_INTERFACE (TP_TYPE_SVC_CHANNEL_INTERFACE_TUBE,
       NULL);
     G_IMPLEMENT_INTERFACE (TP_TYPE_SVC_CHANNEL_INTERFACE_GROUP,
       tp_external_group_mixin_iface_init);
@@ -86,14 +86,14 @@ static const gchar *gabble_tube_dbus_interfaces[] = {
     /* If more interfaces are added, either keep Group as the first, or change
      * the implementations of gabble_tube_dbus_get_interfaces () and
      * gabble_tube_dbus_get_property () too */
-    GABBLE_IFACE_CHANNEL_INTERFACE_TUBE,
+    TP_IFACE_CHANNEL_INTERFACE_TUBE,
     NULL
 };
 
 static const gchar * const gabble_tube_dbus_channel_allowed_properties[] = {
     TP_IFACE_CHANNEL ".TargetHandle",
     TP_IFACE_CHANNEL ".TargetID",
-    GABBLE_IFACE_CHANNEL_TYPE_DBUS_TUBE ".ServiceName",
+    TP_IFACE_CHANNEL_TYPE_DBUS_TUBE ".ServiceName",
     NULL
 };
 
@@ -135,6 +135,7 @@ enum
   PROP_TARGET_ID,
   PROP_INITIATOR_ID,
   PROP_MUC,
+  PROP_SUPPORTED_ACCESS_CONTROLS,
   LAST_PROPERTY
 };
 
@@ -152,6 +153,9 @@ struct _GabbleTubeDBusPrivate
   gchar *service;
   GHashTable *parameters;
   GabbleMucChannel *muc;
+  TpSocketAccessControl access_control;
+  /* GArray of guint */
+  GArray *supported_access_controls;
 
   /* For outgoing tubes, TRUE if the offer has been sent over the network. For
    * incoming tubes, always TRUE.
@@ -240,7 +244,8 @@ filter_cb (DBusConnection *conn,
 
   if (priv->dbus_local_name != NULL)
     {
-      dbus_message_set_sender (msg, priv->dbus_local_name);
+      if (!dbus_message_set_sender (msg, priv->dbus_local_name))
+        DEBUG ("dbus_message_set_sender failed");
     }
 
   if (!dbus_message_marshal (msg, &marshalled, &len))
@@ -283,6 +288,14 @@ out:
   return DBUS_HANDLER_RESULT_HANDLED;
 }
 
+static dbus_bool_t
+allow_all_connections (DBusConnection *conn,
+                       unsigned long uid,
+                       void *data)
+{
+  return TRUE;
+}
+
 static void
 new_connection_cb (DBusServer *server,
                    DBusConnection *conn,
@@ -305,6 +318,14 @@ new_connection_cb (DBusServer *server,
   dbus_connection_add_filter (conn, filter_cb, tube, NULL);
   priv->dbus_conn = conn;
 
+  if (priv->access_control == TP_SOCKET_ACCESS_CONTROL_LOCALHOST)
+    {
+      /* By default libdbus use Credentials access control. If user wants
+       * to use the Localhost access control, we need to bypass this check. */
+      dbus_connection_set_unix_user_function (conn, allow_all_connections,
+          NULL, NULL);
+    }
+
   /* We may have received messages to deliver before the local connection is
    * established. Theses messages are kept in the dbus_msg_queue list and are
    * delivered as soon as we get the connection. */
@@ -455,32 +476,32 @@ unref_handle_foreach (gpointer key,
   tp_handle_unref (contact_repo, handle);
 }
 
-static GabbleTubeChannelState
+static TpTubeChannelState
 get_tube_state (GabbleTubeDBus *self)
 {
   GabbleTubeDBusPrivate *priv = GABBLE_TUBE_DBUS_GET_PRIVATE (self);
   GabbleBytestreamState bytestream_state;
 
   if (!priv->offered)
-    return GABBLE_TUBE_CHANNEL_STATE_NOT_OFFERED;
+    return TP_TUBE_CHANNEL_STATE_NOT_OFFERED;
 
   if (priv->bytestream == NULL)
     /* bytestream not yet created as we're waiting for the SI reply */
-    return GABBLE_TUBE_CHANNEL_STATE_REMOTE_PENDING;
+    return TP_TUBE_CHANNEL_STATE_REMOTE_PENDING;
 
   g_object_get (priv->bytestream, "state", &bytestream_state, NULL);
 
   switch (bytestream_state)
     {
       case GABBLE_BYTESTREAM_STATE_OPEN:
-        return GABBLE_TUBE_CHANNEL_STATE_OPEN;
+        return TP_TUBE_CHANNEL_STATE_OPEN;
         break;
       case GABBLE_BYTESTREAM_STATE_LOCAL_PENDING:
       case GABBLE_BYTESTREAM_STATE_ACCEPTED:
-        return GABBLE_TUBE_CHANNEL_STATE_LOCAL_PENDING;
+        return TP_TUBE_CHANNEL_STATE_LOCAL_PENDING;
         break;
       case GABBLE_BYTESTREAM_STATE_INITIATING:
-        return GABBLE_TUBE_CHANNEL_STATE_REMOTE_PENDING;
+        return TP_TUBE_CHANNEL_STATE_REMOTE_PENDING;
         break;
       default:
         g_return_val_if_reached (0);
@@ -509,8 +530,8 @@ bytestream_state_changed_cb (GabbleBytestreamIface *bytestream,
     {
       tube_dbus_open (self);
 
-      gabble_svc_channel_interface_tube_emit_tube_channel_state_changed (self,
-          GABBLE_TUBE_CHANNEL_STATE_OPEN);
+      tp_svc_channel_interface_tube_emit_tube_channel_state_changed (self,
+          TP_TUBE_CHANNEL_STATE_OPEN);
 
       g_signal_emit (G_OBJECT (self), signals[OPENED], 0);
     }
@@ -607,6 +628,7 @@ gabble_tube_dbus_finalize (GObject *object)
   g_free (priv->stream_id);
   g_free (priv->service);
   g_hash_table_destroy (priv->parameters);
+  g_array_free (priv->supported_access_controls, TRUE);
 
   if (priv->muc != NULL)
     {
@@ -633,7 +655,7 @@ gabble_tube_dbus_get_property (GObject *object,
         break;
       case PROP_CHANNEL_TYPE:
         g_value_set_static_string (value,
-            GABBLE_IFACE_CHANNEL_TYPE_DBUS_TUBE);
+            TP_IFACE_CHANNEL_TYPE_DBUS_TUBE);
         break;
       case PROP_CONNECTION:
         g_value_set_object (value, priv->conn);
@@ -708,7 +730,8 @@ gabble_tube_dbus_get_property (GObject *object,
               TP_IFACE_CHANNEL, "InitiatorID",
               TP_IFACE_CHANNEL, "Requested",
               TP_IFACE_CHANNEL, "Interfaces",
-              GABBLE_IFACE_CHANNEL_TYPE_DBUS_TUBE, "ServiceName",
+              TP_IFACE_CHANNEL_TYPE_DBUS_TUBE, "ServiceName",
+              TP_IFACE_CHANNEL_TYPE_DBUS_TUBE, "SupportedAccessControls",
               NULL);
 
           if (priv->initiator != priv->self_handle)
@@ -719,12 +742,12 @@ gabble_tube_dbus_get_property (GObject *object,
               /* FIXME: use tp_dbus_properties_mixin_add_properties once it's
                * added in tp-glib */
               tp_dbus_properties_mixin_get (object,
-                  GABBLE_IFACE_CHANNEL_INTERFACE_TUBE, "Parameters",
+                  TP_IFACE_CHANNEL_INTERFACE_TUBE, "Parameters",
                   prop_value, NULL);
               g_assert (G_IS_VALUE (prop_value));
 
               g_hash_table_insert (properties,
-                  g_strdup_printf ("%s.%s", GABBLE_IFACE_CHANNEL_INTERFACE_TUBE,
+                  g_strdup_printf ("%s.%s", TP_IFACE_CHANNEL_INTERFACE_TUBE,
                     "Parameters"), prop_value);
             }
 
@@ -760,6 +783,9 @@ gabble_tube_dbus_get_property (GObject *object,
       case PROP_MUC:
         g_value_set_object (value, priv->muc);
         break;
+      case PROP_SUPPORTED_ACCESS_CONTROLS:
+        g_value_set_boxed (value, priv->supported_access_controls);
+        break;
       default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
         break;
@@ -854,6 +880,7 @@ gabble_tube_dbus_constructor (GType type,
   DBusGConnection *bus;
   TpHandleRepoIface *contact_repo;
   TpBaseConnection *base;
+  guint access_control;
 
   obj = G_OBJECT_CLASS (gabble_tube_dbus_parent_class)->
     constructor (type, n_props, props);
@@ -936,6 +963,18 @@ gabble_tube_dbus_constructor (GType type,
       priv->offered = TRUE;
     }
 
+  /* default access control is Credentials as that's the one used by the old
+   * tube API */
+  priv->access_control = TP_SOCKET_ACCESS_CONTROL_CREDENTIALS;
+
+  /* Set SupportedAccessesControl */
+  priv->supported_access_controls = g_array_sized_new (FALSE, FALSE,
+      sizeof (guint), 1);
+  access_control = TP_SOCKET_ACCESS_CONTROL_CREDENTIALS;
+  g_array_append_val (priv->supported_access_controls, access_control);
+  access_control = TP_SOCKET_ACCESS_CONTROL_LOCALHOST;
+  g_array_append_val (priv->supported_access_controls, access_control);
+
   return obj;
 }
 
@@ -956,6 +995,7 @@ gabble_tube_dbus_class_init (GabbleTubeDBusClass *gabble_tube_dbus_class)
   static TpDBusPropertiesMixinPropImpl dbus_tube_props[] = {
       { "ServiceName", "service", NULL },
       { "DBusNames", "dbus-names", NULL },
+      { "SupportedAccessControls", "supported-access-controls", NULL },
       { NULL }
   };
   static TpDBusPropertiesMixinPropImpl tube_iface_props[] = {
@@ -969,12 +1009,12 @@ gabble_tube_dbus_class_init (GabbleTubeDBusClass *gabble_tube_dbus_class)
         NULL,
         channel_props,
       },
-      { GABBLE_IFACE_CHANNEL_TYPE_DBUS_TUBE,
+      { TP_IFACE_CHANNEL_TYPE_DBUS_TUBE,
         tp_dbus_properties_mixin_getter_gobject_properties,
         NULL,
         dbus_tube_props,
       },
-      { GABBLE_IFACE_CHANNEL_INTERFACE_TUBE,
+      { TP_IFACE_CHANNEL_INTERFACE_TUBE,
         tp_dbus_properties_mixin_getter_gobject_properties,
         NULL,
         tube_iface_props,
@@ -1071,7 +1111,7 @@ gabble_tube_dbus_class_init (GabbleTubeDBusClass *gabble_tube_dbus_class)
       "dbus-names",
       "D-Bus names",
       "Mapping of contact handles to D-Bus names (used for muc tubes only).",
-      GABBLE_HASH_TYPE_DBUS_TUBE_PARTICIPANTS,
+      TP_HASH_TYPE_DBUS_TUBE_PARTICIPANTS,
       G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
   g_object_class_install_property (object_class, PROP_DBUS_NAMES, param_spec);
 
@@ -1108,6 +1148,15 @@ gabble_tube_dbus_class_init (GabbleTubeDBusClass *gabble_tube_dbus_class)
 
   g_object_class_install_property (object_class, PROP_MUC, param_spec);
 
+  param_spec = g_param_spec_boxed (
+      "supported-access-controls",
+      "Supported access-controls",
+      "GArray containing supported access controls.",
+      DBUS_TYPE_G_UINT_ARRAY,
+      G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
+  g_object_class_install_property (object_class,
+      PROP_SUPPORTED_ACCESS_CONTROLS, param_spec);
+
   signals[OPENED] =
     g_signal_new ("tube-opened",
                   G_OBJECT_CLASS_TYPE (gabble_tube_dbus_class),
@@ -1241,8 +1290,8 @@ gabble_tube_dbus_offer (GabbleTubeDBus *tube,
       if (!result)
         return FALSE;
 
-      gabble_svc_channel_interface_tube_emit_tube_channel_state_changed (tube,
-          GABBLE_TUBE_CHANNEL_STATE_REMOTE_PENDING);
+      tp_svc_channel_interface_tube_emit_tube_channel_state_changed (tube,
+          TP_TUBE_CHANNEL_STATE_REMOTE_PENDING);
 
     }
   else
@@ -1659,7 +1708,7 @@ gabble_tube_dbus_add_name (GabbleTubeDBus *self,
 
   g_hash_table_insert (added, GUINT_TO_POINTER (handle), (gchar *) name);
 
-  gabble_svc_channel_type_dbus_tube_emit_d_bus_names_changed (self, added,
+  tp_svc_channel_type_dbus_tube_emit_dbus_names_changed (self, added,
       removed);
 
   g_hash_table_destroy (added);
@@ -1697,7 +1746,7 @@ gabble_tube_dbus_remove_name (GabbleTubeDBus *self,
 
   g_array_append_val (removed, handle);
 
-  gabble_svc_channel_type_dbus_tube_emit_d_bus_names_changed (self, added,
+  tp_svc_channel_type_dbus_tube_emit_dbus_names_changed (self, added,
       removed);
 
   g_hash_table_destroy (added);
@@ -1794,7 +1843,7 @@ gabble_tube_dbus_get_channel_type (TpSvcChannel *iface,
                                        DBusGMethodInvocation *context)
 {
   tp_svc_channel_return_from_get_channel_type (context,
-      GABBLE_IFACE_CHANNEL_TYPE_DBUS_TUBE);
+      TP_IFACE_CHANNEL_TYPE_DBUS_TUBE);
 }
 
 /**
@@ -1840,25 +1889,56 @@ gabble_tube_dbus_get_interfaces (TpSvcChannel *iface,
     }
 }
 
+static gboolean
+gabble_tube_dbus_check_access_control (GabbleTubeDBus *self,
+    guint access_control,
+    GError **error)
+{
+  switch (access_control)
+    {
+      case TP_SOCKET_ACCESS_CONTROL_CREDENTIALS:
+      case TP_SOCKET_ACCESS_CONTROL_LOCALHOST:
+        break;
+
+      default:
+        g_set_error (error, TP_ERRORS, TP_ERROR_INVALID_ARGUMENT,
+            "%u socket access control is not supported", access_control);
+        return FALSE;
+    }
+
+  return TRUE;
+}
+
 /**
- * gabble_tube_dbus_offer_d_bus_tube
+ * gabble_tube_dbus_offer_async
  *
- * Implemnets D-Bus method OfferDBusTube on interface
+ * Implemnets D-Bus method Offer on interface
  * org.freedesktop.Telepathy.Channel.Type.DBusTube
  */
 static void
-gabble_tube_dbus_offer_d_bus_tube (GabbleSvcChannelTypeDBusTube *self,
-                                   GHashTable *parameters,
-                                   DBusGMethodInvocation *context)
+gabble_tube_dbus_offer_async (TpSvcChannelTypeDBusTube *self,
+    GHashTable *parameters,
+    guint access_control,
+    DBusGMethodInvocation *context)
 {
   GabbleTubeDBus *tube = GABBLE_TUBE_DBUS (self);
+  GabbleTubeDBusPrivate *priv = GABBLE_TUBE_DBUS_GET_PRIVATE (tube);
   GError *error = NULL;
 
+  if (!gabble_tube_dbus_check_access_control (tube, access_control, &error))
+    {
+      dbus_g_method_return_error (context, error);
+      g_error_free (error);
+      return;
+    }
+
+  priv->access_control = access_control;
+
   g_object_set (self, "parameters", parameters, NULL);
 
   if (gabble_tube_dbus_offer (tube, &error))
     {
-      gabble_svc_channel_type_dbus_tube_return_from_offer_d_bus_tube (context,
+      tp_svc_channel_type_dbus_tube_return_from_offer (context,
           tube->priv->dbus_srv_addr);
     }
   else
@@ -1870,21 +1950,29 @@ gabble_tube_dbus_offer_d_bus_tube (GabbleSvcChannelTypeDBusTube *self,
 }
 
 /**
- * gabble_tube_dbus_accept_d_bus_tube
+ * gabble_tube_dbus_accept_async
  *
- * Implements D-Bus method AcceptDBusTube on interface
+ * Implements D-Bus method Accept on interface
  * org.freedesktop.Telepathy.Channel.Type.DBusTube
  */
 static void
-gabble_tube_dbus_accept_d_bus_tube (GabbleSvcChannelTypeDBusTube *self,
-                                    DBusGMethodInvocation *context)
+gabble_tube_dbus_accept_async (TpSvcChannelTypeDBusTube *self,
+    guint access_control,
+    DBusGMethodInvocation *context)
 {
   GabbleTubeDBus *tube = GABBLE_TUBE_DBUS (self);
   GError *error = NULL;
 
+  if (!gabble_tube_dbus_check_access_control (tube, access_control, &error))
+    {
+      dbus_g_method_return_error (context, error);
+      g_error_free (error);
+      return;
+    }
+
   if (gabble_tube_dbus_accept (GABBLE_TUBE_IFACE (tube), &error))
     {
-      gabble_svc_channel_type_dbus_tube_return_from_accept_d_bus_tube (context,
+      tp_svc_channel_type_dbus_tube_return_from_accept (context,
           tube->priv->dbus_srv_addr);
       ;
     }
@@ -1932,12 +2020,12 @@ static void
 dbustube_iface_init (gpointer g_iface,
                      gpointer iface_data)
 {
-  GabbleSvcChannelTypeDBusTubeClass *klass =
-      (GabbleSvcChannelTypeDBusTubeClass *) g_iface;
+  TpSvcChannelTypeDBusTubeClass *klass =
+      (TpSvcChannelTypeDBusTubeClass *) g_iface;
 
-#define IMPLEMENT(x) gabble_svc_channel_type_dbus_tube_implement_##x (\
-    klass, gabble_tube_dbus_##x)
-  IMPLEMENT(offer_d_bus_tube);
-  IMPLEMENT(accept_d_bus_tube);
+#define IMPLEMENT(x, suffix) tp_svc_channel_type_dbus_tube_implement_##x (\
+    klass, gabble_tube_dbus_##x##suffix)
+  IMPLEMENT(offer,_async);
+  IMPLEMENT(accept,_async);
 #undef IMPLEMENT
 }
