@@ -49,6 +49,7 @@
 #include <gibber/gibber-tcp-transport.h>
 #include <gibber/gibber-transport.h>
 #include <gibber/gibber-unix-transport.h>
+#include <gibber/gibber-util.h>
 
 #include "bytestream-factory.h"
 #include "bytestream-iface.h"
@@ -72,9 +73,9 @@ G_DEFINE_TYPE_WITH_CODE (GabbleTubeStream, gabble_tube_stream, G_TYPE_OBJECT,
       tp_dbus_properties_mixin_iface_init);
     G_IMPLEMENT_INTERFACE (TP_TYPE_SVC_CHANNEL, channel_iface_init);
     G_IMPLEMENT_INTERFACE (GABBLE_TYPE_TUBE_IFACE, tube_iface_init);
-    G_IMPLEMENT_INTERFACE (GABBLE_TYPE_SVC_CHANNEL_TYPE_STREAM_TUBE,
+    G_IMPLEMENT_INTERFACE (TP_TYPE_SVC_CHANNEL_TYPE_STREAM_TUBE,
       streamtube_iface_init);
-    G_IMPLEMENT_INTERFACE (GABBLE_TYPE_SVC_CHANNEL_INTERFACE_TUBE,
+    G_IMPLEMENT_INTERFACE (TP_TYPE_SVC_CHANNEL_INTERFACE_TUBE,
       NULL);
     G_IMPLEMENT_INTERFACE (TP_TYPE_SVC_CHANNEL_INTERFACE_GROUP,
       tp_external_group_mixin_iface_init);
@@ -84,7 +85,7 @@ G_DEFINE_TYPE_WITH_CODE (GabbleTubeStream, gabble_tube_stream, G_TYPE_OBJECT,
 static const gchar * const gabble_tube_stream_channel_allowed_properties[] = {
     TP_IFACE_CHANNEL ".TargetHandle",
     TP_IFACE_CHANNEL ".TargetID",
-    GABBLE_IFACE_CHANNEL_TYPE_STREAM_TUBE ".Service",
+    TP_IFACE_CHANNEL_TYPE_STREAM_TUBE ".Service",
     NULL
 };
 
@@ -93,7 +94,7 @@ static const gchar *gabble_tube_stream_interfaces[] = {
     /* If more interfaces are added, either keep Group as the first, or change
      * the implementations of gabble_tube_stream_get_interfaces () and
      * gabble_tube_stream_get_property () too */
-    GABBLE_IFACE_CHANNEL_INTERFACE_TUBE,
+    TP_IFACE_CHANNEL_INTERFACE_TUBE,
     NULL
 };
 
@@ -175,14 +176,18 @@ struct _GabbleTubeStreamPrivate
 
   /* (GibberTransport *) -> (GabbleBytestreamIface *)
    *
-   * The (t->b) is inserted when the bytestream is open.
+   * The (t->b) is also inserted as soon as they are created.
    */
   GHashTable *transport_to_bytestream;
 
+  /* (GibberTransport *) -> guint */
+  GHashTable *transport_to_id;
+  guint last_connection_id;
+
   TpHandle initiator;
   gchar *service;
   GHashTable *parameters;
-  GabbleTubeChannelState state;
+  TpTubeChannelState state;
 
   TpSocketAddressType address_type;
   GValue *address;
@@ -200,8 +205,16 @@ struct _GabbleTubeStreamPrivate
 
 #define GABBLE_TUBE_STREAM_GET_PRIVATE(obj) ((obj)->priv)
 
+typedef struct
+{
+  GabbleTubeStream *self;
+  TpHandle contact;
+} transport_connected_data;
+
 static void data_received_cb (GabbleBytestreamIface *ibb, TpHandle sender,
     GString *data, gpointer user_data);
+static void transport_connected_cb (GibberTransport *transport,
+    transport_connected_data *data);
 
 static void
 generate_ascii_string (guint len,
@@ -240,6 +253,31 @@ transport_handler (GibberTransport *transport,
       (const gchar *) data->data);
 }
 
+static void
+fire_connection_closed (GabbleTubeStream *self,
+    GibberTransport *transport,
+    const gchar *error,
+    const gchar *debug_msg)
+{
+  GabbleTubeStreamPrivate *priv = GABBLE_TUBE_STREAM_GET_PRIVATE (self);
+  guint connection_id;
+
+  connection_id = GPOINTER_TO_UINT (g_hash_table_lookup (priv->transport_to_id,
+        transport));
+  if (connection_id == 0)
+    {
+      DEBUG ("ConnectionClosed has already been fired for this connection");
+      return;
+    }
+
+  /* remove the ID so we are sure we won't fire ConnectionClosed twice for the
+   * same connection. */
+  g_hash_table_remove (priv->transport_to_id, transport);
+
+  tp_svc_channel_type_stream_tube_emit_connection_closed (self,
+      connection_id, error, debug_msg);
+}
+
 static void
 transport_disconnected_cb (GibberTransport *transport,
                            GabbleTubeStream *self)
@@ -247,6 +285,9 @@ transport_disconnected_cb (GibberTransport *transport,
   GabbleTubeStreamPrivate *priv = GABBLE_TUBE_STREAM_GET_PRIVATE (self);
   GabbleBytestreamIface *bytestream;
 
+  fire_connection_closed (self, transport, TP_ERROR_STR_CANCELLED,
+      "local socket has been disconnected");
+
   bytestream = g_hash_table_lookup (priv->transport_to_bytestream, transport);
   if (bytestream == NULL)
     return;
@@ -266,14 +307,20 @@ remove_transport (GabbleTubeStream *self,
   DEBUG ("disconnect and remove transport");
   g_signal_handlers_disconnect_matched (transport, G_SIGNAL_MATCH_DATA,
       0, 0, NULL, NULL, self);
+  /* The callback on the "connected" signal doesn't match the above
+   * disconnection as it receives a transport_connected_data as user_data
+   * and not the self pointer. */
+  g_signal_handlers_disconnect_matched (transport, G_SIGNAL_MATCH_FUNC,
+      0, 0, NULL, G_CALLBACK (transport_connected_cb), NULL);
 
   gibber_transport_disconnect (transport);
 
-  /* the transport may not be in transport_to_bytestream if the bytestream was
-   * not fully open */
-  g_hash_table_remove (priv->transport_to_bytestream, transport);
+  fire_connection_closed (self, transport, TP_ERROR_STR_CONNECTION_LOST,
+      "bytestream has been broken");
 
+  g_hash_table_remove (priv->transport_to_bytestream, transport);
   g_hash_table_remove (priv->bytestream_to_transport, bytestream);
+  g_hash_table_remove (priv->transport_to_id, transport);
 }
 
 static void
@@ -305,13 +352,8 @@ add_transport (GabbleTubeStream *self,
                GibberTransport *transport,
                GabbleBytestreamIface *bytestream)
 {
-  GabbleTubeStreamPrivate *priv = GABBLE_TUBE_STREAM_GET_PRIVATE (self);
-
   gibber_transport_set_handler (transport, transport_handler, self);
 
-  g_hash_table_insert (priv->transport_to_bytestream,
-      g_object_ref (transport), g_object_ref (bytestream));
-
   g_signal_connect (transport, "disconnected",
       G_CALLBACK (transport_disconnected_cb), self);
   g_signal_connect (transport, "buffer-empty",
@@ -409,6 +451,9 @@ extra_bytestream_negotiate_cb (GabbleBytestreamIface *bytestream,
     {
       DEBUG ("initiator refused new bytestream");
 
+      fire_connection_closed (self, transport,
+          TP_ERROR_STR_CONNECTION_REFUSED, "connection has been refused");
+
       g_object_unref (transport);
       return;
     }
@@ -416,8 +461,13 @@ extra_bytestream_negotiate_cb (GabbleBytestreamIface *bytestream,
   DEBUG ("extra bytestream accepted");
 
   /* transport has been refed in start_stream_initiation () */
+  g_assert (gibber_transport_get_state (transport) ==
+      GIBBER_TRANSPORT_CONNECTED);
   g_hash_table_insert (priv->bytestream_to_transport, g_object_ref (bytestream),
       transport);
+  g_hash_table_insert (priv->transport_to_bytestream,
+      g_object_ref (transport), g_object_ref (bytestream));
+
 
   g_signal_connect (bytestream, "state-changed",
                 G_CALLBACK (extra_bytestream_state_changed_cb), self);
@@ -526,6 +576,175 @@ start_stream_initiation (GabbleTubeStream *self,
   return result;
 }
 
+static guint
+generate_connection_id (GabbleTubeStream *self,
+                        GibberTransport *transport)
+{
+  GabbleTubeStreamPrivate *priv = GABBLE_TUBE_STREAM_GET_PRIVATE (self);
+
+  priv->last_connection_id++;
+
+  g_hash_table_insert (priv->transport_to_id, transport,
+      GUINT_TO_POINTER (priv->last_connection_id));
+
+  return priv->last_connection_id;
+}
+
+static void
+fire_new_local_connection (GabbleTubeStream *self,
+    GibberTransport *transport)
+{
+  guint connection_id;
+
+  connection_id = generate_connection_id (self, transport);
+
+  tp_svc_channel_type_stream_tube_emit_new_local_connection (self,
+      connection_id);
+}
+
+static void
+credentials_received_cb (GibberUnixTransport *transport,
+                         GibberBuffer *buffer,
+                         GibberCredentials *credentials,
+                         GError *error,
+                         gpointer user_data)
+{
+  GabbleTubeStream *self = GABBLE_TUBE_STREAM (user_data);
+
+  /* Credentials received; reblock the transport */
+  gibber_transport_block_receiving (GIBBER_TRANSPORT (transport), TRUE);
+
+  if (error != NULL)
+    {
+      DEBUG ("Didn't receive credentials (%s). Closing transport",
+          error->message);
+      goto credentials_received_cb_out;
+    }
+
+  g_assert (credentials != NULL);
+
+  if (buffer->length != 1)
+    {
+      DEBUG ("Got more than one byte (%" G_GSIZE_FORMAT "). Rejecting",
+          buffer->length);
+      goto credentials_received_cb_out;
+    }
+
+  if (credentials->uid != getuid ())
+    {
+      DEBUG ("Wrong uid (%u). Rejecting", credentials->uid);
+      goto credentials_received_cb_out;
+    }
+
+  DEBUG ("Connection properly authentificated");
+
+  if (!start_stream_initiation (self, GIBBER_TRANSPORT (transport), NULL))
+    {
+      DEBUG ("SI failed. Closing connection");
+    }
+  else
+    {
+      fire_new_local_connection (self, GIBBER_TRANSPORT (transport));
+    }
+
+credentials_received_cb_out:
+  /* start_stream_initiation reffed the transport if everything went fine */
+  g_object_unref (transport);
+}
+
+static gboolean
+check_incoming_connection (GabbleTubeStream *self,
+                           GibberTransport *transport)
+{
+  GabbleTubeStreamPrivate *priv = GABBLE_TUBE_STREAM_GET_PRIVATE (self);
+
+  if (priv->access_control == TP_SOCKET_ACCESS_CONTROL_LOCALHOST)
+    {
+      return TRUE;
+    }
+  else if (priv->access_control == TP_SOCKET_ACCESS_CONTROL_CREDENTIALS)
+    {
+      if (!gibber_unix_transport_recv_credentials (
+            GIBBER_UNIX_TRANSPORT (transport), credentials_received_cb, self))
+        {
+          DEBUG ("Can't receive credentials. Closing transport");
+          return FALSE;
+        }
+
+      /* Temporarly unblock the transport to be able to receive credentials */
+      gibber_transport_block_receiving (transport, FALSE);
+
+      /* We ref the transport so it won't be destroyed by GibberListener */
+      g_object_ref (transport);
+
+      /* Returns FALSE as we are waiting for credentials so SI can't be
+       * started yet. */
+      return FALSE;
+    }
+  else if (priv->access_control == TP_SOCKET_ACCESS_CONTROL_PORT)
+    {
+      struct sockaddr_storage addr;
+      socklen_t len = sizeof (addr);
+      int ret;
+      char peer_host[NI_MAXHOST];
+      char peer_port[NI_MAXSERV];
+      guint port;
+      gchar *host;
+      gchar *tmp;
+
+      if (!gibber_transport_get_peeraddr (transport, &addr, &len))
+        {
+          DEBUG ("gibber_transport_get_peeraddr failed");
+          return FALSE;
+        }
+
+      gibber_normalize_address (&addr);
+
+      g_assert (addr.ss_family == AF_INET || addr.ss_family == AF_INET6);
+
+      ret = getnameinfo ((struct sockaddr *) &addr, len,
+          peer_host, NI_MAXHOST, peer_port, NI_MAXSERV,
+          NI_NUMERICHOST | NI_NUMERICSERV);
+
+      if (ret != 0)
+        {
+          DEBUG ("getnameinfo failed: %s", gai_strerror(ret));
+          return FALSE;
+        }
+
+      dbus_g_type_struct_get (priv->access_control_param,
+          0, &host,
+          1, &port,
+          G_MAXUINT);
+
+      if (tp_strdiff (host, peer_host))
+        {
+          DEBUG ("Wrong ip: %s (%s was expected)", peer_host, host);
+          g_free (host);
+          return FALSE;
+        }
+      g_free (host);
+
+      tmp = g_strdup_printf ("%u", port);
+      if (tp_strdiff (tmp, peer_port))
+        {
+          DEBUG ("Wrong port: %s (%u was expected)", peer_port, port);
+          g_free (tmp);
+          return FALSE;
+        }
+      g_free (tmp);
+
+      return TRUE;
+    }
+  else
+    {
+      /* access_control has already been checked when accepting the tube */
+      g_assert_not_reached ();
+    }
+
+  return FALSE;
+}
+
 /* callback for listening connections from the local application */
 static void
 local_new_connection_cb (GibberListener *listener,
@@ -540,6 +759,13 @@ local_new_connection_cb (GibberListener *listener,
    * its data. */
   gibber_transport_block_receiving (transport, TRUE);
 
+  if (!check_incoming_connection (self, transport))
+    {
+      /* We didn't ref the connection so it will be destroyed by the
+       * GibberListener if needed. */
+      return;
+    }
+
   /* Streams in stream tubes are established with stream initiation (XEP-0095).
    * We use SalutSiBytestreamManager.
    */
@@ -547,11 +773,166 @@ local_new_connection_cb (GibberListener *listener,
     {
       DEBUG ("closing new client connection");
     }
+  else
+    {
+      fire_new_local_connection (self, transport);
+    }
 }
 
 static gboolean
+set_credentials_access_control_param (GValue *access_control_param,
+    GibberTransport *transport)
+{
+  guint8 credentials;
+
+  credentials = g_random_int_range (0, G_MAXUINT8);
+
+  /* The Credentials access control would have be rejected earlier if the
+   * socket type wasn't UNIX. */
+  if (!gibber_unix_transport_send_credentials (
+        GIBBER_UNIX_TRANSPORT (transport), &credentials, sizeof (guint8)))
+    {
+      DEBUG ("send_credentials failed");
+      return FALSE;
+    }
+
+  g_value_init (access_control_param,
+      G_TYPE_UCHAR);
+  g_value_set_uchar (access_control_param, credentials);
+
+  return TRUE;
+}
+
+static gboolean
+set_port_access_control_param (GValue *access_control_param,
+    GibberTransport *transport)
+{
+  struct sockaddr_storage addr;
+  socklen_t addrlen = sizeof (struct sockaddr_storage);
+  char host[NI_MAXHOST];
+  char port_str[NI_MAXSERV];
+  int ret;
+  guint16 port;
+  unsigned long tmp;
+  gchar *endptr;
+
+  if (!gibber_transport_get_sockaddr (transport, &addr, &addrlen))
+    {
+      DEBUG ("Failed to get connection address");
+      return FALSE;
+    }
+
+  ret = getnameinfo ((struct sockaddr *) &addr, addrlen,
+      host, NI_MAXHOST, port_str, NI_MAXSERV,
+      NI_NUMERICHOST | NI_NUMERICSERV);
+  if (ret != 0)
+    {
+      DEBUG ("getnameinfo failed: %s", g_strerror (ret));
+      return FALSE;
+    }
+
+  tmp = strtoul (port_str, &endptr, 10);
+  if (!endptr || *endptr || tmp > G_MAXUINT16)
+    {
+      DEBUG ("invalid port: %s", port_str);
+      return FALSE;
+    }
+  port = (guint16) tmp;
+
+  g_value_init (access_control_param,
+      TP_STRUCT_TYPE_SOCKET_ADDRESS_IPV4);
+  g_value_take_boxed (access_control_param,
+      dbus_g_type_specialized_construct (
+                TP_STRUCT_TYPE_SOCKET_ADDRESS_IPV4));
+
+  dbus_g_type_struct_set (access_control_param,
+      0, host,
+      1, port,
+      G_MAXUINT);
+
+  return TRUE;
+}
+
+static transport_connected_data *
+transport_connected_data_new (GabbleTubeStream *self,
+    TpHandle contact)
+{
+  transport_connected_data *data = g_slice_new (transport_connected_data);
+  data->self = self;
+  data->contact = contact;
+  return data;
+}
+
+static void
+transport_connected_data_free (transport_connected_data *data)
+{
+  g_slice_free (transport_connected_data, data);
+}
+
+static void
+fire_new_remote_connection (GabbleTubeStream *self,
+    GibberTransport *transport,
+    TpHandle contact)
+{
+  GabbleTubeStreamPrivate *priv = GABBLE_TUBE_STREAM_GET_PRIVATE (self);
+  GValue access_control_param = {0,};
+  guint connection_id;
+
+  if (priv->access_control == TP_SOCKET_ACCESS_CONTROL_CREDENTIALS)
+    {
+      if (!set_credentials_access_control_param (&access_control_param,
+            transport))
+        {
+          gibber_transport_disconnect (transport);
+          return;
+        }
+    }
+  else if (priv->access_control == TP_SOCKET_ACCESS_CONTROL_PORT)
+    {
+      if (!set_port_access_control_param (&access_control_param, transport))
+        {
+          gibber_transport_disconnect (transport);
+          return;
+        }
+    }
+  else
+    {
+      /* set a dummy value */
+      g_value_init (&access_control_param, G_TYPE_INT);
+      g_value_set_int (&access_control_param, 0);
+    }
+
+  /* fire NewConnection D-Bus signal */
+
+  connection_id = GPOINTER_TO_UINT (g_hash_table_lookup (priv->transport_to_id,
+        transport));
+  g_assert (connection_id != 0);
+
+  tp_svc_channel_type_stream_tube_emit_new_remote_connection (self,
+      contact, &access_control_param, connection_id);
+  g_value_unset (&access_control_param);
+}
+
+static void
+transport_connected_cb (GibberTransport *transport,
+    transport_connected_data *data)
+{
+  GabbleTubeStreamPrivate *priv = GABBLE_TUBE_STREAM_GET_PRIVATE (data->self);
+  GabbleBytestreamIface *bytestream;
+
+  fire_new_remote_connection (data->self, transport, data->contact);
+
+  bytestream = g_hash_table_lookup (priv->transport_to_bytestream, transport);
+  if (bytestream == NULL)
+    return;
+
+  gabble_bytestream_iface_block_reading (bytestream, FALSE);
+}
+
+static GibberTransport *
 new_connection_to_socket (GabbleTubeStream *self,
-                          GabbleBytestreamIface *bytestream)
+                          GabbleBytestreamIface *bytestream,
+                          TpHandle contact)
 {
   GabbleTubeStreamPrivate *priv = GABBLE_TUBE_STREAM_GET_PRIVATE (self);
   GibberTransport *transport;
@@ -599,14 +980,21 @@ new_connection_to_socket (GabbleTubeStream *self,
    * its data. */
   gibber_transport_block_receiving (transport, TRUE);
 
+  generate_connection_id (self, transport);
+
+  gabble_bytestream_iface_block_reading (bytestream, TRUE);
   g_hash_table_insert (priv->bytestream_to_transport, g_object_ref (bytestream),
       g_object_ref (transport));
+  g_hash_table_insert (priv->transport_to_bytestream,
+      g_object_ref (transport), g_object_ref (bytestream));
+
 
   g_signal_connect (bytestream, "state-changed",
       G_CALLBACK (extra_bytestream_state_changed_cb), self);
 
   g_object_unref (transport);
-  return TRUE;
+
+  return transport;
 }
 
 static gboolean
@@ -659,6 +1047,13 @@ tube_stream_open (GabbleTubeStream *self,
           g_free (path);
           return FALSE;
         }
+
+      if (priv->access_control == TP_SOCKET_ACCESS_CONTROL_LOCALHOST)
+        {
+          /* Everyone can use the socket */
+          chmod (path, 0777);
+        }
+
       g_free (path);
     }
   else if (priv->address_type == TP_SOCKET_ADDRESS_TYPE_IPV4)
@@ -733,6 +1128,10 @@ gabble_tube_stream_init (GabbleTubeStream *self)
       g_direct_equal, (GDestroyNotify) g_object_unref,
       (GDestroyNotify) g_object_unref);
 
+  priv->transport_to_id = g_hash_table_new_full (g_direct_hash,
+      g_direct_equal, NULL, NULL);
+  priv->last_connection_id = 0;
+
   priv->address_type = TP_SOCKET_ADDRESS_TYPE_UNIX;
   priv->address = NULL;
   priv->access_control = TP_SOCKET_ACCESS_CONTROL_LOCALHOST;
@@ -757,11 +1156,18 @@ close_each_extra_bytestream (gpointer key,
    * called */
   g_signal_handlers_disconnect_matched (bytestream, G_SIGNAL_MATCH_DATA,
       0, 0, NULL, NULL, self);
- g_signal_handlers_disconnect_matched (transport, G_SIGNAL_MATCH_DATA,
+  g_signal_handlers_disconnect_matched (transport, G_SIGNAL_MATCH_DATA,
       0, 0, NULL, NULL, self);
+  /* The callback on the "connected" signal doesn't match the above
+   * disconnection as it receives a transport_connected_data as user_data
+   * and not the self pointer. */
+  g_signal_handlers_disconnect_matched (transport, G_SIGNAL_MATCH_FUNC,
+      0, 0, NULL, G_CALLBACK (transport_connected_cb), NULL);
 
   gabble_bytestream_iface_close (bytestream, NULL);
   gibber_transport_disconnect (transport);
+  fire_connection_closed (self, transport, TP_ERROR_STR_CANCELLED,
+      "tube is closing");
 
   g_hash_table_remove (priv->transport_to_bytestream, transport);
 
@@ -812,6 +1218,12 @@ gabble_tube_stream_dispose (GObject *object)
       priv->bytestream_to_transport = NULL;
     }
 
+  if (priv->transport_to_id != NULL)
+    {
+      g_hash_table_destroy (priv->transport_to_id);
+      priv->transport_to_id = NULL;
+    }
+
   tp_handle_unref (contact_repo, priv->initiator);
 
   if (priv->local_listener != NULL)
@@ -873,7 +1285,7 @@ gabble_tube_stream_get_property (GObject *object,
         break;
       case PROP_CHANNEL_TYPE:
         g_value_set_static_string (value,
-            GABBLE_IFACE_CHANNEL_TYPE_STREAM_TUBE);
+            TP_IFACE_CHANNEL_TYPE_STREAM_TUBE);
         break;
       case PROP_CONNECTION:
         g_value_set_object (value, priv->conn);
@@ -945,8 +1357,8 @@ gabble_tube_stream_get_property (GObject *object,
               TP_IFACE_CHANNEL, "InitiatorID",
               TP_IFACE_CHANNEL, "Requested",
               TP_IFACE_CHANNEL, "Interfaces",
-              GABBLE_IFACE_CHANNEL_TYPE_STREAM_TUBE, "Service",
-              GABBLE_IFACE_CHANNEL_TYPE_STREAM_TUBE, "SupportedSocketTypes",
+              TP_IFACE_CHANNEL_TYPE_STREAM_TUBE, "Service",
+              TP_IFACE_CHANNEL_TYPE_STREAM_TUBE, "SupportedSocketTypes",
               NULL);
 
           if (priv->initiator != priv->self_handle)
@@ -957,12 +1369,12 @@ gabble_tube_stream_get_property (GObject *object,
               /* FIXME: use tp_dbus_properties_mixin_add_properties once it's
                * added in tp-glib */
               tp_dbus_properties_mixin_get (object,
-                  GABBLE_IFACE_CHANNEL_INTERFACE_TUBE, "Parameters",
+                  TP_IFACE_CHANNEL_INTERFACE_TUBE, "Parameters",
                   prop_value, NULL);
               g_assert (G_IS_VALUE (prop_value));
 
               g_hash_table_insert (properties,
-                  g_strdup_printf ("%s.%s", GABBLE_IFACE_CHANNEL_INTERFACE_TUBE,
+                  g_strdup_printf ("%s.%s", TP_IFACE_CHANNEL_INTERFACE_TUBE,
                     "Parameters"), prop_value);
             }
 
@@ -1111,11 +1523,11 @@ gabble_tube_stream_constructor (GType type,
   if (priv->initiator == priv->self_handle)
     {
       /* We initiated this tube */
-      priv->state = GABBLE_TUBE_CHANNEL_STATE_NOT_OFFERED;
+      priv->state = TP_TUBE_CHANNEL_STATE_NOT_OFFERED;
     }
   else
     {
-      priv->state = GABBLE_TUBE_CHANNEL_STATE_LOCAL_PENDING;
+      priv->state = TP_TUBE_CHANNEL_STATE_LOCAL_PENDING;
     }
 
   if (priv->handle_type == TP_HANDLE_TYPE_CONTACT)
@@ -1166,12 +1578,12 @@ gabble_tube_stream_class_init (GabbleTubeStreamClass *gabble_tube_stream_class)
         NULL,
         channel_props,
       },
-      { GABBLE_IFACE_CHANNEL_TYPE_STREAM_TUBE,
+      { TP_IFACE_CHANNEL_TYPE_STREAM_TUBE,
         tp_dbus_properties_mixin_getter_gobject_properties,
         NULL,
         stream_tube_props,
       },
-      { GABBLE_IFACE_CHANNEL_INTERFACE_TUBE,
+      { TP_IFACE_CHANNEL_INTERFACE_TUBE,
         tp_dbus_properties_mixin_getter_gobject_properties,
         NULL,
         tube_iface_props,
@@ -1444,7 +1856,7 @@ gabble_tube_stream_accept (GabbleTubeIface *tube,
       goto fail;
     }
 
-  if (priv->state != GABBLE_TUBE_CHANNEL_STATE_LOCAL_PENDING)
+  if (priv->state != TP_TUBE_CHANNEL_STATE_LOCAL_PENDING)
     {
       g_set_error (error, TP_ERRORS, TP_ERROR_INVALID_ARGUMENT,
           "Tube is not in the local pending state");
@@ -1457,10 +1869,10 @@ gabble_tube_stream_accept (GabbleTubeIface *tube,
       goto fail;
     }
 
-  priv->state = GABBLE_TUBE_CHANNEL_STATE_OPEN;
+  priv->state = TP_TUBE_CHANNEL_STATE_OPEN;
 
-  gabble_svc_channel_interface_tube_emit_tube_channel_state_changed (self,
-      GABBLE_TUBE_CHANNEL_STATE_OPEN);
+  tp_svc_channel_interface_tube_emit_tube_channel_state_changed (self,
+      TP_TUBE_CHANNEL_STATE_OPEN);
 
   g_signal_emit (G_OBJECT (self), signals[OPENED], 0);
 
@@ -1555,6 +1967,8 @@ gabble_tube_stream_add_bytestream (GabbleTubeIface *tube,
 {
   GabbleTubeStream *self = GABBLE_TUBE_STREAM (tube);
   GabbleTubeStreamPrivate *priv = GABBLE_TUBE_STREAM_GET_PRIVATE (self);
+  TpHandle contact;
+  GibberTransport *transport;
 
   if (priv->initiator != priv->self_handle)
     {
@@ -1565,18 +1979,19 @@ gabble_tube_stream_add_bytestream (GabbleTubeIface *tube,
       return;
     }
 
-  /* New bytestream, let's connect to the socket */
-  if (new_connection_to_socket (self, bytestream))
-    {
-      TpHandle contact;
+  g_object_get (bytestream, "peer-handle", &contact, NULL);
 
-      if (priv->state == GABBLE_TUBE_CHANNEL_STATE_REMOTE_PENDING)
+  /* New bytestream, let's connect to the socket */
+  transport = new_connection_to_socket (self, bytestream, contact);
+  if (transport != NULL)
+    {
+      if (priv->state == TP_TUBE_CHANNEL_STATE_REMOTE_PENDING)
         {
           DEBUG ("Received first connection. Tube is now open");
-          priv->state = GABBLE_TUBE_CHANNEL_STATE_OPEN;
+          priv->state = TP_TUBE_CHANNEL_STATE_OPEN;
 
-          gabble_svc_channel_interface_tube_emit_tube_channel_state_changed (
-              self, GABBLE_TUBE_CHANNEL_STATE_OPEN);
+          tp_svc_channel_interface_tube_emit_tube_channel_state_changed (
+              self, TP_TUBE_CHANNEL_STATE_OPEN);
 
           g_signal_emit (G_OBJECT (self), signals[OPENED], 0);
         }
@@ -1585,9 +2000,26 @@ gabble_tube_stream_add_bytestream (GabbleTubeIface *tube,
 
       gabble_bytestream_iface_accept (bytestream, augment_si_accept_iq, self);
 
-      g_object_get (bytestream, "peer-handle", &contact, NULL);
-
       g_signal_emit (G_OBJECT (self), signals[NEW_CONNECTION], 0, contact);
+
+      if (gibber_transport_get_state (transport) == GIBBER_TRANSPORT_CONNECTED)
+        {
+          gabble_bytestream_iface_block_reading (bytestream, FALSE);
+          fire_new_remote_connection (self, transport, contact);
+        }
+      else
+        {
+          /* NewConnection will be fired once the transport is connected.
+           * We can't get access_control_param (as the source port for example)
+           * until it's connected. */
+          transport_connected_data *data;
+
+          data = transport_connected_data_new (self, contact);
+
+          g_signal_connect_data (transport, "connected",
+              G_CALLBACK (transport_connected_cb), data,
+              (GClosureNotify) transport_connected_data_free, 0);
+        }
     }
   else
     {
@@ -1666,15 +2098,16 @@ check_unix_params (TpSocketAddressType address_type,
       g_string_free (socket_address, TRUE);
     }
 
-  if (access_control != TP_SOCKET_ACCESS_CONTROL_LOCALHOST)
+  if (access_control == TP_SOCKET_ACCESS_CONTROL_LOCALHOST ||
+      access_control == TP_SOCKET_ACCESS_CONTROL_CREDENTIALS)
   {
-    g_set_error (error, TP_ERRORS, TP_ERROR_INVALID_ARGUMENT,
-        "Only the Localhost access control method is supported for Unix"
-        " sockets");
-    return FALSE;
+    /* no variant associated */
+    return TRUE;
   }
 
-  return TRUE;
+  g_set_error (error, TP_ERRORS, TP_ERROR_INVALID_ARGUMENT,
+      "%u socket access control is not supported", access_control);
+  return FALSE;
 }
 
 static gboolean
@@ -1743,15 +2176,28 @@ check_ip_params (TpSocketAddressType address_type,
       freeaddrinfo (result);
     }
 
-  if (access_control != TP_SOCKET_ACCESS_CONTROL_LOCALHOST)
+  if (access_control == TP_SOCKET_ACCESS_CONTROL_LOCALHOST)
     {
-      g_set_error (error, TP_ERRORS, TP_ERROR_INVALID_ARGUMENT,
-          "%s sockets only support localhost control access",
-          (address_type == TP_SOCKET_ADDRESS_TYPE_IPV4 ? "IPv4" : "IPv6"));
-      return FALSE;
+      return TRUE;
+    }
+  else if (access_control == TP_SOCKET_ACCESS_CONTROL_PORT)
+    {
+      if (access_control_param != NULL)
+        {
+          if (G_VALUE_TYPE (access_control_param) !=
+              TP_STRUCT_TYPE_SOCKET_ADDRESS_IPV4)
+            {
+              g_set_error (error, TP_ERRORS, TP_ERROR_INVALID_ARGUMENT,
+                  "Port access param is supposed to be sq");
+              return FALSE;
+            }
+        }
+      return TRUE;
     }
 
-  return TRUE;
+  g_set_error (error, TP_ERRORS, TP_ERROR_INVALID_ARGUMENT,
+      "%u socket access control is not supported", access_control);
+  return FALSE;
 }
 
 /* used to check access control parameters both for OfferStreamTube and
@@ -1859,7 +2305,7 @@ send_tube_offer (GabbleTubeStream *self,
   result = _gabble_connection_send (priv->conn, msg, error);
   if (result)
     {
-      priv->state = GABBLE_TUBE_CHANNEL_STATE_REMOTE_PENDING;
+      priv->state = TP_TUBE_CHANNEL_STATE_REMOTE_PENDING;
     }
 
   lm_message_unref (msg);
@@ -1873,7 +2319,7 @@ gabble_tube_stream_offer (GabbleTubeStream *self,
 {
   GabbleTubeStreamPrivate *priv = GABBLE_TUBE_STREAM_GET_PRIVATE (self);
 
-  g_assert (priv->state == GABBLE_TUBE_CHANNEL_STATE_NOT_OFFERED);
+  g_assert (priv->state == TP_TUBE_CHANNEL_STATE_NOT_OFFERED);
 
   if (priv->handle_type == TP_HANDLE_TYPE_CONTACT)
     {
@@ -1884,7 +2330,7 @@ gabble_tube_stream_offer (GabbleTubeStream *self,
   else
     {
       /* muc tube is open as soon it's offered */
-      priv->state = GABBLE_TUBE_CHANNEL_STATE_OPEN;
+      priv->state = TP_TUBE_CHANNEL_STATE_OPEN;
       g_signal_emit (G_OBJECT (self), signals[OPENED], 0);
     }
 
@@ -1922,6 +2368,8 @@ gabble_tube_stream_get_supported_socket_types (void)
       1);
   access_control = TP_SOCKET_ACCESS_CONTROL_LOCALHOST;
   g_array_append_val (unix_tab, access_control);
+  access_control = TP_SOCKET_ACCESS_CONTROL_CREDENTIALS;
+  g_array_append_val (unix_tab, access_control);
   g_hash_table_insert (ret, GUINT_TO_POINTER (TP_SOCKET_ADDRESS_TYPE_UNIX),
       unix_tab);
 
@@ -1930,6 +2378,8 @@ gabble_tube_stream_get_supported_socket_types (void)
       1);
   access_control = TP_SOCKET_ACCESS_CONTROL_LOCALHOST;
   g_array_append_val (ipv4_tab, access_control);
+  access_control = TP_SOCKET_ACCESS_CONTROL_PORT;
+  g_array_append_val (ipv4_tab, access_control);
   g_hash_table_insert (ret, GUINT_TO_POINTER (TP_SOCKET_ADDRESS_TYPE_IPV4),
       ipv4_tab);
 
@@ -1938,44 +2388,33 @@ gabble_tube_stream_get_supported_socket_types (void)
       1);
   access_control = TP_SOCKET_ACCESS_CONTROL_LOCALHOST;
   g_array_append_val (ipv6_tab, access_control);
+  access_control = TP_SOCKET_ACCESS_CONTROL_PORT;
+  g_array_append_val (ipv6_tab, access_control);
   g_hash_table_insert (ret, GUINT_TO_POINTER (TP_SOCKET_ADDRESS_TYPE_IPV6),
       ipv6_tab);
 
   return ret;
 }
 
-/* Callback plugged only if the tube has been offered with the new
- * Channel.Type.StreamTube API. */
-static void
-stream_unix_tube_new_connection_cb (GabbleTubeStream *self,
-                                    guint contact,
-                                    gpointer user_data)
-{
-  gabble_svc_channel_type_stream_tube_emit_stream_tube_new_connection (self,
-      contact);
-}
-
-
 /**
- * gabble_tube_stream_offer_stream_tube
+ * gabble_tube_stream_offer_async
  *
- * Implements D-Bus method OfferStreamTube
+ * Implements D-Bus method Offer
  * on org.freedesktop.Telepathy.Channel.Type.StreamTube
  */
 static void
-gabble_tube_stream_offer_stream_tube (GabbleSvcChannelTypeStreamTube *iface,
-                                      guint address_type,
-                                      const GValue *address,
-                                      guint access_control,
-                                      const GValue *access_control_param,
-                                      GHashTable *parameters,
-                                      DBusGMethodInvocation *context)
+gabble_tube_stream_offer_async (TpSvcChannelTypeStreamTube *iface,
+    guint address_type,
+    const GValue *address,
+    guint access_control,
+    GHashTable *parameters,
+    DBusGMethodInvocation *context)
 {
   GabbleTubeStream *self = GABBLE_TUBE_STREAM (iface);
   GabbleTubeStreamPrivate *priv = GABBLE_TUBE_STREAM_GET_PRIVATE (self);
   GError *error = NULL;
 
-  if (priv->state != GABBLE_TUBE_CHANNEL_STATE_NOT_OFFERED)
+  if (priv->state != TP_TUBE_CHANNEL_STATE_NOT_OFFERED)
     {
       g_set_error (&error, TP_ERRORS, TP_ERROR_INVALID_ARGUMENT,
           "Tube is not in the not offered state");
@@ -1985,7 +2424,7 @@ gabble_tube_stream_offer_stream_tube (GabbleSvcChannelTypeStreamTube *iface,
     }
 
   if (!gabble_tube_stream_check_params (address_type, address,
-        access_control, access_control_param, &error))
+        access_control, NULL, &error))
     {
       dbus_g_method_return_error (context, error);
       g_error_free (error);
@@ -2000,8 +2439,6 @@ gabble_tube_stream_offer_stream_tube (GabbleSvcChannelTypeStreamTube *iface,
   priv->address = tp_g_value_slice_dup (address);
   g_assert (priv->access_control == TP_SOCKET_ACCESS_CONTROL_LOCALHOST);
   priv->access_control = access_control;
-  g_assert (priv->access_control_param == NULL);
-  priv->access_control_param = tp_g_value_slice_dup (access_control_param);
 
   g_object_set (self, "parameters", parameters, NULL);
 
@@ -2017,33 +2454,30 @@ gabble_tube_stream_offer_stream_tube (GabbleSvcChannelTypeStreamTube *iface,
 
   if (priv->handle_type == TP_HANDLE_TYPE_CONTACT)
     {
-      gabble_svc_channel_interface_tube_emit_tube_channel_state_changed (
-          self, GABBLE_TUBE_CHANNEL_STATE_REMOTE_PENDING);
+      tp_svc_channel_interface_tube_emit_tube_channel_state_changed (
+          self, TP_TUBE_CHANNEL_STATE_REMOTE_PENDING);
     }
   else
     {
-      gabble_svc_channel_interface_tube_emit_tube_channel_state_changed (
-          self, GABBLE_TUBE_CHANNEL_STATE_OPEN);
+      tp_svc_channel_interface_tube_emit_tube_channel_state_changed (
+          self, TP_TUBE_CHANNEL_STATE_OPEN);
     }
 
-  g_signal_connect (self, "tube-new-connection",
-      G_CALLBACK (stream_unix_tube_new_connection_cb), self);
-
-  gabble_svc_channel_type_stream_tube_return_from_offer_stream_tube (context);
+  tp_svc_channel_type_stream_tube_return_from_offer (context);
 }
 
 /**
- * gabble_tube_stream_accept_stream_tube
+ * gabble_tube_stream_accept_async
  *
- * Implements D-Bus method AcceptStreamTube
+ * Implements D-Bus method Accept
  * on org.freedesktop.Telepathy.Channel.Type.StreamTube
  */
 static void
-gabble_tube_stream_accept_stream_tube (GabbleSvcChannelTypeStreamTube *iface,
-                                       guint address_type,
-                                       guint access_control,
-                                       const GValue *access_control_param,
-                                       DBusGMethodInvocation *context)
+gabble_tube_stream_accept_async (TpSvcChannelTypeStreamTube *iface,
+    guint address_type,
+    guint access_control,
+    const GValue *access_control_param,
+    DBusGMethodInvocation *context)
 {
   GabbleTubeStream *self = GABBLE_TUBE_STREAM (iface);
   GabbleTubeStreamPrivate *priv = GABBLE_TUBE_STREAM_GET_PRIVATE (self);
@@ -2069,7 +2503,7 @@ gabble_tube_stream_accept_stream_tube (GabbleSvcChannelTypeStreamTube *iface,
     gabble_muc_channel_send_presence (self->muc, NULL);
 #endif
 
-  gabble_svc_channel_type_stream_tube_return_from_accept_stream_tube (context,
+  tp_svc_channel_type_stream_tube_return_from_accept (context,
       priv->address);
 }
 
@@ -2098,7 +2532,7 @@ gabble_tube_stream_get_channel_type (TpSvcChannel *iface,
                                        DBusGMethodInvocation *context)
 {
   tp_svc_channel_return_from_get_channel_type (context,
-      GABBLE_IFACE_CHANNEL_TYPE_STREAM_TUBE);
+      TP_IFACE_CHANNEL_TYPE_STREAM_TUBE);
 }
 
 /**
@@ -2180,12 +2614,12 @@ static void
 streamtube_iface_init (gpointer g_iface,
                        gpointer iface_data)
 {
-  GabbleSvcChannelTypeStreamTubeClass *klass =
-      (GabbleSvcChannelTypeStreamTubeClass *) g_iface;
+  TpSvcChannelTypeStreamTubeClass *klass =
+      (TpSvcChannelTypeStreamTubeClass *) g_iface;
 
-#define IMPLEMENT(x) gabble_svc_channel_type_stream_tube_implement_##x (\
-    klass, gabble_tube_stream_##x)
-  IMPLEMENT(offer_stream_tube);
-  IMPLEMENT(accept_stream_tube);
+#define IMPLEMENT(x, suffix) tp_svc_channel_type_stream_tube_implement_##x (\
+    klass, gabble_tube_stream_##x##suffix)
+  IMPLEMENT(offer,_async);
+  IMPLEMENT(accept,_async);
 #undef IMPLEMENT
 }
