@@ -568,7 +568,7 @@ create_new_tube (GabbleTubesChannel *self,
 
   /* The old API doesn't know the "not offered" state, so we have to wait that
    * the tube is offered before announcing it. */
-  if (state != GABBLE_TUBE_CHANNEL_STATE_NOT_OFFERED)
+  if (state != TP_TUBE_CHANNEL_STATE_NOT_OFFERED)
     {
       tp_svc_channel_type_tubes_emit_new_tube (self,
           tube_id,
@@ -735,8 +735,7 @@ foreach_slave (gpointer key,
   TpTubeType type;
 
   g_object_get (tube, "type", &type, NULL);
-  if (type == TP_TUBE_TYPE_STREAM)
-    data->foreach (TP_EXPORTABLE_CHANNEL (tube), data->user_data);
+  data->foreach (TP_EXPORTABLE_CHANNEL (tube), data->user_data);
 }
 
 void gabble_tubes_channel_foreach (GabbleTubesChannel *self,
@@ -809,9 +808,10 @@ gabble_tubes_channel_presence_updated (GabbleTubesChannel *self,
                                        LmMessage *presence)
 {
   GabbleTubesChannelPrivate *priv = GABBLE_TUBES_CHANNEL_GET_PRIVATE (self);
-  LmMessageNode *tubes_node, *tube_node;
+  LmMessageNode *tubes_node;
   TpHandleRepoIface *contact_repo = tp_base_connection_get_handles (
       (TpBaseConnection *) priv->conn, TP_HANDLE_TYPE_CONTACT);
+  NodeIter i;
   const gchar *presence_type;
   GHashTable *old_dbus_tubes;
   struct _add_in_old_dbus_tubes_data add_data;
@@ -843,9 +843,9 @@ gabble_tubes_channel_presence_updated (GabbleTubesChannel *self,
   add_data.contact = contact;
   g_hash_table_foreach (priv->tubes, add_in_old_dbus_tubes, &add_data);
 
-  for (tube_node = tubes_node->children; tube_node != NULL;
-      tube_node = tube_node->next)
+  for (i = node_iter (tubes_node); i; i = node_iter_next (i))
     {
+      LmMessageNode *tube_node = node_iter_data (i);
       const gchar *stream_id;
       GabbleTubeIface *tube;
       guint tube_id;
@@ -968,7 +968,7 @@ copy_tube_in_ptr_array (gpointer key,
   TpHandle initiator;
   gchar *service;
   GHashTable *parameters;
-  GabbleTubeChannelState state;
+  TpTubeChannelState state;
   TpTubeType type;
   GPtrArray *array = (GPtrArray *) user_data;
   GValue entry = {0,};
@@ -980,7 +980,7 @@ copy_tube_in_ptr_array (gpointer key,
   /* The old interface has no way to represent unoffered tubes, so they
    * shouldn't appear in the result of ListTubes()
    */
-  if (state == GABBLE_TUBE_CHANNEL_STATE_NOT_OFFERED)
+  if (state == TP_TUBE_CHANNEL_STATE_NOT_OFFERED)
     return;
 
   g_object_get (tube,
@@ -1506,18 +1506,18 @@ GabbleTubeIface *gabble_tubes_channel_tube_request (GabbleTubesChannel *self,
   channel_type = tp_asv_get_string (request_properties,
             TP_IFACE_CHANNEL ".ChannelType");
 
-  if (! tp_strdiff (channel_type, GABBLE_IFACE_CHANNEL_TYPE_STREAM_TUBE))
+  if (! tp_strdiff (channel_type, TP_IFACE_CHANNEL_TYPE_STREAM_TUBE))
     {
       type = TP_TUBE_TYPE_STREAM;
       service = tp_asv_get_string (request_properties,
-                GABBLE_IFACE_CHANNEL_TYPE_STREAM_TUBE ".Service");
+                TP_IFACE_CHANNEL_TYPE_STREAM_TUBE ".Service");
 
     }
-  else if (! tp_strdiff (channel_type, GABBLE_IFACE_CHANNEL_TYPE_DBUS_TUBE))
+  else if (! tp_strdiff (channel_type, TP_IFACE_CHANNEL_TYPE_DBUS_TUBE))
     {
       type = TP_TUBE_TYPE_DBUS;
       service = tp_asv_get_string (request_properties,
-                GABBLE_IFACE_CHANNEL_TYPE_DBUS_TUBE ".ServiceName");
+                TP_IFACE_CHANNEL_TYPE_DBUS_TUBE ".ServiceName");
     }
   else
     /* This assertion is safe: this function's caller only calls it in one of
