@@ -47,6 +47,8 @@ typedef struct _GabbleJingleFactory GabbleJingleFactory;
 typedef struct _GabbleJingleSession GabbleJingleSession;
 typedef struct _GabbleJingleContent GabbleJingleContent;
 typedef struct _GabbleJingleTransportGoogle GabbleJingleTransportGoogle;
+typedef struct _GabbleJingleTransportRawUdp GabbleJingleTransportRawUdp;
+typedef struct _GabbleJingleTransportIceUdp GabbleJingleTransportIceUdp;
 typedef struct _GabbleJingleMediaRtp GabbleJingleMediaRtp;
 
 typedef struct _JingleCandidate JingleCandidate;
@@ -69,13 +71,17 @@ typedef enum {
     PRESENCE_CAP_BYTESTREAMS = 1 << 7,
     PRESENCE_CAP_IBB = 1 << 8,
     PRESENCE_CAP_SI_TUBES = 1 << 9,
-    PRESENCE_CAP_FILE_TRANSFER = 1 << 10,
     PRESENCE_CAP_OLPC_1 = 1 << 11,
     PRESENCE_CAP_JINGLE_RTP = 1 << 12,
     PRESENCE_CAP_JINGLE032 = 1 << 13,
-    PRESENCE_CAP_JINGLE_TRANSPORT_ICE = 1 << 14,
+    PRESENCE_CAP_JINGLE_TRANSPORT_ICEUDP = 1 << 14,
     PRESENCE_CAP_JINGLE_TRANSPORT_RAWUDP = 1 << 15,
     PRESENCE_CAP_GEOLOCATION = 1 << 16,
+    PRESENCE_CAP_SI_FILE_TRANSFER = 1 << 17,
+    PRESENCE_CAP_JINGLE_OMITS_CONTENT_CREATOR = 1 << 18,
+    PRESENCE_CAP_GOOGLE_VIDEO = 1 << 19,
+    PRESENCE_CAP_JINGLE_RTP_AUDIO = 1 << 20,
+    PRESENCE_CAP_JINGLE_RTP_VIDEO = 1 << 21,
 } GabblePresenceCapabilities;
 
 G_END_DECLS
