@@ -29,6 +29,10 @@
 
 #include <telepathy-glib/handle-repo-dynamic.h>
 
+#ifdef HAVE_UUID
+# include <uuid.h>
+#endif
+
 #define DEBUG_FLAG GABBLE_DEBUG_JID
 
 #include "base64.h"
@@ -70,6 +74,29 @@ sha1_bin (const gchar *bytes,
   g_checksum_free (checksum);
 }
 
+gchar *
+gabble_generate_id (void)
+{
+#ifdef HAVE_UUID
+  /* generate random UUIDs */
+  uuid_t uu;
+  gchar *str;
+
+  str = g_new0 (gchar, 37);
+  uuid_generate_random (uu);
+  uuid_unparse_lower (uu, str);
+  return str;
+#else
+  /* generate from the time, a counter, and a random integer */
+  static gulong last = 0;
+  GTimeVal tv;
+
+  g_get_current_time (&tv);
+  return g_strdup_printf ("%lx.%lx/%lx/%x", tv.tv_sec, tv.tv_usec,
+      last++, g_random_int ());
+#endif
+}
+
 static void
 lm_message_node_add_nick (LmMessageNode *node, const gchar *nick)
 {
@@ -97,10 +124,11 @@ lm_message_node_add_own_nick (LmMessageNode *node,
 }
 
 void
-lm_message_node_unlink (LmMessageNode *orphan)
+lm_message_node_unlink (LmMessageNode *orphan,
+    LmMessageNode *parent)
 {
-  if (orphan->parent && orphan == orphan->parent->children)
-    orphan->parent->children = orphan->next;
+  if (parent && orphan == parent->children)
+    parent->children = orphan->next;
   if (orphan->prev)
     orphan->prev->next = orphan->next;
   if (orphan->next)
@@ -111,7 +139,7 @@ void
 lm_message_node_steal_children (LmMessageNode *snatcher,
                                 LmMessageNode *mum)
 {
-  LmMessageNode *baby;
+  NodeIter i;
 
   g_return_if_fail (snatcher->children == NULL);
 
@@ -121,10 +149,11 @@ lm_message_node_steal_children (LmMessageNode *snatcher,
   snatcher->children = mum->children;
   mum->children = NULL;
 
-  for (baby = snatcher->children;
-       baby != NULL;
-       baby = baby->next)
-    baby->parent = snatcher;
+  for (i = node_iter (snatcher); i; i = node_iter_next (i))
+    {
+      LmMessageNode *baby = node_iter_data (i);
+      baby->parent = snatcher;
+    }
 }
 
 /* variant of lm_message_node_get_child() which ignores node namespace
@@ -132,10 +161,12 @@ lm_message_node_steal_children (LmMessageNode *snatcher,
 LmMessageNode *
 lm_message_node_get_child_any_ns (LmMessageNode *node, const gchar *name)
 {
-  LmMessageNode *child;
+  NodeIter i;
 
-  for (child = node->children; child != NULL; child = child->next)
+  for (i = node_iter (node); i; i = node_iter_next (i))
     {
+      LmMessageNode *child = node_iter_data (i);
+
       if (!tp_strdiff (lm_message_node_get_name (child), name))
           return child;
     }
@@ -143,6 +174,23 @@ lm_message_node_get_child_any_ns (LmMessageNode *node, const gchar *name)
   return NULL;
 }
 
+static const gchar *
+find_namespace_of_prefix (LmMessageNode *node,
+    const gchar *prefix)
+{
+  gchar *attr = g_strdup_printf ("xmlns:%s", prefix);
+  const gchar *node_ns = NULL;
+
+  /* find the namespace in this node or its parents */
+  for (; (node != NULL) && (node_ns == NULL); node = node->parent)
+    {
+      node_ns = lm_message_node_get_attribute (node, attr);
+    }
+
+  g_free (attr);
+  return node_ns;
+}
+
 const gchar *
 lm_message_node_get_namespace (LmMessageNode *node)
 {
@@ -152,16 +200,9 @@ lm_message_node_get_namespace (LmMessageNode *node)
   if (x != NULL)
     {
       gchar *prefix = g_strndup (node->name, (x - node->name));
-      gchar *attr = g_strdup_printf ("xmlns:%s", prefix);
-
-      /* find the namespace in this node or its parents */
-      for (node_ns = NULL; (node != NULL) && (node_ns == NULL); node = node->parent)
-        {
-          node_ns = lm_message_node_get_attribute (node, attr);
-        }
 
+      node_ns = find_namespace_of_prefix (node, prefix);
       g_free (prefix);
-      g_free (attr);
     }
   else
     {
@@ -195,12 +236,11 @@ lm_message_node_get_child_with_namespace (LmMessageNode *node,
                                           const gchar *name,
                                           const gchar *ns)
 {
-  LmMessageNode *tmp;
+  NodeIter i;
 
-  for (tmp = node->children;
-       tmp != NULL;
-       tmp = tmp->next)
+  for (i = node_iter (node); i; i = node_iter_next (i))
     {
+      LmMessageNode *tmp = node_iter_data (i);
       gchar *tag = NULL;
       gboolean found;
 
@@ -232,51 +272,6 @@ lm_message_node_get_child_with_namespace (LmMessageNode *node,
   return NULL;
 }
 
-static LmMessageNode *
-message_node_last_child (LmMessageNode *node)
-{
-  LmMessageNode *l;
-
-  g_return_val_if_fail (node != NULL, NULL);
-
-  if (!node->children)
-    {
-      return NULL;
-    }
-
-  l = node->children;
-
-  while (l->next)
-    {
-      l = l->next;
-    }
-
-  return l;
-}
-
-void
-lm_message_node_add_child_node (LmMessageNode *node, LmMessageNode *child)
-{
-  LmMessageNode *prev;
-
-  g_return_if_fail (node != NULL);
-
-  prev = message_node_last_child (node);
-  lm_message_node_ref (child);
-
-  if (prev)
-    {
-      prev->next = child;
-      child->prev = prev;
-    }
-  else
-    {
-      node->children = child;
-    }
-
-  child->parent = node;
-}
-
 /* note: these are only used internally for readability, not part of the API
  */
 enum {
@@ -668,7 +663,7 @@ lm_message_node_extract_properties (LmMessageNode *node,
                                     const gchar *prop)
 {
   GHashTable *properties;
-  LmMessageNode *child;
+  NodeIter i;
 
   properties = g_hash_table_new_full (g_str_hash, g_str_equal, g_free,
       (GDestroyNotify) tp_g_value_slice_free);
@@ -676,8 +671,9 @@ lm_message_node_extract_properties (LmMessageNode *node,
   if (node == NULL)
     return properties;
 
-  for (child = node->children; child; child = child->next)
+  for (i = node_iter (node); i; i = node_iter_next (i))
     {
+      LmMessageNode *child = node_iter_data (i);
       const gchar *name;
       const gchar *type;
       const gchar *value;
@@ -1015,3 +1011,117 @@ gabble_signal_connect_weak (gpointer instance,
   g_object_weak_ref (instance_obj, instance_destroyed_cb, ctx);
   g_object_weak_ref (user_data, user_data_destroyed_cb, ctx);
 }
+
+typedef struct {
+    GSourceFunc function;
+    GObject *object;
+    guint source_id;
+} WeakIdleCtx;
+
+static void
+idle_weak_ref_notify (gpointer data,
+                      GObject *dead_object)
+{
+  g_source_remove (GPOINTER_TO_UINT (data));
+}
+
+static void
+idle_removed (gpointer data)
+{
+  WeakIdleCtx *ctx = (WeakIdleCtx *) data;
+
+  g_slice_free (WeakIdleCtx, ctx);
+}
+
+static gboolean
+idle_callback (gpointer data)
+{
+  WeakIdleCtx *ctx = (WeakIdleCtx *) data;
+
+  if (ctx->function ((gpointer) ctx->object))
+    {
+      return TRUE;
+    }
+  else
+    {
+      g_object_weak_unref (
+          ctx->object, idle_weak_ref_notify, GUINT_TO_POINTER (ctx->source_id));
+      return FALSE;
+    }
+}
+
+/* Like g_idle_add(), but cancel the callback if the provided object is
+ * finalized.
+ */
+guint
+gabble_idle_add_weak (GSourceFunc function,
+                      GObject *object)
+{
+  WeakIdleCtx *ctx;
+
+  ctx = g_slice_new0 (WeakIdleCtx);
+  ctx->function = function;
+  ctx->object = object;
+  ctx->source_id = g_idle_add_full (
+      G_PRIORITY_DEFAULT_IDLE, idle_callback, ctx, idle_removed);
+
+  g_object_weak_ref (
+      object, idle_weak_ref_notify, GUINT_TO_POINTER (ctx->source_id));
+  return ctx->source_id;
+}
+
+typedef struct {
+    gchar *key;
+    gchar *value;
+} Attribute;
+
+const gchar *
+lm_message_node_get_attribute_with_namespace (LmMessageNode *node,
+    const gchar *attribute,
+    const gchar *ns)
+{
+  GSList *l;
+  const gchar *result = NULL;
+
+  g_return_val_if_fail (node != NULL, NULL);
+  g_return_val_if_fail (attribute != NULL, NULL);
+  g_return_val_if_fail (ns != NULL, NULL);
+
+  for (l = node->attributes; l != NULL && result == NULL; l = g_slist_next (l))
+    {
+      /* This is NOT part of loudmouth API; it depends LM internals */
+      Attribute *attr = (Attribute *) l->data;
+      gchar **pair;
+
+      pair = g_strsplit (attr->key, ":", 2);
+
+      if (tp_strdiff (pair[1], attribute))
+        /* no prefix (pair[1] == NULL) or the local-name is not the
+         * attribute we are looking for */
+        goto next_attribute;
+
+      if (tp_strdiff (find_namespace_of_prefix (node, pair[0]), ns))
+        /* wrong namespace */
+        goto next_attribute;
+
+      result = attr->value;
+
+next_attribute:
+      g_strfreev (pair);
+      continue;
+    }
+
+  return result;
+}
+
+GPtrArray *
+gabble_g_ptr_array_copy (GPtrArray *source)
+{
+  GPtrArray *ret = g_ptr_array_sized_new (source->len);
+  guint i;
+
+  for (i = 0; i < source->len; i++)
+    g_ptr_array_add (ret, g_ptr_array_index (source, i));
+
+  return ret;
+}
