@@ -37,7 +37,7 @@
 #include "request-pipeline.h"
 #include "util.h"
 
-#define DEFAULT_REQUEST_TIMEOUT 60000
+#define DEFAULT_REQUEST_TIMEOUT 180
 #define VCARD_CACHE_ENTRY_TTL 60
 
 static const gchar *NO_ALIAS = "none";
@@ -171,10 +171,10 @@ gabble_vcard_manager_cache_quark (void)
   return quark;
 }
 
-#define GABBLE_VCARD_MANAGER_GET_PRIVATE(o) ((o)->priv)
-
 static void cache_entry_free (void *data);
 static gint cache_entry_compare (gconstpointer a, gconstpointer b);
+static void manager_patch_vcard (
+    GabbleVCardManager *self, LmMessageNode *vcard_node);
 
 static void
 gabble_vcard_manager_init (GabbleVCardManager *obj)
@@ -247,8 +247,8 @@ gabble_vcard_manager_get_property (GObject *object,
                                    GValue *value,
                                    GParamSpec *pspec)
 {
-  GabbleVCardManager *chan = GABBLE_VCARD_MANAGER (object);
-  GabbleVCardManagerPrivate *priv = GABBLE_VCARD_MANAGER_GET_PRIVATE (chan);
+  GabbleVCardManager *self = GABBLE_VCARD_MANAGER (object);
+  GabbleVCardManagerPrivate *priv = self->priv;
 
   switch (property_id) {
     case PROP_CONNECTION:
@@ -269,8 +269,8 @@ gabble_vcard_manager_set_property (GObject *object,
                                    const GValue *value,
                                    GParamSpec *pspec)
 {
-  GabbleVCardManager *chan = GABBLE_VCARD_MANAGER (object);
-  GabbleVCardManagerPrivate *priv = GABBLE_VCARD_MANAGER_GET_PRIVATE (chan);
+  GabbleVCardManager *self = GABBLE_VCARD_MANAGER (object);
+  GabbleVCardManagerPrivate *priv = self->priv;
 
   switch (property_id) {
     case PROP_CONNECTION:
@@ -298,8 +298,7 @@ static void
 cache_entry_free (gpointer data)
 {
   GabbleVCardCacheEntry *entry = data;
-  GabbleVCardManagerPrivate *priv =
-      GABBLE_VCARD_MANAGER_GET_PRIVATE (entry->manager);
+  GabbleVCardManagerPrivate *priv = entry->manager->priv;
   TpHandleRepoIface *contact_repo = tp_base_connection_get_handles
       ((TpBaseConnection *) priv->connection, TP_HANDLE_TYPE_CONTACT);
 
@@ -326,7 +325,7 @@ cache_entry_free (gpointer data)
 static GabbleVCardCacheEntry *
 cache_entry_get (GabbleVCardManager *manager, TpHandle handle)
 {
-  GabbleVCardManagerPrivate *priv = GABBLE_VCARD_MANAGER_GET_PRIVATE (manager);
+  GabbleVCardManagerPrivate *priv = manager->priv;
   TpHandleRepoIface *contact_repo = tp_base_connection_get_handles (
       (TpBaseConnection *) priv->connection, TP_HANDLE_TYPE_CONTACT);
   GabbleVCardCacheEntry *entry;
@@ -349,7 +348,7 @@ static gboolean
 cache_entry_timeout (gpointer data)
 {
   GabbleVCardManager *manager = data;
-  GabbleVCardManagerPrivate *priv = GABBLE_VCARD_MANAGER_GET_PRIVATE (manager);
+  GabbleVCardManagerPrivate *priv = manager->priv;
   GabbleVCardCacheEntry *entry;
 
   time_t now = time (NULL);
@@ -369,9 +368,8 @@ cache_entry_timeout (gpointer data)
 
   if (entry)
     {
-      priv->cache_timer = g_timeout_add (
-          1000 * (entry->expires - time (NULL)),
-          cache_entry_timeout, manager);
+      priv->cache_timer = g_timeout_add_seconds (
+          entry->expires - time (NULL), cache_entry_timeout, manager);
     }
 
   return FALSE;
@@ -381,8 +379,7 @@ cache_entry_timeout (gpointer data)
 static void
 cache_entry_attempt_to_free (GabbleVCardCacheEntry *entry)
 {
-  GabbleVCardManagerPrivate *priv = GABBLE_VCARD_MANAGER_GET_PRIVATE
-      (entry->manager);
+  GabbleVCardManagerPrivate *priv = entry->manager->priv;
   TpBaseConnection *base = (TpBaseConnection *) priv->connection;
 
   if (entry->vcard_node != NULL)
@@ -410,7 +407,7 @@ cache_entry_attempt_to_free (GabbleVCardCacheEntry *entry)
     {
       /* if we do have some pending edits, we should also have
        * some pipeline items or pending requests */
-      g_assert (priv->edits == NULL);
+      g_assert (priv->edit_pipeline_item || priv->edits == NULL);
     }
 
   tp_heap_remove (priv->timed_cache, entry);
@@ -422,7 +419,7 @@ void
 gabble_vcard_manager_invalidate_cache (GabbleVCardManager *manager,
                                        TpHandle handle)
 {
-  GabbleVCardManagerPrivate *priv = GABBLE_VCARD_MANAGER_GET_PRIVATE (manager);
+  GabbleVCardManagerPrivate *priv = manager->priv;
   GabbleVCardCacheEntry *entry = g_hash_table_lookup (priv->cache,
       GUINT_TO_POINTER (handle));
   TpHandleRepoIface *contact_repo = tp_base_connection_get_handles (
@@ -497,7 +494,7 @@ static void
 gabble_vcard_manager_dispose (GObject *object)
 {
   GabbleVCardManager *self = GABBLE_VCARD_MANAGER (object);
-  GabbleVCardManagerPrivate *priv = GABBLE_VCARD_MANAGER_GET_PRIVATE (self);
+  GabbleVCardManagerPrivate *priv = self->priv;
 
   if (priv->dispose_has_run)
     return;
@@ -584,8 +581,8 @@ initial_request_cb (GabbleVCardManager *self,
                     GError *error,
                     gpointer user_data)
 {
-  GabbleVCardManagerPrivate *priv = GABBLE_VCARD_MANAGER_GET_PRIVATE (self);
-  gchar *alias = (gchar *) user_data;
+  GabbleVCardManagerPrivate *priv = self->priv;
+  gchar *alias = user_data;
   gchar *sha1;
 
   if (vcard)
@@ -615,7 +612,7 @@ status_changed_cb (GObject *object,
                    gpointer user_data)
 {
   GabbleVCardManager *self = GABBLE_VCARD_MANAGER (user_data);
-  GabbleVCardManagerPrivate *priv = GABBLE_VCARD_MANAGER_GET_PRIVATE (self);
+  GabbleVCardManagerPrivate *priv = self->priv;
   GabbleConnection *conn = GABBLE_CONNECTION (object);
   TpBaseConnection *base = (TpBaseConnection *) conn;
 
@@ -640,8 +637,7 @@ status_changed_cb (GObject *object,
               priv->edits = g_hash_table_new_full (g_str_hash, g_str_equal,
                   g_free, g_free);
 
-          g_hash_table_insert (priv->edits, g_strdup ("NICKNAME"),
-              alias);
+          g_hash_table_insert (priv->edits, g_strdup ("NICKNAME"), alias);
         }
 
       /* FIXME: we happen to know that synchronous errors can't happen */
@@ -801,12 +797,7 @@ observe_vcard (GabbleConnection *conn,
 
   if (old_alias != NULL && !tp_strdiff (old_alias, alias))
     {
-#ifdef ENABLE_DEBUG
-      if (alias != NULL)
-        DEBUG ("no change to vCard alias \"%s\" for handle %u", alias, handle);
-      else
-        DEBUG ("still no vCard alias for handle %u", handle);
-#endif
+      DEBUG ("no change to vCard alias \"%s\" for handle %u", alias, handle);
 
       g_free (alias);
       return;
@@ -833,60 +824,37 @@ observe_vcard (GabbleConnection *conn,
       g_signal_emit (G_OBJECT (manager), signals[NICKNAME_UPDATE], 0, handle);
 }
 
-static GError *
-get_error_from_pipeline_reply (LmMessage *reply_msg, GError *error)
-{
-    GError *err = NULL;
-
-    if (error)
-        return g_error_copy (error);
-
-    if (lm_message_get_sub_type (reply_msg) != LM_MESSAGE_SUB_TYPE_ERROR)
-        return NULL;
-
-    err = gabble_message_get_xmpp_error (reply_msg);
-
-    if (err == NULL)
-      {
-        err = g_error_new (GABBLE_VCARD_MANAGER_ERROR,
-            GABBLE_VCARD_MANAGER_ERROR_UNKNOWN, "An unknown error occurred");
-      }
-
-    return err;
-}
-
+/* Called when a pre-set get request failed, or when a set request succeeded
+ * or failed.
+ */
 static void
 replace_reply_cb (GabbleConnection *conn,
                   LmMessage *reply_msg,
                   gpointer user_data,
                   GError *error)
 {
-  GabbleVCardManager *manager = GABBLE_VCARD_MANAGER (user_data);
-  GabbleVCardManagerPrivate *priv = GABBLE_VCARD_MANAGER_GET_PRIVATE (manager);
+  GabbleVCardManager *self = GABBLE_VCARD_MANAGER (user_data);
+  GabbleVCardManagerPrivate *priv = self->priv;
   TpBaseConnection *base = (TpBaseConnection *) conn;
-
-  GError *err = get_error_from_pipeline_reply (reply_msg, error);
   GList *li;
-  LmMessageNode *node;
+  LmMessageNode *node = NULL;
 
+  /* If we sent a SET request, it's dead now. */
   priv->edit_pipeline_item = NULL;
 
   DEBUG ("called: %s error", (error) ? "some" : "no");
 
-  if (err)
+  if (error)
     {
       /* We won't need our patched vcard after all */
       if (priv->patched_vcard != NULL)
           lm_message_node_unref (priv->patched_vcard);
 
       priv->patched_vcard = NULL;
-
-      node = NULL;
     }
   else
     {
-      GabbleVCardCacheEntry *entry = cache_entry_get (manager,
-          base->self_handle);
+      GabbleVCardCacheEntry *entry = cache_entry_get (self, base->self_handle);
 
       /* We must have patched vcard by now */
       g_assert (priv->patched_vcard != NULL);
@@ -899,7 +867,7 @@ replace_reply_cb (GabbleConnection *conn,
       priv->patched_vcard = NULL;
 
       /* observe it so we pick up alias updates */
-      observe_vcard (conn, manager, base->self_handle, entry->vcard_node);
+      observe_vcard (conn, self, base->self_handle, entry->vcard_node);
 
       node = entry->vcard_node;
     }
@@ -909,21 +877,21 @@ replace_reply_cb (GabbleConnection *conn,
   li = priv->edit_requests;
   while (li)
     {
-      GabbleVCardManagerEditRequest *req = (GabbleVCardManagerEditRequest *) li->data;
+      GabbleVCardManagerEditRequest *req = li->data;
       li = g_list_next (li);
-      if (req->set_in_pipeline)
+      if (req->set_in_pipeline || error)
         {
           if (req->callback)
             {
-              (req->callback) (req->manager, req, node, err, req->user_data);
+              (req->callback) (req->manager, req, node, error, req->user_data);
             }
 
           gabble_vcard_manager_remove_edit_request (req);
         }
     }
 
-  if (err != NULL)
-    g_error_free (err);
+  /* If we've received more edit requests in the meantime, send them off. */
+  manager_patch_vcard (self, node);
 }
 
 static void
@@ -940,7 +908,7 @@ patch_vcard_foreach (gpointer k, gpointer v, gpointer user_data)
       node = lm_message_node_get_child (vcard_node, "PHOTO");
       if (node != NULL)
         {
-          lm_message_node_unlink (node);
+          lm_message_node_unlink (node, vcard_node);
           lm_message_node_unref (node);
         }
 
@@ -979,39 +947,37 @@ patch_vcard_foreach (gpointer k, gpointer v, gpointer user_data)
 static LmMessageNode *
 vcard_copy (LmMessageNode *parent, LmMessageNode *src)
 {
-    LmMessageNode *child;
     LmMessageNode *new = lm_message_node_add_child (parent, src->name,
-        src->value);
+        lm_message_node_get_value (src));
     const gchar *xmlns;
+    NodeIter i;
 
     xmlns = lm_message_node_get_attribute (src, "xmlns");
     if (xmlns != NULL)
       lm_message_node_set_attribute (new, "xmlns", xmlns);
 
-    for (child = src->children; child; child = child->next)
-        vcard_copy (new, child);
+    for (i = node_iter (src); i; i = node_iter_next (i))
+      vcard_copy (new, node_iter_data (i));
 
     return new;
 }
 
 static void
-manager_patch_vcard (GabbleVCardManager *manager,
+manager_patch_vcard (GabbleVCardManager *self,
                      LmMessageNode *vcard_node)
 {
-  GabbleVCardManagerPrivate *priv = GABBLE_VCARD_MANAGER_GET_PRIVATE (manager);
+  GabbleVCardManagerPrivate *priv = self->priv;
   LmMessage *msg;
   LmMessageNode *patched_vcard;
   GList *li;
 
-  g_assert (priv->edits != NULL);
+  /* Bail out if we don't have outstanding edits to make, or if we already
+   * have a set request in progress.
+   */
+  if (priv->edits == NULL || priv->edit_pipeline_item != NULL)
+      return;
 
-  /* There can be only one SET request at any given time,
-   * because XMPP server will process requests sequentially.
-   * This function can only be called when GET request for
-   * our vcard returns, but if we do SET and then GET for
-   * our vcard, SET will always return first, and clear the
-   * pipeline item. Or so I hope. */
-  g_assert (priv->edit_pipeline_item == NULL);
+  DEBUG("patching vcard");
 
   msg = lm_message_new_with_sub_type (NULL, LM_MESSAGE_TYPE_IQ,
       LM_MESSAGE_SUB_TYPE_SET);
@@ -1024,11 +990,11 @@ manager_patch_vcard (GabbleVCardManager *manager,
   /* We'll save the patched vcard, and if the server says
    * we're ok, put it into the cache. But we want to leave the
    * original vcard in the cache until that happens. */
-  lm_message_node_ref (patched_vcard);
-  priv->patched_vcard = patched_vcard;
+  priv->patched_vcard = lm_message_node_ref (patched_vcard);
 
   priv->edit_pipeline_item = gabble_request_pipeline_enqueue (
-      priv->connection->req_pipeline, msg, 0, replace_reply_cb, manager);
+      priv->connection->req_pipeline, msg, DEFAULT_REQUEST_TIMEOUT,
+      replace_reply_cb, self);
 
   lm_message_unref (msg);
 
@@ -1053,13 +1019,12 @@ pipeline_reply_cb (GabbleConnection *conn,
                    GError *error)
 {
   GabbleVCardCacheEntry *entry = user_data;
-  GabbleVCardManager *manager = GABBLE_VCARD_MANAGER (entry->manager);
-  GabbleVCardManagerPrivate *priv = GABBLE_VCARD_MANAGER_GET_PRIVATE (manager);
+  GabbleVCardManager *self = GABBLE_VCARD_MANAGER (entry->manager);
+  GabbleVCardManagerPrivate *priv = self->priv;
   TpBaseConnection *base = (TpBaseConnection *) conn;
-  TpHandleRepoIface *contact_repo = tp_base_connection_get_handles (base,
-      TP_HANDLE_TYPE_CONTACT);
+  TpHandleRepoIface *contact_repo =
+      tp_base_connection_get_handles (base, TP_HANDLE_TYPE_CONTACT);
   LmMessageNode *vcard_node = NULL;
-  GError *err = NULL;
 
   DEBUG("called for entry %p", entry);
 
@@ -1069,8 +1034,7 @@ pipeline_reply_cb (GabbleConnection *conn,
 
   entry->pipeline_item = NULL;
 
-  err = get_error_from_pipeline_reply (reply_msg, error);
-  if (err)
+  if (error)
     {
       /* If request for our own vCard failed, and we do have
        * pending edits to make, cancel those and return error
@@ -1081,13 +1045,11 @@ pipeline_reply_cb (GabbleConnection *conn,
           g_hash_table_destroy (priv->edits);
           priv->edits = NULL;
 
-          replace_reply_cb (conn, reply_msg, manager, error);
+          replace_reply_cb (conn, reply_msg, self, error);
         }
 
       /* Complete pending GET requests */
-      cache_entry_complete_requests (entry, err);
-
-      g_error_free (err);
+      cache_entry_complete_requests (entry, error);
       return;
     }
 
@@ -1107,8 +1069,7 @@ pipeline_reply_cb (GabbleConnection *conn,
     }
 
   /* Put the message in the cache */
-  lm_message_node_ref (vcard_node);
-  entry->vcard_node = vcard_node;
+  entry->vcard_node = lm_message_node_ref (vcard_node);
 
   entry->expires = time (NULL) + VCARD_CACHE_ENTRY_TTL;
   tp_heap_add (priv->timed_cache, entry);
@@ -1117,21 +1078,20 @@ pipeline_reply_cb (GabbleConnection *conn,
       GabbleVCardCacheEntry *first =
           tp_heap_peek_first (priv->timed_cache);
 
-      priv->cache_timer = g_timeout_add
-          ((first->expires - time (NULL)) * 1000, cache_entry_timeout,
-           manager);
+      priv->cache_timer = g_timeout_add_seconds (
+          first->expires - time (NULL), cache_entry_timeout, self);
     }
 
   /* We have freshly updated cache for our vCard, edit it if
-   * there are any pending edits */
-  if (entry->handle == base->self_handle && priv->edits != NULL)
+   * there are any pending edits and no outstanding set request.
+   */
+  if (entry->handle == base->self_handle)
     {
-      DEBUG("will patch vcard");
-      manager_patch_vcard (manager, vcard_node);
+      manager_patch_vcard (self, vcard_node);
     }
 
   /* Observe the vCard as it goes past */
-  observe_vcard (priv->connection, manager, entry->handle, vcard_node);
+  observe_vcard (priv->connection, self, entry->handle, vcard_node);
 
   /* Complete all pending requests successfully */
   cache_entry_complete_requests (entry, NULL);
@@ -1149,8 +1109,7 @@ notify_delete_request (gpointer data, GObject *obj)
 static void
 cache_entry_ensure_queued (GabbleVCardCacheEntry *entry, guint timeout)
 {
-  GabbleConnection *conn =
-    GABBLE_VCARD_MANAGER_GET_PRIVATE (entry->manager)->connection;
+  GabbleConnection *conn = entry->manager->priv->connection;
   TpBaseConnection *base = (TpBaseConnection *) conn;
   TpHandleRepoIface *contact_repo = tp_base_connection_get_handles (base,
       TP_HANDLE_TYPE_CONTACT);
@@ -1182,8 +1141,8 @@ cache_entry_ensure_queued (GabbleVCardCacheEntry *entry, guint timeout)
           ')',
           NULL);
 
-      entry->pipeline_item = gabble_request_pipeline_enqueue
-          (conn->req_pipeline, msg, timeout, pipeline_reply_cb, entry);
+      entry->pipeline_item = gabble_request_pipeline_enqueue (
+          conn->req_pipeline, msg, timeout, pipeline_reply_cb, entry);
 
       lm_message_unref (msg);
 
@@ -1205,7 +1164,7 @@ gabble_vcard_manager_request (GabbleVCardManager *self,
                               gpointer user_data,
                               GObject *object)
 {
-  GabbleVCardManagerPrivate *priv = GABBLE_VCARD_MANAGER_GET_PRIVATE (self);
+  GabbleVCardManagerPrivate *priv = self->priv;
   TpBaseConnection *connection = (TpBaseConnection *) priv->connection;
   TpHandleRepoIface *contact_repo = tp_base_connection_get_handles (
       connection, TP_HANDLE_TYPE_CONTACT);
@@ -1233,7 +1192,8 @@ gabble_vcard_manager_request (GabbleVCardManager *self,
   request->entry->pending_requests = g_slist_prepend
       (request->entry->pending_requests, request);
 
-  request->timer_id = g_timeout_add (timeout, timeout_request, request);
+  request->timer_id =
+      g_timeout_add_seconds (timeout, timeout_request, request);
   cache_entry_ensure_queued (request->entry, timeout);
   return request;
 }
@@ -1249,7 +1209,7 @@ gabble_vcard_manager_edit (GabbleVCardManager *self,
 {
   va_list ap;
   size_t i;
-  GabbleVCardManagerPrivate *priv = GABBLE_VCARD_MANAGER_GET_PRIVATE (self);
+  GabbleVCardManagerPrivate *priv = self->priv;
   TpBaseConnection *base = (TpBaseConnection *) priv->connection;
   GabbleVCardManagerEditRequest *req;
   GabbleVCardCacheEntry *entry;
@@ -1260,7 +1220,7 @@ gabble_vcard_manager_edit (GabbleVCardManager *self,
   gabble_vcard_manager_invalidate_cache (self, base->self_handle);
   DEBUG ("checking if we have pending requests already");
   entry = cache_entry_get (self, base->self_handle);
-  if (!entry->pending_requests)
+  if (!priv->edit_pipeline_item && !entry->pending_requests)
     {
       DEBUG ("we don't, create one");
       /* create dummy GET request if neccessary */
@@ -1307,7 +1267,7 @@ gabble_vcard_manager_edit (GabbleVCardManager *self,
 void
 gabble_vcard_manager_remove_edit_request (GabbleVCardManagerEditRequest *request)
 {
-  GabbleVCardManagerPrivate *priv = GABBLE_VCARD_MANAGER_GET_PRIVATE (request->manager);
+  GabbleVCardManagerPrivate *priv = request->manager->priv;
 
   DEBUG("request == %p", request);
 
@@ -1334,9 +1294,9 @@ notify_delete_edit_request (gpointer data, GObject *obj)
 }
 
 static void
-cancel_all_edit_requests (GabbleVCardManager *manager)
+cancel_all_edit_requests (GabbleVCardManager *self)
 {
-  GabbleVCardManagerPrivate *priv = GABBLE_VCARD_MANAGER_GET_PRIVATE (manager);
+  GabbleVCardManagerPrivate *priv = self->priv;
   GError cancelled = { GABBLE_VCARD_MANAGER_ERROR,
       GABBLE_VCARD_MANAGER_ERROR_CANCELLED,
       "Request cancelled" };
@@ -1356,12 +1316,12 @@ cancel_all_edit_requests (GabbleVCardManager *manager)
 
 
 void
-gabble_vcard_manager_cancel_request (GabbleVCardManager *manager,
+gabble_vcard_manager_cancel_request (GabbleVCardManager *self,
                                      GabbleVCardManagerRequest *request)
 {
-  g_return_if_fail (GABBLE_IS_VCARD_MANAGER (manager));
+  g_return_if_fail (GABBLE_IS_VCARD_MANAGER (self));
   g_return_if_fail (NULL != request);
-  g_return_if_fail (manager == request->manager);
+  g_return_if_fail (self == request->manager);
 
   cancel_request (request);
 }
@@ -1374,7 +1334,7 @@ gabble_vcard_manager_get_cached (GabbleVCardManager *self,
                                  TpHandle handle,
                                  LmMessageNode **node)
 {
-  GabbleVCardManagerPrivate *priv = GABBLE_VCARD_MANAGER_GET_PRIVATE (self);
+  GabbleVCardManagerPrivate *priv = self->priv;
   GabbleVCardCacheEntry *entry = g_hash_table_lookup (priv->cache,
       GUINT_TO_POINTER (handle));
   TpHandleRepoIface *contact_repo = tp_base_connection_get_handles (
@@ -1397,16 +1357,16 @@ gabble_vcard_manager_get_cached (GabbleVCardManager *self,
  * if any. If there is no cached alias, return NULL.
  */
 const gchar *
-gabble_vcard_manager_get_cached_alias (GabbleVCardManager *manager,
+gabble_vcard_manager_get_cached_alias (GabbleVCardManager *self,
                                        TpHandle handle)
 {
   GabbleVCardManagerPrivate *priv;
   TpHandleRepoIface *contact_repo;
   const gchar *s;
 
-  g_return_val_if_fail (GABBLE_IS_VCARD_MANAGER (manager), NULL);
+  g_return_val_if_fail (GABBLE_IS_VCARD_MANAGER (self), NULL);
 
-  priv = GABBLE_VCARD_MANAGER_GET_PRIVATE (manager);
+  priv = self->priv;
   contact_repo = tp_base_connection_get_handles (
       (TpBaseConnection *) priv->connection, TP_HANDLE_TYPE_CONTACT);
 
@@ -1425,16 +1385,16 @@ gabble_vcard_manager_get_cached_alias (GabbleVCardManager *manager,
  * Return TRUE if we've tried looking up an alias for this handle before.
  */
 gboolean
-gabble_vcard_manager_has_cached_alias (GabbleVCardManager *manager,
+gabble_vcard_manager_has_cached_alias (GabbleVCardManager *self,
                                        TpHandle handle)
 {
   GabbleVCardManagerPrivate *priv;
   TpHandleRepoIface *contact_repo;
   gpointer p;
 
-  g_return_val_if_fail (GABBLE_IS_VCARD_MANAGER (manager), FALSE);
+  g_return_val_if_fail (GABBLE_IS_VCARD_MANAGER (self), FALSE);
 
-  priv = GABBLE_VCARD_MANAGER_GET_PRIVATE (manager);
+  priv = self->priv;
   contact_repo = tp_base_connection_get_handles (
       (TpBaseConnection *) priv->connection, TP_HANDLE_TYPE_CONTACT);
 
