@@ -5,13 +5,16 @@ Infrastructure code for testing Gabble by pretending to be a Jabber server.
 
 import base64
 import os
-import sha
+import hashlib
 import sys
-import time
 import random
 
 import ns
+import constants as cs
 import servicetest
+from servicetest import (
+    assertEquals, assertLength, assertContains, wrap_channel, EventPattern,
+    )
 import twisted
 from twisted.words.xish import domish, xpath
 from twisted.words.protocols.jabber.client import IQ
@@ -26,6 +29,9 @@ NS_XMPP_BIND = 'urn:ietf:params:xml:ns:xmpp-bind'
 def make_result_iq(stream, iq):
     result = IQ(stream, "result")
     result["id"] = iq["id"]
+    to = iq.getAttribute('to')
+    if to is not None:
+        result["from"] = to
     query = iq.firstChildElement()
 
     if query:
@@ -36,14 +42,20 @@ def make_result_iq(stream, iq):
 def acknowledge_iq(stream, iq):
     stream.send(make_result_iq(stream, iq))
 
-def send_error_reply(stream, iq):
+def send_error_reply(stream, iq, error_stanza=None):
     result = IQ(stream, "error")
     result["id"] = iq["id"]
     query = iq.firstChildElement()
+    to = iq.getAttribute('to')
+    if to is not None:
+        result["from"] = to
 
     if query:
         result.addElement((query.uri, query.name))
 
+    if error_stanza:
+        result.addChild(error_stanza)
+
     stream.send(result)
 
 def request_muc_handle(q, conn, stream, muc_jid):
@@ -57,13 +69,15 @@ def request_muc_handle(q, conn, stream, muc_jid):
     event = q.expect('dbus-return', method='RequestHandles')
     return event.value[0][0]
 
-def make_muc_presence(affiliation, role, muc_jid, alias):
+def make_muc_presence(affiliation, role, muc_jid, alias, jid=None):
     presence = domish.Element((None, 'presence'))
     presence['from'] = '%s/%s' % (muc_jid, alias)
     x = presence.addElement((ns.MUC_USER, 'x'))
     item = x.addElement('item')
     item['affiliation'] = affiliation
     item['role'] = role
+    if jid is not None:
+        item['jid'] = jid
     return presence
 
 def sync_stream(q, stream):
@@ -80,9 +94,10 @@ def sync_stream(q, stream):
 class JabberAuthenticator(xmlstream.Authenticator):
     "Trivial XML stream authenticator that accepts one username/digest pair."
 
-    def __init__(self, username, password):
+    def __init__(self, username, password, resource=None):
         self.username = username
         self.password = password
+        self.resource = resource
         xmlstream.Authenticator.__init__(self)
 
     # Patch in fix from http://twistedmatrix.com/trac/changeset/23418.
@@ -126,11 +141,13 @@ class JabberAuthenticator(xmlstream.Authenticator):
         assert map(str, username) == [self.username]
 
         digest = xpath.queryForNodes('/iq/query/digest', iq)
-        expect = sha.sha(self.xmlstream.sid + self.password).hexdigest()
+        expect = hashlib.sha1(self.xmlstream.sid + self.password).hexdigest()
         assert map(str, digest) == [expect]
 
         resource = xpath.queryForNodes('/iq/query/resource', iq)
-        assert map(str, resource) == ['Resource']
+        assertLength(1, resource)
+        if self.resource is not None:
+            assertEquals(self.resource, str(resource[0]))
 
         result = IQ(self.xmlstream, "result")
         result["id"] = iq["id"]
@@ -139,10 +156,11 @@ class JabberAuthenticator(xmlstream.Authenticator):
 
 
 class XmppAuthenticator(xmlstream.Authenticator):
-    def __init__(self, username, password):
+    def __init__(self, username, password, resource=None):
         xmlstream.Authenticator.__init__(self)
         self.username = username
         self.password = password
+        self.resource = resource
         self.authenticated = False
 
     def streamStarted(self, root=None):
@@ -178,12 +196,16 @@ class XmppAuthenticator(xmlstream.Authenticator):
         self.authenticated = True
 
     def bindIq(self, iq):
-        assert xpath.queryForString('/iq/bind/resource', iq) == 'Resource'
+        resource = xpath.queryForString('/iq/bind/resource', iq)
+        if self.resource is not None:
+            assertEquals(self.resource, resource)
+        else:
+            assert resource is not None
 
         result = IQ(self.xmlstream, "result")
         result["id"] = iq["id"]
         bind = result.addElement((NS_XMPP_BIND, 'bind'))
-        jid = bind.addElement('jid', content='test@localhost/Resource')
+        jid = bind.addElement('jid', content=('test@localhost/%s' % resource))
         self.xmlstream.send(result)
 
         self.xmlstream.dispatch(self.xmlstream, xmlstream.STREAM_AUTHD_EVENT)
@@ -196,6 +218,7 @@ def make_stream_event(type, stanza):
 def make_iq_event(iq):
     event = make_stream_event('stream-iq', iq)
     event.iq_type = iq.getAttribute("type")
+    event.iq_id = iq.getAttribute("id")
     query = iq.firstChildElement()
 
     if query:
@@ -205,6 +228,8 @@ def make_iq_event(iq):
 
         if query.getAttribute("node"):
             event.query_node = query.getAttribute("node")
+    else:
+        event.query = None
 
     return event
 
@@ -252,6 +277,7 @@ class BaseXmlStream(xmlstream.XmlStream):
             identity['type'] = 'pep'
 
             iq['type'] = 'result'
+            iq['from'] = iq['to']
             self.send(iq)
 
 class JabberXmlStream(BaseXmlStream):
@@ -261,8 +287,7 @@ class XmppXmlStream(BaseXmlStream):
     version = (1, 0)
 
 class GoogleXmlStream(BaseXmlStream):
-    # ???
-    version = (0, 9)
+    version = (1, 0)
 
     def _cb_disco_iq(self, iq):
         if iq.getAttribute('to') == 'localhost':
@@ -276,6 +301,7 @@ class GoogleXmlStream(BaseXmlStream):
             feature['var'] = ns.GOOGLE_JINGLE_INFO
 
             iq['type'] = 'result'
+            iq['from'] = 'localhost'
             self.send(iq)
 
 def make_connection(bus, event_func, params=None):
@@ -285,6 +311,7 @@ def make_connection(bus, event_func, params=None):
         'resource': 'Resource',
         'server': 'localhost',
         'port': dbus.UInt32(4242),
+        'fallback-socks5-proxies': dbus.Array([], signature='s'),
         }
 
     if params:
@@ -293,14 +320,14 @@ def make_connection(bus, event_func, params=None):
     return servicetest.make_connection(bus, event_func, 'gabble', 'jabber',
         default_params)
 
-def make_stream(event_func, authenticator=None, protocol=None, port=4242):
+def make_stream(event_func, authenticator=None, protocol=None, port=4242, resource=None):
     # set up Jabber server
 
     if authenticator is None:
-        authenticator = JabberAuthenticator('test', 'pass')
+        authenticator = XmppAuthenticator('test', 'pass', resource=resource)
 
     if protocol is None:
-        protocol = JabberXmlStream
+        protocol = XmppXmlStream
 
     stream = protocol(event_func, authenticator)
     factory = twisted.internet.protocol.Factory()
@@ -308,34 +335,6 @@ def make_stream(event_func, authenticator=None, protocol=None, port=4242):
     port = reactor.listenTCP(port, factory)
     return (stream, port)
 
-def go(params=None, authenticator=None, protocol=None, start=None):
-    # hack to ease debugging
-    domish.Element.__repr__ = domish.Element.toXml
-
-    bus = dbus.SessionBus()
-    handler = servicetest.EventTest()
-    conn = make_connection(bus, handler.handle_event, params)
-    (stream, _) = make_stream(handler.handle_event, authenticator, protocol)
-    handler.data = {
-        'bus': bus,
-        'conn': conn,
-        'conn_iface': dbus.Interface(conn,
-            'org.freedesktop.Telepathy.Connection'),
-        'stream': stream}
-    handler.data['test'] = handler
-    handler.verbose = (os.environ.get('CHECK_TWISTED_VERBOSE', '') != '')
-    map(handler.expect, servicetest.load_event_handlers())
-
-    if '-v' in sys.argv:
-        handler.verbose = True
-
-    if start is None:
-        handler.data['conn'].Connect()
-    else:
-        start(handler.data)
-
-    reactor.run()
-
 def install_colourer():
     def red(s):
         return '\x1b[31m%s\x1b[0m' % s
@@ -360,8 +359,17 @@ def install_colourer():
     sys.stdout = Colourer(sys.stdout, patterns)
     return sys.stdout
 
+def disconnect_conn(q, conn, stream, expected=[]):
+    conn.Disconnect()
 
-def exec_test_deferred (funs, params, protocol=None, timeout=None):
+    tmp = expected + [EventPattern('dbus-signal', signal='StatusChanged',
+            args=[cs.CONN_STATUS_DISCONNECTED, cs.CSR_REQUESTED])]
+
+    events = q.expect_many(*tmp)
+    return events[:-1]
+
+def exec_test_deferred(funs, params, protocol=None, timeout=None,
+                        authenticator=None):
     # hack to ease debugging
     domish.Element.__repr__ = domish.Element.toXml
     colourer = None
@@ -376,7 +384,9 @@ def exec_test_deferred (funs, params, protocol=None, timeout=None):
 
     bus = dbus.SessionBus()
     # conn = make_connection(bus, queue.append, params)
-    (stream, port) = make_stream(queue.append, protocol=protocol)
+    resource = params.get('resource') if params is not None else None
+    (stream, port) = make_stream(queue.append, protocol=protocol,
+        authenticator=authenticator, resource=resource)
 
     error = None
 
@@ -389,74 +399,63 @@ def exec_test_deferred (funs, params, protocol=None, timeout=None):
         traceback.print_exc()
         error = e
 
+    if colourer:
+        sys.stdout = colourer.fh
+
+    d = port.stopListening()
+
+    if error is None:
+        d.addBoth((lambda *args: reactor.crash()))
+    else:
+        # please ignore the POSIX behind the curtain
+        d.addBoth((lambda *args: os._exit(1)))
+
     try:
-        if colourer:
-          sys.stdout = colourer.fh
-        d = port.stopListening()
-        if error is None:
-            d.addBoth((lambda *args: reactor.crash()))
-        else:
-            # please ignore the POSIX behind the curtain
-            d.addBoth((lambda *args: os._exit(1)))
-
-        conn.Disconnect()
-
+        disconnect_conn(queue, conn, stream)
     except dbus.DBusException, e:
+        # Connection has already been disconnected
         pass
 
-def exec_tests(funs, params=None, protocol=None, timeout=None):
-  reactor.callWhenRunning (exec_test_deferred, funs, params, protocol, timeout)
-  reactor.run()
+def exec_tests(funs, params=None, protocol=None, timeout=None,
+               authenticator=None):
+    reactor.callWhenRunning(
+        exec_test_deferred, funs, params, protocol, timeout, authenticator)
+    reactor.run()
 
-def exec_test(fun, params=None, protocol=None, timeout=None):
-  exec_tests([fun], params, protocol, timeout)
+def exec_test(fun, params=None, protocol=None, timeout=None,
+              authenticator=None):
+  exec_tests([fun], params, protocol, timeout, authenticator)
 
 # Useful routines for server-side vCard handling
 current_vcard = domish.Element(('vcard-temp', 'vCard'))
 
-def handle_get_vcard(event, data):
-    iq = event.stanza
+def expect_and_handle_get_vcard(q, stream):
+    get_vcard_event = q.expect('stream-iq', query_ns=ns.VCARD_TEMP,
+        query_name='vCard', iq_type='get')
 
-    if iq['type'] != 'get':
-        return False
-
-    if iq.uri != 'jabber:client':
-        return False
-
-    vcard = list(iq.elements())[0]
-
-    if vcard.name != 'vCard':
-        return False
+    iq = get_vcard_event.stanza
+    vcard = iq.firstChildElement()
+    assert vcard.name == 'vCard', vcard.toXml()
 
     # Send back current vCard
-    new_iq = IQ(data['stream'], 'result')
-    new_iq['id'] = iq['id']
-    new_iq.addChild(current_vcard)
-    data['stream'].send(new_iq)
-    return True
+    result = make_result_iq(stream, iq)
+    result.addChild(current_vcard)
+    stream.send(result)
 
-def handle_set_vcard(event, data):
-    global current_vcard
-    iq = event.stanza
+def expect_and_handle_set_vcard(q, stream, check=None):
+    set_vcard_event = q.expect('stream-iq', query_ns=ns.VCARD_TEMP,
+        query_name='vCard', iq_type='set')
+    iq = set_vcard_event.stanza
+    vcard = iq.firstChildElement()
+    assert vcard.name == 'vCard', vcard.toXml()
 
-    if iq['type'] != 'set':
-        return False
+    if check is not None:
+        check(vcard)
 
-    if iq.uri != 'jabber:client':
-        return False
-
-    vcard = list(iq.elements())[0]
-
-    if vcard.name != 'vCard':
-        return False
-
-    current_vcard = iq.firstChildElement()
-
-    new_iq = IQ(data['stream'], 'result')
-    new_iq['id'] = iq['id']
-    data['stream'].send(new_iq)
-    return True
+    # Update current vCard
+    current_vcard = vcard
 
+    stream.send(make_result_iq(stream, iq))
 
 def _elem_add(elem, *children):
     for child in children:
@@ -465,7 +464,8 @@ def _elem_add(elem, *children):
         elif isinstance(child, unicode):
             elem.addContent(child)
         else:
-            raise ValueError('invalid child object %r', child)
+            raise ValueError(
+                'invalid child object %r (must be element or unicode)', child)
 
 def elem(a, b=None, **kw):
     r"""
@@ -513,3 +513,95 @@ def elem_iq(server, type, **kw):
             iq[k] = v
 
     return iq
+
+def make_presence(_from, to='test@localhost', type=None, status=None, caps=None):
+    presence = domish.Element((None, 'presence'))
+    presence['from'] = _from
+    presence['to'] = to
+
+    if type is not None:
+        presence['type'] = type
+
+    if status is not None:
+        presence.addElement('status', content=status)
+
+    if caps is not None:
+        cel = presence.addElement(('http://jabber.org/protocol/caps', 'c'))
+        for key,value in caps.items():
+            cel[key] = value
+
+    return presence
+
+def expect_list_channel(q, bus, conn, name, contacts, lp_contacts=[],
+                        rp_contacts=[]):
+    return expect_contact_list_channel(q, bus, conn, cs.HT_LIST, name,
+        contacts, lp_contacts=lp_contacts, rp_contacts=rp_contacts)
+
+def expect_group_channel(q, bus, conn, name, contacts, lp_contacts=[],
+                         rp_contacts=[]):
+    return expect_contact_list_channel(q, bus, conn, cs.HT_GROUP, name,
+        contacts, lp_contacts=lp_contacts, rp_contacts=rp_contacts)
+
+def expect_contact_list_channel(q, bus, conn, ht, name, contacts,
+                                lp_contacts=[], rp_contacts=[]):
+    """
+    Expects NewChannel and NewChannels signals for the
+    contact list with handle type 'ht' and ID 'name', and checks that its
+    members, lp members and rp members are exactly 'contacts', 'lp_contacts'
+    and 'rp_contacts'.
+    Returns a proxy for the channel.
+    """
+
+    old_signal, new_signal = q.expect_many(
+            EventPattern('dbus-signal', signal='NewChannel'),
+            EventPattern('dbus-signal', signal='NewChannels'),
+            )
+
+    path, type, handle_type, handle, suppress_handler = old_signal.args
+
+    assertEquals(cs.CHANNEL_TYPE_CONTACT_LIST, type)
+    assertEquals(name, conn.InspectHandles(handle_type, [handle])[0])
+
+    chan = wrap_channel(bus.get_object(conn.bus_name, path),
+        cs.CHANNEL_TYPE_CONTACT_LIST)
+    members = chan.Group.GetMembers()
+
+    assertEquals(sorted(contacts),
+        sorted(conn.InspectHandles(cs.HT_CONTACT, members)))
+
+    lp_handles = conn.RequestHandles(cs.HT_CONTACT, lp_contacts)
+    rp_handles = conn.RequestHandles(cs.HT_CONTACT, rp_contacts)
+
+    # NB. comma: we're unpacking args. Thython!
+    info, = new_signal.args
+    assertLength(1, info) # one channel
+    path_, emitted_props = info[0]
+
+    assertEquals(path_, path)
+
+    assertEquals(cs.CHANNEL_TYPE_CONTACT_LIST, emitted_props[cs.CHANNEL_TYPE])
+    assertEquals(ht, emitted_props[cs.TARGET_HANDLE_TYPE])
+    assertEquals(handle, emitted_props[cs.TARGET_HANDLE])
+
+    channel_props = chan.Properties.GetAll(cs.CHANNEL)
+    assertEquals(handle, channel_props.get('TargetHandle'))
+    assertEquals(ht, channel_props.get('TargetHandleType'))
+    assertEquals(cs.CHANNEL_TYPE_CONTACT_LIST, channel_props.get('ChannelType'))
+    assertContains(cs.CHANNEL_IFACE_GROUP, channel_props.get('Interfaces'))
+    assertEquals(name, channel_props['TargetID'])
+    assertEquals(False, channel_props['Requested'])
+    assertEquals('', channel_props['InitiatorID'])
+    assertEquals(0, channel_props['InitiatorHandle'])
+
+    group_props = chan.Properties.GetAll(cs.CHANNEL_IFACE_GROUP)
+    assertContains('HandleOwners', group_props)
+    assertContains('Members', group_props)
+    assertEquals(members, group_props['Members'])
+    assertContains('LocalPendingMembers', group_props)
+    actual_lp_handles = [x[0] for x in group_props['LocalPendingMembers']]
+    assertEquals(sorted(lp_handles), sorted(actual_lp_handles))
+    assertContains('RemotePendingMembers', group_props)
+    assertEquals(sorted(rp_handles), sorted(group_props['RemotePendingMembers']))
+    assertContains('GroupFlags', group_props)
+
+    return chan
