@@ -2,68 +2,132 @@
 Test the Hold API.
 """
 
-from gabbletest import exec_test, make_result_iq, acknowledge_iq, sync_stream
-from servicetest import make_channel_proxy, unwrap, tp_path_prefix, \
-        call_async, EventPattern
-import jingletest
-import gabbletest
-import dbus
-import time
+from gabbletest import make_result_iq, sync_stream
+from servicetest import (
+    assertEquals, wrap_channel,
+    make_channel_proxy, call_async, EventPattern, sync_dbus, tp_path_prefix,
+    )
+
+import constants as cs
+
+from jingletest2 import JingleTest2, test_all_dialects
+
+def mutable_stream_tests(jp, jt, q, bus, conn, stream, chan, handle):
+    # ---- Test 13: while the call's on hold, we add a new stream ---
+    # We shouldn't go off hold locally as a result, and the new StreamHandler
+    # should tell s-e to hold the stream.
+
+    pending_hold = [
+        EventPattern('dbus-signal', signal='HoldStateChanged',
+            predicate=lambda e: e.args[0] == cs.HS_PENDING_HOLD),
+        ]
+    q.forbid_events(pending_hold)
+
+    call_async(q, chan.StreamedMedia, 'RequestStreams', handle,
+        [cs.MEDIA_STREAM_TYPE_AUDIO])
+
+    e = q.expect('dbus-signal', signal='NewStreamHandler')
+    audio_stream_path = e.args[0]
+    audio_stream_handler = make_channel_proxy(conn, e.args[0],
+            'Media.StreamHandler')
+
+    # Syncing here to make sure SetStreamHeld happens after Ready...
+    sync_dbus(bus, q, conn)
+
+    audio_stream_handler.Ready(jt.get_audio_codecs_dbus())
+    audio_stream_handler.NewNativeCandidate("fake",
+            jt.get_remote_transports_dbus())
+    audio_stream_handler.StreamState(cs.MEDIA_STREAM_STATE_CONNECTED)
+
+    path_suffix = audio_stream_path[len(tp_path_prefix):]
+
+    q.expect_many(
+        EventPattern('dbus-signal', signal='SetStreamHeld', args=[True],
+            path=path_suffix),
+        EventPattern('dbus-signal', signal='SetStreamSending', args=[False],
+            path=path_suffix),
+        )
+
+    assertEquals(cs.HS_HELD, chan.Hold.GetHoldState()[0])
+
+    sync_dbus(bus, q, conn)
+
+    # ---- Test 14: while the call's on hold, the peer adds a new stream ----
+    # Again, we shouldn't go off hold locally as a result, and the new
+    # StreamHandler should tell s-e to hold the stream.
+
+    node = jp.SetIq(jt.peer, jt.jid, [
+        jp.Jingle(jt.sid, jt.peer, 'content-add', [
+            jp.Content('videostream', 'initiator', 'both', [
+                jp.Description('video', [
+                    jp.PayloadType(name, str(rate), str(id)) for
+                        (name, id, rate) in jt.video_codecs ]),
+            jp.TransportGoogleP2P() ]) ]) ])
+    stream.send(jp.xml(node))
+
+    e = q.expect('dbus-signal', signal='NewStreamHandler')
+    video_stream_path = e.args[0]
+    video_stream_handler = make_channel_proxy(conn, e.args[0],
+            'Media.StreamHandler')
+
+    # Syncing here to make sure SetStreamHeld happens after Ready...
+    sync_dbus(bus, q, conn)
+
+    video_stream_handler.Ready(jt.get_video_codecs_dbus())
+    video_stream_handler.NewNativeCandidate("fake",
+            jt.get_remote_transports_dbus())
+    video_stream_handler.StreamState(cs.MEDIA_STREAM_STATE_CONNECTED)
+
+    path_suffix = video_stream_path[len(tp_path_prefix):]
+
+    q.expect_many(
+        EventPattern('dbus-signal', signal='SetStreamHeld', args=[True],
+            path=path_suffix),
+        EventPattern('dbus-signal', signal='SetStreamSending', args=[False],
+            path=path_suffix),
+        )
+
+    assertEquals(cs.HS_HELD, chan.Hold.GetHoldState()[0])
+
+    sync_dbus(bus, q, conn)
+    q.unforbid_events(pending_hold)
 
 
-MEDIA_STREAM_TYPE_AUDIO = 0
-MEDIA_STREAM_TYPE_VIDEO = 0
-# Hold states
-S_UNHELD = 0
-S_HELD = 1
-S_PENDING_HOLD = 2
-S_PENDING_UNHOLD = 3
-# Reasons
-R_NONE = 0
-R_REQUESTED = 1
-R_RESOURCE_NOT_AVAILABLE = 2
 
+def test(jp, q, bus, conn, stream):
+    # These are 0- (for old dialects) or 1- (for new dialects) element lists
+    # that can be splatted into expect_many with *
+    hold_event = jp.rtp_info_event_list("hold")
+    unhold_event = jp.rtp_info_event_list("unhold")
 
-def test(q, bus, conn, stream):
-    jt = jingletest.JingleTest(stream, 'test@localhost', 'foo@bar.com/Foo')
+    # Let's forbid them until we're ready to start holding, to check that
+    # Gabble doesn't send spurious notifications.
+    q.forbid_events(hold_event)
+    q.forbid_events(unhold_event)
 
-    # If we need to override remote caps, feats, codecs or caps,
-    # this is a good time to do it
+    jt = JingleTest2(jp, conn, q, stream, 'test@localhost', 'foo@bar.com/Foo')
 
-    # Connecting
-    conn.Connect()
+    jt.prepare()
 
-    q.expect('dbus-signal', signal='StatusChanged', args=[1, 1])
-    q.expect('stream-authenticated')
-    q.expect('dbus-signal', signal='PresenceUpdate',
-        args=[{1L: (0L, {u'available': {}})}])
-    q.expect('dbus-signal', signal='StatusChanged', args=[0, 1])
+    self_handle = conn.GetSelfHandle()
+    handle = conn.RequestHandles(cs.HT_CONTACT, [jt.peer])[0]
+    path = conn.RequestChannel(cs.CHANNEL_TYPE_STREAMED_MEDIA,
+        cs.HT_CONTACT, handle, True)
 
-    # We need remote end's presence for capabilities
-    jt.send_remote_presence()
+    chan = wrap_channel(bus.get_object(conn.bus_name, path), 'StreamedMedia',
+        ['Hold'])
 
-    # Gabble doesn't trust it, so makes a disco
-    event = q.expect('stream-iq', query_ns='http://jabber.org/protocol/disco#info',
-             to='foo@bar.com/Foo')
+    call_async(q, chan.StreamedMedia, 'RequestStreams', handle,
+        [cs.MEDIA_STREAM_TYPE_AUDIO, cs.MEDIA_STREAM_TYPE_VIDEO])
 
-    jt.send_remote_disco_reply(event.stanza)
-
-    # Force Gabble to process the caps before calling RequestChannel
-    sync_stream(q, stream)
-
-    handle = conn.RequestHandles(1, [jt.remote_jid])[0]
-
-    path = conn.RequestChannel(
-        'org.freedesktop.Telepathy.Channel.Type.StreamedMedia',
-        1, handle, True)
-
-    signalling_iface = make_channel_proxy(conn, path, 'Channel.Interface.MediaSignalling')
-    media_iface = make_channel_proxy(conn, path, 'Channel.Type.StreamedMedia')
-    group_iface = make_channel_proxy(conn, path, 'Channel.Interface.Group')
-    hold_iface = make_channel_proxy(conn, path, 'Channel.Interface.Hold')
-
-    media_iface.RequestStreams(handle, [MEDIA_STREAM_TYPE_AUDIO,
-        MEDIA_STREAM_TYPE_VIDEO])
+    if not jp.can_do_video():
+        # Video on GTalk? Not so much.
+        e = q.expect('dbus-error', method='RequestStreams')
+        # The spec and implemention say this should be NotAvailable, but wjt
+        # thinks it should be NotCapable. The spec bug is #20920.
+        name = e.error.get_dbus_name()
+        #assert name == cs.NOT_CAPABLE, name
+        return
 
     # S-E gets notified about new session handler, and calls Ready on it
     e = q.expect('dbus-signal', signal='NewSessionHandler')
@@ -84,7 +148,7 @@ def test(q, bus, conn, stream):
     audio_stream_handler.NewNativeCandidate("fake",
             jt.get_remote_transports_dbus())
     audio_stream_handler.Ready(jt.get_audio_codecs_dbus())
-    audio_stream_handler.StreamState(2)
+    audio_stream_handler.StreamState(cs.MEDIA_STREAM_STATE_CONNECTED)
 
     e = q.expect('dbus-signal', signal='NewStreamHandler')
 
@@ -95,31 +159,35 @@ def test(q, bus, conn, stream):
     video_stream_handler.NewNativeCandidate("fake",
             jt.get_remote_transports_dbus())
     video_stream_handler.Ready(jt.get_video_codecs_dbus())
-    video_stream_handler.StreamState(2)
+    video_stream_handler.StreamState(cs.MEDIA_STREAM_STATE_CONNECTED)
 
-    e = q.expect('stream-iq')
-    assert e.query.name == 'jingle'
-    assert e.query['action'] == 'session-initiate'
-    stream.send(gabbletest.make_result_iq(stream, e.stanza))
+    e = q.expect('stream-iq', predicate=jp.action_predicate('session-initiate'))
+    stream.send(make_result_iq(stream, e.stanza))
 
-    jt.outgoing_call_reply(e.query['sid'], True)
+    jt.parse_session_initiate(e.query)
+    jt.accept(with_video=True)
 
     q.expect('stream-iq', iq_type='result')
 
     # ---- Test 1: GetHoldState returns unheld and unhold is a no-op ----
 
-    hold_state = hold_iface.GetHoldState()
-    assert hold_state[0] == S_UNHELD, hold_state
-    hold_iface.RequestHold(False)
+    hold_state = chan.Hold.GetHoldState()
+    assert hold_state[0] == cs.HS_UNHELD, hold_state
+    chan.Hold.RequestHold(False)
+
+    # We're about to start holding, so remove the ban on <hold/>.
+    sync_stream(q, stream)
+    q.unforbid_events(hold_event)
 
     # ---- Test 2: successful hold ----
 
-    call_async(q, hold_iface, 'RequestHold', True)
+    call_async(q, chan.Hold, 'RequestHold', True)
     q.expect_many(
         EventPattern('dbus-signal', signal='HoldStateChanged',
-            args=[S_PENDING_HOLD, R_REQUESTED]),
+            args=[cs.HS_PENDING_HOLD, cs.HSR_REQUESTED]),
         EventPattern('dbus-signal', signal='SetStreamHeld', args=[True]),
         EventPattern('dbus-return', method='RequestHold', value=()),
+        *hold_event
         )
 
     call_async(q, audio_stream_handler, 'HoldState', True)
@@ -127,52 +195,62 @@ def test(q, bus, conn, stream):
     q.expect_many(
         EventPattern('dbus-return', method='HoldState', value=()),
         EventPattern('dbus-signal', signal='HoldStateChanged',
-            args=[S_HELD, R_REQUESTED]),
+            args=[cs.HS_HELD, cs.HSR_REQUESTED]),
         )
 
     # ---- Test 3: GetHoldState returns held and hold is a no-op ----
 
-    hold_state = hold_iface.GetHoldState()
-    assert hold_state[0] == S_HELD, hold_state
-    hold_iface.RequestHold(True)
+    q.forbid_events(hold_event)
+
+    hold_state = chan.Hold.GetHoldState()
+    assert hold_state[0] == cs.HS_HELD, hold_state
+    chan.Hold.RequestHold(True)
+
+    sync_stream(q, stream)
+    q.unforbid_events(hold_event)
 
     # ---- Test 4: successful unhold ----
 
-    call_async(q, hold_iface, 'RequestHold', False)
+    call_async(q, chan.Hold, 'RequestHold', False)
     q.expect_many(
         EventPattern('dbus-signal', signal='HoldStateChanged',
-            args=[S_PENDING_UNHOLD, R_REQUESTED]),
+            args=[cs.HS_PENDING_UNHOLD, cs.HSR_REQUESTED]),
         EventPattern('dbus-signal', signal='SetStreamHeld', args=[False]),
         EventPattern('dbus-return', method='RequestHold', value=()),
         )
 
+    sync_stream(q, stream)
+    q.unforbid_events(unhold_event)
+
     call_async(q, audio_stream_handler, 'HoldState', False)
     call_async(q, video_stream_handler, 'HoldState', False)
     q.expect_many(
         EventPattern('dbus-return', method='HoldState', value=()),
         EventPattern('dbus-signal', signal='HoldStateChanged',
-            args=[S_UNHELD, R_REQUESTED]),
+            args=[cs.HS_UNHELD, cs.HSR_REQUESTED]),
+        *unhold_event
         )
 
     # ---- Test 5: GetHoldState returns False and unhold is a no-op ----
 
-    hold_state = hold_iface.GetHoldState()
-    assert hold_state[0] == S_UNHELD, hold_state
-    hold_iface.RequestHold(False)
+    hold_state = chan.Hold.GetHoldState()
+    assert hold_state[0] == cs.HS_UNHELD, hold_state
+    chan.Hold.RequestHold(False)
 
     # ---- Test 6: 3 parallel calls to hold ----
 
-    hold_state = hold_iface.GetHoldState()
-    assert hold_state[0] == S_UNHELD, hold_state
+    hold_state = chan.Hold.GetHoldState()
+    assert hold_state[0] == cs.HS_UNHELD, hold_state
 
-    call_async(q, hold_iface, 'RequestHold', True)
-    call_async(q, hold_iface, 'RequestHold', True)
-    call_async(q, hold_iface, 'RequestHold', True)
+    call_async(q, chan.Hold, 'RequestHold', True)
+    call_async(q, chan.Hold, 'RequestHold', True)
+    call_async(q, chan.Hold, 'RequestHold', True)
     q.expect_many(
         EventPattern('dbus-signal', signal='HoldStateChanged',
-            args=[S_PENDING_HOLD, R_REQUESTED]),
+            args=[cs.HS_PENDING_HOLD, cs.HSR_REQUESTED]),
         EventPattern('dbus-signal', signal='SetStreamHeld', args=[True]),
         EventPattern('dbus-return', method='RequestHold', value=()),
+        *hold_event
         )
 
     call_async(q, audio_stream_handler, 'HoldState', True)
@@ -180,93 +258,112 @@ def test(q, bus, conn, stream):
     q.expect_many(
         EventPattern('dbus-return', method='HoldState', value=()),
         EventPattern('dbus-signal', signal='HoldStateChanged',
-            args=[S_HELD, R_REQUESTED]),
+            args=[cs.HS_HELD, cs.HSR_REQUESTED]),
         )
 
     # ---- Test 7: 3 parallel calls to unhold ----
 
-    call_async(q, hold_iface, 'RequestHold', False)
-    call_async(q, hold_iface, 'RequestHold', False)
-    call_async(q, hold_iface, 'RequestHold', False)
+    q.forbid_events(unhold_event)
+
+    call_async(q, chan.Hold, 'RequestHold', False)
+    call_async(q, chan.Hold, 'RequestHold', False)
+    call_async(q, chan.Hold, 'RequestHold', False)
     q.expect_many(
         EventPattern('dbus-signal', signal='HoldStateChanged',
-            args=[S_PENDING_UNHOLD, R_REQUESTED]),
+            args=[cs.HS_PENDING_UNHOLD, cs.HSR_REQUESTED]),
         EventPattern('dbus-signal', signal='SetStreamHeld', args=[False]),
         EventPattern('dbus-return', method='RequestHold', value=()),
         )
 
+    sync_stream(q, stream)
+    q.unforbid_events(unhold_event)
+
     call_async(q, audio_stream_handler, 'HoldState', False)
     call_async(q, video_stream_handler, 'HoldState', False)
     q.expect_many(
         EventPattern('dbus-return', method='HoldState', value=()),
         EventPattern('dbus-signal', signal='HoldStateChanged',
-            args=[S_UNHELD, R_REQUESTED]),
+            args=[cs.HS_UNHELD, cs.HSR_REQUESTED]),
+        *unhold_event
         )
 
     # ---- Test 8: hold, then change our minds before s-e has responded ----
 
-    hold_state = hold_iface.GetHoldState()
-    assert hold_state[0] == S_UNHELD, hold_state
+    hold_state = chan.Hold.GetHoldState()
+    assert hold_state[0] == cs.HS_UNHELD, hold_state
 
-    call_async(q, hold_iface, 'RequestHold', True)
-    call_async(q, hold_iface, 'RequestHold', False)
+    call_async(q, chan.Hold, 'RequestHold', True)
     q.expect_many(
         EventPattern('dbus-signal', signal='HoldStateChanged',
-            args=[S_PENDING_HOLD, R_REQUESTED]),
+            args=[cs.HS_PENDING_HOLD, cs.HSR_REQUESTED]),
         EventPattern('dbus-signal', signal='SetStreamHeld', args=[True]),
+        *hold_event
         )
+
+    # Gabble can't send <unhold/> until s-e confirms it has the resources
+    q.forbid_events(unhold_event)
+
+    call_async(q, chan.Hold, 'RequestHold', False)
     q.expect_many(
         EventPattern('dbus-signal', signal='HoldStateChanged',
-            args=[S_PENDING_UNHOLD, R_REQUESTED]),
+            args=[cs.HS_PENDING_UNHOLD, cs.HSR_REQUESTED]),
         EventPattern('dbus-signal', signal='SetStreamHeld', args=[False]),
         )
 
     call_async(q, audio_stream_handler, 'HoldState', True)
     call_async(q, video_stream_handler, 'HoldState', True)
     call_async(q, audio_stream_handler, 'HoldState', False)
+
+    sync_stream(q, stream)
+    q.unforbid_events(unhold_event)
+
     call_async(q, video_stream_handler, 'HoldState', False)
     q.expect_many(
         EventPattern('dbus-return', method='HoldState', value=()),
         EventPattern('dbus-signal', signal='HoldStateChanged',
-            args=[S_UNHELD, R_REQUESTED]),
+            args=[cs.HS_UNHELD, cs.HSR_REQUESTED]),
+        *unhold_event
         )
 
-    hold_state = hold_iface.GetHoldState()
-    assert hold_state[0] == S_UNHELD, hold_state
+    hold_state = chan.Hold.GetHoldState()
+    assert hold_state[0] == cs.HS_UNHELD, hold_state
 
     # ---- Test 9: unhold, then change our minds before s-e has responded ----
 
     # Go to state "held" first
-    call_async(q, hold_iface, 'RequestHold', True)
+    call_async(q, chan.Hold, 'RequestHold', True)
     q.expect_many(
         EventPattern('dbus-signal', signal='HoldStateChanged',
-            args=[S_PENDING_HOLD, R_REQUESTED]),
+            args=[cs.HS_PENDING_HOLD, cs.HSR_REQUESTED]),
         EventPattern('dbus-signal', signal='SetStreamHeld', args=[True]),
         EventPattern('dbus-return', method='RequestHold', value=()),
+        *hold_event
         )
     call_async(q, audio_stream_handler, 'HoldState', True)
     call_async(q, video_stream_handler, 'HoldState', True)
     q.expect_many(
         EventPattern('dbus-return', method='HoldState', value=()),
         EventPattern('dbus-signal', signal='HoldStateChanged',
-            args=[S_HELD, R_REQUESTED]),
+            args=[cs.HS_HELD, cs.HSR_REQUESTED]),
         )
 
     # Actually do test 9
 
-    hold_state = hold_iface.GetHoldState()
-    assert hold_state[0] == S_HELD, hold_state
+    q.forbid_events(hold_event + unhold_event)
 
-    call_async(q, hold_iface, 'RequestHold', False)
-    call_async(q, hold_iface, 'RequestHold', True)
+    hold_state = chan.Hold.GetHoldState()
+    assert hold_state[0] == cs.HS_HELD, hold_state
+
+    call_async(q, chan.Hold, 'RequestHold', False)
+    call_async(q, chan.Hold, 'RequestHold', True)
     q.expect_many(
         EventPattern('dbus-signal', signal='HoldStateChanged',
-            args=[S_PENDING_UNHOLD, R_REQUESTED]),
+            args=[cs.HS_PENDING_UNHOLD, cs.HSR_REQUESTED]),
         EventPattern('dbus-signal', signal='SetStreamHeld', args=[False]),
         )
     q.expect_many(
         EventPattern('dbus-signal', signal='HoldStateChanged',
-            args=[S_PENDING_HOLD, R_REQUESTED]),
+            args=[cs.HS_PENDING_HOLD, cs.HSR_REQUESTED]),
         EventPattern('dbus-signal', signal='SetStreamHeld', args=[True]),
         )
 
@@ -277,18 +374,23 @@ def test(q, bus, conn, stream):
     q.expect_many(
         EventPattern('dbus-return', method='HoldState', value=()),
         EventPattern('dbus-signal', signal='HoldStateChanged',
-            args=[S_HELD, R_REQUESTED]),
+            args=[cs.HS_HELD, cs.HSR_REQUESTED]),
         )
 
-    hold_state = hold_iface.GetHoldState()
-    assert hold_state[0] == S_HELD, hold_state
+    sync_stream(q, stream)
+    q.unforbid_events(hold_event + unhold_event)
+
+    hold_state = chan.Hold.GetHoldState()
+    assert hold_state[0] == cs.HS_HELD, hold_state
 
     # ---- Test 10: attempting to unhold fails (both streams) ----
 
-    call_async(q, hold_iface, 'RequestHold', False)
+    q.forbid_events(hold_event + unhold_event)
+
+    call_async(q, chan.Hold, 'RequestHold', False)
     q.expect_many(
         EventPattern('dbus-signal', signal='HoldStateChanged',
-            args=[S_PENDING_UNHOLD, R_REQUESTED]),
+            args=[cs.HS_PENDING_UNHOLD, cs.HSR_REQUESTED]),
         EventPattern('dbus-signal', signal='SetStreamHeld', args=[False]),
         EventPattern('dbus-return', method='RequestHold', value=()),
         )
@@ -299,15 +401,20 @@ def test(q, bus, conn, stream):
     q.expect_many(
         EventPattern('dbus-return', method='UnholdFailure', value=()),
         EventPattern('dbus-signal', signal='HoldStateChanged',
-            args=[S_HELD, R_RESOURCE_NOT_AVAILABLE]),
+            args=[cs.HS_HELD, cs.HSR_RESOURCE_NOT_AVAILABLE]),
         )
 
+    sync_stream(q, stream)
+    q.unforbid_events(hold_event + unhold_event)
+
     # ---- Test 11: attempting to unhold fails (first stream) ----
 
-    call_async(q, hold_iface, 'RequestHold', False)
+    q.forbid_events(hold_event + unhold_event)
+
+    call_async(q, chan.Hold, 'RequestHold', False)
     q.expect_many(
         EventPattern('dbus-signal', signal='HoldStateChanged',
-            args=[S_PENDING_UNHOLD, R_REQUESTED]),
+            args=[cs.HS_PENDING_UNHOLD, cs.HSR_REQUESTED]),
         EventPattern('dbus-signal', signal='SetStreamHeld', args=[False]),
         EventPattern('dbus-return', method='RequestHold', value=()),
         )
@@ -317,15 +424,20 @@ def test(q, bus, conn, stream):
     q.expect_many(
         EventPattern('dbus-return', method='UnholdFailure', value=()),
         EventPattern('dbus-signal', signal='HoldStateChanged',
-            args=[S_HELD, R_RESOURCE_NOT_AVAILABLE]),
+            args=[cs.HS_HELD, cs.HSR_RESOURCE_NOT_AVAILABLE]),
         )
 
+    sync_stream(q, stream)
+    q.unforbid_events(hold_event + unhold_event)
+
     # ---- Test 12: attempting to unhold partially fails, so roll back ----
 
-    call_async(q, hold_iface, 'RequestHold', False)
+    q.forbid_events(hold_event + unhold_event)
+
+    call_async(q, chan.Hold, 'RequestHold', False)
     q.expect_many(
         EventPattern('dbus-signal', signal='HoldStateChanged',
-            args=[S_PENDING_UNHOLD, R_REQUESTED]),
+            args=[cs.HS_PENDING_UNHOLD, cs.HSR_REQUESTED]),
         EventPattern('dbus-signal', signal='SetStreamHeld', args=[False]),
         EventPattern('dbus-return', method='RequestHold', value=()),
         )
@@ -339,7 +451,7 @@ def test(q, bus, conn, stream):
         EventPattern('dbus-signal', signal='SetStreamHeld', args=[True]),
         EventPattern('dbus-return', method='UnholdFailure', value=()),
         EventPattern('dbus-signal', signal='HoldStateChanged',
-            args=[S_PENDING_HOLD, R_RESOURCE_NOT_AVAILABLE]),
+            args=[cs.HS_PENDING_HOLD, cs.HSR_RESOURCE_NOT_AVAILABLE]),
         )
 
     call_async(q, audio_stream_handler, 'HoldState', True)
@@ -347,23 +459,22 @@ def test(q, bus, conn, stream):
     q.expect_many(
         EventPattern('dbus-return', method='HoldState', value=()),
         EventPattern('dbus-signal', signal='HoldStateChanged',
-            args=[S_HELD, R_RESOURCE_NOT_AVAILABLE]),
+            args=[cs.HS_HELD, cs.HSR_RESOURCE_NOT_AVAILABLE]),
         )
 
+    sync_stream(q, stream)
+    q.unforbid_events(hold_event + unhold_event)
+
+    if jp.has_mutable_streams():
+        mutable_stream_tests(jp, jt, q, bus, conn, stream, chan, handle)
+
     # ---- The end ----
 
-    group_iface.RemoveMembers([dbus.UInt32(1)], 'closed')
+    chan.Group.RemoveMembers([self_handle], 'closed')
 
     # Test completed, close the connection
 
     e = q.expect('dbus-signal', signal='Close') #XXX - match against the path
 
-    conn.Disconnect()
-    q.expect('dbus-signal', signal='StatusChanged', args=[2, 1])
-
-    return True
-
-
 if __name__ == '__main__':
-    exec_test(test)
-
+    test_all_dialects(test)
