@@ -2,17 +2,15 @@
 Test everything related to contents
 """
 
-from gabbletest import exec_test, make_result_iq, sync_stream, exec_tests
-from servicetest import make_channel_proxy, unwrap, tp_path_prefix, \
-        EventPattern
-import gabbletest
-import dbus
-import time
-from twisted.words.xish import xpath
-
-from jingletest2 import *
-
+from gabbletest import sync_stream
+from servicetest import (
+    make_channel_proxy, tp_path_prefix, assertEquals, EventPattern,
+    )
 import constants as cs
+from jingletest2 import (
+    JingleTest2, JingleProtocol015, JingleProtocol031, test_dialects)
+
+from twisted.words.xish import xpath
 
 def worker(jp, q, bus, conn, stream):
 
@@ -37,23 +35,18 @@ def worker(jp, q, bus, conn, stream):
     remote_handle = conn.RequestHandles(cs.HT_CONTACT, ["foo@bar.com/Foo"])[0]
 
     # Remote end calls us
-    # jt.incoming_call()
-    node = jp.SetIq(jt2.peer, jt2.jid, [
-        jp.Jingle(jt2.sid, jt2.peer, 'session-initiate', [
-            jp.Content('stream1', 'initiator', 'both', [
-                jp.Description('audio', [
-                    jp.PayloadType(name, str(rate), str(id)) for
-                        (name, id, rate) in jt2.audio_codecs ]),
-            jp.TransportGoogleP2P() ]) ]) ])
-    stream.send(jp.xml(node))
+    jt2.incoming_call()
 
+    # FIXME: these signals are not observable by real clients, since they
+    #        happen before NewChannels.
     # The caller is in members
     e = q.expect('dbus-signal', signal='MembersChanged',
              args=[u'', [remote_handle], [], [], [], 0, 0])
 
     # We're pending because of remote_handle
     e = q.expect('dbus-signal', signal='MembersChanged',
-             args=[u'', [], [], [self_handle], [], remote_handle, 0])
+             args=[u'', [], [], [self_handle], [], remote_handle,
+                   cs.GC_REASON_INVITED])
 
     media_chan = make_channel_proxy(conn, tp_path_prefix + e.path, 'Channel.Interface.Group')
     signalling_iface = make_channel_proxy(conn, tp_path_prefix + e.path, 'Channel.Interface.MediaSignalling')
@@ -76,7 +69,8 @@ def worker(jp, q, bus, conn, stream):
 
     # We are now in members too
     e = q.expect('dbus-signal', signal='MembersChanged',
-             args=[u'', [self_handle], [], [], [], 0, 0])
+             args=[u'', [self_handle], [], [], [], self_handle,
+                   cs.GC_REASON_NONE])
 
     # we are now both in members
     members = media_chan.GetMembers()
@@ -87,9 +81,8 @@ def worker(jp, q, bus, conn, stream):
     stream_handler.StreamState(cs.MEDIA_STREAM_STATE_CONNECTED)
 
     # First one is transport-info
-    e = q.expect('stream-iq')
-    assert jp.match_jingle_action(e.query, 'transport-info')
-    assert e.query['initiator'] == 'foo@bar.com/Foo'
+    e = q.expect('stream-iq', predicate=jp.action_predicate('transport-info'))
+    assertEquals('foo@bar.com/Foo', e.query['initiator'])
 
     # stream.send(gabbletest.make_result_iq(stream, e.stanza))
     stream.send(jp.xml(jp.ResultIq('test@localhost', e.stanza, [])))
@@ -98,8 +91,7 @@ def worker(jp, q, bus, conn, stream):
     stream_handler.SupportedCodecs(jt2.get_audio_codecs_dbus())
 
     # Second one is session-accept
-    e = q.expect('stream-iq')
-    assert jp.match_jingle_action(e.query, 'session-accept')
+    e = q.expect('stream-iq', predicate=jp.action_predicate('session-accept'))
 
     # stream.send(gabbletest.make_result_iq(stream, e.stanza))
     stream.send(jp.xml(jp.ResultIq('test@localhost', e.stanza, [])))
@@ -132,7 +124,7 @@ def worker(jp, q, bus, conn, stream):
         q.expect('stream-iq', iq_type='error')
 
 
-    # Remote end then tries to create content which we already have
+    # Remote end then tries to create a content with a name it's already used
     node = jp.SetIq(jt2.peer, jt2.jid, [
         jp.Jingle(jt2.sid, jt2.peer, 'content-add', [
             jp.Content('stream1', 'initiator', 'both', [
@@ -145,6 +137,7 @@ def worker(jp, q, bus, conn, stream):
     # Gabble should return error (content already exists)
     q.expect('stream-iq', iq_type='error')
 
+    # We try to add a stream
     (stream_handler2, id2) = make_stream_request(cs.MEDIA_STREAM_TYPE_VIDEO)
 
     # Gabble should now send content-add
@@ -157,8 +150,20 @@ def worker(jp, q, bus, conn, stream):
 
     stream.send(jp.xml(jp.ResultIq('test@localhost', e.stanza, [])))
 
+    # We try to add yet another stream
+    (stream_handler3, id3) = make_stream_request(cs.MEDIA_STREAM_TYPE_VIDEO)
 
-    # Remote end rejects it
+    # Gabble should send another content-add
+    e = q.expect('stream-iq', iq_type='set', predicate=lambda x:
+        xpath.queryForNodes("/iq/jingle[@action='content-add']",
+            x.stanza))
+
+    d = e.query.firstChildElement()
+    assertEquals('responder', d['creator'])
+
+    stream.send(jp.xml(jp.ResultIq('test@localhost', e.stanza, [])))
+
+    # Remote end rejects the first stream we tried to add.
     node = jp.SetIq(jt2.peer, jt2.jid, [
         jp.Jingle(jt2.sid, jt2.peer, 'content-reject', [
             jp.Content(c['name'], c['creator'], c['senders'], []) ]) ])
@@ -166,23 +171,49 @@ def worker(jp, q, bus, conn, stream):
 
     # Gabble removes the stream
     q.expect('dbus-signal', signal='StreamRemoved',
-        interface='org.freedesktop.Telepathy.Channel.Type.StreamedMedia')
+        interface=cs.CHANNEL_TYPE_STREAMED_MEDIA)
 
+    # Remote end tries to add a content with the same name as the second one we
+    # just added
+    node = jp.SetIq(jt2.peer, jt2.jid, [
+        jp.Jingle(jt2.sid, jt2.peer, 'content-add', [
+            jp.Content(d['name'], 'initiator', 'both', [
+                jp.Description('audio', [
+                    jp.PayloadType(name, str(rate), str(id)) for
+                        (name, id, rate) in jt2.audio_codecs ]),
+            jp.TransportGoogleP2P() ]) ]) ])
+    stream.send(jp.xml(node))
 
-    # We try to make the request again, and succeed
-    (stream_handler3, id3) = make_stream_request(cs.MEDIA_STREAM_TYPE_VIDEO)
+    # Because stream names are namespaced by creator, Gabble should be okay
+    # with that.
+    q.expect_many(
+        EventPattern('stream-iq', iq_type='result', iq_id=node[2]['id']),
+        EventPattern('dbus-signal', signal='StreamAdded'),
+        )
 
-    # Gabble should again send content-add
-    e = q.expect('stream-iq', iq_type='set', predicate=lambda x:
-        xpath.queryForNodes("/iq/jingle[@action='content-add']",
-            x.stanza))
-    stream.send(jp.xml(jp.ResultIq('test@localhost', e.stanza, [])))
-    c = e.query.firstChildElement()
+    # Remote end thinks better of that, and removes the similarly-named stream
+    # it tried to add.
+    node = jp.SetIq(jt2.peer, jt2.jid, [
+        jp.Jingle(jt2.sid, jt2.peer, 'content-remove', [
+            jp.Content(d['name'], 'initiator', d['senders'], []) ]) ])
+    stream.send(jp.xml(node))
 
-    # Remote end finally accepts
+    q.expect_many(
+        EventPattern('stream-iq', iq_type='result', iq_id=node[2]['id']),
+        EventPattern('dbus-signal', signal='StreamRemoved'),
+        )
+
+    # Remote end finally accepts. When Gabble did not namespace contents by
+    # their creator, it would NAK this IQ:
+    #  - Gabble (responder) created a stream called 'foo';
+    #  - test suite (initiator) created a stream called 'foo', which Gabble
+    #    decided would replace its own stream called 'foo';
+    #  - test suite removed its 'foo';
+    #  - test suite accepted Gabble's 'foo', but Gabble didn't believe a stream
+    #    called 'foo' existed any more.
     node = jp.SetIq(jt2.peer, jt2.jid, [
         jp.Jingle(jt2.sid, jt2.peer, 'content-accept', [
-            jp.Content(c['name'], c['creator'], c['senders'], [
+            jp.Content(d['name'], d['creator'], d['senders'], [
                 jp.Description('video', [
                     jp.PayloadType(name, str(rate), str(id)) for
                         (name, id, rate) in jt2.audio_codecs ]),
@@ -192,43 +223,31 @@ def worker(jp, q, bus, conn, stream):
     # We get remote codecs
     e = q.expect('dbus-signal', signal='SetRemoteCodecs')
 
-    # Now, both we and remote peer try to remove the content simultaneously
+    # Now, both we and remote peer try to remove the content simultaneously:
+    # Telepathy client calls RemoveStreams...
     media_iface.RemoveStreams([id3])
+
+    # ...so Gabble sends a content-remove...
+    e = q.expect('stream-iq', iq_type='set', predicate=lambda x:
+        xpath.queryForNodes("/iq/jingle[@action='content-remove']",
+            x.stanza))
+
+    # ...but before it's acked the peer sends its own content-remove...
     node = jp.SetIq(jt2.peer, jt2.jid, [
         jp.Jingle(jt2.sid, jt2.peer, 'content-remove', [
             jp.Content(c['name'], c['creator'], c['senders'], []) ]) ])
     stream.send(jp.xml(node))
 
-    # Gabble should ignore our content-remove and send it's own
-    # (fixme: this could be racy)
-    e = q.expect('stream-iq', iq_type='set', predicate=lambda x:
-        xpath.queryForNodes("/iq/jingle[@action='content-remove']",
-            x.stanza))
+    # ...and we don't want Gabble to break when that happens.
+    sync_stream(q, stream)
 
     # Now we want to remove the first stream
     media_iface.RemoveStreams([id1])
 
-    # The remote peer still hasn't ackd the first stream removal, but since
-    # gabble knows no streams will be left after the removal completes,
-    # it will just terminate the session.
-
+    # Since this is the last stream, Gabble will just terminate the session.
     e = q.expect('stream-iq', iq_type='set', predicate=lambda x:
         xpath.queryForNodes("/iq/jingle[@action='session-terminate']",
             x.stanza))
 
-    conn.Disconnect()
-    q.expect('dbus-signal', signal='StatusChanged', args=[2, 1])
-
-    return True
-
-
-def test015(q, bus, conn, stream):
-    return worker(JingleProtocol015(), q, bus, conn, stream)
-
-def test031(q, bus, conn, stream):
-    return worker(JingleProtocol031(),q, bus, conn, stream)
-
 if __name__ == '__main__':
-    exec_test(test015)
-    exec_test(test031)
-
+    test_dialects(worker, [JingleProtocol015, JingleProtocol031])
