@@ -19,6 +19,8 @@
 
 #include "gabble.h"
 #include "connection.h"
+#include "jingle-factory.h"
+#include "jingle-session.h"
 
 #include <lib/gibber/gibber-resolver.h>
 
@@ -28,11 +30,13 @@ int
 main (int argc,
       char **argv)
 {
+  gabble_init ();
+
   /* needed for test-disco-no-reply.py */
-  gabble_connection_set_disco_reply_timeout (3000);
+  gabble_connection_set_disco_reply_timeout (3);
 
-  g_type_init ();
   gibber_resolver_set_resolver (GABBLE_TYPE_RESOLVER_FAKE);
+  gabble_jingle_factory_set_test_mode ();
 
   return gabble_main (argc, argv);
 }
