@@ -1,9 +1,10 @@
 import dbus
 
 from servicetest import call_async, EventPattern
-from gabbletest import make_result_iq, acknowledge_iq, elem, elem_iq
+from gabbletest import make_result_iq, elem, elem_iq
 from twisted.words.xish import domish, xpath
 from twisted.words.protocols.jabber.client import IQ
+import constants as cs
 import ns
 
 # Copied from Gadget
@@ -26,10 +27,10 @@ def parse_properties(elems):
             continue
 
         if type not in valid_types:
-            raise PropertyTypeError(type, elems.uri)
+            raise ValueError
 
         if type == 'bool' and value not in ['1', '0', 'true', 'false']:
-            raise PropertyTypeError(type, elems.uri)
+            raise ValueError
 
         properties[name] = (type, value)
 
@@ -58,7 +59,7 @@ def announce_gadget(q, stream, disco_stanza):
     # wait for Gadget disco#info query
     event = q.expect('stream-iq', to='gadget.localhost', query_ns=ns.DISCO_INFO)
 
-    reply = elem_iq(stream, 'result', id=event.stanza['id'])(
+    reply = elem_iq(stream, 'result', id=event.stanza['id'], from_='gadget.localhost')(
         elem(ns.DISCO_INFO, 'query')(
             elem('identity', category='collaboration', type='gadget', name='OLPC Gadget')(),
             elem('feature', var=ns.OLPC_BUDDY)(),
@@ -171,10 +172,10 @@ def send_reply_to_activity_view_request(stream, stanza, activities):
     stream.send(reply)
 
 def request_random_activity_view(q, stream, conn, max, id, activities):
-    requests_iface = dbus.Interface(conn, 'org.freedesktop.Telepathy.Connection.Interface.Requests')
+    requests_iface = dbus.Interface(conn, cs.CONN_IFACE_REQUESTS)
 
     call_async(q, requests_iface, 'CreateChannel',
-        { 'org.freedesktop.Telepathy.Channel.ChannelType':
+        { cs.CHANNEL_TYPE:
             'org.laptop.Telepathy.Channel.Type.ActivityView',
             'org.laptop.Telepathy.Channel.Interface.View.MaxSize': max
           })
@@ -199,7 +200,7 @@ def request_random_activity_view(q, stream, conn, max, id, activities):
     return return_event.value[0]
 
 def close_view(q, view, id):
-    chan_iface = dbus.Interface(view, 'org.freedesktop.Telepathy.Channel')
+    chan_iface = dbus.Interface(view, cs.CHANNEL)
     call_async(q, chan_iface, 'Close')
     event, _, _ = q.expect_many(
         EventPattern('stream-message', to='gadget.localhost'),
