@@ -6,12 +6,11 @@ import dbus
 
 from servicetest import call_async, EventPattern
 from gabbletest import exec_test, make_result_iq, acknowledge_iq, sync_stream
+import constants as cs
 
-from twisted.words.xish import domish, xpath
-from twisted.words.protocols.jabber.client import IQ
+from twisted.words.xish import xpath
 
-from util import (announce_gadget, properties_to_xml, parse_properties,
-    create_gadget_message, close_view, elem)
+from util import announce_gadget, create_gadget_message, elem
 import ns
 
 tp_name_prefix = 'org.freedesktop.Telepathy'
@@ -35,7 +34,8 @@ def test(q, bus, conn, stream):
     conn.Connect()
 
     _, iq_event, disco_event = q.expect_many(
-        EventPattern('dbus-signal', signal='StatusChanged', args=[0, 1]),
+        EventPattern('dbus-signal', signal='StatusChanged',
+            args=[cs.CONN_STATUS_CONNECTED, cs.CSR_REQUESTED]),
         EventPattern('stream-iq', to=None, query_ns='vcard-temp',
             query_name='vCard'),
         EventPattern('stream-iq', to='localhost', query_ns=ns.DISCO_ITEMS))
@@ -209,7 +209,7 @@ def test(q, bus, conn, stream):
     assert presence[handles['fred']] == (2, 'available', '')
 
     # close view 1
-    view1.Close(dbus_interface='org.freedesktop.Telepathy.Channel')
+    view1.Close(dbus_interface=cs.CHANNEL)
 
     # Eric's presence is changed as he's not in the view anymore
     event, _ = q.expect_many(
@@ -225,7 +225,7 @@ def test(q, bus, conn, stream):
     assert presence[handles['fred']] == (2, 'available', '')
 
     # close view 2
-    view2.Close(dbus_interface='org.freedesktop.Telepathy.Channel')
+    view2.Close(dbus_interface=cs.CHANNEL)
 
    # and now Fred is offline
     event, _ = q.expect_many(
