@@ -4,44 +4,26 @@ Test broken groups on the roster (regression test for fd.o #12791)
 
 import dbus
 
-from gabbletest import exec_test
-
-
-HT_CONTACT_LIST = 3
-HT_GROUP = 4
-
-
-def _expect_contact_list_channel(q, bus, conn, name, contacts):
-    event = q.expect('dbus-signal', signal='NewChannel')
-    path, type, handle_type, handle, suppress_handler = event.args
-    assert type == u'org.freedesktop.Telepathy.Channel.Type.ContactList', type
-    assert handle_type == HT_CONTACT_LIST, handle_type
-    inspected = conn.InspectHandles(handle_type, [handle])[0]
-    assert inspected == name, (inspected, name)
-    chan = bus.get_object(conn.bus_name, path)
-    group_iface = dbus.Interface(chan,
-        u'org.freedesktop.Telepathy.Channel.Interface.Group')
-    inspected = conn.InspectHandles(1, group_iface.GetMembers())
-    assert inspected == contacts, (inspected, contacts)
+from gabbletest import exec_test, expect_list_channel
+import constants as cs
+import ns
 
 def _expect_group_channel(q, bus, conn, name, contacts):
     event = q.expect('dbus-signal', signal='NewChannel')
     path, type, handle_type, handle, suppress_handler = event.args
-    assert type == u'org.freedesktop.Telepathy.Channel.Type.ContactList', type
-    assert handle_type == HT_GROUP, handle_type
+    assert type == cs.CHANNEL_TYPE_CONTACT_LIST, type
+    assert handle_type == cs.HT_GROUP, handle_type
     inspected = conn.InspectHandles(handle_type, [handle])[0]
     assert inspected == name, (inspected, name)
     chan = bus.get_object(conn.bus_name, path)
-    group_iface = dbus.Interface(chan,
-        u'org.freedesktop.Telepathy.Channel.Interface.Group')
-    inspected = conn.InspectHandles(1, group_iface.GetMembers())
+    group_iface = dbus.Interface(chan, cs.CHANNEL_IFACE_GROUP)
+    inspected = conn.InspectHandles(cs.HT_CONTACT, group_iface.GetMembers())
     assert inspected == contacts, (inspected, contacts)
 
 def test(q, bus, conn, stream):
     conn.Connect()
-    # q.expect('dbus-signal', signal='StatusChanged', args=[0, 1])
 
-    event = q.expect('stream-iq', query_ns='jabber:iq:roster')
+    event = q.expect('stream-iq', query_ns=ns.ROSTER)
     event.stanza['type'] = 'result'
 
     item = event.query.addElement('item')
@@ -82,19 +64,15 @@ def test(q, bus, conn, stream):
     # FIXME: this is somewhat fragile - it's asserting the exact order that
     # things currently happen in roster.c. In reality the order is not
     # significant
-    _expect_contact_list_channel(q, bus, conn, 'publish',
+    expect_list_channel(q, bus, conn, 'publish',
         ['amy@foo.com', 'bob@foo.com'])
-    _expect_contact_list_channel(q, bus, conn, 'subscribe',
+    expect_list_channel(q, bus, conn, 'subscribe',
         ['amy@foo.com', 'che@foo.com'])
-    _expect_contact_list_channel(q, bus, conn, 'known',
+    expect_list_channel(q, bus, conn, 'stored',
         ['amy@foo.com', 'bob@foo.com', 'che@foo.com'])
     _expect_group_channel(q, bus, conn, 'women', ['amy@foo.com'])
     _expect_group_channel(q, bus, conn, 'affected-by-fdo-12791', [])
     _expect_group_channel(q, bus, conn, 'men', ['bob@foo.com', 'che@foo.com'])
 
-    conn.Disconnect()
-    q.expect('dbus-signal', signal='StatusChanged', args=[2, 1])
-
 if __name__ == '__main__':
     exec_test(test)
-
