@@ -7,13 +7,15 @@ import dbus
 
 from twisted.words.xish import domish
 
-from servicetest import EventPattern
+from servicetest import EventPattern, wrap_channel, assertLength, assertEquals
 from gabbletest import acknowledge_iq, exec_test
+import constants as cs
+import ns
 
 def test(q, bus, conn, stream):
     conn.Connect()
 
-    event = q.expect('stream-iq', query_ns='jabber:iq:roster')
+    event = q.expect('stream-iq', query_ns=ns.ROSTER)
     # send back empty roster
     event.stanza['type'] = 'result'
     stream.send(event.stanza)
@@ -22,7 +24,7 @@ def test(q, bus, conn, stream):
         event = q.expect('dbus-signal', signal='NewChannel')
         path, type, handle_type, handle, suppress_handler = event.args
 
-        if type != u'org.freedesktop.Telepathy.Channel.Type.ContactList':
+        if type != cs.CHANNEL_TYPE_CONTACT_LIST:
             continue
 
         chan_name = conn.InspectHandles(handle_type, [handle])[0]
@@ -30,17 +32,16 @@ def test(q, bus, conn, stream):
         if chan_name == 'subscribe':
             break
 
-    # request subscription
-    chan = bus.get_object(conn.bus_name, path)
-    group_iface = dbus.Interface(chan,
-        u'org.freedesktop.Telepathy.Channel.Interface.Group')
-    assert group_iface.GetMembers() == []
-    handle = conn.RequestHandles(1, ['bob@foo.com'])[0]
-    group_iface.AddMembers([handle], '')
+    chan = wrap_channel(bus.get_object(conn.bus_name, path), 'ContactList')
+    assertLength(0, chan.Group.GetMembers())
 
-    event = q.expect('stream-iq', iq_type='set', query_ns='jabber:iq:roster')
+    # request subscription
+    handle = conn.RequestHandles(cs.HT_CONTACT, ['bob@foo.com'])[0]
+    chan.Group.AddMembers([handle], '')
+
+    event = q.expect('stream-iq', iq_type='set', query_ns=ns.ROSTER)
     item = event.query.firstChildElement()
-    assert item["jid"] == 'bob@foo.com'
+    assertEquals('bob@foo.com', item["jid"])
 
     acknowledge_iq(stream, event.stanza)
 
@@ -53,14 +54,9 @@ def test(q, bus, conn, stream):
 
     q.expect_many(
             EventPattern('dbus-signal', signal='MembersChanged',
-                args=['', [2], [], [], [], 0, 0]),
+                args=['', [handle], [], [], [], 0, 0]),
             EventPattern('stream-presence'),
             )
 
-    conn.Disconnect()
-
-    q.expect('dbus-signal', signal='StatusChanged', args=[2, 1])
-
 if __name__ == '__main__':
     exec_test(test)
-
