@@ -6,14 +6,17 @@ Test that Gabble responds to disco#info queries.
 from twisted.words.xish import domish
 
 from gabbletest import exec_test
+import constants as cs
 
 def test(q, bus, conn, stream):
     conn.Connect()
-    q.expect('dbus-signal', signal='StatusChanged', args=[0, 1])
+    q.expect('dbus-signal', signal='StatusChanged',
+            args=[cs.CONN_STATUS_CONNECTED, cs.CSR_REQUESTED])
 
     m = domish.Element((None, 'iq'))
     m['from'] = 'foo@bar.com'
     m['id'] = '1'
+    m['type'] = 'get'
     m.addElement(('http://jabber.org/protocol/disco#info', 'query'))
     stream.send(m)
 
@@ -21,9 +24,5 @@ def test(q, bus, conn, stream):
     elem = event.stanza
     assert elem['id'] == '1'
 
-    conn.Disconnect()
-    q.expect('dbus-signal', signal='StatusChanged', args=[2, 1])
-
 if __name__ == '__main__':
     exec_test(test)
-
