@@ -8,44 +8,45 @@ import dbus
 from twisted.words.xish import domish
 
 from gabbletest import exec_test
-from servicetest import EventPattern
+from servicetest import EventPattern, wrap_channel
+import constants as cs
 
 def test(q, bus, conn, stream):
     conn.Connect()
-    q.expect('dbus-signal', signal='StatusChanged', args=[0, 1])
+    q.expect('dbus-signal', signal='StatusChanged',
+            args=[cs.CONN_STATUS_CONNECTED, cs.CSR_REQUESTED])
 
     # <message type="chat"><body>hello</body</message>
     m = domish.Element((None, 'message'))
     m['from'] = 'foo@bar.com/Pidgin'
+    m['id'] = '1845a1a9-f7bc-4a2e-a885-633aadc81e1b'
     m['type'] = 'chat'
     m.addElement('body', content='hello')
     stream.send(m)
 
     event = q.expect('dbus-signal', signal='NewChannel')
-    text_chan = bus.get_object(conn.bus_name, event.args[0])
-    assert event.args[1] == u'org.freedesktop.Telepathy.Channel.Type.Text'
-    # check that handle type == contact handle
-    assert event.args[2] == 1
+    text_chan = wrap_channel(
+        bus.get_object(conn.bus_name, event.args[0]), 'Text', ['Messages'])
+    assert event.args[1] == cs.CHANNEL_TYPE_TEXT
+    assert event.args[2] == cs.HT_CONTACT
     foo_at_bar_dot_com_handle = event.args[3]
     jid = conn.InspectHandles(1, [foo_at_bar_dot_com_handle])[0]
     assert jid == 'foo@bar.com'
     assert event.args[4] == False   # suppress handler
 
     # Exercise basic Channel Properties from spec 0.17.7
-    channel_props = text_chan.GetAll(
-            'org.freedesktop.Telepathy.Channel',
-            dbus_interface=dbus.PROPERTIES_IFACE)
+    channel_props = text_chan.Properties.GetAll(cs.CHANNEL)
     assert channel_props.get('TargetHandle') == event.args[3],\
             (channel_props.get('TargetHandle'), event.args[3])
-    assert channel_props.get('TargetHandleType') == 1,\
+    assert channel_props.get('TargetHandleType') == cs.HT_CONTACT,\
             channel_props.get('TargetHandleType')
     assert channel_props.get('ChannelType') == \
-            'org.freedesktop.Telepathy.Channel.Type.Text',\
+            cs.CHANNEL_TYPE_TEXT,\
             channel_props.get('ChannelType')
-    assert 'org.freedesktop.Telepathy.Channel.Interface.ChatState' in \
+    assert cs.CHANNEL_IFACE_CHAT_STATE in \
             channel_props.get('Interfaces', ()), \
             channel_props.get('Interfaces')
-    assert 'org.freedesktop.Telepathy.Channel.Interface.Messages' in \
+    assert cs.CHANNEL_IFACE_MESSAGES in \
             channel_props.get('Interfaces', ()), \
             channel_props.get('Interfaces')
     assert channel_props['TargetID'] == jid,\
@@ -78,9 +79,10 @@ def test(q, bus, conn, stream):
     header, body = message
 
     assert header['message-sender'] == foo_at_bar_dot_com_handle, header
-    # the spec says that message-type "SHOULD be omitted for normal chat
+    # the spec says that message-type "MAY be omitted for normal chat
     # messages."
-    assert 'message-type' not in header, header
+    assert 'message-type' not in header or header['message-type'] == 0, header
+    assert header['message-token'] == '1845a1a9-f7bc-4a2e-a885-633aadc81e1b'
 
     assert body['content-type'] == 'text/plain', body
     assert body['content'] == 'hello', body
@@ -89,9 +91,7 @@ def test(q, bus, conn, stream):
     # PendingMessagesRemoved fires.
     message_id = header['pending-message-id']
 
-    dbus.Interface(text_chan,
-        u'org.freedesktop.Telepathy.Channel.Type.Text'
-        ).AcknowledgePendingMessages([message_id])
+    text_chan.Text.AcknowledgePendingMessages([message_id])
 
     removed = q.expect('dbus-signal', signal='PendingMessagesRemoved')
 
@@ -108,9 +108,7 @@ def test(q, bus, conn, stream):
         }
     ]
 
-    dbus.Interface(text_chan,
-        u'org.freedesktop.Telepathy.Channel.Interface.Messages'
-        ).SendMessage(greeting, dbus.UInt32(0))
+    sent_token = text_chan.Messages.SendMessage(greeting, dbus.UInt32(0))
 
     stream_message, sent, message_sent = q.expect_many(
         EventPattern('stream-message'),
@@ -129,17 +127,19 @@ def test(q, bus, conn, stream):
     assert len(sent_message) == 2, sent_message
     header = sent_message[0]
     assert header['message-type'] == 2, header # Notice
+    assert header['message-token'] == sent_token, header
     body = sent_message[1]
     assert body['content-type'] == 'text/plain', body
     assert body['content'] == u'what up', body
 
+    assert message_sent.args[2] == sent_token
+
     assert sent.args[1] == 2, sent.args # Notice
     assert sent.args[2] == u'what up', sent.args
 
 
     # Send a message using Channel.Type.Text API
-    dbus.Interface(text_chan,
-        u'org.freedesktop.Telepathy.Channel.Type.Text').Send(0, 'goodbye')
+    text_chan.Text.Send(0, 'goodbye')
 
     stream_message, sent, message_sent = q.expect_many(
         EventPattern('stream-message'),
@@ -157,8 +157,9 @@ def test(q, bus, conn, stream):
     sent_message = message_sent.args[0]
     assert len(sent_message) == 2, sent_message
     header = sent_message[0]
-    # if it would be Normal, message-type SHOULD be omitted
-    assert 'message-type' not in header, header
+    # the spec says that message-type "MAY be omitted for normal chat
+    # messages."
+    assert 'message-type' not in header or header['message-type'] == 0, header
     body = sent_message[1]
     assert body['content-type'] == 'text/plain', body
     assert body['content'] == u'goodbye', body
@@ -166,10 +167,5 @@ def test(q, bus, conn, stream):
     assert sent.args[1] == 0, sent.args # message type normal
     assert sent.args[2] == u'goodbye', sent.args
 
-
-    conn.Disconnect()
-    q.expect('dbus-signal', signal='StatusChanged', args=[2, 1])
-
 if __name__ == '__main__':
     exec_test(test)
-
