@@ -12,6 +12,7 @@ from gabbletest import exec_test, acknowledge_iq, sync_stream
 
 import constants as cs
 import tubetestutil as t
+import ns
 
 def props(ct, extra=None):
     ret = { cs.TARGET_HANDLE_TYPE: cs.HT_CONTACT,
@@ -23,15 +24,14 @@ def props(ct, extra=None):
     return ret
 
 def test(q, bus, conn, stream):
-    echo_path = t.set_up_echo("")
-
     conn.Connect()
 
     _, vcard_event, roster_event = q.expect_many(
-        EventPattern('dbus-signal', signal='StatusChanged', args=[0, 1]),
+        EventPattern('dbus-signal', signal='StatusChanged',
+            args=[cs.CONN_STATUS_CONNECTED, cs.CSR_REQUESTED]),
         EventPattern('stream-iq', to=None, query_ns='vcard-temp',
             query_name='vCard'),
-        EventPattern('stream-iq', query_ns='jabber:iq:roster'))
+        EventPattern('stream-iq', query_ns=ns.ROSTER))
 
     acknowledge_iq(stream, vcard_event.stanza)
 
@@ -81,8 +81,10 @@ def test(q, bus, conn, stream):
     e = q.expect('dbus-error', method='OfferDBusTube').error
     assert e.get_dbus_name() == cs.NOT_AVAILABLE, e.get_dbus_name()
 
+    address = t.create_server(q, cs.SOCKET_ADDRESS_TYPE_UNIX)
+
     call_async(q, tubes, 'OfferStreamTube', 'echo', {},
-        cs.SOCKET_ADDRESS_TYPE_UNIX, dbus.ByteArray(echo_path),
+        cs.SOCKET_ADDRESS_TYPE_UNIX, address,
         cs.SOCKET_ACCESS_CONTROL_LOCALHOST, "")
     e = q.expect('dbus-error', method='OfferStreamTube').error
     assert e.get_dbus_name() == cs.NOT_AVAILABLE, e.get_dbus_name()
@@ -94,20 +96,18 @@ def test(q, bus, conn, stream):
         {cs.STREAM_TUBE_SERVICE: "newecho"}))
     st_chan = bus.get_object(conn.bus_name, st_path)
     st = dbus.Interface(st_chan, cs.CHANNEL_TYPE_STREAM_TUBE)
-    call_async(q, st, 'OfferStreamTube', cs.SOCKET_ADDRESS_TYPE_UNIX,
-        dbus.ByteArray(echo_path), cs.SOCKET_ACCESS_CONTROL_LOCALHOST, "", {})
-    e = q.expect('dbus-error', method='OfferStreamTube').error
+    call_async(q, st, 'Offer', cs.SOCKET_ADDRESS_TYPE_UNIX,
+        address, cs.SOCKET_ACCESS_CONTROL_LOCALHOST, {})
+    e = q.expect('dbus-error', method='Offer').error
     assert e.get_dbus_name() == cs.NOT_AVAILABLE, e.get_dbus_name()
 
     dt_path, _ = requests.CreateChannel(props(cs.CHANNEL_TYPE_DBUS_TUBE,
         { cs.DBUS_TUBE_SERVICE_NAME: "com.newecho" }))
     dt_chan = bus.get_object(conn.bus_name, dt_path)
     dt = dbus.Interface(dt_chan, cs.CHANNEL_TYPE_DBUS_TUBE)
-    call_async(q, dt, 'OfferDBusTube', {})
-    e = q.expect('dbus-error', method='OfferDBusTube').error
+    call_async(q, dt, 'Offer', {}, cs.SOCKET_ACCESS_CONTROL_CREDENTIALS)
+    e = q.expect('dbus-error', method='Offer').error
     assert e.get_dbus_name() == cs.NOT_AVAILABLE, e.get_dbus_name()
 
-    conn.Disconnect()
-
 if __name__ == '__main__':
     exec_test(test)
